function [ objCtrl ] = XFqpmaxDec(objCtrl,Q,R,options)
% XFQPMAXDEC designs an ellipsoidal terminal constraint rectangular in the dimension of subsystems and set a quadratic block-diagonal cost function with quadratic terminal penalty
%
% [ objCtrl ] = XFqpmaxDec(objCtrl,Q,R,options)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Designs an ellipsoidal terminal constraint rectangular in the dimension of 
% subsystems and set a quadratic block-diagonal cost function with quadratic 
% terminal penalty
% 
% The cost function is 
% J = \sum{i=0}^(N-1)(x(i)'Q x(i)+u(i)'Ru(i))+x(N)'S x(N)
%
% The terminal set is an ellipsoidal set centered in the origin, described 
% by the block-diagonal matrix S. The ellipsoid is computed as the maximal 
% ellipsoid that guarantees state and input constraints and asymptotic 
% stability of the origin. 
%
%
% USAGE:
%
% [ objCtrl ] = objCtrl.XFqpmaxDec(Q,R)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% objCtrl - cenmpc object
% Q and R - matrices such that Q>=0 and R>0.
% options from YALMIP, i.e. sdpsettings object (optional)
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objCtrl - updated cenmpc object
%
%

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Stefano Riverso, Alberto Battocchio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, alberto.battocchio01@universitadipavia.it,
%     giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if isempty(objCtrl.N)
        warning('XFQPMAX: PnP controller is empty') %#ok<WNTAG>
        return
    end

    if nargin~=3 && nargin~=4
        error('XFQPMAX: input parameters. see help XFQPMAX')
    end
    
    A = objCtrl.A;
    B = objCtrl.B;
    n = size(A,1);
    m = size(B,2);
    
    Cx = objCtrl.Hx;
    cx = objCtrl.Kx;
    Cu = objCtrl.Hu;
    cu = objCtrl.Ku;
    
    if isempty(Cx) && isempty(Cu)
        error('XFQPMAX: there are no state and input constraints')
    end
    
    if any(Cx*zeros(n,1)>cx)
        error('XFQPMAX: 0 is not inside polytope Xc')
    end
    
    if ~isempty(Cu) && any(Cu*zeros(m,1)>cu)
        error('XFQPMAX: 0 is not inside polytope V')
    end
    
    if n~=size(Q,1)
        error('XFQPMAX: dimensions of A and Q')
    end
    if any(any(Q~=Q')) || any(eig(Q)<0)
        error('XFQPMAX: Q<0, Q is not symmetric')
    end
    
    if m~=size(R,1)
        error('XFQPMAX: dimensions of B and R')
    end
    if any(any(R~=R')) || any(eig(R)<0)
        error('XFQPMAX: R<0, R is not symmetric')
    end
    
    if nargin==3
        options = sdpsettings;
    end
    
    mucen = size(objCtrl.B,2)-sum(objCtrl.mi);
    
    maskX = createMaskDec(objCtrl.ni);
    maskU = createMaskDec(objCtrl.mi); maskU = [ maskU ones(size(maskU,1),mucen) ; ones(mucen,size(maskU,2)) ones(mucen) ];
    maskF = createMaskDec(objCtrl.mi,objCtrl.ni); maskF = [ maskF ; ones(mucen,size(maskF,2)) ];
    Q     = Q.*maskX;
    R     = R.*maskU;
    
    Y = sdpvar(n,n,'symmetric');
    F = sdpvar(m,n);
    CONS = [ Y>=0 ; Y.*maskX==Y ; F.*maskF==F ];
    CONS = [ CONS ; [    -Y         A*Y+B*F    zeros(n)     zeros(n,m)  ;
                      (A*Y+B*F)'      -Y          Y             F'      ;
                       zeros(n)        Y       -inv(Q)      zeros(n,m)  ;
                      zeros(m,n)       F      zeros(m,n)      -inv(R)   ]<=0  ];
                  
    if ~isempty(Cu)
        llow = zeros(m,1);
        uup  = zeros(m,1);
        for i=1:m
            temp    = lpbound([zeros(1,i-1) -1 zeros(1,m-i)],Cu,cu,options);
            uup(i)  = temp(i);
            temp    = lpbound([zeros(1,i-1)  1 zeros(1,m-i)],Cu,cu,options);
            llow(i) = temp(i);
        end
        CONS = [ CONS ; [diag(min([uup abs(llow)],[],2)).^2 F;F' Y]>=0 ];
    end
    
    for k = 1:size(Cx,1)
        Cx(k,:) = Cx(k,:)/cx(k);
        cx(k)   = 1;
        CONS = [ CONS ; -Cx(k,:)*Y*Cx(k,:)'+1 >= 0 ];
    end
    
    info = solvesdp(CONS,-geomean(Y),options);
    if info.problem~=0
        cset = checkset(CONS);
        if info.problem==4 && any(cset(3:end)<=-1e-6)
            error(info.info)
        end
    end
    
    S    = sparse(inv(double(Y)));
    N    = objCtrl.N;

    yalmip('clear');
    objCtrl.J  = @(xc,v,xcref,vref) (sum(diag((xc(:,1:N)-xcref(:,1:N))'*Q*(xc(:,1:N)-xcref(:,1:N)))) + sum(diag((v-vref)'*R*(v-vref))) + (xc(:,N+1)-xcref(:,N+1))'*S*(xc(:,N+1)-xcref(:,N+1)) );
    objCtrl.Xf = @(xcN,xcref)( xcN'*S*xcN <= 1 );
    
    objCtrl.TerminalSet  = true;
    objCtrl.CostFunction = true;
    
end



function val = lpbound(f,H,K,options)
    val = sdpvar(size(H,2),1);
    info = solvesdp(H*val<=K,f*val,options);
    if info.problem~=0
        error(info.info)
    end
    val = double(val);
end