function [ u, diagnostics ] = uRH(objCtrl,x0,din,xrefin,urefin,options)
% URH compute the control action
%
% [ u, diagnostics ] = objCtrl.uRH(x0,din,xrefin,urefin,options)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Compute the control action given by the solution of MPC optimization problem
%
% USAGE:
%
% [ u              ] = objCtrl.uRH(x0)
% [ u              ] = objCtrl.uRH(x0,din)
% [ u              ] = objCtrl.uRH(x0,din,xrefin,urefin)
% [ u              ] = objCtrl.uRH(x0,din,xrefin,urefin,options)
% [ u, diagnostics ] = objCtrl.uRH(x0,din,xrefin,urefin,options)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% objCtrl - cenmpc object
% x0 - current state of the subsystem equipped with the pnpmpc object
% din - vector of the exogenous inputs acting on the subsystem
% xrefin - set-points for the state. It could be a vector of n states, or
%           a matrix of n x N+1 elements.
% urefin - set-points for the state. It could be a vector of m states, or
%          a matrix of m x N elements.
% options from YALMIP, i.e. sdpsettings object (optional)
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% u - values of the input variables
% diagnostics - struct variable with field 'mpc' and 'uInv'. 
%      Struct 'mpc' contains:
%          * in 'info' the variable returned by solvesdp for the mpc problem, 
%          * in 'J' value of the cost function
%          * in 'x' and 'u' the optimizers of the mpc problem
%      Struct 'uInv' contains:
%          * in 'info' the variable returned by solvesdp to compute the
%            control action inside the RCI set stored in objCtrl, when
%            apply tube-based MPC
%
    
% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Stefano Riverso, Alberto Battocchio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, alberto.battocchio01@universitadipavia.it,
%     giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------
    
    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if isempty(objCtrl.N)
        warning('URH: centralized MPC controller is empty') %#ok<WNTAG>
        return
    end
    
    if ~objCtrl.TerminalSet || ~objCtrl.CostFunction
        error('URH: cost function and terminal set not defined')
    end
    
    if nargin<2 || nargin>6
        error('URH: parameters in input. see help URH')
    end
    
    if size(x0,1)~=size(objCtrl.A,1)
        error('URH: dimension of x0 is wrong')
    end
    
    
    d = zeros(size(objCtrl.Exo,2),objCtrl.N);
    if nargin>=3
        [dr, dc] = size(din);
        if dr~=numel(objCtrl.Exo)
            error('URH: d must be a vector of %d or a matrix %d x %d',numel(objCtrl.Exo),numel(objCtrl.Exo),objCtrl.N)
        end
        if dc>1 && dc ~= objCtrl.N
            error('URH: d must be a vector of %d or a matrix %d x %d',numel(objCtrl.Exo),numel(objCtrl.Exo),objCtrl.N)
        else
            d = din;
        end
        
        if isvector(din)
            d = repmat(din,1,objCtrl.N);
        end
    end 
    
    
    xcref = zeros(size(objCtrl.A,1),objCtrl.N+1);
    if nargin>=4
        [xr, xc] = size(xrefin);
        if xc>1
            if xr ~= size(objCtrl.A,1) || xc ~= objCtrl.N+1
                error('URH: xcref must be a matrix %d x %d',size(objCtrl.A,1),objCtrl.N+1)
            end
            
            if ~isempty (objCtrl.Hxc)
                if ~all(all(objCtrl.Hxc*xrefin <= repmat(objCtrl.Kxc,1,objCtrl.N+1)))
                    error('URH: xcref does not respect the state constraints')
                end
            end
            xcref = xrefin;
        end
        
        if isvector(xrefin)
            if ~isempty(objCtrl.Hxc)
                if ~all(objCtrl.Hxc*xrefin<=objCtrl.Kxc)
                    error('URH: xcref does not respect the state constraints')
                end
            end
            xcref = repmat(xrefin,1,objCtrl.N+1);
        end
    end
    
    
    vref = zeros(size(objCtrl.B,2),objCtrl.N);
    if nargin>=5
        [ur, uc] = size(urefin);
        if uc>1
            if ur ~= size(objCtrl.B,1) || uc ~= objCtrl.N
                error('URH: vref must be a matrix %d x %d',size(objCtrl.B,1),objCtrl.N)
            end
            if ~isempty (objCtrl.Hv)
                if ~all(all(objCtrl.Hv*urefin <= repmat(objCtrl.Kv,1,objCtrl.N)))
                    error('URH: vref does not respect the input constraints')
                end
            end
            vref = urefin;
        end
        
        if isvector(urefin)
            if ~isempty(objCtrl.Hv)
                if ~all(objCtrl.Hv*urefin<=objCtrl.Kv)
                    error('URH: vref does not respect the input constraints')
                end
            end
            vref = repmat(urefin,1,objCtrl.N);
        end
    end
    
    if nargin<6
        options = sdpsettings;
    end
    
    xcopt  = sdpvar(size(objCtrl.A,1),objCtrl.N+1,'full');
    vopt   = sdpvar(size(objCtrl.B,2),objCtrl.N,'full');
    if isa(objCtrl.Z,'parameterizedRCI')
        lambda = sdpvar(objCtrl.Z.k,size(objCtrl.Z.x0,1),'full');
    else
        lambda = [];
    end
        
    CONS   = [  objCtrl.dynState(xcopt(:),vopt(:),d(:))        ;
                objCtrl.stateCons(xcopt)                       ;
                objCtrl.inputCons(vopt)                        ;
                objCtrl.Xf(xcopt(:,objCtrl.N+1),xcref(:,end))  ;
                objCtrl.initialCons(x0,xcopt(:,1),lambda)      ];
    
    info = solvesdp( CONS , objCtrl.J(xcopt,vopt,xcref,vref) , options );
    
    if info.problem~=0
        error(info.info);
    end
    
    v   = double(vopt(:,1));
    xc  = double(xcopt(:,1));
    
    diagnostics = [];
    if nargout==2
        diagnostics.mpc    = info;
        diagnostics.mpc.x  = double(xcopt);
        diagnostics.mpc.u  = double(vopt);
        diagnostics.mpc.J  = objCtrl.J(diagnostics.mpc.xc,diagnostics.mpc.v,xcref,vref);
    end
    
    if isa(objCtrl.Z,'parameterizedRCI')
        [ uz , infouInv ] = objCtrl.Z.uInv(x0-xc,options);
        if infouInv.problem~=0
            error(infouInv.info)
        end
        uz = double(uz);
        diagnostics.uInv = infouInv;
    else
        uz = 0;
    end
     
    
    diagnostics.u.v  = v;
    diagnostics.u.uz = uz;
    
    u  = v + uz;
    
end