function [ objCtrl ] = zeroTerminal(objCtrl,Q,R)
% ZEROTERMINAL designs a zero terminal constraint and set a quadratic cost function without terminal penalty
%
% [ objCtrl ] = zeroTerminal(objCtrl,Q,R)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Design a zero terminal constraint and set a quadratic cost function 
% without terminal penalty
% 
% The cost function is
% J = \sum{i=0}^(N-1)(x(i)'Q x(i)+u(i)'Ru(i))
%
% The terminal set is
% \hat x(N) = xcrefin, with xcrefin equal to the state set-point
%
% USAGE:
%
% [ objCtrl ] = objCtrl.zeroTerminal(Q,R)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% objCtrl - cenmpc object
% Q and R - matrices such that Q>=0 and R>0.  
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objCtrl - updated cenmpc object
%
%

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Stefano Riverso, Alberto Battocchio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, alberto.battocchio01@universitadipavia.it,
%     giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if isempty(objCtrl.N)
        warning('ZEROTERMINAL: centralized MPC controller is empty') %#ok<WNTAG>
        return
    end
    
    if nargin~=3
        error('ZEROTERMINAL: input parameters. see help ZEROTERMINAL')
    end
    
    n = size(objCtrl.A,1);
    m = size(objCtrl.B,2);
    N = objCtrl.N;
    
    if n~=size(Q,1)
        error('ZEROTERMINAL: dimensions of A and Q')
    end
    if any(any(Q~=Q')) || any(eig(Q)<0)
        error('ZEROTERMINAL: Q<0, Q is not symmetric')
    end
    
    if m~=size(R,1)
        error('ZEROTERMINAL: dimensions of B and R')
    end
    if any(any(R~=R')) || any(eig(R)<0)
        error('ZEROTERMINAL: R<0, R is not symmetric')
    end
    
    objCtrl.J  = @(xc,v,xcref,vref) (sum(diag((xc(:,1:N)-xcref(:,1:N))'*Q*(xc(:,1:N)-xcref(:,1:N)))) + sum(diag((v-vref)'*R*(v-vref))) );
    objCtrl.Xf = @(xcN,xcref)( xcN == xcref );
    
    objCtrl.TerminalSet  = true;
    objCtrl.CostFunction = true;
    
end