function [ H K F ] = doubleHK(obj)
% DOUBLEHK Returns the polytope and H-representation of an epsilon-mRPI object
%
% [ H K F ] = doubleHK(obj)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% The H-representation of a polytope F is F = { x | Hx<=K }
%
% USAGE:
%
% [ H K F ] = obj.doubleHK
%   F       = obj.doubleHK
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% epsilon_mRPI object
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% Matrices H and K
% F updated epsilon_mRPI object with F_epsilon field as a polytope object
%
%
% see also EPSILON_MRPI POLYHEDRON

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, giancarlo.ferrari@unipv.it
% (C) 2012 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if isa(obj.W,'zonotope')
        error('DOUBLEHK: W is a zonotope, the method does not convert the set in a polytopic set')
    end
    
    if isempty(obj.F_epsilon)
        vr = obj.vertMink.V(:,:,1)';
        s  = obj.s;
        n  = size(obj.A,2);
        
        for i=2:s
            xk = obj.vertMink.V(:,:,i);
            if all(all(xk==0))
                continue;
            end
            vr = repmat(vr,size(xk,2),1) + reshape(repmat(xk,size(vr,1),1),n,[])';
            tmp.V = vr;
            tmp = cddmex('reduce_v',tmp);
            vr = tmp.V;
        end
        
        P = Polyhedron(full(vr));
        P = P.computeHRep;
        
        H = P.A;
        K = P.b;
        
        F = obj;
        F.F_epsilon = P;
        
        if nargout==1
            H = F;
        end
    else
        F = obj;
        if nargout==1
            H = F;
            return
        end
        H = F.F_epsilon.A;
        K = F.F_epsilon.b;
    end
    
end