classdef epsilon_mRPI
% EPSILON_MRPI compute an outer-approximation of the minimal Robust Positively Invariant Set as proposed in [1]
% 
% F = epsilon_mRPI(A,W,epsilon,X)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return an outer-approximation of the minimal Robust Positively Invariant 
% Set as proposed in [1]
% 
% R = EPSILON_MRPI(A,W,epsilon,X)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - Matrix A of the linear system x(t+1)=Ax(t) with n states
% - W is a Polyhedron or zonotope object
% - epsilon is the precision of the outer-approximation
% - X is a Polyhedron object, with H-representation and V-representation
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% F is a epsilon_mRPI object
%
% ---------------------------------------------------------------------------
% PROPERTIES
% ---------------------------------------------------------------------------
% A, W, epsilon, X : the inputs are stored in the epsilon_mRPI object
% alpha, s : parameters of the Algorithm
% F_epsilon: Polyhedron object of the epsilon-mRPI
% F_epsilonZ: zonotope object of the epsilon-mRPI
%
% ---------------------------------------------------------------------------
% METHODS
% ---------------------------------------------------------------------------
% DOUBLEHK: return the H-representation of the epsilon-mRPI
% DOUBLEPG: return the Generators-representation of the epsilon-mRPI
% ISINSIDE: checks if a given point x lies in the interior of the epsilon-mRPI
% PLOT: plot the robust control invariant set
% 
%
% [1] S. V. Rakovi?, E. C. Kerrigan, K. I. Kouramas, and D. Q. Mayne, ?Invariant 
% approximations of the minimal robust positively invariant set,? IEEE Trans. 
% Automat. Contr., vol. 50, no. 3, pp. 406?410, 2005.
    
% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Stefano Riverso, Alberto Battocchio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, alberto.battocchio01(at)universitadipavia.it,
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    properties ( SetAccess = private , Hidden = false )
        A          ;
        W          ;
        epsilon    ;
        alpha      ;
        s          ;
    end
    
    properties ( SetAccess = private , Hidden = false )
        F_epsilon  ;
        F_epsilonZ ;
    end
    
    properties ( SetAccess = private , Hidden = true )
        vertMink   ;
    end
    
    methods
        
        function obj = epsilon_mRPI(A,W,epsilon,X)
            
            global pnpmpcToolboxGlobalOptions;
            if ~isstruct(pnpmpcToolboxGlobalOptions),
                pnpmpc_toolbox_err;
            end
            
            if nargin<3
                error('ERROR: at least 3 parameters epsilon_mRPI(A,W,epsilon). See help epsilon_mRPI')
            end
            
            n = size(A,1);
            
            if any(abs(eig(A))>=1)
                error('ERROR: A is not Schur')
            end
            if n~=size(A,2)
                error('ERROR: dimensions of A')
            end
            if epsilon<=0
                error('ERROR: EPSILON is <=0 ')
            end
            if ~isa(W,'Polyhedron') && ~isa(W,'zonotope')
                error('ERROR: W is not a Polyhedron or a zonotope')
            end
            if n~=W.Dim
                error('ERROR: dimensions of A and set W')
            end
            if ~W.isInside(zeros(n,1))
                error('ERROR: 0 is not inside set W')
            end
            if nargin>=4
                if ~(isa(X,'Polyhedron') && n==X.Dim)
                    error('ERROR: dimensions of set X')
                end
            else
                X = [];
            end
            
            s             = 0          ;
            alpha         = inf        ;
            M             = 0          ;
            M_sum         = zeros(n,2) ;
            obj.vertMink  = []         ;
            
            if ~isa(W,'zonotope') && isa(W,'Polyhedron')
                Fw = W.A;
                Gw = W.b;
                while alpha > epsilon/(epsilon+M)
                    % step 3
                    s = s + 1;
                    % step 4
                    alpha_candidate = zeros(1,size(Fw,1));
                    for i=1:size(Fw,1)
                        tmpExtreme = W.extreme((A^s)'*Fw(i,:)');
                        alpha_candidate(i) = tmpExtreme.supp/Gw(i);
                    end
                    alpha = max(alpha_candidate);
                    % step 5
                    for i=1:n
                        e = zeros(n,1);
                        e(i) = 1;
                        tmpExtreme = W.extreme((A^(s-1))'*e);
                        M_sum(i,1) = M_sum(i,1) + tmpExtreme.supp;
                        tmpExtreme = W.extreme(-(A^(s-1))'*e);
                        M_sum(i,2) = M_sum(i,2) + tmpExtreme.supp;
                    end
                    M = max(max(M_sum));
                end
            elseif isa(W,'zonotope')
                px              = W.p;
                Gx              = W.G;                
                index           = sum(abs(Gx))~=0;
                nvert           = sum(index);
                vertex          = zeros(2^nvert,size(Gx,2));
                vertex(:,index) = extreme(polytope([eye(nvert);-eye(nvert)],ones(2*nvert,1)));
                while alpha > epsilon/(epsilon+M)
                    % step 3
                    s = s + 1;
                    % step 4
                    alpha_candidate = zeros(1,size(vertex,1));
                    for i=1:size(vertex,1)
                        alpha_candidate(i) = W.extreme((A^s)'*(px+Gx*vertex(i,:)'))/W.extreme(px+Gx*vertex(i,:)');
                    end
                    alpha = max(alpha_candidate);
                    % step 5
                    for i=1:n
                        e = zeros(n,1);
                        e(i) = 1;
                        M_sum(i,1) = M_sum(i,1) + W.extreme(A^(s-1)'*e);
                        M_sum(i,2) = M_sum(i,2) + W.extreme(-(A^(s-1))'*e);
                    end
                    M = max(max(M_sum));
                end
            else
                error('ERROR: set W is not a Polyhedron or zonotope set')
            end
            
            % epsilon-mRPI inside X
            if ~isempty(X)
                HX = X.A;
                KX = X.b;
                for i=1:size(HX,1)
                    sumSup = 0;
                    for j=0:s-1
                        temp = W.extreme((A^j)'*HX(i,:)');
                        sumSup = sumSup + temp.supp;
                    end
                    if sumSup>(1-alpha)*KX(i)
                        error('ERROR: epsilon_mRPI is not inside X')
                    end
                end
            end
            
            % create points for implicit representation of mRPI
            if ~isa(W,'zonotope')
                W = W.computeVRep;
                vertW    = W.V;
                vertMink = zeros(n,size(vertW,1),s);
                vertMink(:,:,1) = vertW';
                A_mul    = eye(n);
                for i=1:s-1
                    A_mul = A_mul*A;
                    vertMink(:,:,i+1) = reshape(sum(repmat(A_mul,size(vertW,1),1).*reshape(repmat(vertW,1,n)',n,[])',2),2,[]);
                end
                obj.vertMink.V = (1-alpha)^-1*vertMink;
            else
                ng         = ngenerator(W);
                GG         = zeros(n,ng*s);
                A_mul      = eye(n);
                pp         = px;
                GG(:,1:ng) = Gx;
                for i=1:s-1
                    A_mul = A_mul*A;
                    pp = pp + A_mul*px;
                    GG(:,ng*i+1:ng*(i+1)) = A_mul*Gx;
                end
                obj.vertMink.p = (1-alpha)^-1*pp;
                obj.vertMink.G = (1-alpha)^-1*GG;
            end
            
            obj.A         = A        ;
            obj.W         = W        ;
            obj.alpha     = alpha    ;
            obj.s         = s        ;
            obj.epsilon   = epsilon  ;
            
        end
    end
    
    
end