function [ BB,  obj ] = bounding_box(obj,options)
% BOUNDING_BOX Computes a bounding box for a localControlLyapunov set
%
% BB = bounding_box(obj,options)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Compute a bounding box for given localControlLyapunov set object
%
%   BB = BOUNDING_BOX(obj) returns a cell array with elements
%   {HB,KB} such that BB = { x | HB*x<=KB }
%
% USAGE:
%   [ BB obj ] = bounding_box(obj,options)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% obj - localControlLyapunov object
% options - sdpsettings object for YALMIP
%
% ---------------------------------------------------------------------------
% OUTPUT                                                                                                    
% ---------------------------------------------------------------------------
% BB - Cell array with elements {HB,KB} such that BB = { x | HB*x<=KB }
% obj - localControlLyapunov object updated with bounding box matrices
%
% see also LOCALCONTROLLYAPUNOV

% Copyright is with the following author(s):
%
% (C) 2012 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin~=2
        options = sdpsettings;
    end
    
    if isempty(obj.BB)
        
        n = size(obj.A,2);
        
        HBB = [ eye(n,n) ; -eye(n,n) ];
        KBB = ones(2*n,1);
        
        for i = 1:n
            e          = zeros(n,1);
            e(i)       = 1;
            KBB(i,1)   = obj.supportFunction(e,options);
            e(i)       = -1;
            KBB(n+i,1) = obj.supportFunction(e,options);
        end
        
        BB     = {HBB,KBB};
        obj.BB = {HBB,KBB};
        
    else
        BB     = obj.BB;
    end
    
end