function [ H, K, L ] = double(obj,option)
% DOUBLE Return the H-representation of a local Control Lyapunov set (LCL)
%
% [ H K L ] = double(obj)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Return the H-representation of the LCL set LP, i.e. matrices H and K such
% that
% LP = { x | Hx<=K }
%
% USAGE:
%
% [ H K L ] = obj.double
%   L       = obj.double
% [ H K L ] = obj.double(option);
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% localControlLyapunov object
% option=true, compute minimal H-representation (default false)
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% Matrices H and K
% L updated localControlLyapunov, containing the Polyhedron object LP
%
%
% see also LOCALCONTROLLYAPUNOV POLYHEDRON

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, giancarlo.ferrari@unipv.it
%
% (C) 2012 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin==1 || isempty(option) || option~=true
        option = false;
    end
    
    if isempty(obj.LP)
        vr = obj.x0(2:end,:);
        k  = obj.k;
        n  = size(obj.A,2);
        
        tmp = Polyhedron(full(vr));
        
        for i=1:k-1
            xk = obj.x(:,2:end,i);
            if all(all(xk==0))
                continue;
            end
            vr = repmat(vr,size(xk,2),1) + reshape(repmat(xk,size(vr,1),1),n,[])';
            tmp = Polyhedron(full(vr));
            vr = tmp.V;
        end
        
        tmp = full(obj.beta)^-1*( obj.bx + tmp );
        if option
            tmp = tmp.minHRep;
        else
            tmp = tmp.computeHRep;
        end
        
        H = tmp.A;
        K = tmp.b;
        
        L    = obj;
        L.LP = tmp;        
    else
        if option && ~obj.LP.irredundantHRep
            obj.LP = [];
            [ H, K, L ] = obj.double(true);
        else
            L = obj;
            H = L.LP.A;
            K = L.LP.b;
        end
    end
    
    if nargout==1
        H = L;
    end
    
end