function [ H, K, UinvSet ] = doubleUinv(obj)
% DOUBLE Returns the H-representation of the polytope in which the control law of a local Control Lyapunov set (LCL) lies.
%
% [ H K UinvSet ] = doubleUinv(obj)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Return the H-representation of the polytope LCL such that
% UinvSet = { x | H u(x)<=K }
% where u(x) is the control law of an LCL
%
% USAGE:
%
% [ H K UinvSet ] = obj.doubleUinv
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% localControlLyapunov object
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% Matrices H and K
% UinvSet Polyhedron object 
%
%
% see also LOCALCONTROLLYAPUNOV POLYHEDRON

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, giancarlo.ferrari@unipv.it
%
% (C) 2012 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end    

    vr = obj.u(:,2:end,1)';
    k  = obj.k;
    m  = size(obj.B,2);
    
    tmp = Polyhedron(full(vr));
    
    for i=2:k
        uk = obj.u(:,2:end,i);
        if all(all(uk==0))
            continue;
        end  
        vr = repmat(vr,size(uk,2),1) + reshape(repmat(uk,size(vr,1),1),m,[])';
        tmp = Polyhedron(full(vr));
        vr = tmp.V;
    end
    
    tmp = full(obj.beta)^-1*( obj.bu + tmp );
    UinvSet = tmp.computeHRep;
        
    H = UinvSet.A;
    K = UinvSet.b;
    
    if nargout==1
        H = UinvSet;
    end
    
end