classdef localControlLyapunov
% LOCALCONTROLLYAPUNOV compute a parameterized Local Control Invariant and Stable Set for a Linear System as proposed in [1]
% 
% L = localControlLyapunov(A,B,k,X,U,x0,xref,uref,options)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return a parameterized Local Control Invariant and Stable Set
% 
% L = LOCALCONTROLLYAPUNOV(A,B,k,X,U,x0)
% L = LOCALCONTROLLYAPUNOV(A,B,k,X,U,x0,xref,uref)
% L = LOCALCONTROLLYAPUNOV(A,B,k,X,U,x0,xref,uref,options)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - Matrices A and B of the linear system x(t+1)=Ax(t)+Bu(t) with n states
% and m inputs
% - k must be >= of the controllability index of the pair (A,B)
% - X is a cell array defined as {cx,dx} where cx*x<=dx
% - U is a cell array defined as {cu,du} where cu*u<=du
% - x0 is a matrix of dimensions q x n, with q number of points
% - xref and uref to set the center of the local control invariant and stable
%   set otherwise they are optimized
% - options from YALMIP, i.e. sdpsettings object (optional)
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% L is a localControlLyapunov object
%
% ---------------------------------------------------------------------------
% PROPERTIES
% ---------------------------------------------------------------------------
% A, B, k, cx, dx, cu, du, x0, xref, uref : the inputs are stored in the
%                                           localControlLyapunov object
% x, u, beta, lambda, bx, bu : optimized values of the procedure
% info : object with information of the optimization problem (see solvesdp
%        of YALMIP). An LP solver is needed.
% LP: Polyhedron object of the local Control Lyapunov set, with
%     H-representation and V-representation
%
% ---------------------------------------------------------------------------
% METHODS
% ---------------------------------------------------------------------------
% ISINSIDE: checks if a given point x lies in the interior of the RCI
% PLOT: plot the robust control invariant set
% UINV: compute the control input to guarantee robust control invariance
% SUPPORTFUNCTION: Support function computed in c of parameterized Local 
%                  Control Invariant and Stable Set
% BOUNDING_BOX: Compute a bounding box for the given Local Control
%               Invariant Set
% 
%
% [1] S. V. Rakovic and M. Baric, Parameterized Robust Control Invariant
% Sets for Linear Systems: Theoretical Advances and Computational Remarks,
% IEEE Transactions on Automatic Control, vol. 55, n. 7, pp. 1599-1614, 2010
    
% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------
    
    properties ( SetAccess = private , Hidden = false )
        A      ;
        B      ;
        k      ;
        cx     ;
        dx     ;
        cu     ;
        du     ;
        x0     ;
        xref   ;
        uref   ;
        x      ;
        u      ;
        beta   ;
        lambda ;
        bx     ;
        bu     ;
        info   ;
    end
    
    properties ( SetAccess = private , Hidden = false )
        LP     ;
    end
    
    properties ( SetAccess = private , Hidden = true )
        BB     ;
    end
    
    
    methods
        
        function obj = localControlLyapunov(A,B,k,X,U,x0,xref,uref,options)
            
            global pnpmpcToolboxGlobalOptions;
            if ~isstruct(pnpmpcToolboxGlobalOptions),
                pnpmpc_toolbox_err;
            end
            
            if nargin==0
                return;
            end
            
            if nargin~=9
                options = sdpsettings;
            end
            
            if nargin<=7
                xref = [];
                uref = [];
            end
            
            if isempty(B)
                B = zeros(size(A,2),1);
            end
            
            obj.A    = A;
            obj.B    = B;
            
            n        = size(obj.A ,1);
            m        = size(obj.B ,2);
            
            if ~iscell(X) || length(X)~=2
                error('LOCALCONTROLLYAPUNOV: X is not a cell array of two elements. see help localControlLyapunov');
            end
            if ~iscell(U) || length(U)~=2
                error('LOCALCONTROLLYAPUNOV: U is not a cell array of two elements. see help localControlLyapunov');
            end
            
            obj.cx   = sparse(X{1,1});
            obj.dx   = sparse(X{1,2});
            
            obj.cu   = sparse(U{1,1});
            obj.du   = sparse(U{1,2});
            
            obj.k    = k;
            obj.x0   = sparse([zeros(1,n) ; x0]);
            obj.xref = sparse(xref);
            obj.uref = sparse(uref);
            
            rx       = size(obj.cx,1);
            ru       = size(obj.cu,1);
            
            x0       = [zeros(1,n) ; x0]';
            q        = size(x0,2);
            
            CI       = ctrbindex(obj.A,obj.B);
            if k<CI
                error('LOCALCONTROLLYAPUNOV: k is less than controllability index %s',num2str(CI));
            end
            
            if ( ~isempty(obj.cx) && (size(obj.cx,2)~=n || size(obj.dx,1)~=rx || size(obj.dx,2)~=1) ) || size(A,2)~=n || size(B,1)~=n || size(obj.x0,2)~=n || ( ~isempty(obj.cu) && (size(obj.cu,2)~=m || size(obj.du,1)~=ru || size(obj.du,2)~=1))
                error('LOCALCONTROLLYAPUNOV: parameters dimensions');
            end
            if ~isempty(xref) && ( size(xref,1)~=n || size(xref,2)~=1 )
                error('LOCALCONTROLLYAPUNOV: parameters dimensions');
            end
            if ~isempty(uref) && ( size(uref,1)~=m || size(uref,2)~=1 )
                error('LOCALCONTROLLYAPUNOV: parameters dimensions');
            end
            
            obj.info.problem = 1;
            
            CONS   = [];
            
            bx     = sdpvar(n,1);
            bu     = sdpvar(m,1);
            alpha  = sdpvar(1,1);
            beta   = sdpvar(1,1);
            x      = sdpvar(n,q,k);
            u      = sdpvar(m,q,k);
            lambda = sdpvar(q,q);
            gamma  = sdpvar(rx,k);
            phi    = sdpvar(ru,k);
            
            assign(bx,0);
            assign(bu,0);
            assign(alpha,0);
            assign(beta,0);
            assign(x,0);
            assign(u,0);
            assign(lambda,0);
            if ~isempty(gamma)
                assign(gamma,0);
            end
            if ~isempty(phi)
                assign(phi,0);
            end
            
            
            %% (6.5a)
            CONS = [ CONS ; alpha-1<=-eps ; -alpha<=0 ; beta>=eps ];
            
            
            %% (6.5b)
            CONS = [ CONS ; -repmat(alpha,q,1)+sum(lambda,2)<=0 ];
            
            
            %% (6.5c)
            CONS = [ CONS ; -lambda(:) <= 0 ];
            
            if ~isempty(obj.cu)
                %% (6.7b)
                CONS = [ CONS ; obj.cu*bu+sum(phi,2)-beta*obj.du <= 0 ];
                
                %% (6.5e)
                if k==1
                    listper = [1 2];
                else
                    listper = [1 3 2];
                end
                CONS = [ CONS ; sum(reshape(repmat(obj.cu',k*q,1),m,ru*q*k)'.*repmat(reshape(permute(u,listper),m,k*q)',ru,1),2)-reshape(repmat(phi,1,q)',k*q*ru,1) <= 0  ];
            end
            
            
            if ~isempty(obj.cx)
                %% (6.7c)
                CONS = [ CONS ; obj.cx*bx+sum(gamma,2)-beta*obj.dx <= 0 ];
                
                %% (6.5g)
                CONS = [ CONS ; sum(reshape(repmat(obj.cx',q,1),n,rx*q)'.*repmat(x0',rx,1),2) - reshape(repmat(gamma(:,1)',q,1),q*rx,1) <= 0 ];
                if k~=1
                    if k==2
                        listper = [1 2];
                    else
                        listper = [1 3 2];
                    end
                    CONS = [ CONS ; sum(reshape(repmat(obj.cx',(k-1)*q,1),n,rx*q*(k-1))'.*repmat(reshape(permute(x(:,:,1:k-1),listper),n,(k-1)*q)',rx,1),2) - reshape(repmat(gamma(:,2:k),1,q)',(k-1)*q*rx,1) <= 0  ];
                end
            end
            
            
            %% (6.5h)
            for i9=1:k-1
                for i10=1:q
                    CONS = [ CONS ; x(:,i10,i9+1)-A*x(:,i10,i9)-B*u(:,i10,i9+1)==0 ];
                end
            end
            CONS = [ CONS ; reshape(x(:,:,1),n*q,1)-sum(repmat(A,q,1).*reshape(repmat(x0,n,1),n,n*q)',2)-sum(repmat(B,q,1).*reshape(repmat(u(:,:,1),n,1),m,n*q)',2)==0 ];
            
            
            %% (6.5a)
            CONS = [ CONS ; (eye(n)-A)*bx-B*bu==zeros(n,1) ];
            CONS = [ CONS ; reshape(x(:,:,k),n*q,1)-sum(reshape(repmat(lambda,1,n)',q,n*q)'.*repmat(x0,q,1),2)==zeros(n*q,1) ];
            if ~isempty(xref)
                CONS = [ CONS ; bx==xref ];
            end
            if ~isempty(uref)
                CONS = [ CONS ; bu==uref ];
            end
            
            
            %% () 0 is added to partial state and control sets
            for i=1:k
                CONS = [ CONS ; x(:,1,i) == 0 ];
                CONS = [ CONS ; u(:,1,i) == 0 ];
            end
            
            obj.info  = solvesdp(CONS,alpha,options);
            
            if obj.info.problem==0
                obj.x                 = double(x);
                obj.u                 = double(u);
                obj.beta              = double(beta);
                obj.bx                = sparse(double(bx));
                obj.bu                = sparse(double(bu));
                
                obj.u(isnan(obj.u))   = 0;
                obj.x(isnan(obj.x))   = 0;
                obj.bu(isnan(obj.bu)) = 0;
                obj.bx(isnan(obj.bx)) = 0;
                
                % Compute lambda-contractive term
                tmp   = Polyhedron(full(obj.x0)).computeHRep;
                cs0   = tmp.A;
                ds0   = tmp.b;
                rs0   = size(cs0,1);
                psi   = sdpvar(rs0,k);
                delta = sdpvar(1);
                
                if ~isempty(cs0) && isempty(tmp.Ae)
                    CONS = [ delta>=1 ; sum(psi,2)-delta*ds0 <= 0 ];
                    CONS = [ CONS ; sum(reshape(repmat(cs0',q,1),n,rs0*q)'.*repmat(x0',rs0,1),2) - reshape(repmat(psi(:,1)',q,1),q*rs0,1) <= 0 ];
                    if k~=1
                        if k==2
                            listper = [1 2];
                        else
                            listper = [1 3 2];
                        end
                        CONS = [ CONS ; sum(reshape(repmat(cs0',(k-1)*q,1),n,rs0*q*(k-1))'.*repmat(reshape(permute(obj.x(:,:,1:k-1),listper),n,(k-1)*q)',rs0,1),2) - reshape(repmat(psi(:,2:k),1,q)',(k-1)*q*rs0,1) <= 0  ];
                    end
                    info       = solvesdp( CONS , delta , options );
                else
                    obj.lambda = [];
                    warning('LOCALCONTROLLYAPUNOV:contractiveness','WARNING: contractiveness not computed, convex hull of x0 not computed');
                    info.problem = -1;
                end
                
                if info.problem == 0
                    delta      = double(delta);
                    obj.lambda = ( delta + double(alpha) - 1 ) / delta ;
                else
                    obj.lambda = [];
                    warning('LOCALCONTROLLYAPUNOV:contractiveness','WARNING: contractiveness not computed, solver error');
                end
                
            else
                obj.x      = [];
                obj.u      = [];
                obj.beta   = [];
                obj.bx     = [];
                obj.bu     = [];
                obj.lambda = [];
            end
            
            obj.LP = [];
            
        end
        
        
    end
end