function sup = supportFunction(varargin)
% SUPPORTFUNCTION Evaluation in the point c of the support function of a Local Control Invariant set
%
% sup = supportFunction(R,c,options)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% For a Local Control Invariant set R, compute
%
% sup = max_{x\in R} (c'*x)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% R   - parameterized Local Control Invariant and Stable Set
% c   - column vector of the same dimension of R
% options - sdpsettings object (optional)
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% sup value of the support function
%
% see also LOCALCONTROLLYAPUNOV

% Copyright is with the following author(s):
%
% (C) 2012 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
%
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the
%          Free Software Foundation, Inc.,
%          59 Temple Place, Suite 330,
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin<2
        error('SUPPORTFUNCTION: at lest 2 parameters for supportFunction(R,c). See help supportFunction')
    end
    R = varargin{1};
    c = varargin{2};
    if nargin~=3
        options = sdpsettings;
    else
        options = varargin{3};
    end
    n = size(R.A,1);
    if ~isa(R,'localControlLyapunov')
        error('SUPPORTFUNCTION: R is not a localControlLyapunov object')
    end
    if ~isa(c,'double')
        error('SUPPORTFUNCTION: c is not a double vector')
    end
    if n~=size(c,1)
        error('SUPPORTFUNCTION: dimension of R and c are incompatible');
    end
    
    
    
    lambda = sdpvar(R.k,size(R.x0,1));
    mu     = sdpvar(1);
    xV     = [ R.x0' reshape(R.x(:,:,1:R.k-1),size(R.A,1),[]) ];
    x      = sdpvar(n,1);
    info   = solvesdp([ lambda(:)>=0 ; sum(lambda,2)==mu ; mu>=0 ; mu<=1 ; x == R.beta^-1*( R.bx + sum(repmat(reshape(lambda',1,[]),size(R.A,1),1).*xV,2) ) ],-c'*x,options);
    
    if info.problem == 0
        sup = c'*double(x);
    else
        error('SUPPORTFUNCTION: ERROR SOLVER: %s',info.info);
    end
    
end