function objlse = addSubsystem(objlse,objsubss,flag,options)
% ADDSUBSYSTEM update an lse object next the plugging of a new subsystem
%
% objlse = addSubsystem(objlse,objsubss,flag,options)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Update an lse object next the plugging of a new subsystem
% 
% USAGE: 
%  
% objlse = addSubsystem(objlse,objsubss)
% objlse = addSubsystem(objlse,objsubss,flag)
% objlse = addSubsystem(objlse,objsubss,flag,options)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% objlse : lse object
%
% objsubss : subss object
%
% flag   : .norm define the norm for the optimization problem to compute
%                gain Lij, possible values 1,2,inf,'F' (default inf)
%          .Delta vector of numel(objsubss.couplingA) false|true to not 
%                compute|compute matrix Lij (default true)
%          .ExoBounded exogenous inputs that must be considered as
%                      disturbances (default empty)
%
% options : sdpsettings object for YALMIP
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlse : updated lse object
%
%
% see also LSE

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Daria Rubini, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     daria.rubini01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin<2 || nargin>4
        error('ADDSUBSYSTEM: number of input arguments')
    end
    
    if isempty(objsubss.Ts) || objsubss.Ts~=objlse.Ts
        error('ADDSUBSYSTEM: objsubss must be a discrete-time subsystem and compatible with Ts of the large-scale estimator')
    end
    
    if ~isempty(objsubss.couplingB)
        error('ADDSUBSYSTEM: subsystem must be input decoupled')
    end
    
    if ~isempty(objsubss.couplingC)
        error('ADDSUBSYSTEM: subsystem must be output decoupled')
    end
    
    if ~isempty(objsubss.getDcen)
        error('ADDSUBSYSTEM: subsystem has centralized inputs affecting output dynamics')
    end
    
    if nargin~=4 || isempty(options)
        options = sdpsettings;
    end
    
    ni   = size(objsubss.getA,1);
    mi   = size(objsubss.getB,2);
    pi   = size(objsubss.getC,1);
    
    nSys = length(objlse.ni);
    
    if any(objsubss.couplingA>nSys)
        error('ADDSUBSYSTEM: subsystem objsubss is coupled with a subsystem not in the large-scale estimator')
    end
    
    objlse.A = [objlse.getA zeros(sum(objlse.ni),ni); zeros(ni,sum(objlse.ni)+ni)];
    objlse.C = [objlse.getC zeros(sum(objlse.pi),ni); zeros(pi,sum(objlse.ni)+ni)];
    objlse.L = [objlse.getL zeros(sum(objlse.ni),pi); zeros(ni,sum(objlse.pi)+pi)];
    objlse.T = [objlse.T zeros(nSys,1); zeros(1,nSys+1)];
    
    objlse.ni(end+1) = ni;
    objlse.mi(end+1) = mi;
    objlse.pi(end+1) = pi;
    
    objlse.Delta = [objlse.getDelta false(nSys,1); false(1,nSys+1)];
    objlse.Delta(end,end) = true;
    objlse.alpha = [objlse.alpha; 0];
    
    objlse.couplingmatrix = [objlse.couplingmatrix false(nSys,1); false(1,nSys+1)];
    objlse.couplingmatrix(end,end) = true;
    Ni = numel(objsubss.couplingA);
    for j=objsubss.couplingA
        objlse.couplingmatrix(end,j) = true;
    end
    
    if nargin==2 || isempty(flag)
        flag.norm       = inf;
        flag.Delta      = true(1,Ni);
        flag.ExoBounded = [];
    end
    
    if ~isfield(flag,'ExoBounded')
        flag.ExoBounded = [];
    else
        flag.ExoBounded  = unique(flag.ExoBounded);
        flag.ExoBounded  = intersect(objsubss.Exo,flag.ExoBounded);
    end
    
    if ~isfield(flag,'norm')
        flag.norm = inf;
    end
    
    if ~isfield(flag,'Delta')
        flag.Delta = true(1,Ni);
    end
    
    if flag.norm~=inf && flag.norm~=1 && flag.norm~=2 && flag.norm~='F'
        flag.norm = inf;
        warning('ADDSUBSYSTEM: flag.norm is wrong or empty. Default value will be used: flag.norm=inf')
    end
    
    if size(flag.Delta,1)~=1 || size(flag.Delta,2)~=Ni || any(any(flag.Delta>1 | flag.Delta<0))
        error('ADDSUBSYSTEM: flag.Delta is a vector of %d elements',Ni)
    end
    
    
    [ flag.ExoBounded, indexExo ] = intersect(unique(flag.ExoBounded),objsubss.Exo);
    tmp1 = intersect(objlse.ExoBounded,objsubss.Exo);
    if ~isempty(tmp1)
        error('ADDSUBSYSTEM: exogenous inputs in the involved subsystem act also on other subsystems')
    end
    
    nSys = nSys + 1;
    i    = nSys;
    
    %% ALGORITHM 2
    %% PART (A)
    [ He, Ke ] = objsubss.doubleErrorEstimationConstraint();
    if isempty(He)
        He = sparse([eye(objlse.ni(i));-eye(objlse.ni(i))]);
        Ke = sparse(1e6*ones(2*objlse.ni(i),1));
    end
    [ vert, He, Ke ] = lse.computeVHK(He,Ke);
    
    try
        Li = -place(full(objsubss.getA)',full(objsubss.getC)',-floor(ni/2)*10^-3:10^-3:(ni-1)/2*10^-3);
    catch e
        error(e.message)
    end
    objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i))) = objsubss.getA+Li'*objsubss.getC;
    objlse.L(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.pi(1:i-1))+1:sum(objlse.pi(1:i))) = Li';
    
    kmin = rank(full(objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)))));
    kmax = 10*kmin;
    for k=kmin:kmax
        try
            objlse.Si(i) = localControlLyapunov(objlse.getA(i),[],k,{He,Ke},{[],[]},vert,zeros(ni,1),[],options);
        catch e
            warning(e.message)
            objlse.Si(i).info.problem = 1;
        end
        if objlse.Si(i).info.problem == 0 && ~isempty(objlse.Si(i).lambda) && objlse.Si(i).lambda<1 && isinside(objlse.Si(i),zeros(objlse.ni(i),1),options)
            break;
        end
    end
    if objlse.Si(i).info.problem ~= 0 || isempty(objlse.Si(i).lambda)
        error('ADDSUBSYSTEM: computation of set Si')
    end
    objlse.T(i,i) = objlse.Si(i).lambda;
    objlse.Si(i)  = objlse.Si(i).double(true);
    
    
    if ~isempty(flag.ExoBounded)
        Mi = objsubss.getM;
        Mi = Mi(:,indexExo);
        if ~isempty(Mi) && ~all(all(Mi==0))
            [ G, temp ]  = objlse.Si(i).double;
            G            = G./repmat(temp,1,size(G,2));
            w            = sdpvar(size(Mi,2),1);
            objlse.alpha(i) = -inf;
            [ Fw, Kw ]   = objsubss.doubleExoConstraint;
            D = Polyhedron(full(Fw),full(Kw)).projection(indexExo);
            Fw = D.A;
            Kw = D.b;
            if isempty(Fw)
                CONS = [];
            else
                CONS = Fw*w<=Kw;
            end
            for psi=1:size(G,1)
                info     = solvesdp( CONS , -G(psi,:)*Mi*w , options );
                if info.problem == 0
                    objlse.alpha(i) = max(objlse.alpha(i),G(psi,:)*(Mi*double(w)));
                else
                    error('LSE: %s for computation of parameter alpha',info.info)
                end
            end
        end
    end
    
    fprintf('ADDSUBSYSTEM: decentralized operations completed\n')
    
    
    
    
    %% PART (B)
    % distributed steps
    [ G, temp ] = objlse.Si(i).double;
    G           = G./repmat(temp,1,size(G,2));
    
    for j = objsubss.couplingA
        
        Cj    = objlse.getC(j);
        index = find(objsubss.couplingA==j);
        Aij   = full(objsubss.Aij(:,sum(objlse.ni(objsubss.couplingA(1:index-1)))+1:sum(objlse.ni(objsubss.couplingA(1:index)))));
        
        %% Step B.I
        [ Gj, Kj ] = objlse.Si(j).double;
        Gj = Gj./repmat(Kj,1,size(Gj,2));
        Kj = ones(size(Gj,1),1);
        
        if ~objlse.Delta(i,j)
            objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:j-1))+1:sum(objlse.ni(1:j))) = Aij;
        else
            LL = sdpvar(objlse.ni(i),objlse.pi(j));
            if flag.norm==inf
                J = norm(G*(Aij+LL*Cj)*pinv(Gj),inf);
            elseif flag.norm==1
                J = norm(G*(Aij+LL*Cj)*pinv(Gj),1);
            elseif flag.norm==2
                J = norm(G*(Aij+LL*Cj)*pinv(Gj),2);
            elseif flag.norm=='F'
                J = sum(sum( (G*(Aij+LL*Cj)*pinv(Gj)).^2 ));
            end
            info = solvesdp([],J,options);
            if info.problem == 0
                LL = double(LL);
                LL( abs(LL)<=10^-8 ) = 0;
                objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:j-1))+1:sum(objlse.ni(1:j))) = Aij+LL*Cj;
                objlse.L(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.pi(1:j-1))+1:sum(objlse.pi(1:j))) = LL;
                if all(all(LL==0))
                    objlse.Delta(i,j) = false;
                end
            else
                error('ADDSUBSYSTEM: %s for computation of matrix L %d%d',info.info,i,j)
            end
        end
        Aij = objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:j-1))+1:sum(objlse.ni(1:j)));
        Aij( abs(Aij)<=10^-8 ) = 0;
        
        %% Step B.II
        if all(all(Aij==0))
            objlse.T(i,j) = 0;
        else
            objlse.T(i,j) = -inf;
            xj         = sdpvar(size(Gj,2),1);
            for psi=1:size(G,1)
                info     = solvesdp( Gj*xj<=Kj , -G(psi,:)*Aij*xj , options );
                if info.problem == 0
                    objlse.T(i,j) = max(objlse.T(i,j),G(psi,:)*Aij*double(xj));
                else
                    error('ADDSUBSYSTEM: %s for computation of parameter mu %d%d',info.info,i,j)
                end
            end
        end
    end
    
    fprintf('ADDSUBSYSTEM: distributed operations completed\n')
    
    
    
    
    
    %% PART (C)
    
    %% Step C.I
    eigen = eig(full(objlse.T));
    if any(abs(eigen)>=1)
        tmpDistributedEstimator = objlse;
        save tmpDistributedEstimator tmpDistributedEstimator
        error('ADDSUBSYSTEM: matrix T is not stable. FYI: the failed state estimator is saved in the current folder as tmpDistributedEstimator')
    end
    
    %% Step C.II
    thetaMax = 0;
    [ He, Ke ] = objsubss.doubleErrorEstimationConstraint;
    if isempty(He)
        He = sparse([eye(objlse.ni(i));-eye(objlse.ni(i))]);
        Ke = sparse(1e6*ones(2*objlse.ni(i),1));
    end
    for kk=1:size(He,1)
        thetaMax = max(thetaMax,objlse.Si(i).supportFunction((He(kk,:)/Ke(kk))',options));
    end
    thetaMax = thetaMax^-1;
    H0   = [ eye(nSys) ; -eye(nSys) ];
    K0   = [ objlse.Theta0{2}(1:nSys-1) ; thetaMax ; zeros(nSys,1) ];
    objlse.Theta0 = { H0 , K0 };
    
    if any(objlse.alpha~=0)
        if any(H0*((eye(nSys)-objlse.T)\objlse.alpha)>=+K0+10^-6)
            tmpDistributedEstimator = objlse;
            save tmpDistributedEstimator tmpDistributedEstimator
            error('ADDSUBSYSTEM: not feasible stable point for the system of contraction factors. FYI: the failed state estimator is saved in the current folder as tmpDistributedEstimator')
        end
    end
    
    %% Step C.III
    Hk = H0;
    Kk = K0;
    k  = floor(log10(1e-6)/log10(max(abs(eigen))));
    Ptmp = Polyhedron;
    for jj=1:k
        Kk = [ Kk-Hk*objlse.alpha ; K0 ];
        Hk = [ Hk*objlse.T ; H0 ];
        P  = Polyhedron(full(Hk),full(Kk)).minHRep;
        Hk = P.A;
        Kk = P.b;
        if Ptmp.Dim~=0 && Ptmp==P
            break;
        end
        Ptmp = P;
    end
    Hinf = Hk;
    Kinf = Kk;
    
    objlse.Theta = {Hinf,Kinf};
    
    %% Step C.IV
    Kinfdec = zeros(nSys,1);
    theta   = sdpvar(nSys,1);
    for i = 1:nSys
        c    = zeros(nSys,1);
        c(i) = 1;
        info = solvesdp(Hinf*theta<=Kinf,-c'*theta,options);
        if info.problem == 0
            Kinfdec(i) = double(theta(i));
        else
            error('ADDSUBSYSTEM: error computing inner box approximation')
        end
    end
    c    = ones(nSys,1)./Kinfdec;
    info = solvesdp(Hinf*theta<=Kinf,-c'*theta,options);
    if info.problem == 0
        objlse.ThetaDec = {[eye(nSys);-eye(nSys)],[double(theta);zeros(nSys,1)]};
    else
        error('ADDSUBSYSTEM: error computing inner box approximation')
    end
    
    fprintf('ADDSUBSYSTEM: centralized operations completed\n')
    
    tmp = objlse.Exo;
    [ tmp2 , index2 ] = setdiff(objsubss.Exo,flag.ExoBounded);
    objlse.Exo = reshape(sort(unique(union(objlse.Exo,tmp2))),1,[]);
    objlse.ExoBounded = reshape(sort(unique(union(objlse.ExoBounded,flag.ExoBounded))),1,[]);
    if isempty(tmp2)
        if isempty(tmp)
            Mtmp = [];
            objlse.Exo = [];
        else
            Mtmp = [objlse.B(:,sum(objlse.pi(1:i-1))+sum(objlse.mi(1:i-1))+(1:numel(tmp)));zeros(objlse.ni(i),numel(tmp))];
        end
    else
        Mtmp = zeros(sum(objlse.ni),numel(objlse.Exo));
        for jj=1:numel(tmp)
            Mtmp(1:sum(objlse.ni(1:i-1)),tmp(jj)==objlse.Exo) = objlse.B(:,sum(objlse.pi(1:i-1))+sum(objlse.mi(1:i-1))+jj);
        end
        Mi = objsubss.getM;
        Mi = Mi(:,index2);
        Mtmp(sum(objlse.ni(1:i-1))+1:end,numel(tmp)+1:end) = Mi;
    end
    
    Bnewu = [ objlse.B(:,sum(objlse.pi(1:i-1))+1:sum(objlse.pi(1:i-1))+sum(objlse.mi(1:i-1))) zeros(sum(objlse.ni(1:i-1)),mi) ; zeros(ni,sum(objlse.mi(1:i-1))) objsubss.getB ];
    
    if isempty(objsubss.ICen)
        if size(objlse.B,2)-(sum(objlse.pi(1:i-1))+sum(objlse.mi(1:i-1))+numel(tmp))~=0
            Bcentmp = [objlse.B(:,size(objlse.B,2)-(sum(objlse.pi(1:i-1))+sum(objlse.mi(1:i-1))+numel(tmp)));zeros(objlse.ni(i),numel(objlse.ICen))];
        else
            Bcentmp = [];
            objlse.ICen = [];
        end
    else
        tmp = objlse.ICen;
        objlse.ICen = reshape(sort(unique(union(objlse.ICen,objsubss.ICen))),1,[]);
        Bcentmp = zeros(sum(objlse.ni),numel(objlse.ICen));
        for jj=1:numel(tmp)
            Bcentmp(1:sum(objlse.ni(1:i-1)),tmp(jj)==objlse.ICen) = objlse.B(:,sum(objlse.pi(1:i-1))+sum(objlse.mi(1:i-1))+numel(tmp)+jj);
        end
        Bcentmp(sum(objlse.ni(1:i-1))+1:end,objsubss.ICen) = objsubss.getBcen;
    end
    
    
    objlse.B = sparse([-objlse.L Bnewu Mtmp Bcentmp ]);
    
end