function test = checkErrorEstimation(obj,e,i,options)
% CHECKERRORESTIMATION check if the value e of estimation error is acceptable
%
% test = checkErrorEstimation(obj,e,i,options)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Check if the value of estimation error specified in the input e is
% acceptable. If input i is specified, the function considers e as the
% estimation error of the i-th subsystem, otherwise it considers e as the
% global estimation error of the system.
% 
% USAGE: 
%  
% test = CHECKERRORESTIMATION(obj,e)
% test = CHECKERRORESTIMATION(obj,e,i)
% test = CHECKERRORESTIMATION(obj,e,[],options)
% test = CHECKERRORESTIMATION(obj,e,i,options)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% obj : lse object
%
% e : estimation error
% 
% i : index of the subsystem
% 
% options : sdpsettings object for YALMIP
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% test : true if e is an admissible estimation error, false otherwise
%
%
% see also LSE

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Daria Rubini, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     daria.rubini01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------     

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin<2 || nargin>4
        error('CHECKERRORESTIMATION: number of input parameters')
    end
    
    if nargin>=3 && (~isempty(i))
        if ( size(e,1)~=obj.ni(i) || size(e,2)~=1 )
            error('CHECKERRORESTIMATION: e must have %d rows and 1 column',obj.ni(i))
        end
        if ~isa(i,'double')
            error('CHECKERRORESTIMATION: indeces of systems')
        end
        if i<1 || i>length(obj.ni)
            error('CHECKERRORESTIMATION: i must be positive and less than %d',length(obj.ni))
        end
    else
        if ( size(e,1)~=sum(obj.ni) || size(e,2)~=1 )
            error('CHECKERRORESTIMATION: e must have %d rows and 1 column',sum(obj.ni))
        end
    end
    
    if nargin~=4
        options = sdpsettings;
    end
    
    if nargin>=3 && (~isempty(i))
        theta = sdpvar(1);
        H = obj.Si(i).LP.A;
        K = obj.Si(i).LP.b;
        
        info = solvesdp([H*e<=theta*K; theta<=obj.ThetaDec{2}(i)],0,options);
        
        if info.problem==0
            test = true;
        elseif info.problem==1
            test = false;
        else
            error(info.info)
        end
    else
        theta = sdpvar(length(obj.ni),1);
        CONS  = obj.Theta{1}*theta<=obj.Theta{2};
        
        for i=1:length(obj.ni)
            H = obj.Si(i).LP.A;
            K = obj.Si(i).LP.b;
            ei        = e(sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)));
            CONS      = [ CONS ; H*ei<=theta(i)*K ]; %#ok<AGROW>
        end
        
        info = solvesdp(CONS,0,options);
        
        if info.problem==0
            test = true;
        elseif info.problem==1
            test = false;
        else
            error(['CHECKERRORESTIMATION: ' info.info])
        end
    end
    
end