function [ xt, yt , ytf ] = globalEstimator(obj,xt,y,u,d,ucen)
% GLOBALESTIMATOR compute the global state and output estimation
%
% [ xt, yt , ytf ] = globalEstimator(obj,xt,u,y,d,ucen)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Compute the global state and output estimation, i.e. the state
% and output estimation of the overall system
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% obj : lse object
% 
% xt : state estimation at time t
%
% u : input at time t
%
% y : output at time t
%
% d : measurable exogenous inputs at time t
%
% ucen : centralized inputs at time t
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% xt  : state estimation at time t+1
%
% yt  : output estimation at time t
% 
% ytf : output estimation using state estimation at time t+1
%
%
% see also LSE

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Daria Rubini, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     daria.rubini01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin~=4 && nargin~=5 && nargin~=6
        error('GLOBALESTIMATOR: number of input parameters')
    end
    
    if size(xt,1)~=sum(obj.ni) || size(xt,2)~=1
        error('GLOBALESTIMATOR: xt must have %d rows and 1 column',sum(obj.ni))
    end
    
    if size(u,1)~=sum(obj.mi) || size(u,2)~=1
        error('GLOBALESTIMATOR: u must have %d rows and 1 column',sum(obj.mi))
    end
    
    if size(y,1)~=sum(obj.pi)  || size(y,2)~=1
        error('GLOBALESTIMATOR: y must have %d rows and 1 column',sum(obj.pi))
    end
    
    r = numel(obj.Exo);
    if r~=0 && ( nargin<5 || size(d,1)~=r  || size(d,2)~=1 )
        error('GLOBALESTIMATOR: d must have %d rows and 1 column',r)
    else
        if r==0
            d = [];
        end
    end
    
    r = size(obj.getB,2)-sum(obj.pi)-sum(obj.mi)-r;
    if r~=0 && ( nargin<6 || size(ucen,1)~=r  || size(ucen,2)~=1 )
        error('GLOBALESTIMATOR: ucen must have %d rows and 1 column',r)
    else
        if r==0
            ucen = [];
        end
    end
    
    
    if nargout>=2
        yt = obj.getC*xt;
    end
    
    xt = obj.getA*xt + obj.getB*[y;u;d;ucen];
    
    if nargout>=3
        ytf = obj.getC*xt;
    end
    
end