function [ xit, yit , yitf ] = localEstimator(obj,i,xit,xjt,yi,ui,yj,d,ucen)
% LOCALESTIMATOR compute the local state and output estimation
%
% [ xit, yit , yitf ] = localEstimator(obj,i,xit,xjt,yi,ui,yj,d,ucen)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Compute the local state and output estimation, i.e. the state
% and output estimation of the i-th subsystem
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% obj : lse object
%
% i : index of the subsystem
% 
% xit : state estimation of the i-th subsystem at time t
%
% xjt : state estimation of the other subsystems at time t
%
% ui : input of the i-th subsystem at time t
%
% yi : output of the i-th subsystem at time t
%
% yj : output of the other subsystems at time t
%
% d : measurable exogenous inputs at time t
%
% ucen : centralized inputs at time t
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% xit : state estimation of the i-th subsystem at time t+1
%
% yit : output estimation of the i-th subsystem at time t
%
% yitf : output estimation of the i-th subsystem using state estimation at 
%        time t+1
%
%
% see also LSE

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Daria Rubini, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     daria.rubini01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------     

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin<6 || nargin>8
        error('LOCALESTIMATOR: number of input parameters')
    end
    
    if ~isa(i,'double')
        error('LOCALESTIMATOR: index of subsystem');
    end
    
    if i<1 || i>length(obj.ni)
        error('LOCALESTIMATOR: i must be positive and less than %d',length(obj.ni))
    end
    
    nxj    = setdiff(find(obj.couplingmatrix(i,:)==1),i);
    nyj    = setdiff(find(obj.Delta(i,:)==1),i);
    
    ntotxj = sum(obj.ni(nxj));
    ntotyj = sum(obj.pi(nyj));
    
    if size(xit,1)~=obj.ni(i) || size(xit,2)~=1
        error('LOCALESTIMATOR: xit must have %d rows and 1 column',obj.ni(i))
    end
    
    if size(xjt,1)~=ntotxj    || size(xjt,2)~=1
        error('LOCALESTIMATOR: xjt must have %d rows and 1 column',ntotxj)
    end
    
    if size(ui,1)~=obj.mi(i)  || size(ui,2)~=1
        error('LOCALESTIMATOR: ui must have %d rows and 1 column',obj.mi(i))
    end
    
    if size(yi,1)~=obj.pi(i)  || size(yi,2)~=1
        error('LOCALESTIMATOR: yi must have %d rows and 1 column',obj.pi(i))
    end
    
    if ntotyj~=0 && ( nargin<7 || size(yj,1)~=ntotyj || size(yj,2)~=1 )
        error('LOCALESTIMATOR: yj must have %d rows and 1 column',ntotyj)
    elseif ntotyj==0
        yj = [];
    end
    
    r = numel(obj.Exo);
    if r~=0 && ( nargin<8 || size(d,1)~=r  || size(d,2)~=1 )
        error('GLOBALESTIMATOR: d must have %d rows and 1 column',r)
    else
        if r==0
            d = [];
        end
    end
    
    r = size(obj.getB,2)-sum(obj.pi)-sum(obj.mi)-numel(obj.Exo);
    if r~=0 && ( nargin<9 || size(ucen,1)~=r  || size(ucen,2)~=1 )
        error('GLOBALESTIMATOR: ucen must have %d rows and 1 column',r)
    else
        if r==0
            ucen = [];
        end
    end
    
    
    Ai = obj.getA(i);
    Bi = obj.getB(i);
    
    if nargout>=2
        yit = obj.getC(i)*xit;
    end
    
    xit = Ai*xit + Bi*[yi;yj;ui;xjt;d;ucen];
    
    if nargout>=3
        yitf = obj.getC(i)*xit;
    end
    
end