classdef lse
% LSE design a large-scale estimator as proposed in [1] and [2]
% 
% objlse = lse(objlss,flag,options)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return a large-scale estimator
% 
% objlse = LSE(objlss)
% objlse = LSE(objlss,flag)
% objlse = LSE(objlss,flag,options)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - objlss is a large-scale system (lss) object
% - flag   .norm define the norm for the optimization problem to compute
%                gain Lij, possible values 1,2,inf,'F' (default inf)
%          .Delta square matrix of dimension objlss.numSys. Element ij is
%                 false if subsystem j does not send outputs to subsystem i,
%                 i.e. Lij is 0, true otherwise (default true)
%          .ExoBounded exogenous inputs that must be considered as
%                      disturbances (default empty)
% - options is a sdpsettings object for YALMIP
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlse is a large-scale estimator for the large-scale system in input
%
% ---------------------------------------------------------------------------
% PROPERTIES
% ---------------------------------------------------------------------------
% Most important properties.
% A, B, C : dynamics of the large-scale estimator
% L : gain of the large-scale estimator
% Theta , alpha : dynamics of contraction factors
%
% ---------------------------------------------------------------------------
% METHODS
% ---------------------------------------------------------------------------
% ADDSUBSYSTEM: add a subsystem in a previous lse object
% CHECKERRORESTIMATION: check if the initial error is allowed
% GETA, GETB, GETC, GETL: returns matrices A, B, C, L or a block
% GETDELTA: returns matrix Delta
% GETDYNTHETA: returns dynamics of contraction factors
% GLOBALESTIMATOR: compute state estimation for the overall lse
% LOCALESTIMATOR: compute state estimation for a subsystem in the lse
% LSE2SS: compute the ss object of the large-state estimator
% REMOVESUBSYSTEM: remove a subsystem in a previous lse object
% SETCOUPLING: set a new coupling between two subsystems
%
% [1] S. Riverso, D. Rubini, and G. Ferrari-Trecate, "Distributed bounded-error 
%     state estimation for partitioned systems based on practical robust 
%     positive invariance," in Proceedings of the 12th European Control 
%     Conference, 2013, pp. 2633-2638.
% [2] S. Riverso, "Distributed and plug-and-play control for constrained 
%     systems," PhD dissertation, Universita' degli studi di Pavia, 2014.
    
% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Daria Rubini, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     daria.rubini01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    properties ( SetAccess = private , Hidden = false )
        A              ;    % dynamics matrix of the large-scale estimator
        B              ;    % input matrix of the large-scale estimator
        C              ;    % ouput matrix of the large-scale estimator
        L              ;    % gains of the large-scale estimator
        Si             ;    % local sets Si for each subsystem
        T              ;    % dynamics of contraction factors
        alpha          ;    % dynamics of contraction factors
        Theta          ;    % feasible contraction factors for the overall dynamics
        ThetaDec       ;    % feasible contraction factors for the local dynamics
        couplingmatrix ;    % matrix, ij is true if subsystems i and j are dynamically coupled
        Delta          ;    % matrix, ij is true if subsystems j sends outputs to subsystem i
        Exo            ;    % indices of exogenous signals acting on the subsystems
        ExoBounded     ;    % indices of exogenous signals acting on the subsystems as disturbances
        ICen           ;    % indeces of centralized signals acting on subsystems
    end
    
    properties ( SetAccess = private , Hidden = true )
        ni             ;    % number of states for each subsystem
        mi             ;    % number of control inputs for each subsystem
        pi             ;    % number of outputs for each subsystem
        Theta0         ;    % feasible initial contraction factors for the overall dynamics
        Ts             ;    % sampling time
    end
    
    
    methods
        
        function objlse = lse(objlss,flag,options)
            
            global pnpmpcToolboxGlobalOptions;
            if ~isstruct(pnpmpcToolboxGlobalOptions),
                pnpmpc_toolbox_err;
            end
            
            if nargin==0
                return;
            end
            
            yalmip('clear')
            
            if nargin>3 || nargin<1
                error('LSE: parameters in input. see help LSE');
            end
            
            if ~isa (objlss,'lss')
                error('LSE: first argument must be lss object')
            end
            
            if isempty(objlss.Ts)
                error('LSE: objlss object must be a discrete-time system')
            end
            
            if ~all(all(objlss.getCc==0))
                error('LSE: objlss must be output decoupled')
            end
            
            if ~all(all(objlss.getBc==0))
                error('LSE: objlss must be input decoupled')
            end
            
            if objlss.numICen~=0
                error('LSE: objlss must be without centralized inputs')
            end
            
            if any(any(objlss.getD~=0))
                error('LSE: matrix D of objlss object must be 0')
            end
            
            if any(any(objlss.getDcen~=0))
                error('LSE: matrix Dcen of objlss object must be 0')
            end
            
            if any(any(objlss.getN~=0))
                error('LSE: matrix N of objlss object must be 0')
            end
            
            if nargin~=3 || isempty(options)
                options = sdpsettings;
            end
            
            objlse.A              = sparse(zeros(sum(objlss.ni)));
            objlse.T              = sparse(zeros(objlss.numSys));
            objlse.alpha          = sparse(zeros(objlss.numSys,1));
            objlse.C              = objlss.getC;
            objlse.ni             = objlss.ni;
            objlse.mi             = objlss.mi;
            if isempty(objlss.pi)
                objlse.pi         = objlss.ni;
            else
                objlse.pi         = objlss.pi;
            end
            objlse.L              = sparse(zeros(sum(objlse.ni),sum(objlse.pi)));
            objlse.couplingmatrix = objlss.couplingmatrix;
            objlse.Ts             = objlss.Ts;
            objlse.Si             = localControlLyapunov;
            
            if nargin==1 || isempty(flag)
                flag.norm       = inf;
                flag.Delta      = objlss.couplingmatrix;
                flag.ExoBounded = [];
            end
            
            if ~isfield(flag,'ExoBounded')
                flag.ExoBounded = [];
            else
                flag.ExoBounded  = intersect(unique(flag.ExoBounded),1:objlss.numExo);
            end
            
            if ~isfield(flag,'norm')
                flag.norm = inf;
            end
            
            if ~isfield(flag,'Delta')
                flag.Delta = objlss.couplingmatrix;
            end
            
            if flag.norm~=inf && flag.norm~=1 && flag.norm~=2 && flag.norm~='F'
                flag.norm = inf;
                warning('LSE: flag.norm is wrong or empty. Default value will be used: flag.norm=inf' )
            end
            
            if size(flag.Delta,1)~=objlss.numSys || size(flag.Delta,2)~=objlss.numSys || any(any(flag.Delta>1 | flag.Delta<0))
                error('LSE: flag.Delta is a square matrix of numSys elements')
            end
            
            objlse.Delta = flag.Delta;
            objlse.Delta = diag(true(objlss.numSys,1)) | flag.Delta ;
            objlse.Delta = sparse( objlse.Delta );
            
            M = objlss.getM;
            for j=flag.ExoBounded
                memoj = false;
                for i=1:objlss.numSys
                    if any(M(sum(objlss.ni(1:i-1))+1:sum(objlss.ni(1:i)),j)~=0)
                        if memoj
                            error('LSE: exogenous input %d acts on more subsystems',j)
                        else
                            memoj = true;
                        end
                    end
                end
            end
            
            
            %% PART (A)
            for i = 1:objlss.numSys
                %% Step A.I
                [ He, Ke ]       = objlss.doubleErrorEstimationConstraint(i);
                if isempty(He)
                    He = sparse([eye(objlse.ni(i));-eye(objlse.ni(i))]);
                    Ke = sparse(1e6*ones(2*objlse.ni(i),1));
                end
                [ vert, He, Ke ] = lse.computeVHK(He,Ke);
                
                try
                    Li    = -place(full(objlss.getA(i))',full(objlss.getC(i))',-floor(objlse.ni(i)/2)*10^-3:10^-3:(objlse.ni(i)-1)/2*10^-3);
                catch e
                    error(e.message)
                end
                objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i))) = objlss.getA(i)+Li'*objlss.getC(i);
                objlse.L(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.pi(1:i-1))+1:sum(objlse.pi(1:i))) = Li';
                
                %% Step A.II
                kmin = rank(full(objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)))));
                kmax = 10*kmin;
                for k=kmin:kmax
                    try
                        objlse.Si(i) = localControlLyapunov(objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i))),[],k,{He,Ke},{[],[]},0.1*vert,zeros(objlse.ni(i),1),[],options);
                        % objlse.Si(i) = localControlLyapunov(objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i))),[],k,{He,Ke},{[],[]},vert,zeros(objlse.ni(i),1),[],options);
                    catch e
                        warning(e.message)
                        objlse.Si(i).info.problem = 1;
                    end
                    if objlse.Si(i).info.problem == 0 && ~isempty(objlse.Si(i).lambda) && objlse.Si(i).lambda<1 && isinside(objlse.Si(i),zeros(objlse.ni(i),1),options)
                        break;
                    end
                end
                if objlse.Si(i).info.problem ~= 0 || isempty(objlse.Si(i).lambda)
                    error('LSE: computation of set Si for subsystem %d',i)
                end
                objlse.T(i,i)   = objlse.Si(i).lambda;
                objlse.Si(i)    = objlse.Si(i).double(true);
                
                %% Step A.III
                if ~isempty(flag.ExoBounded)
                    [ Fw, Kw, Dindex ]   = objlss.doubleExoConstraintOnSystem(i);
                    [ tmpExoi , indexExoi ] = intersect(Dindex,flag.ExoBounded);
                    D = Polyhedron(full(Fw),full(Kw)).projection(indexExoi);
                    Fw = D.A;
                    Kw = D.b;
                    Mi = objlss.getM(i,tmpExoi);
                    if ~isempty(Mi) && ~all(all(Mi==0))
                        [ G, temp ]  = objlse.Si(i).double;
                        G            = G./repmat(temp,1,size(G,2));
                        w            = sdpvar(size(Mi,2),1);
                        objlse.alpha(i) = -inf;
                        if isempty(Fw)
                            CONS = [];
                        else
                            CONS = Fw*w<=Kw;
                        end
                        for psi=1:size(G,1)
                            info     = solvesdp( CONS , -G(psi,:)*Mi*w , options );
                            if info.problem == 0
                                objlse.alpha(i) = max(objlse.alpha(i),G(psi,:)*(Mi*double(w)));
                            else
                                error('LSE: %s for computation of %d-th parameter alpha',info.info,i)
                            end
                        end
                    end
                end
                fprintf('LSE: decentralized operations for subsystem %d completed\n',i);
            end
            
            
            
            
            
            %% PART (B)
            for i = 1:objlss.numSys
                [ G, temp ] = objlse.Si(i).double;
                G           = G./repmat(temp,1,size(G,2));
                
                for j = 1:objlss.numSys
                    if i==j || ~objlss.isCoupled(i,j)
                        continue;
                    end
                    Cj  = objlss.getC(j);
                    Aij = objlss.getA(i,j);
                    
                    %% Step B.I
                    [ Gj, Kj ] = objlse.Si(j).double;
                    Gj = Gj./repmat(Kj,1,size(Gj,2));
                    Kj = ones(size(Gj,1),1);
                    
                    if ~objlse.Delta(i,j)
                        objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:j-1))+1:sum(objlse.ni(1:j))) = Aij;
                    else
                        LL = sdpvar(objlse.ni(i),objlse.pi(j));
                        if flag.norm==inf
                            J = norm(G*(Aij+LL*Cj)*pinv(Gj),inf);
                        elseif flag.norm==1
                            J = norm(G*(Aij+LL*Cj)*pinv(Gj),1);
                        elseif flag.norm==2
                            J = norm(G*(Aij+LL*Cj)*pinv(Gj),2);
                        elseif flag.norm=='F'
                            J = sum(sum( (G*(Aij+LL*Cj)*pinv(Gj)).^2 ));
                        end
                        info = solvesdp([],J,options);
                        if info.problem == 0
                            LL = double(LL);
                            LL( abs(LL)<=10^-8 ) = 0;
                            objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:j-1))+1:sum(objlse.ni(1:j))) = Aij+LL*Cj;
                            objlse.L(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.pi(1:j-1))+1:sum(objlse.pi(1:j))) = LL;
                            if all(all(LL==0))
                                objlse.Delta(i,j) = false;
                            end
                        else
                            error('LSE: %s for computation of matrix L %d%d',info.info,i,j)
                        end
                    end
                    Aij = objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:j-1))+1:sum(objlse.ni(1:j)));
                    Aij( abs(Aij)<=10^-8 ) = 0;
                    
                    %% Step B.II
                    if all(all(Aij==0))
                        objlse.T(i,j) = 0;
                    else
                        objlse.T(i,j) = -inf;
                        xj         = sdpvar(size(Gj,2),1);
                        for psi=1:size(G,1)
                            info     = solvesdp( Gj*xj<=Kj , -G(psi,:)*Aij*xj , options );
                            if info.problem == 0
                                objlse.T(i,j) = max(objlse.T(i,j),G(psi,:)*Aij*double(xj));
                            else
                                error('LSE: %s for computation of parameter mu %d%d',info.info,i,j)
                            end
                        end
                    end
                end
                fprintf('LSE: distributed operations for subsystem %d completed\n',i);
            end
            
            
            
            
            
            
            %% PART (C)
            
            %% Step C.I
            eigen = eig(full(objlse.T));
            if any(abs(eigen)>=1)
                tmpDistributedEstimator = objlse;
                save tmpDistributedEstimator tmpDistributedEstimator
                error('LSE: matrix T is not stable. FYI: the failed state estimator is saved in the current folder as tmpDistributedEstimator')
            end
            
            %% Step C.II
            thetaMax = zeros(objlss.numSys,1);
            for i = 1:objlss.numSys
                [ He, Ke ] = objlss.doubleErrorEstimationConstraint(i);
                if isempty(He)
                    He = sparse([eye(objlse.ni(i));-eye(objlse.ni(i))]);
                    Ke = sparse(1e6*ones(2*objlse.ni(i),1));
                end
                for kk=1:size(He,1)
                    thetaMax(i) = max(thetaMax(i),objlse.Si(i).supportFunction((He(kk,:)/Ke(kk))',options));
                end
                thetaMax(i) = thetaMax(i)^-1;
            end
            H0   = [ eye(objlss.numSys) ; -eye(objlss.numSys) ];
            K0   = [ thetaMax ; zeros(objlss.numSys,1) ];
            objlse.Theta0 = { H0 , K0 };
            
            if any(objlse.alpha~=0)
                if any(H0*((eye(objlss.numSys)-objlse.T)\objlse.alpha)>=+K0+10^-6)
                    tmpDistributedEstimator = objlse;
                    save tmpDistributedEstimator tmpDistributedEstimator
                    error('LSE: not feasible stable point for the system of contraction factors. FYI: the failed state estimator is saved in the current folder as tmpDistributedEstimator')
                end
            end
            
            %% Step C.III
            Hk = H0;
            Kk = K0;
            k  = floor(log10(1e-6)/log10(max(abs(eigen))));
            Ptmp = Polyhedron;
            for i=1:k
                Kk = [ Kk-Hk*objlse.alpha ; K0 ];
                Hk = [ Hk*objlse.T ; H0 ];
                P  = Polyhedron(full(Hk),full(Kk)).minHRep;
                Hk = P.A;
                Kk = P.b;
                if Ptmp.Dim~=0 && Ptmp==P
                    break;
                end
                Ptmp = P;
            end
            Hinf = Hk;
            Kinf = Kk;
            
            objlse.Theta = {Hinf,Kinf};
            
            %% Step C.IV
            Kinfdec = zeros(objlss.numSys,1);
            theta   = sdpvar(objlss.numSys,1);
            for i = 1:objlss.numSys
                c    = zeros(objlss.numSys,1);
                c(i) = 1;
                info = solvesdp(Hinf*theta<=Kinf,-c'*theta,options);
                if info.problem == 0
                    Kinfdec(i) = double(theta(i));
                else
                    error('LSE: error computing inner box approximation')
                end
            end
            c    = ones(objlss.numSys,1)./Kinfdec;
            info = solvesdp(Hinf*theta<=Kinf,-c'*theta,options);
            if info.problem == 0
                objlse.ThetaDec = {[eye(objlss.numSys);-eye(objlss.numSys)],[double(theta);zeros(objlss.numSys,1)]};
            else
                error('LSE: error computing inner box approximation')
            end
            
            fprintf('LSE: centralized operations completed\n')
            
            Mtmp = objlss.getM;
            objlse.Exo = setdiff(1:objlss.numExo,flag.ExoBounded);
            objlse.ExoBounded = flag.ExoBounded;
            objlse.B = sparse([-objlse.L objlss.getB Mtmp(:,objlse.Exo) objlss.getBcen ]);
            
            objlse.ICen = 1:objlss.numICen;
            
        end
        
    end
    
    
    
    
    
    methods(Static)
        
        function [ v, H, K ] = computeVHK(H,K)
            n     = size(H,2);
            tmp.A = [ full(H) ; eye(n) ; -eye(n) ];
            tmp.B = [ full(K) ; ones(2*n,1)*10^5 ];
            v     = cddmex('extreme',tmp);
            v     = v.V;
            if isempty(H)
                H = [ eye(n) ; -eye(n) ];
                K = ones(2*n,1)*10^5;
            end
        end
        
    end
    
end