function [ sys ] = lse2ss( obj, i )
% LSE2SS Obtain an ss object from an lse object
%
% [ sys ] = lse2ss( obj, i )
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Obtain an ss object from an lse object. The dynamics of the
% overall estimator is given by 
%           x(t+1) = Ax(t) + B [u(t);d(t);ucen(t)] - Ly(t)
% where x is the state estimation and A, B and L are the matrices of the
% lse object. The dynamics of the i-th local estimator is given by
% xi(t+1) = Aii*xi + sum(Aij*xj) + Bi*[ui;d(t);ucen(t)] - Li*yi - sum(Lij*yj)
% 
% USAGE: 
%  
% [ sys ] = lse2ss( obj ) consider the overall estimator
% [ sys ] = lse2ss( obj , i ) consider the i-th local estimator
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% obj : lse object
%
% i : index of the subsystem
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% sys : ss object
%
% see also LSE SS

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Daria Rubini, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     daria.rubini01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin<1||nargin>2
        error('LSE2SS: number of input parameters')
    end
    
    if nargin==2 && (~isa(i,'double'))
        error('LSE2SS: indeces of systems');
    end
    
    if nargin==2 && (i<1 || i>length(obj.ni))
        error('LSE2SS: i must be positive and less than %d',length(obj.ni))
    end
    
    if nargin==1
        sys = ss(full(obj.getA),full(obj.getB),[full(obj.getC);eye(size(obj.A,1))],0,obj.Ts);
        maxi = max([size(obj.A,1);size(obj.B,2);size(obj.C,1);max(obj.Exo)]);
        stri = cell(maxi,1);
        for i=1:maxi
            stri{i,1} = num2str(i);
        end
        
        strx = cell(size(obj.A,1),1);
        for i=1:length(obj.ni)
            for j=1:obj.ni(i)
                strx{sum(obj.ni(1:i-1))+j,1} = [ 'x' stri{i} '_' stri{j} 'e' ];
            end
        end
        set(sys,'StateName',strx)
        stry = cell(size(obj.A,1)+size(obj.C,1),1);
        for i=1:length(obj.pi)
            for j=1:obj.pi(i)
                stry{sum(obj.pi(1:i-1))+j,1} = [ 'y' stri{i} '_' stri{j} 'e' ];
            end
        end
        stry(size(obj.C,1)+1:end,1) = strx;
        set(sys,'OutputName',stry)
        
        
        stru = cell(size(obj.B,2),1);
        for i=1:length(obj.pi)
            for j=1:obj.pi(i)
                stru{sum(obj.pi(1:i-1))+j,1} = [ 'y' stri{i} '_' stri{j} ];
            end
        end
        for i=1:length(obj.mi)
            for j=1:obj.mi(i)
                stru{sum(obj.pi)+sum(obj.mi(1:i-1))+j,1} = [ 'u' stri{i} '_' stri{j} ];
            end
        end
        for i=1:length(obj.Exo)
            stru{sum(obj.pi)+sum(obj.mi)+i,1} = [ 'd_' stri{obj.Exo(i)} ];
        end
        for i=sum(obj.pi)+sum(obj.mi)+length(obj.Exo)+1:size(obj.B,2)
            stru{i,1} = [ 'ucen_' stri{i-(sum(obj.pi)+sum(obj.mi)+length(obj.Exo))} ];
        end
        set(sys,'InputName',stru)
        
        for i=1:length(obj.ni)
            eval(['InputGroup.Measurement_y' stri{i} '=sum(obj.pi(1:i-1))+1:sum(obj.pi(1:i));']);
            eval(['InputGroup.KnownInput_u' stri{i} '=sum(obj.pi)+(sum(obj.mi(1:i-1))+1:sum(obj.mi(1:i)));']);
            eval(['OutputGroup.OutputEstimate_y' stri{i} 'e=sum(obj.pi(1:i-1))+1:sum(obj.pi(1:i));']);
            eval(['OutputGroup.StateEstimate_x' stri{i} 'e=sum(obj.pi)+(sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)));']);
        end
        InputGroup.KnownInput_d = sum(obj.pi)+sum(obj.mi)+1:sum(obj.pi)+sum(obj.mi)+length(obj.Exo);
        InputGroup.KnownInput_ucen = sum(obj.pi)+sum(obj.mi)+length(obj.Exo)+1:size(obj.B,2);
        
        set(sys,'InputGroup',InputGroup)
        set(sys,'OutputGroup',OutputGroup)
    end
    
    
    if nargin==2
        Ai = full(obj.getA(i));
        Bi = full(obj.getB(i));
        Ci = full(obj.getC(i));
        
        sys = ss(Ai,Bi,[Ci;eye(obj.ni(i))],0,obj.Ts);
        maxi = max([size(Ai,1);size(Bi,2);size(Ci,1);max(obj.Exo)]);
        stri = cell(maxi,1);
        for j=1:maxi
            stri{j,1} = num2str(j);
        end
        
        strx = cell(size(Ai,1),1);
        for j=1:obj.ni(i)
            strx{j,1} = [ 'x' stri{i} '_' stri{j} 'e' ];
        end
        set(sys,'StateName',strx)
        stry = cell(size(Ai,1)+size(Ci,1),1);
        for j=1:obj.pi(i)
            stry{j,1} = [ 'y' stri{i} '_' stri{j} 'e' ];
        end
        stry(size(Ci,1)+1:end,1) = strx;
        set(sys,'OutputName',stry)
        
        stru = cell(size(Bi,2),1);
        for j=1:obj.pi(i)
            stru{j,1} = [ 'y' stri{i} '_' stri{j} ];
        end
        cnt = 0;
        eval(['InputGroup.Measurement_y' stri{i} '=1:obj.pi(i);']);
        eval(['InputGroup.KnownInput_u' stri{i} '=obj.pi(i)+cnt+(1:obj.mi(i));']);
        for jj = setdiff(find(obj.Delta(i,:)==1),i)
            for j=1:obj.pi(jj)
                stru{obj.pi(i)+cnt+j,1} = [ 'y' stri{jj} '_' stri{j} ];
            end
            eval(['InputGroup.Measurement_y' stri{jj} '=obj.pi(i)+cnt+(1:obj.pi(jj));']);
            cnt = cnt + obj.pi(jj);
        end
        for j=1:obj.mi(i)
            stru{obj.pi(i)+cnt+j,1} = [ 'u' stri{i} '_' stri{j} ];
        end
        cntx = 0;
        for jj=setdiff(find(obj.couplingmatrix(i,:)==1),i)
            for j=1:obj.ni(jj)
                stru{obj.pi(i)+obj.mi(i)+cnt+cntx+j,1} = [ 'x' stri{jj} '_' stri{j} ];
            end
            eval(['InputGroup.StateEstimate_x' stri{jj} 'e=obj.pi(i)+obj.mi(i)+cnt+cntx+(1:obj.ni(jj));']);
            cntx = cntx + obj.ni(jj);
        end
        for jj=1:length(obj.Exo)
            stru{obj.pi(i)+obj.mi(i)+cnt+cntx+jj,1} = [ 'd_' stri{obj.Exo(jj)} ];
        end
        for jj=obj.pi(i)+obj.mi(i)+cnt+cntx+length(obj.Exo)+1:size(Bi,2)
            stru{jj,1} = [ 'ucen_' stri{jj-(obj.pi(i)+obj.mi(i)+cnt+cntx+length(obj.Exo))} ];
        end
        InputGroup.KnownInput_d = obj.pi(i)+obj.mi(i)+cnt+cntx+1:obj.pi(i)+obj.mi(i)+cnt+cntx+length(obj.Exo);
        InputGroup.KnownInput_ucen = obj.pi(i)+obj.mi(i)+cnt+cntx+length(obj.Exo)+1:size(Bi,2);
        set(sys,'InputName',stru)
        
        
        eval(['OutputGroup.OutputEstimate_y' stri{i} 'e=1:obj.pi(i);']);
        eval(['OutputGroup.StateEstimate_x' stri{i} 'e=1:obj.ni(i);']);
        
        set(sys,'InputGroup',InputGroup)
        set(sys,'OutputGroup',OutputGroup)
    end
    
end