function [ objlse ] = removeSubsystem(objlse,i)
% REMOVESUBSYSTEM update an lse object when a subsystem is unplugged
%
% objlse = removeSubsystem(objlse,i)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Update an lse object when a subsystem is unplugged
% 
% USAGE: 
%  
% objlse = removeSubsystem(objlse,i)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% objlse : lse object
%
% i : index of the subsystem to remove
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlse = updated lse object
%
%
% see also LSE

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Daria Rubini, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     daria.rubini01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin~=2
        error('REMOVESUBSYSTEM: number of input parameters')
    end
    
    if ~isa(i,'double')
        error('REMOVESUBSYSTEM: indeces of systems')
    end
    
    if i<1 || i>length(objlse.ni)
        error('REMOVESUBSYSTEM: i must be positive and less than %d',length(objlse.ni))
    end
    
    objlse.T(i,:) = [];
    objlse.T(:,i) = [];
    objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),:)                  = [];
    objlse.A(:,sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)))                  = [];
    objlse.B(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),:)                  = [];
    objlse.B(:,sum(objlse.pi(1:i-1))+1:sum(objlse.pi(1:i)))                  = [];
    objlse.B(:,sum(objlse.pi)-objlse.pi(i)+(sum(objlse.mi(1:i-1))+1:sum(objlse.mi(1:i)))) = [];
    objlse.C(sum(objlse.pi(1:i-1))+1:sum(objlse.pi(1:i)),:)                  = [];
    objlse.C(:,sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)))                  = [];
    objlse.L(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),:)                  = [];
    objlse.L(:,sum(objlse.pi(1:i-1))+1:sum(objlse.pi(1:i)))                  = [];
    
    objlse.Delta(i,:)      = [];
    objlse.Delta(:,i)      = [];
    objlse.couplingmatrix(:,i) = [];
    objlse.couplingmatrix(i,:) = [];
    
    objlse.alpha(i) = [];
    
    objlse.Si(i) = [];
    
    % centralized steps
    eigen = eig(full(objlse.T));
    if any(abs(eigen)>=1)
        tmpDistributedEstimator = objlse;
        save tmpDistributedEstimator tmpDistributedEstimator
        error('REMOVESUBSYSTEM: matrix T is not stable. FYI: the failed state estimator is saved in the current folder as tmpDistributedEstimator')
    end
    
    
    H0 = [ eye(length(objlse.ni)-1) ; -eye(length(objlse.ni)-1) ];
    K0 = objlse.Theta0{2};
    K0(i) = [];
    K0(i-1+length(objlse.ni)) = [];
    
    objlse.Theta0 = { H0 , K0 };
    
    if any(objlse.alpha~=0)
        if any(H0*((eye(length(objlse.ni)-1)-objlse.T)\objlse.alpha)>=+K0+10^-6)
            tmpDistributedEstimator = objlse;
            save tmpDistributedEstimator tmpDistributedEstimator
            error('REMOVESUBSYSTEM: not feasible stable point for the system of contraction factors. FYI: the failed state estimator is saved in the current folder as tmpDistributedEstimator')
        end
    end
    
    Hk = H0;
    Kk = K0;
    k  = floor(log10(1e-6)/log10(max(abs(eigen))));
    Ptmp = Polyhedron;
    for i=1:k
        Kk = [ Kk-Hk*objlse.alpha ; K0 ];
        Hk = [ Hk*objlse.T ; H0 ];
        P  = Polyhedron(full(Hk),full(Kk)).minHRep;
        Hk = P.A;
        Kk = P.b;
        if Ptmp.Dim~=0 && Ptmp==P
            break;
        end
        Ptmp = P;
    end
    Hinf = Hk;
    Kinf = Kk;
    
    objlse.Theta = {Hinf,Kinf};
    
    Kinfdec = zeros(length(objlse.ni)-1,1);
    theta   = sdpvar(length(objlse.ni)-1,1);
    
    for i = 1:length(objlse.ni)-1
        c    = zeros(length(objlse.ni)-1,1);
        c(i) = 1;
        info = solvesdp(Hinf*theta<=Kinf,-c'*theta,sdpsettings('verbose',0));
        if info.problem == 0
            Kinfdec(i) = double(theta(i));
        else
            error('REMOVESUBSYSTEM: error computing inner box approximation')
        end
    end
    
    c    = ones(length(objlse.ni)-1,1)./Kinfdec;
    info = solvesdp(Hinf*theta<=Kinf,-c'*theta,sdpsettings('verbose',0));
    if info.problem == 0
        objlse.ThetaDec = {[eye(length(objlse.ni)-1);-eye(length(objlse.ni)-1)],[double(theta);zeros(length(objlse.ni)-1,1)]};
    else
        error('REMOVESUBSYSTEM: error computing inner box approximation')
    end
    
    objlse.ni(i) = [];
    objlse.mi(i) = [];
    objlse.pi(i) = [];
    
    
end