function objlse = setCoupling(objlse,Aij,i,j,flag,options)
% SETCOUPLING define a new coupling Aij between subsystems i and j then recompute gains for the large-scale estimator
%
% objlse = setCoupling(objlse,Aij,i,j,flag,options)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Update, if it is possible, the gains of the large-scale estimator
% 
% USAGE: 
%  
% objlse = setCoupling(objlse,Aij,i,j)
% objlse = setCoupling(objlse,Aij,i,j,flag)
% objlse = setCoupling(objlse,Aij,i,j,flag,options)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% objlse : lse object
%
% Aij    : dynamics coupling matrix between subsystems i and j
%
% i and j : indices of involved subsystems i and j  
%
% flag   : .norm define the norm for the optimization problem to compute
%                gain Lij, possible values 1,2,inf,'F' (default inf)
%          .Delta false|true to not compute|compute matrix Lij (default
%                true)
%
% options : sdpsettings object for YALMIP
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlse : updated lse object
%
%
% see also LSE

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin<4 || nargin>6
        error('SETCOUPLING: number of input arguments')
    end
    
    if nargin~=6 || isempty(options)
        options = sdpsettings;
    end
    
    if i==j
        error('SETCOUPLING: it is impossible set the local dynamics of subsystem %d',i)
    end
    
    nSys = numel(objlse.ni);
    
    if i>nSys
        error('SETCOUPLING: subsystem %d does not exists',i)
    end
    
    if j>nSys
        error('SETCOUPLING: subsystem %d does not exists',j)
    end
    
    if objlse.ni(i)~=size(Aij,1) || objlse.ni(j)~=size(Aij,2)
        error('SETCOUPLING: A%d%d should be a matrix %d x %d ',i,j,objlse.ni(i),objlse.ni(j))
    end
    
    objlse.couplingmatrix(i,j) = true;
    
    if nargin==4 || isempty(flag)
        flag.norm       = inf;
        flag.Delta      = true;
    end
    
    if ~isfield(flag,'norm')
        flag.norm = inf;
    end
    
    if ~isfield(flag,'Delta')
        flag.Delta = true;
    end
    
    if flag.norm~=inf && flag.norm~=1 && flag.norm~=2 && flag.norm~='F'
        flag.norm = inf;
        warning('SETCOUPLING: flag.norm is wrong or empty. Default value will be used: flag.norm=inf')
    end
    
    if size(flag.Delta,1)~=1 || size(flag.Delta,2)~=1 || any(any(flag.Delta>1 | flag.Delta<0))
        error('SETCOUPLING: flag.Delta is a vector of %d elements',Ni)
    end
    
    
    
    
    %% PART (B)
    % distributed steps
    [ G, temp ] = objlse.Si(i).double;
    G           = G./repmat(temp,1,size(G,2));
    
    Cj    = objlse.getC(j);
    
    %% Step B.I
    [ Gj, Kj ] = objlse.Si(j).double;
    Gj = Gj./repmat(Kj,1,size(Gj,2));
    Kj = ones(size(Gj,1),1);
    
    if ~objlse.Delta(i,j)
        objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:j-1))+1:sum(objlse.ni(1:j))) = Aij;
    else
        LL = sdpvar(objlse.ni(i),objlse.pi(j));
        if flag.norm==inf
            J = norm(G*(Aij+LL*Cj)*pinv(Gj),inf);
        elseif flag.norm==1
            J = norm(G*(Aij+LL*Cj)*pinv(Gj),1);
        elseif flag.norm==2
            J = norm(G*(Aij+LL*Cj)*pinv(Gj),2);
        elseif flag.norm=='F'
            J = sum(sum( (G*(Aij+LL*Cj)*pinv(Gj)).^2 ));
        end
        info = solvesdp([],J,options);
        if info.problem == 0
            LL = double(LL);
            LL( abs(LL)<=10^-8 ) = 0;
            objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:j-1))+1:sum(objlse.ni(1:j))) = Aij+LL*Cj;
            objlse.L(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.pi(1:j-1))+1:sum(objlse.pi(1:j))) = LL;
            if all(all(LL==0))
                objlse.Delta(i,j) = false;
            end
        else
            error('SETCOUPLING: %s for computation of matrix L %d%d',info.info,i,j)
        end
    end
    Aij = objlse.A(sum(objlse.ni(1:i-1))+1:sum(objlse.ni(1:i)),sum(objlse.ni(1:j-1))+1:sum(objlse.ni(1:j)));
    Aij( abs(Aij)<=10^-8 ) = 0;
    
    %% Step B.II
    if all(all(Aij==0))
        objlse.T(i,j) = 0;
    else
        objlse.T(i,j) = -inf;
        xj            = sdpvar(size(Gj,2),1);
        for psi=1:size(G,1)
            info     = solvesdp( Gj*xj<=Kj , -G(psi,:)*Aij*xj , options );
            if info.problem == 0
                objlse.T(i,j) = max(objlse.T(i,j),G(psi,:)*Aij*double(xj));
            else
                error('SETCOUPLING: %s for computation of parameter mu %d%d',info.info,i,j)
            end
        end
    end
    
    fprintf('SETCOUPLING: distributed operations completed\n')
    
    
    %% PART (C)
    
    %% Step C.I
    eigen = eig(full(objlse.T));
    if any(abs(eigen)>=1)
        tmpDistributedEstimator = objlse;
        save tmpDistributedEstimator tmpDistributedEstimator
        error('SETCOUPLING: matrix T is not stable. FYI: the failed state estimator is saved in the current folder as tmpDistributedEstimator')
    end
    
    %% Step C.II
    H0 = objlse.Theta0{1};
    K0 = objlse.Theta0{2};
    if any(objlse.alpha~=0)
        if any(H0*((eye(nSys)-objlse.T)\objlse.alpha)>=+K0+10^-6)
            tmpDistributedEstimator = objlse;
            save tmpDistributedEstimator tmpDistributedEstimator
            error('SETCOUPLING: not feasible stable point for the system of contraction factors. FYI: the failed state estimator is saved in the current folder as tmpDistributedEstimator')
        end
    end
    
    %% Step C.III
    Hk = H0;
    Kk = K0;
    k  = floor(log10(1e-6)/log10(max(abs(eigen))));
    Ptmp = Polyhedron;
    for jj=1:k
        Kk = [ Kk-Hk*objlse.alpha ; K0 ];
        Hk = [ Hk*objlse.T ; H0 ];
        P  = Polyhedron(full(Hk),full(Kk)).minHRep;
        Hk = P.A;
        Kk = P.b;
        if Ptmp.Dim~=0 && Ptmp==P
            break;
        end
        Ptmp = P;
    end
    Hinf = Hk;
    Kinf = Kk;
    
    objlse.Theta = {Hinf,Kinf};
    
    %% Step C.IV
    Kinfdec = zeros(nSys,1);
    theta   = sdpvar(nSys,1);
    for i = 1:nSys
        c    = zeros(nSys,1);
        c(i) = 1;
        info = solvesdp(Hinf*theta<=Kinf,-c'*theta,options);
        if info.problem == 0
            Kinfdec(i) = double(theta(i));
        else
            error('SETCOUPLING: error computing inner box approximation')
        end
    end
    c    = ones(nSys,1)./Kinfdec;
    info = solvesdp(Hinf*theta<=Kinf,-c'*theta,options);
    if info.problem == 0
        objlse.ThetaDec = {[eye(nSys);-eye(nSys)],[double(theta);zeros(nSys,1)]};
    else
        error('SETCOUPLING: error computing inner box approximation')
    end
    
    fprintf('SETCOUPLING: centralized operations completed\n')
    
end