function obj = addCentralizedInput(obj,Bcen,Dcen,i,j)
% ADDCENTRALIZEDINPUT defines connection between subsystem i and centralized input j
%
% objlss = addCentralizedInput(objlss,Bcen,Dcen,i,j) 
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Define matrices Bcen and Dcen or matrices Bcenij and Dcenij
% 
% objlss = objlss.ADDCENTRALIZEDINPUT(Bcen,Dcen,i,j)
% Define the relation between system i and centralized input j
%
% USAGE:
% objlss = objlss.addCentralizedInput(Bcen,Dcen) define overall matrices
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% Bcen and Dcen matrices with opportune dimensions
% 
% i = number (or name) of the system
% 
% j = number of the centralized input
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlss with centralized inputs updated
% 
%
% see also: ADDCENTRALIZEDINPUTCONSTRAINT REMOVECENTRALIZEDINPUT

% Copyright is with the following author(s):
%
% (C) 2013 
%     Fabio Sarzo, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     fabio.sarzo01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------


    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin<=3 % define overall M and N
        if nargin==2
            Dcen = [];
        end
        
        if size(Bcen,1)~=sum(obj.ni)
            error('ADDCENTRALIZEDINPUT: Bcen rows=%d ~= sum(ni)=%d',size(Bcen,1),sum(obj.ni))
        end
        
        if isempty (obj.pi)
            pi = obj.ni;
        else
            pi = obj.pi;
        end
        
        if ~isempty(Dcen) && (size(Dcen,1)~=sum(pi))
            error('ADDCENTRALIZEDINPUT: Dcen rows=%d ~= sum(pi)=%d',size(Dcen,1),sum(pi))
        end
        
        if ~isempty(Dcen) && (size(Bcen,2)~=size(Dcen,2))
            error('ADDCENTRALIZEDINPUT: Bcen e Dcen must have same number of columns.')
        end
        
        obj.numICen = size(Bcen,2);
        obj.Bcen      = sparse(Bcen);
        obj.Dcen      = sparse(Dcen);
        if ~isempty(obj.Hcen)
            if obj.numICen ~= size(obj.Bcen,2)
                obj.Hcen = [ obj.Hcen zeros(size(obj.Hcen,1),size(obj.Bcen,2)-obj.numHcen) ];
            end
        end
        return
    end
    
    if nargin==4
        j = size(obj.Bcen,2)+1 : size(obj.Bcen,2)+size(Bcen,2) ;
    end
    
    if nargin>=4 % modular procedure
        if ischar(i)
            i = obj.name2index(i);
        end
        
        if size(Bcen,1)~=obj.ni(i)
            error('ADDCENTRALIZEDINPUT: Bcen rows=%d ~= ni(%d)=%d',size(Bcen,1),i,obj.ni(i))
        end
        
        if size(Bcen,2)~=length(j)
            error('ADDCENTRALIZEDINPUT: Bcen columns=%d ~=%d',size(Bcen,2),length(j));
        end
        
        if isempty(obj.Bcen)
            obj.Bcen = sparse(zeros(sum(obj.ni),obj.numICen));
        end
        
        for k=1:length(j)
            obj.Bcen(sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)),j(k)) = Bcen(:,k);
        end
        
        if isempty(obj.pi)
            pi = obj.ni;
        else
            pi = obj.pi;
            obj.Dcen = sparse(zeros(sum(pi),obj.numICen+size(Bcen,2)));
        end
        
        if ~isempty(Dcen) && ~isequal(Dcen,zeros(pi(i),size(Bcen,2)))
            %enter if Dcen is not empty and is ~=0
            if size(Dcen,1) ~= pi(i)
                error('ADDCENTRALIZEDINPUT: Dcen rows=%d ~= pi(%d)=%d',size(Dcen,1),i,pi(i))
            end
            
            if size(Bcen,2)~=size(Dcen,2)
                error('ADDCENTRALIZEDINPUT: Bcen e Dcen must have same sumbers of columns')
            end
            
            if isempty(obj.Dcen)
                obj.Dcen = sparse(zeros(sum(pi),obj.numICen));
            end
            
            for k=1:length(j)
                obj.Dcen(sum(pi(1:i-1))+1:sum(pi(1:i)),j(k)) = Dcen(:,k);
            end
            
            if isempty(obj.C)
                obj.C  = sparse(eye(sum(obj.ni)));
                obj.D  = sparse(zeros(sum(obj.ni),sum(obj.mi)));
                if obj.numExo~=0
                    obj.N  = sparse(zeros(sum(pi),obj.numExo));
                end
                obj.pi = pi;
            end
            
        end
        
        
        
        if ~isempty(obj.Hcen)
            if obj.numICen ~= size(obj.Bcen,2)
                obj.Hcen = [ obj.Hcen zeros(size(obj.Hcen,1),size(obj.Bcen,2)-obj.numICen) ];
            end
        end
        obj.numICen = size(obj.Bcen,2);
        return
    end
    
    error('ADDCENTRALIZEDINPUT: number of parameters in input')
    
end