function obj = addCentralizedInputConstraint(obj,ceninputindex,H,K)
% ADDCENTRALIZEDINPUTCONSTRAINT defines polyhedral constraints for a centralized input 
%
% objlss = addCentralizedInputConstraint(objlss,ceninputindex,H,K)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% H and K are such that H*ucen(ceninputindex)<=K
% 
% USAGE: 
% objlss = objlss.ADDCENTRALIZEDINPUTCONSTRAINT(ceninputindex,H,K)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% ceninputindex = number of the input centralized interested involved in the
% constraint. ceninputindex can be a scalar or a vector 
% 
% H and K are such that H*ucen(ceninputindex)<=K
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlss with the constraint saved in Hcen and Kcen
%
%
% see also DOUBLECENINPUTCONSTRAINT REMOVECONSTRAINT

% Copyright is with the following author(s):
%
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------      

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin~=4
        error('ADDCENTRALIZEDINPUTCONSTRAINT: number of input')
    end
    
    [ hr, hc ] = size(H);
    [ kr, kc ] = size(K);
    
    if hr~=kr
        error('ADDCENTRALIZEDINPUTCONSTRAINT: h rows=%d ~= k rows=%d',hr,kr)
    end
    
    if kc~=1
        error('ADDCENTRALIZEDINPUTCONSTRAINT: k col=%d ~= 1',kc)
    end
    
    if ~isa(ceninputindex,'double')
        error('ADDCENTRALIZEDINPUTCONSTRAINT: indeces of systems');
    end
    
    if hc ~= length(ceninputindex)
        error('ADDCENTRALIZEDINPUTCONSTRAINT: H columns')
    end
    
    if isempty(obj.rcen)
        obj.rcen=zeros(1,obj.numICen);
    end
    
    if size(obj.rcen,2)~=obj.numICen
        obj.rcen(1,obj.numICen)=0;
    end
    
    Haux = zeros(hr,obj.numICen);
    vtmp = ones(length(ceninputindex),1)==1;
    for i = 1:length(ceninputindex)
        if ceninputindex(i)>obj.numICen
            error('ADDCENTRALIZEDINPUTCONSTRAINT: Icen number %d not yet defined',ceninputindex(i))
        end
        Htmp = H(:, i);
        Haux(:,ceninputindex(i)) = Htmp;
        if all(all(Htmp==0))
            vtmp(i) = 0;
        end
    end
    
    if ~isscalar(ceninputindex) && sum(vtmp)==1
        ceninputindex = ceninputindex(vtmp);
        warning('ADDCENTRALIZEDINPUTCONSTRAINT: coupled constraints are reduced to local constraints for subsystem %d',ceninputindex); %#ok<WNTAG>
    end
    
    if isscalar(ceninputindex)
        obj.Hcen = sparse( [obj.Hcen(1:sum(obj.rcen(1:ceninputindex)),:) ; Haux ; obj.Hcen(sum(obj.rcen(1:ceninputindex))+1:end,:) ]);
        obj.Kcen = sparse( [obj.Kcen(1:sum(obj.rcen(1:ceninputindex)))   ; K    ; obj.Kcen(sum(obj.rcen(1:ceninputindex))+1:end)   ]);
        obj.rcen(ceninputindex) = obj.rcen(ceninputindex)+hr;
        return
    end
    
    
    obj.Hcen = sparse([ obj.Hcen ; Haux ]);
    obj.Kcen = sparse([ obj.Kcen ; K    ]);
    
end