function obj = addCoupling(obj,i,j,Aij,Bij,Cij,Dij)
% ADDCOUPLING define coupling between two subsystems
%
% objlss = addCoupling(objlss,i,j,Aij,Bij,Cij,Dij)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Set a coupling on the matrices A,B,C,D, i.e. set the connection between
% system i and system j
%
% objlss = objlss.ADDCOUPLING(i,j,Aij,Bij,Cij,Dij)
%
% USAGE:
% if the connection does not exist, [] can used as input parameter
% 
% example:
% objlss = objlss.addCoupling (i,j,Aij) 
% objlss = objlss.addCoupling (i,j,[],Bij)
% 
% ---------------------------------------------------------------------------
% INPUT
% --------------------------------------------------------------------------- 
% i = number or name of the first system
% j = number or name of the second system
% 
% Aij Bij Cij Dij matrices with opportune dimensions
% 
% ---------------------------------------------------------------------------
% OUTPUT
% --------------------------------------------------------------------------- 
% objlss updated
% 
%
% see also ADDSYSTEM REMOVECOUPLING

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------        

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin<=3 && nargin~=2
        error('ADDCOUPLING: Input argument must be 2 or 3 or 5 or 7')
    end
    
    ni = obj.ni;
    mi = obj.mi;
    pi = obj.pi;
    
    if ischar(i)
        i = name2index(obj,i);
    end
    
    if ischar(j)==1
        j = name2index(obj,j);
    end
    
    if(i==j)
        error('ADDCOUPLING: i=j')
    end
    
    if(i>obj.numSys || j>obj.numSys)
        error('ADDCOUPLING: Insert before the decoupled system')
    end
    
    if nargin==3
        return
    end
    
    if ~isempty(Aij)
        if ~isequal(size(Aij,1),ni(i))
            error('ADDCOUPLING: Aij rows=%d ~= ni(%d)=%d',size(Aij,1),i,ni(i))
        end
        if ~isequal(size(Aij,2),ni(j))
            error('ADDCOUPLING: Aij columns=%d ~=ni(%d)=%d',size(Aij,2),j,ni(j))
        end
        obj.A(sum(ni(1:i-1))+1:sum(ni(1:i-1))+ni(i),(sum(ni(1:j-1))+1):(sum(ni(1:j-1))+ni(j)))  = Aij;
    end
    
    if nargin>=5
        if ~isempty(Bij)
            if ~isequal(size(Bij,1),ni(i))
                error('ADDCOUPLING: Bij rows=%d ~=ni(%d)=%d',size(Bij,1),i,ni(i))
            end
            if ~isequal(size(Bij,2),obj.mi(j))
                error('ADDCOUPLING: Bij columns=%d ~= mi(%d)=%d',size(Bij,2),j,obj.mi(j))
            end
            obj.B(sum(ni(1:i-1))+1:sum(ni(1:i-1))+ni(i),(sum(mi(1:j-1))+1):(sum(mi(1:j-1))+mi(j)))  = Bij;
        end
    end
    
    if nargin>=6
        
        if ~isempty(Cij)
            if isempty(obj.C)
                obj.C       = sparse(eye(sum(obj.ni)));
                obj.pi      = obj.ni;
                pi          = obj.pi;
            end
            if isequal(size(Cij,1),obj.pi(i))==0
                error('ADDCOUPLING: Cij rows=%d ~= pi(%d)=%d',size(C,1),i,obj.pi(i))
            end
            if isequal(size(Cij,2),obj.ni(j))==0
                error('ADDCOUPLING: Cij columns=%d ~= ni(%d)=%d',size(Cij,2),j,obj.ni(j))
            end
            obj.C ( (sum(pi(1:i-1))+1:sum(pi(1:i-1))+pi(i)), (sum(ni(1:j-1))+1):(sum(ni(1:j-1))+ni(j)) )  = Cij;
        end
    end
    
    if nargin>=7
        if ~isempty(Dij)
            if isempty(obj.D)
                obj.D=zeros(sum(obj.pi),sum(obj.mi));
            end
            if isequal(size(Dij,1),obj.pi(i))==0
                error('ADDCOUPLING: Dij rows=%d ~= pi(%d)=%d',size(Dij,1),i,obj.pi(i))
            end
            if isequal(size(Dij,2),obj.mi(j))==0
                error('ADDCOUPLING: Dij columns=%d ~= mi(%d)=%d',size(Dij,2),j,obj.mi(j))
            end
            obj.D(sum(pi(1:i-1))+1:sum(pi(1:i-1))+pi(i),(sum(mi(1:j-1))+1):(sum(mi(1:j-1))+mi(j)))  = Dij;
        end
    end
    
    obj.couplingmatrix(i,j) = true;
    [a,b,c,d] = obj.isCoupled(i,j);
    
    if a || b || c || d
        obj.couplingmatrix(i,j) = true;
    else
        obj.couplingmatrix(i,j) = false;
    end
    
end