function obj = addExoInput(obj,M,N,i,j)
% ADDEXOINPUT defines connection between subsystem i and exogenous signal j
%
% objlss = addExoInput(objlss,M,N,i,j) 
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Define matrices M and N or matrices Mij and Nij
% 
% objlss = objlss.ADDEXOINPUT(M,N,i,j)
% Define the relation between subssystem i and exogenous signal j
%
% USAGE:
% objlss = objlss.addExoInput(M,N) define overall matrices M and N
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% M and N matrices with opportune dimensions
% 
% i = number (or name) of the system
% 
% j = number of the exogenous signal
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlss with exogenous signals updated
% 
%
% see also ADDEXOCONSTRAINT REMOVEEXOINPUT
    
% Copyright is with the following author(s):
% (C) 2013 
%     Fabio Sarzo, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     fabio.sarzo01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin<=3 % define overall M and N
        if nargin==2
            N = [];
        end
        
        if size(M,1)~=sum(obj.ni)
            error('ADDEXOINPUT: M rows=%d ~= sum(ni)=%d',size(M,1),sum(obj.ni))
        end
        
        if isempty (obj.pi)
            pi = obj.ni;
        else
            pi = obj.pi;
        end
        
        if ~isempty(N) && (size(N,1)~=sum(pi))
            error('ADDEXOINPUT: N rows=%d ~= sum(pi)=%d',size(N,1),sum(pi))
        end
        
        if ~isempty(N) && (size(M,2)~=size(N,2))
            error('ADDEXOINPUT: M e N must have same number of columns.')
        end
        
        obj.numExo = size(M,2);
        obj.M      = sparse(M);
        obj.N      = sparse(N);
        if ~isempty(obj.Hd)
            if obj.numExo ~= size(obj.M,2)
                obj.Hd = [ obj.Hd zeros(size(obj.Hd,1),size(obj.M,2)-obj.numExo) ];
            end
        end
        obj.named = [];
        return
    end
    
    if nargin==4
        j = size(obj.M,2)+1 : size(obj.M,2)+size(M,2) ;
    end
    
    if nargin>=4 % modular procedure
        if ischar(i)
            i = obj.name2index(i);
        end
        
        if size(M,1)~=obj.ni(i)
            error('ADDEXOINPUT: M rows=%d ~= ni(%d)=%d',size(M,1),i,obj.ni(i))
        end
        
        if size(M,2)~=length(j)
            error('ADDEXOINPUT: M columns=%d ~=%d',size(M,2),length(j));
        end
        
        if isempty(obj.M)
            obj.M = sparse(zeros(sum(obj.ni),obj.numExo));
        end
        
        for k=1:length(j)
            obj.M(sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)),j(k)) = M(:,k);
        end
        
        if isempty (obj.pi)
            pi = obj.ni;
        else
            pi = obj.pi;
            obj.N = sparse(zeros(sum(pi),obj.numExo+size(M,2)));
        end
        
        if ~isempty(N) && ~isequal(N,zeros(pi(i),size(M,2)))
            %enter if N is not empty and is ~=0
            if size(N,1) ~= pi(i)
                error('ADDEXOINPUT: N rows=%d ~= pi(%d)=%d',size(N,1),i,pi(i))
            end
            
            if size(M,2)~=size(N,2)
                error('ADDEXOINPUT: M e N must have same sumbers of columns')
            end
            
            if isempty(obj.N)
                obj.N = sparse(zeros(sum(pi),obj.numExo));
            end
            
            for k=1:length(j)
                obj.N(sum(pi(1:i-1))+1:sum(pi(1:i)),j(k)) = N(:,k);
            end
            
            if isempty(obj.C)
                obj.C  = sparse(eye(sum(obj.ni)));
                obj.D  = sparse(zeros(sum(obj.ni),sum(obj.mi)));
                if obj.numICen~=0;
                    obj.Dcen = sparse(zeros(sum(pi),obj.numICen));
                end
                obj.pi = pi;
            end
        end
        
        if ~isempty(obj.Hd)
            if obj.numExo ~= size(obj.M,2)
                obj.Hd = [ obj.Hd zeros(size(obj.Hd,1),size(obj.M,2)-obj.numExo) ];
                obj.rd = [ obj.rd zeros(1,size(obj.M,2)) ];
            end
        end
        obj.numExo = size(obj.M,2);
        if ~isempty(obj.named)
            obj.named = [ obj.named , cell(1,size(M,2)) ];
        end
        return
    end
    
    error('ADDEXOINPUT: number of parameters in input')
    
end