function obj = addLocalInput(obj,index,Bi,Di,namenewU)
% ADDLOCALINPUT Defines a new input for i-th subsystem of an lss object
%
%  objlss = addLocalInput(obj,index,Bi,Di,namenewU)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% -------------------------------------------------------------------------
% Update matrices B,D,Hu and nameu consistently to the new input of i-th 
% susbsystem
%
% USAGE:
% objlss = objlss.addLocalInput(index,Bi,Di)
% objlss = objlss.addLocalInput(index,Bi,Di,namenewU)
%
% ---------------------------------------------------------------------------
% INPUT
% -------------------------------------------------------------------------
% Bi Di column matrices with opportune dimensions (number of rows = number
% of states of i-th subsystem (i.e n(i))
% 
% index = number (or name) of the subsystem to which is added the new input 
%
% namenewU = name of the new input as a string
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlss updated
% 

% Copyright is with the following author(s):
%
% (C) 2013 
%     Fabio Sarzo, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     fabio.sarzo01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
    
    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin~=4 && nargin~=5
        error('ADDLOCALINPUT: check number of inputs, see help addLocalInput');
    end
    
    if ~isa(obj,'lss')
        error('ADDLOCALINPUT: System input must be lss objects');
    end
    
    if ischar(index)
        index = obj.name2index(index);
    end
    
    if length(index)~=1
        error('ADDLOCALINPUT: index must be one');
    end
    
    if index>obj.numSys
        error('ADDLOCALINPUT: Subsystem %d does not exist',index);
    end
    
    if ~isequal(obj.ni(index),size(Bi,1))
        error('ADDLOCALINPUT: Ai rows=%d ~= Bi rows=%d',obj.ni(index),size(Bi,1));
    end
    
    Bcol = [zeros(sum(obj.ni(1:(index-1))),1);Bi;zeros(sum(obj.ni((index+1):end)),1)];
    Btemp = zeros(sum(obj.ni),sum(obj.mi)+1);
    Btemp(:,1:sum(obj.mi(1:index))) = obj.B(:,1:sum(obj.mi(1:index)));
    Btemp(:,sum(obj.mi(1:index))+1) = Bcol;
    Btemp(:,sum(obj.mi(1:index))+2:end) = obj.B(:,sum(obj.mi(1:index))+1:end);
    obj.B = sparse(Btemp);
    
    if isempty(obj.pi)
        
        if ~isempty(Di)
            
            obj.pi = obj.ni;
            if ~isequal(obj.pi(index),size(Di,1))
                error('ADDLOCALINPUT: Ci rows=%d ~= Di rows=%d',obj.pi(index),size(Di,1));
            end
            obj.C  = sparse(eye(sum(obj.ni)));
            obj.D  = sparse(zeros(sum(obj.ni),sum(obj.mi)));
            if obj.numICen~=0;
                obj.Dcen = sparse(zeros(sum(obj.pi),obj.numICen));
            end
            if obj.numExo~=0
                obj.N  = sparse(zeros(sum(obj.pi),obj.numExo));
            end
            
            
            Dcol = [zeros(sum(obj.pi(1:(index-1))),1);Di;zeros(sum(obj.pi((index+1):end)),1)];
            Dtemp = zeros(sum(obj.pi),sum(obj.mi)+1);
            Dtemp(:,1:sum(obj.mi(1:index))) = obj.D(:,1:sum(obj.mi(1:index)));
            Dtemp(:,sum(obj.mi(1:index))+1) = Dcol;
            Dtemp(:,sum(obj.mi(1:index))+2:end) = obj.D(:,sum(obj.mi(1:index))+1:end);
            obj.D = sparse(Dtemp);
            
            
        end
        
        
    else
        
        if ~isempty(Di)
            if ~isequal(obj.pi(index),size(Di,1))
                error('ADDLOCALINPUT: Ci rows=%d ~= Di rows=%d',obj.pi(index),size(Di,1));
            end
            Dcol = [zeros(sum(obj.pi(1:(index-1))),1);Di;zeros(sum(obj.pi((index+1):end)),1)];
            Dtemp = zeros(sum(obj.pi),sum(obj.mi)+1);
            Dtemp(:,1:sum(obj.mi(1:index))) = obj.D(:,1:sum(obj.mi(1:index)));
            Dtemp(:,sum(obj.mi(1:index))+1) = Dcol;
            Dtemp(:,sum(obj.mi(1:index))+2:end) = obj.D(:,sum(obj.mi(1:index))+1:end);
            obj.D = sparse(Dtemp);
        else
            Di = zeros(obj.pi(index),1);
            Dcol = [zeros(sum(obj.pi(1:(index-1))),1);Di;zeros(sum(obj.pi((index+1):end)),1)];
            Dtemp = zeros(sum(obj.pi),sum(obj.mi)+1);
            Dtemp(:,1:sum(obj.mi(1:index))) = obj.D(:,1:sum(obj.mi(1:index)));
            Dtemp(:,sum(obj.mi(1:index))+1) = Dcol;
            Dtemp(:,sum(obj.mi(1:index))+2:end) = obj.D(:,sum(obj.mi(1:index))+1:end);
            obj.D = sparse(Dtemp);
            
        end
        
    end
    
    if ~isempty(obj.Hu)
        Hutemp = zeros(size(obj.Hu,1),sum(obj.mi)+1);
        Hutemp(:,1:sum(obj.mi(1:index))) = obj.Hu(:,1:sum(obj.mi(1:index)));
        Hutemp(:,sum(obj.mi(1:index))+1) = zeros(size(obj.Hu,1),1);
        Hutemp(:,sum(obj.mi(1:index))+2:end) = obj.Hu(:,sum(obj.mi(1:index))+1:end);
        obj.Hu = sparse(Hutemp);
    end
    
    obj.mi(index) = obj.mi(index)+1;
    
    if nargin==5
        if ~ischar(namenewU)
            error('ADDLOCALINPUT: new name must be a string')
        end
        if isempty(obj.nameu)
            obj.nameu = cell(1,obj.numSys);
            tmp = cell(obj.mi(index),1);
            tmp{1:end-1} = '';
            tmp{end} = namenewU;
            obj = obj.setNameu(index,tmp);
        else
            if isempty(obj.nameu{index})
                tmp = cell(obj.mi(index),1);
                tmp{1:end-1} = '';
                tmp{end} = namenewU;
                obj = obj.setNameu(index,tmp);
            else
                obj = obj.setNameu(index,[obj.nameu{index};cellstr(namenewU)]);
            end
        end
    else
        if ~isempty(obj.nameu) && ~isempty(obj.nameu{index})
            obj = obj.setNameu(index,[obj.nameu{index};{''}]);
        end
    end
    
end