function obj = addOutConstraint(obj,sysindex,H,K)
% ADDOUTCONSTRAINT defines polyhedral constraints on local outputs
%
% objlss = addOutConstraint (objlss,sysindex,H,K)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% H and K are such that H*y(sysindex)<=K
%
% objlss = objlss.ADDOUTCONSTRAINT(sysindex,H,K)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% sysindex = name or number of the system (vector or cell array of strings
% for coupled constraints)
% 
% H and K are such that H*y(sysindex)<=K
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlss with the constraint saved in Hy and Ky
% 
%
% see also ADDSTATECONSTRAINT ADDINPUTCONSTRAINT REMOVECONSTRAINT

% Copyright is with the following author(s):
%
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if isempty(obj.C) && isempty(obj.D)
        warning('MATLAB:paramAmbiguous','State feedback y = x: the output constraints are state constraints')
        obj = obj.addStateConstraint(sysindex,H,K);
        return
    end
    if nargin~=4
        error('ADDOUTCONSTRAINT: number of input')
    end
    
    [ hr, hc ] = size(H);
    [ kr, kc ] = size(K);
    
    if hr~=kr
        error('ADDOUTCONSTRAINT: h rows=%d ~= k rows=%d',hr,kr)
    end
    
    if kc~=1
        error('ADDOUTCONSTRAINT: k col=%d ~= 1',kc)
    end
    
    if iscellstr(sysindex)
        vect=[];
        for k=1:size(sysindex,2)
            vect(k)=obj.name2index(sysindex(k));
        end
        sysindex=vect;
    end
    
    if ~ischar(sysindex) && ~isa(sysindex,'double')
        error('ADDOUTCONSTRAINT: indeces of systems');
    end
    
    if ischar(sysindex)
        sysindex = obj.name2index(sysindex);
    end
    
    if ~isempty(obj.pi)
        pi = obj.pi;
    else
        pi = obj.ni;
    end
    
    piindex = pi(sysindex);
    
    if hc ~= sum(piindex)
        error('ADDOUTCONSTRAINT: h columns')
    end
    
    if isempty(obj.ry)
        obj.ry=zeros(1,obj.numSys);
    end
    
    Haux = zeros(hr,sum(pi));
    vtmp = ones(length(sysindex),1)==1;
    for i = 1:length(sysindex)
        Htmp = H(:,sum(piindex(1:i-1))+1:sum(piindex(1:i)));
        Haux(:,sum(pi(1:sysindex(i)-1))+1:sum(pi(1:sysindex(i)))) = Htmp;
        if all(all(Htmp==0))
            vtmp(i) = 0;
        end
    end
    
    if ~isscalar(sysindex) && sum(vtmp)==1
        sysindex = sysindex(vtmp);
        warning('ADDOUTCONSTRAINT: coupled constraints are reduced to local constraints for subsystem %d',sysindex); %#ok<WNTAG>
    end    
    
    if isscalar(sysindex)
        obj.Hy = sparse( [obj.Hy(1:sum(obj.ry(1:sysindex)),:) ; Haux ; obj.Hy(sum(obj.ry(1:sysindex))+1:end,:) ]);
        obj.Ky = sparse( [obj.Ky(1:sum(obj.ry(1:sysindex)))   ; K    ; obj.Ky(sum(obj.ry(1:sysindex))+1:end)   ]);
        obj.ry(sysindex) = obj.ry(sysindex)+hr;
        return
    end
    
    
    obj.Hy = sparse([ obj.Hy ; Haux ]);
    obj.Ky = sparse([ obj.Ky ; K    ]);
    
end