function obj = addStateConstraint(obj,sysindex,H,K)
% ADDSTATECONSTRAINT defines polyhedral constraints on state variables
%
% objlss = addStateConstraint(objlss,sysindex,H,K)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% H and K are such that H*x(sysindex)<=K
% 
% objlss = objlss.ADDSTATECONSTRAINT(sysindex,H,K)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% sysindex = name or number of the system (vector or cell array of strings
% for coupled constraint)
% 
% H and K are such that H*x(sysindex)<=K
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlss with the constraint saved in Hx and Kx
% 
%
% see also ADDINPUTCONSTRAINT ADDEXOCONSTRAINT REMOVECONSTRAINT

% Copyright is with the following author(s):
%
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin~=4
        error('ADDSTATECONSTRAINT: number of inputs')
    end
    
    [ hr, hc ] = size(H);
    [ kr, kc ] = size(K);
    
    if hr~=kr
        error('ADDSTATECONSTRAINT: h rows=%d ~= k rows=%d',hr,kr)
    end
    
    if kc~=1
        error('ADDSTATECONSTRAINT: k col=%d ~= 1',kc)
    end
    
    if iscellstr(sysindex)
        vect=[];
        for k=1:size(sysindex,2)
            vect(k)=obj.name2index(sysindex(k));
        end
        sysindex=vect;
    end
    
    if ~ischar(sysindex) && ~isa(sysindex,'double')
        error('ADDSTATECONSTRAINT: indeces of systems');
    end
    
    if ischar(sysindex)
        sysindex = obj.name2index(sysindex);
    end
    
    niindex = obj.ni(sysindex);
    
    if hc ~= sum(niindex)
        error('ADDSTATECONSTRAINT: h columns')
    end
    
    if isempty(obj.rx)
        obj.rx=zeros(1,obj.numSys);
    end
    
    Haux = zeros(hr,sum(obj.ni));
    vtmp = ones(length(sysindex),1)==1;
    for i = 1:length(sysindex)
        Htmp = H(:,sum(niindex(1:i-1))+1:sum(niindex(1:i)));
        Haux(:,sum(obj.ni(1:sysindex(i)-1))+1:sum(obj.ni(1:sysindex(i)))) = Htmp;
        if all(all(Htmp==0))
            vtmp(i) = 0;
        end
    end
    
    if ~isscalar(sysindex) && sum(vtmp)==1
        sysindex = sysindex(vtmp);
        warning('ADDSTATECONSTRAINT: coupled constraints are reduced to local constraints for subsystem %d',sysindex); %#ok<WNTAG>
    end
    
    if isscalar(sysindex)
        obj.Hx = sparse( [obj.Hx(1:sum(obj.rx(1:sysindex)),:) ; Haux ; obj.Hx(sum(obj.rx(1:sysindex))+1:end,:) ]);
        obj.Kx = sparse( [obj.Kx(1:sum(obj.rx(1:sysindex)))   ; K    ; obj.Kx(sum(obj.rx(1:sysindex))+1:end)   ]);
        obj.rx(sysindex) = obj.rx(sysindex)+hr;
        return
    end 
    
    obj.Hx = sparse([ obj.Hx ; Haux ]);
    obj.Kx = sparse([ obj.Kx ; K    ]);
    
end