function obj = addSystem(obj,Ai,Bi,Ci,Di,name,Ts)
% ADDSYSTEM adds a system into a large-scale system
% 
% objlss = addSystem(objlss,Ai,Bi,Ci,Di,name,Ts)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Add a new system into a large-scale system. The linear system is
% described as 
% x_i(t+1)|dx_i = Ai x_i(t) + Bi u_i(t)
% y(t)          = Ci x_i(t) + Di u_i(t)
% 
% objlss = objlss.ADDSYSTEM(Ai,Bi,Ci,Di,name,Ts) return the large-scale
% system with the new system
%
% USAGE:
% 
% objlss = objlss.addSystem(Ai)
% objlss = objlss.addSystem(Ai,Bi)
% objlss = objlss.addSystem(Ai,Bi,Ci,Di)
% objlss = objlss.addSystem(Ai,Bi,Ci,Di,name) 
% objlss = objlss.addSystem(Ai,Bi,Ci,Di,name,Ts) 
% objlss = objlss.addSystem(objsubss)
% objlss = objlss.addSystem(objss)
% objlss = objlss.addSystem(objtf)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% Ai Bi Ci Di = matrices of the i-th system
%      if Bi (or Ci) is 0, the i-th subsystem has 0 input, i.e m(i)=0 
%         (or 0 output, i.e. p(i)=0)
%  
% name = name of the system as a string
% 
% Ts = sample time, it must be equal to the objlss.Ts
%       Ts double number --> continuous-time system discretized with Ts
%       Ts -1            --> discrete-time system
%       Ts empty         --> continuous-time system
% 
% objsubss : if the system is described as a subss object
% objss    : if the system is described as an ss object
% objtf    : if the system is described as a tf object
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlss updated with the new system
% 
%
% see also ADDCOUPLING GETSYSTEM REMOVESYSTEM
 

%Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------    

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin~=2 && nargin~=3 && nargin~=5 && nargin~=6 && nargin~=7
        error('ADDSYSTEM: inputs arguments, see help addSystem')
    end
    
    if nargin==2        
        if isa(Ai,'subss')
            subss = Ai;
            if isempty (subss.C) && isempty (subss.D);
                obj=addSystem(obj,subss.A,subss.B);
            else
                
                obj=addSystem(obj,subss.A,subss.B,subss.getC,subss.getD);
            end
            
            if ~isempty (subss.M)
                if ~isempty (subss.N)
                    obj=obj.addExoInput(subss.M,subss.N,obj.numSys);
                else
                    obj=obj.addExoInput(subss.M,[],obj.numSys);
                end
            end
            
            if ~isempty (subss.Bcen)
                if ~isempty (subss.Dcen)
                    obj=obj.addCentralizedInput(subss.Bcen,subss.Dcen,obj.numSys);
                else
                    obj=obj.addCentrlizedInput(subss.Bcen,[],obj.numSys);
                end
            end
            
            
            if ~all(all(subss.Hx==0))
                obj=obj.addStateConstraint(obj.numSys,subss.Hx,subss.Kx);
            end
            if ~all(all(subss.Hu==0))
                obj=obj.addInputConstraint(obj.numSys,subss.Hu,subss.Ku);
            end
            if ~all(all(subss.Hy==0))
                obj=obj.addOutConstraint(obj.numSys,subss.Hy,subss.Ky);
            end
            
            if ~all(all(subss.Hd==0))
                obj=obj.addExoConstraint(obj.numExo-size(subss.M,2)+1:obj.numExo,subss.Hd,subss.Kd);
            end
            
            if subss.Ts~=obj.Ts
                warning('MATLAB:paramAmbiguous','WARNING: Ts of the subss object is different from the lss object. Ts=%d is used',obj.Ts)
            end
            return
            
        elseif isa(Ai,'double') || isa(Ai,'sparse')
            if ~isequal(size(Ai,1),size(Ai,2))
                error('addsystem: ai must be a square matrix')
            end
            
            name = [];
            Ts = [];
            
            if isempty(name)
                obj.name{1,end+1} = ['ss_{',num2str(obj.numSys+1),'}'];
            elseif ischar(name)
                obj.name{1,end+1} = name;
                for k=1:(size(obj.name,2)-1) % check if the name of the system is already used
                    if strcmp(obj.name{1,k},name)==1
                        obj.name{1,end} = strcat('ss_{',int2str(obj.numSys+1),'}');
                        warning('MATLAB:paramAmbiguous','Proposed name is used: %s will be replaced with the default name %s',name,obj.name{1,end})
                    end
                end
            else
                error('ADDSYSTEM: The name must be a string')
            end
            if isempty(obj.Ts) && isempty(obj.ni)
                obj.Ts = Ts;
            else
                if ~isempty(obj.Ts) && ~isempty(Ts)
                    if (obj.Ts~=Ts)
                        warning('MATLAB:paramAmbiguous','WARNING: different Ts. Ts of the lss object is used.')
                    end
                end
            end
            obj.couplingmatrix(end+1,end+1) = true;
            obj.A           = sparse(blkdiag(obj.A,Ai));
            obj.ni(end+1)   = size(Ai,1);
            obj.mi(end+1)   = 0;
            if ~isempty(obj.B)
                obj.B = [ obj.B ; sparse(zeros(obj.ni,sum(obj.mi))) ];
            end
            obj.numSys      = obj.numSys+1;
            return
            
        elseif isa(Ai,'ss')
            sssys       = Ai;
            [ a, b, c, d ] = ssdata(sssys);
            Ts          = sssys.Ts;
            name        = sssys.Name;
            obj         = obj.addSystem(a,b,c,d,name,Ts);
            i           = obj.numSys;
            obj         = obj.setNamex(i,sssys.StateName);
            obj         = obj.setNameu(i,sssys.InputName);
            obj         = obj.setNamey(i,sssys.OutputName);
            return
            
        elseif isa(Ai,'tf')
            obj         = obj.addSystem(ss(Ai));
            return
            
        else
            error('ADDSYSTEM: input must be a subss object')
        end
    end
    
    if nargin<6
        name = [];
    end
    if nargin<7
        Ts = [];
    end
    
    if isempty(name)
        obj.name{1,end+1} = ['ss_{',num2str(obj.numSys+1),'}'];
    elseif ischar(name)
        obj.name{1,end+1} = name;
        for k=1:(size(obj.name,2)-1) % check if the name of the system is already used
            if strcmp(obj.name{1,k},name)==1
                obj.name{1,end} = strcat('ss_{',int2str(obj.numSys+1),'}');
                warning('MATLAB:paramAmbiguous','Proposed name is used: %s will be replaced with the default name %s',name,obj.name{1,end})
            end
        end
    else
        error('ADDSYSTEM: The name must be a string')
    end
    
    if all(all(Bi==0)) || isempty(Bi)
        Bi = zeros(size(Ai,1),0);
    end
    
    if nargin>3
        if Ci==0
            if obj.numSys==0
                error('ADDSYSTEM: no outputs in lss')
            end
            Ci = zeros(0,size(Ai,1));
        end
        if Di==0
            Di = zeros(size(Ci,1),size(Bi,2));
        end
    end
    
    if ~isequal(size(Ai,1),size(Bi,1)) && ~isempty(Bi)
        error('ADDSYSTEM: Ai rows=%d ~= Bi rows=%d',size(Ai,1),size(Bi,1))
    end
    if ~isequal(size(Ai,1),size(Ai,2))
        error('ADDSYSTEM: Ai must be a square matrix')
    end
    
    if ~(isequal(Ts,-1) || isscalar(Ts) || isempty(Ts))
        error('ADDSYSTEM: Ts must be -1 or scalar or empty')
    end
    
    if isempty(obj.Ts) && isempty(obj.ni)
        obj.Ts = Ts;
    else
        if ~isempty(obj.Ts) && ~isempty(Ts)
            if (obj.Ts~=Ts)
                warning('MATLAB:paramAmbiguous','WARNING: different Ts. Ts of the lss object is used.')
            end
        end
    end
    
    obj.couplingmatrix(end+1,end+1) = true;
    
    if nargin>=3
        obj.A           = sparse(blkdiag(obj.A,Ai));
        obj.ni(end+1)   = size(Ai,1);
        obj.numSys      = obj.numSys+1;
        obj.mi(end+1)   = size(Bi,2);
        if isempty(obj.B)
            obj.B       = sparse(zeros(sum(obj.ni(1:end-1)),sum(obj.mi(1:end-1))));
            obj.mi      = [ zeros(1,obj.numSys-1) size(Bi,2) ];
        end
        obj.B           = sparse(blkdiag(obj.B,Bi));
        if ~isempty(Bi)
            if ~isempty(obj.C) && nargin==3
                obj.C       = sparse(blkdiag(obj.C,eye(sum(obj.ni(end)))));
                if ~isempty(obj.D)
                    obj.D       = sparse(blkdiag(obj.D,zeros(obj.ni(end),obj.mi(end))));
                end
                obj.pi      = [ obj.pi , obj.ni(end) ];
            end
        end
    end
    
    if nargin>=5
        if isempty(Ci)
            if ~isempty(Di)
                error('ADDSYSTEM: dimensions of matrices Ci and Di')
            end
            if ~isempty(obj.C)
                obj.C = sparse(blkdiag(obj.C,eye(obj.ni(end))));
            end
            obj.pi(end+1) = obj.ni(end);
        else
            if ~isempty(Di) && ~isequal(size(Ci,1),size(Di,1))
                error('ADDSYSTEM: Ci rows=%d ~= Di rows=%d',size(Ci,1),size(Di,1))
            end
            if ~isequal(size(Ci,2),size(Ai,2))
                error('ADDSYSTEM: Ci columns=%d ~= Ai columns=%d',size(Ci,2),size(Ai,2))
            end
            if ~isempty(Di) && ~isequal (size(Di,2),size(Bi,2))
                error('ADDSYSTEM: Di columns=%d ~=Bi columns=%d',size(Di,2),size(Bi,2))
            end
            if isempty(obj.C)
                obj.C       = sparse(eye(sum(obj.ni(1:end-1))));
            end
            if ~isempty(Di) && isempty(obj.D)
                obj.D       = sparse(zeros(sum(obj.ni(1:end-1)),sum(obj.mi(1:end-1))));
            end
            if isempty(obj.pi)
                obj.pi = obj.ni(1:end-1);
            end
            obj.C           = sparse(blkdiag(obj.C,Ci));
            obj.D           = sparse(blkdiag(obj.D,Di));
            obj.pi(end+1)   = size(Ci,1);
        end
    end
    
    if isempty(obj.N) && isempty(obj.Dcen)
        if isequal(obj.C,eye(sum(obj.ni)))
            obj.C = [];
        end
        
        if isequal (obj.D,zeros(sum(obj.ni),sum(obj.mi)))
            obj.D = [];
        end
        
        if isempty(obj.C) && isempty(obj.D)
            obj.pi = [];
        end
    end
    if ~isempty(obj.M)
        obj.M=[obj.M;zeros(obj.ni(end),obj.numExo)];
    end
    
    if ~isempty(obj.N)
        if isempty(obj.pi)
            pi = obj.ni;
        else
            pi = obj.pi;
        end
        obj.N=[obj.N;zeros(pi(end),obj.numExo)];
    end
    
    if ~isempty(obj.Bcen)
        obj.Bcen=[obj.Bcen;zeros(obj.ni(end),obj.numICen)];
    end
    
    if ~isempty(obj.Dcen)
        if isempty(obj.pi)
            pi = obj.ni;
        else
            pi = obj.pi;
        end
        obj.Dcen=[obj.Dcen;zeros(pi(end),obj.numICen)];
    end
    
    if ~isempty(obj.Hx)
        obj.Hx=[obj.Hx,zeros(size(obj.Hx,1),obj.ni(end))];
        obj.rx(end+1)=0;
    end
    
    if ~isempty(obj.Hu)
        obj.Hu=[obj.Hu,zeros(size(obj.Hu,1),obj.mi(end))];
        obj.ru(end+1)=0;
    end
    
    if ~isempty(obj.Hy)
        obj.Hy=[obj.Hy,zeros(size(obj.Hy,1),obj.pi(end))];
        obj.ry(end+1)=0;
    end
    
    if ~isempty(obj.Hdeltau)
        obj.Hdeltau=[obj.Hdeltau,zeros(size(obj.Hdeltau,1),obj.mi(end))];
        obj.rdeltau(end+1)=0;
    end
    
    if ~isempty(obj.He)
        obj.He=[obj.He,zeros(size(obj.He,1),obj.ni(end))];
        obj.re(end+1)=0;
    end
    
    if all(all(obj.B==0)) && isempty(obj.D)
        obj.B = [];
    end
    
end