function dobjlss = clss2dlss(cobjlss,Ts,method)
% CLSS2DLSS Converts continuous-time dynamic large-scale system to discrete time.
% 
% dobjlss = clss2dlss(objlss,Ts,method)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Computes a discrete-time model dobjlss with sampling time Ts that 
% approximates the continuous-time model cobjlss.
%
% Dobjlss = objlss.CLSS2DLSS(Ts,method)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - Ts is the sample time
% - The string method selects the discretization method among the following:
%       'zoh'       Zero-order hold on the inputs
%       'foh'       Linear interpolation of inputs
%       'impulse'   Impulse-invariant discretization
%       'tustin'    Bilinear (Tustin) approximation.
%       'matched'   Matched pole-zero method (for SISO systems only).
%       'subsystem' Discretization system by system
%       The default is 'zoh' when method is omitted.
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% Dobjlss = discretized continuos-time large-scale system
% 
% 
% see also C2D 

% Copyright is with the following author(s):
%
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------


    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if ~isempty(cobjlss.Ts) && (cobjlss.Ts>0 || cobjlss.Ts==-1)
        error('CLSS2DLSS: Large-scale system is already discretized')
    end
    
    if nargin==2
        method='zoh';
    end
    
    if strcmp(method,'subsystem')
        dobjlss = cobjlss;
        dobjlss.Ts = Ts;
        for i=1:cobjlss.numSys
            sub = cobjlss.getSystem(i);
            Ass = sub.getA;
            Bss = [sub.Aij,sub.getB,sub.Bij,sub.getM,sub.getBcen];
            Css = sub.getC;
            Dss = [sub.Cij,sub.getD,sub.Dij,sub.getN,sub.getDcen];
            
            [BssR, BssC] = size(Bss);
            [DssR, DssC] = size(Dss);
            
            flag = BssC-DssC;
            
            if flag > 0
                Dss = [Dss,zeros(DssR,flag)];
            end
            
            if flag < 0
                Bss = [Bss,zeros(BssR,abs(flag))];
            end
            
            sys = ss(full(Ass),full(Bss),full(Css),full(Dss));
            
            sysd = c2d(sys,Ts);
            
            [Ad,Bd,Cd,Dd]   = ssdata(sysd);
            
            %new matrices
            
            %diag A
            dobjlss = dobjlss.setA(Ad,i);
            
            % Aij
            for a=1:length(sub.couplingA)
                dobjlss = dobjlss.setA(Bd(:,1:cobjlss.ni(sub.couplingA(a))),i,sub.couplingA(a)) ;
                Bd = circshift(Bd,[0,-(cobjlss.ni(sub.couplingA(a)))]);
            end
            
            %diag B
            dobjlss = dobjlss.setB(Bd(:,1:cobjlss.mi(i)),i);
            Bd = circshift(Bd,[0,-(cobjlss.mi(i))]);
            
            %Bij
            for b = 1: length(sub.couplingB)
                dobjlss = dobjlss.setB(Bd(:,1:cobjlss.mi(sub.couplingB(b))),i,sub.couplingB(b)) ;
                Bd = circshift(Bd,[0,-(cobjlss.mi(sub.couplingB(b)))]);
            end
            
            %M
            for m=1: length(sub.Exo)
                dobjlss = dobjlss.setM(Bd(:,1),i,sub.Exo(m));
                Bd = circshift(Bd,[0, -1]);
            end
            
            %Bcen
            for bcen = 1:length( sub.ICen)
                dobjlss = dobjlss.setBcen(Bd(:,1),i,sub.ICen(bcen));
                Bd = circshift(Bd,[0, -1]);
            end
            
            %diag C
            dobjlss = dobjlss.setC(Cd,i);
            
            %Cij
            for c=1:length(sub.couplingC)
                dobjlss = dobjlss.setC(Dd(:,1:cobjlss.ni(sub.couplingC(c))),i,sub.couplingC(c)) ;
                Dd = circshift(Dd,[0,-(cobjlss.ni(sub.couplingC(c)))]);
            end
            
            %diag D
            dobjlss = dobjlss.setD (Dd(:,1:cobjlss.mi(i)) , i);
            Dd = circshift(Dd,[0 , -(cobjlss.mi(i))]);
            
            %Dij
            for d = 1: length(sub.couplingD)
                dobjlss = dobjlss.setD(Dd(:,1:cobjlss.mi(sub.couplingD(d))),i,sub.couplingD(d)) ;
                Dd = circshift(Dd,[0,-(cobjlss.mi(sub.couplingD(d)))]);
            end
            
            %N
            for n=1: length(sub.Exo)
                dobjlss = dobjlss.setN(Dd(:,1),i,sub.Exo(n));
                Dd = circshift(Dd,[0, -1]);
            end
            
            %Dcen
            for dcen = 1:length( sub.ICen)
                dobjlss = dobjlss.setDcen(Dd(:,1),i,sub.ICen(dcen));
                Dd = circshift(Dd,[0, -1]);
            end
        end
        if dobjlss.D==zeros(size(dobjlss.D))
            dobjlss.D=[];
            if dobjlss.C==eye(size(dobjlss.C))
                dobjlss.C=[];
            end
        end
        
        if all(all(dobjlss.M==0))
            dobjlss.M = [];
        end
        
        if all(all(dobjlss.N==0))
            dobjlss.N = [];
        end
        
        if all(all(dobjlss.Bcen==0))
            dobjlss.Bcen = [];
        end
        
        if all(all(dobjlss.Dcen==0))
            dobjlss.Dcen = [];
        end
        return
    end
    
    if ~ischar(method)
        error('CLSS2DLSS: method must be a string')
    end
    
    if Ts<=0
        error('CLSS2DLSS: Ts must be >0')
    end
    
    A               = cobjlss.getA;
    B               = [ cobjlss.getB,cobjlss.getM,cobjlss.getBcen];
    C               = cobjlss.getC;
    D               = [cobjlss.getD,cobjlss.getN,cobjlss.getDcen];
    sys             = ss(full(A),full(B),full(C),full(D));
    sysd            = c2d(sys,Ts,method);
    [A,Btot,C,Dtot] = ssdata(sysd);
    dobjlss         = cobjlss;
    dobjlss.Ts      = Ts;
    dobjlss.name    = cobjlss.name;
    dobjlss.A       = sparse(A);
    dobjlss.B       = sparse(Btot(:,1:sum(cobjlss.mi)));
    dobjlss.C       = sparse(C);
    dobjlss.D       = sparse(Dtot(:,1:sum(cobjlss.mi)));
    dobjlss.M       = sparse(Btot(:,sum(cobjlss.mi)+1:sum(cobjlss.mi)+cobjlss.numExo));
    dobjlss.N       = sparse(Dtot(:,sum(cobjlss.mi)+1:sum(cobjlss.mi)+cobjlss.numExo));
    dobjlss.Bcen    = sparse(Btot(:,sum(cobjlss.mi)+cobjlss.numExo+1:end));
    dobjlss.Dcen    = sparse(Dtot(:,sum(cobjlss.mi)+cobjlss.numExo+1:end));
    
    if dobjlss.D==zeros(size(dobjlss.D))
        dobjlss.D=[];
        if dobjlss.C==eye(size(dobjlss.C))
            dobjlss.C=[];
        end
    end
    
    if all(all(dobjlss.M==0))
        dobjlss.M = [];
    end
    
    if all(all(dobjlss.N==0))
        dobjlss.N = [];
    end
    
    if all(all(dobjlss.Bcen==0))
        dobjlss.Bcen = [];
    end
    
    if all(all(dobjlss.Dcen==0))
        dobjlss.Dcen = [];
    end
    
    dobjlss.couplingmatrix = sparse(true(dobjlss.numSys));
    
end