function [ ctrl, K ] = createCtrlPnP( objlss , flag )
% CREATECTRLPNP design Plug-and-Play controllers for each subsystem of a large-scale system as proposed in [1] and [2]
%
% ctrl = createCtrlPnPMPC( objlss , flag )
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Design Plug-and-Play controllers for each subsystem of a large-scale system
%
% The function is written for Parallel Computing Toolbox.
% To take advantages of parallelization, before this function one can run 
% matlabpool(Ncore), with Ncore number of core.
%
% USAGE:
% ctrl = objlss.CREATECTRLPNP
% ctrl = objlss.CREATECTRLPNP( flag )
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - objlss is an lss object
% - flag is an optional parameter. it could be a vector of optional
%    parameters. (i-th element for i-th subsystem), see flag parameter for
%    function pnpctrl
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% ctrl is an array of lss.numSys elements. The i-th element is a pnpctrl object
% for the i-th subsystem
% 
%
% [1] S. Riverso and G. Ferrari-Trecate, "Plug-and-Play distributed model 
%     predictive control with coupling attenuation," Optimal Control 
%     Applications and Methods, p. Submitted, 2013.
% [2] S. Riverso, "Distributed and plug-and-play control for constrained 
%     systems," PhD dissertation, Universita' degli studi di Pavia, 2014.
%
% see also PNPCTRL PNPMPC PNPESTIMATOR
    
% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if ~isa(objlss,'lss')
        error('CREATECTRLPNP: first argument must be a lss object')
    end
    
    if nargin<1 || nargin>2
        error('CREATECTRLPNP: number of input arguments')
    end
    
    if isempty(objlss.Ts)
        error('CREATECTRLPNP: lss object must be a discrete-time system')
    end
    
    if nargin==1
        flag.distributedParents = [];
        flag.Qi                   = [];
        flag.Ri                   = [];
        flag.useConstraints       = 0 ;
        flag.norm                 = inf;
        flag.boundKij             = inf;
    end
    
    if ~(( size(flag,1)==1 && size(flag,2)==1 ) || ( size(flag,1)==objlss.numSys && size(flag,2)==1 ) || ( size(flag,1)==1 && size(flag,2)==objlss.numSys )) && ~isempty(flag)
        error('CREATEPNPESTIMATORS: input parameter flag')
    end
    
    if size(flag,1)==1 && size(flag,2)==1
        flag = repmat(flag,objlss.numSys,1);
    end
    
    ctrl(1,objlss.numSys) = pnpctrl;
    
    subssobj = cell(1,objlss.numSys);
    Xj       = cell(objlss.numSys,1);
    Hi       = cell(objlss.numSys,1);
    Ki       = cell(objlss.numSys,1);
    
    warning off %#ok<WNOFF>
    for i = 1:objlss.numSys
        subssobj{i} = objlss.getSystem(i);
        [ Hi{i}, Ki{i} ] = objlss.doubleStateConstraint(i);
        if isempty(Hi{i})
            Hi{i} = sparse(zeros(1,objlss.ni(i)));
            Ki{i} = 0;
        end
    end
    warning on %#ok<WNON>
    
    for i = 1:objlss.numSys
        Xj{i} = {Hi{subssobj{i}.couplingA};Ki{subssobj{i}.couplingA}}';
    end
    clear Hi Ki
    
    % Parallelization of the plug-and-play's design
    parfor i = 1:objlss.numSys
        try
            ctrl(i) = pnpctrl(subssobj{i},Xj{i},flag(i));
            disp(['CREATECTRLPNP: Controller ' subssobj{i}.name ' done']);
            Xj{i}       = [];
        catch e
            error(['SUBSYSTEM ' subssobj{i}.name '...' e.message]);
        end
    end
    
    K = zeros(sum(objlss.mi),sum(objlss.ni));
    for i=1:objlss.numSys
        K(sum(objlss.mi(1:i-1))+1:sum(objlss.mi(1:i)),sum(objlss.ni(1:i-1))+1:sum(objlss.ni(1:i))) = ctrl(i).Ki ;
        if ~isempty(ctrl(i).Kij)
            for jj=1:length(subssobj{i}.couplingA)
                j = subssobj{i}.couplingA(jj);
                K(sum(objlss.mi(1:i-1))+1:sum(objlss.mi(1:i)),sum(objlss.ni(1:j-1))+1:sum(objlss.ni(1:j))) = ctrl(i).Kij(:,sum(subssobj{i}.nj(1:jj-1))+1:sum(subssobj{i}.nj(1:jj))) ;
            end
        end
    end
end