function ctrl = createCtrlPnPMPC( objlss , N , flag , options )
% CREATECTRLPNPMPC design Plug-and-Play controllers for each subsystem of a large-scale system as proposed in [1], [2] and [3]
%
% ctrl = createCtrlPnPMPC( objlss , N , k , options )
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Design Plug-and-Play controllers for each subsystem of a large-scale system
%
% The function is written for Parallel Computing Toolbox.
% To take advantages of parallelization, before this function one can run 
% matlabpool(Ncore), with Ncore number of core.
% 
% ctrl = objlss.CREATECTRLPNPMPC( N , k , options )
%
% USAGE:
% ctrl = objlss.CREATECTRLPNPMPC( N , k )
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - objlss is an lss object
% - N is the horizon of the prediction. It can be a vector of lss.numSys
% elements
% - k must be >= of the controllability index of the pair (Aii,Bii). One can
% also set a cell array {kmin,kmax} in order to try different k. If k, kmin
% and kmax are vectors of lss.numSys elements, the i-th k, kmin and kmax are
% used to design the Plug-and-Play controller
% - options from YALMIP, i.e. sdpsettings object (optional)
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% ctrl is an array of lss.numSys elements. The i-th element is a pnpmpc object
% for the i-th subsystem
% 
%
% [1] S. Riverso, M. Farina, and G. Ferrari-Trecate, "Plug-and-Play Model 
%     Predictive Control based on robust control invariant sets,"
%     Automatica, p. Submitted, 2013.
% [2] S. Riverso, M. Farina, and G. Ferrari-Trecate, "Plug-and-Play 
%     Decentralized Model Predictive Control for Linear Systems," IEEE 
%     Trans. Automat. Contr., vol. 58, no. 10, pp. 2608-2614, 2013.
% [3] S. Riverso, "Distributed and plug-and-play control for constrained 
%     systems," Universita' degli studi di Pavia, 2014.
%
% see also PNPMPC
    
% Copyright is with the following author(s):
%
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if ~isa(objlss,'lss')
        error('CREATECTRLPNPMPC: first argument must be a lss object')
    end
    
    if nargin<2 || nargin>4
        error('CREATECTRLPNPMPC: number of input arguments')
    end
    
    if isempty(objlss.Ts)
        error('CREATECTRLPNPMPC: lss object must be a discrete-time system')
    end
    
    if ~isempty(objlss.Bcen)
        error('CREATECTRLPNPMPC: no centralized inputs to design PnPMPC controllers')
    end
    
    if ~isempty(objlss.C)
        warning('CREATECTRLPNPMPC: PNPMPC class will design a state-feedback PnPMPC controller')
    end
    
    if ~isempty(objlss.Hx)
        if size(objlss.Hx,1)~=sum(objlss.rx)
            error('CREATECTRLPNPMPC: no coupling constraints on state variables')
        end
    end
    
    if ~isempty(objlss.Hu)
        if size(objlss.Hu,1)~=sum(objlss.ru)
            error('CREATECTRLPNPMPC: no coupling constraints on input variables')
        end
    end    
    
    % Check flag
    if nargin<=2 || isempty(flag)
        flag.distributedParents = [];
        flag.LPdesign             = true ;
        flag.Qi                   = [];
        flag.Ri                   = [];
        flag.ExoBounded           = [];
        flag.norm                 = inf;
        flag.boundKij             = inf;
    end
    
    if ~(( size(flag,1)==1 && size(flag,2)==1 ) || ( size(flag,1)==objlss.numSys && size(flag,2)==1 ) || ( size(flag,1)==1 && size(flag,2)==objlss.numSys )) && ~isempty(flag)
        error('CREATECTRLPNPMPC: input parameter flag')
    end
    
    if size(flag,1)==1 && size(flag,2)==1
        flag = repmat(flag,objlss.numSys,1);
    end
    
    % Check N
    if length(N)==1
        if ~(N>0)
            error('CREATECTRLPNPMPC: N (receding horizon) must be positive')
        end
        N = repmat(N,1,objlss.numSys);
    else
        if length(N)~=objlss.numSys
            error('CREATECTRLPNPMPC: N should be a vector of objlss.numSys elements')
        end
        if any(N<=0)
            error('CREATECTRLPNPMPC: N (receding horizon) must be positive')
        end
    end
    
    if nargin <=3 || isempty(options)
        options = sdpsettings;
    end
    
    for i = 1:objlss.numSys
        ctrl(i) = pnpmpc;
    end
    
    subssobj = cell(objlss.numSys,1);    
    Xj       = cell(objlss.numSys,1);
    Uj       = cell(objlss.numSys,1);
    Hi       = cell(objlss.numSys,1);
    Ki       = cell(objlss.numSys,1); 
    HiU      = cell(objlss.numSys,1);
    KiU      = cell(objlss.numSys,1);
    for i = 1:objlss.numSys
        subssobj{i} = objlss.getSystem(i);
        [ Hi{i}  , Ki{i}  ] = objlss.doubleStateConstraint(i);
        [ HiU{i} , KiU{i} ] = objlss.doubleInputConstraint(i);
        if isempty(Hi{i})
            Hi{i} = sparse(zeros(1,objlss.ni(i)));
            Ki{i} = 0;
        end
        if isempty(HiU{i})
            HiU{i} = sparse(zeros(1,objlss.mi(i)));
            KiU{i} = 0;
        end
    end
    for i = 1:objlss.numSys
        Xj{i} = {Hi{subssobj{i}.couplingA};Ki{subssobj{i}.couplingA}}';
    end
    for i = 1:objlss.numSys
        Uj{i} = {HiU{subssobj{i}.couplingB};KiU{subssobj{i}.couplingB}}';
    end
    clear Hi Ki
    
    
    % Parallelization of the plug-and-play's design
    parfor i = 1:objlss.numSys
        try
            ctrl(i) = pnpmpc(subssobj{i},N(i),Xj{i},Uj{i},flag(i),options);
            disp(['CREATECTRLPNPMPC: Controller ' subssobj{i}.name ' done']);
            subssobj{i} = [];
            Xj{i}       = [];
        catch e
            error(['CREATECTRLPNPMPC: SUBSYSTEM ' subssobj{i}.name ' ... ' e.message]);
        end
    end
end