function [ kest , L ] = createPnPEstimators( objlss , flag )
% CREATEPNPESTIMATORS design Plug-and-Play state estimators for each subsystem of a large-scale system as proposed in [1]
% 
% kest = createPnPEstimators( objlss , flag )
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Design Plug-and-Play state estimators for each subsystem of a large-scale 
% system
%
% The function is written for Parallel Computing Toolbox.
% To take advantages of parallelization, before this function one can run 
% matlabpool(Ncore), with Ncore number of core.
% 
% kest = objlss.CREATEPNPESTIMATORS( flag )
%
% USAGE:
% kest = objlss.CREATEPNPESTIMATORS
% kest = objlss.CREATEPNPESTIMATORS( flag )
% [ kest , L ] = objlss.CREATEPNPESTIMATORS
%
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - objlss is an lss object
% - flag is a single struct or a cell array of struct where the fields are
%   as in pnpEstimator constructor
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% - kest is an array of lss.numSys elements. The i-th element is a pnpEstimator
%   object for the i-th subsystem
% - L is the overall gain matrix such that A+LC is stable
%
% [1] S. Riverso, M. Farina, R. Scattolini, and G. Ferrari-Trecate, 
%     "Plug-and-play distributed state estimation for linear systems," 
%     in Proceedings of the 52nd IEEE Conference on Decision and Control, 
%     2013, pp. 4889-4894.
%
% see also PNPESTIMATOR PNPMPC
    
% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if ~isa(objlss,'lss')
        error('CREATEPNPESTIMATORS: first argument must be a lss object')
    end
    
    if nargin<1 || nargin>2
        error('CREATEPNPESTIMATORS: number of input arguments')
    end
    
    if isempty(objlss.Ts)
        error('CREATEPNPESTIMATORS: lss object must be a discrete-time system')
    end
    
    if nargin==1
        flag.distributedParents = [];
        flag.Qi                   = [];
        flag.Ri                   = [];
        flag.boundedEstimation    = 0 ;
        flag.ExoBounded           = [];
        flag.norm                 = inf;
    end
    
    if isempty(objlss.pi)
        pi = objlss.ni;
    else
        pi = objlss.pi;
    end
    
    if ~(( size(flag,1)==1 && size(flag,2)==1 ) || ( size(flag,1)==objlss.numSys && size(flag,2)==1 ) || ( size(flag,1)==1 && size(flag,2)==objlss.numSys )) && ~isempty(flag)
        error('CREATEPNPESTIMATORS: input parameter flag')
    end
    
    if size(flag,1)==1 && size(flag,2)==1
        flag = repmat(flag,objlss.numSys,1);
    end
    
    kest(objlss.numSys) = pnpEstimator;
    
    subssobj = cell(objlss.numSys,1);
    Ej       = cell(objlss.numSys,1);
    Oj       = cell(objlss.numSys,1);
    Ci       = cell(objlss.numSys,1);
    Cj       = cell(objlss.numSys,1);
    Hi       = cell(objlss.numSys,1);
    Ki       = cell(objlss.numSys,1);
    Hoi      = cell(objlss.numSys,1);
    Koi      = cell(objlss.numSys,1);
    
    warning off %#ok<WNOFF>
    for i = 1:objlss.numSys
        subssobj{i} = objlss.getSystem(i);
        [ Hi{i}, Ki{i} ] = objlss.doubleErrorEstimationConstraint(i);
        if isempty(Hi{i})
            Hi{i} = sparse(zeros(1,objlss.ni(i)));
            Ki{i} = 0;
        end
        Ci{i} = objlss.getC(i);
        
        [ Htmp , Ktmp , Dindex ] = objlss.doubleExoConstraintOnSystem(i);
        if isempty(Dindex)
            Hoi{i} = [];
            Koi{i} = [];
        else
            Ntmp   = objlss.getN(i,Dindex);
            V.V    = pnpmpc.computeExtreme( Ntmp , Htmp , Ktmp , [] , [] , [] , [] , [] , [] );
            if all(all(V.V<=1e-6))
                Hoi{i} = [];
                Koi{i} = [];
            else
                st     = cddmex('hull',V);
                Hoi{i} = st.A;
                Koi{i} = st.B;
            end
        end
    end
    warning on %#ok<WNON>
    
    for i = 1:objlss.numSys
        Ej{i} = {Hi{subssobj{i}.couplingA};Ki{subssobj{i}.couplingA}}';
        Cj{i} = {Ci{subssobj{i}.couplingA}}';
        Oj{i} = {Hoi{subssobj{i}.couplingA};Koi{subssobj{i}.couplingA}}';
    end
    clear Hi Ki Ci Hoi Koi
        
    % Parallelization of the plug-and-play's design
    parfor i = 1:objlss.numSys
        try
            kest(i) = pnpEstimator(subssobj{i},Ej{i},Cj{i},Oj{i},flag(i));
            disp(['CREATEPNPESTIMATORS: Estimator ' subssobj{i}.name ' done']);
            Ej{i}       = [];
            Cj{i}       = [];
        catch e
            error(['CREATEPNPESTIMATORS: SUBSYSTEM ' subssobj{i}.name ' ... ' e.message]);
        end
    end
    
    L = zeros(sum(objlss.ni),sum(pi));
    for i=1:objlss.numSys
        L(sum(objlss.ni(1:i-1))+1:sum(objlss.ni(1:i)),sum(pi(1:i-1))+1:sum(pi(1:i))) = kest(i).Li ;
        if ~isempty(kest(i).Lij)
            for jj=1:length(subssobj{i}.couplingA)
                j = subssobj{i}.couplingA(jj);
                L(sum(objlss.ni(1:i-1))+1:sum(objlss.ni(1:i)),sum(pi(1:j-1))+1:sum(pi(1:j))) = kest(i).Lij(:,sum(pi(1:jj-1))+1:sum(pi(1:jj))) ;
            end
        end
    end
    L = sparse(L);
end