function [Hcen,Kcen] = doubleCenInputConstraint(obj,index,selection)
% DOUBLECENINPUTCONSTRAINT returns Hcen and Kcen matrices defining the Polyhedron Ucen
% 
% [Hcen,Kcen] = doubleCenInputConstraint(obj,index,selection)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return Hcen and Kcen matrix and the Polyhedron Uc
%
% [Hcen,Kcen] = objlss.DOUBLEINPUTCONSTRAINT(index,selection)
% 
% USAGE:
% [Hcen,Kcen] = objlss.doubleCenInputConstraint() return overall centralized 
% input constraints
% [Hcen,Kcen] = objlss.doubleCenInputConstraint(index)
% Uc = objlss.doubleCenInputConstraint(index,selection)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% index = vector of numbers or names (as cellarray) of involved centralized
% inputs 
% 
% selection = 0 exclusive: return matrices H and K of constraints that 
% involve ONLY systems specified in index
%
% selection = 1 all: return matrices H and K of constraints that 
% involve ALSO systems specified in index
% 
% example: constraint 'Uc' between system [1,3,5]
% with index = [1,5]
% selection=0--> return Hcen=[] and Kcen=[] because there is no constraint 
% between systems 1 and 5 only
% selection=1--> return constraint 'Uc' [1,3,5] because it contains indeces 
% 1 and 5
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% [Hcen, Kcen]  Hcen and Kcen matrices defining the Polyhedron Uc
% = { Hcen*ucen<=Kcen }
% 
% [Uc] Polyhedron object [1]
% 
%
% see also ADDCENTRALIZEDINPUTCONSTRAINT REMOVECONSTRAINT
%
% [1] M. Herceg, M. Kvasnica, C. N. Jones, and M. Morari, "Multi-Parametric 
% Toolbox 3.0," in Proceedings of the 12th European Control Conference, 
% 2013, pp. 502-510, http://control.ee.ethz.ch/~mpt

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin>3
        error('DOUBLECENINPUTCONSTRAINT: numbers of input')
    end
        
    if nargin==1
        Hcen=obj.Hcen;
        Kcen=obj.Kcen;
        if nargout==1 || nargout==0
            Hcen=Polyhedron(full(Hcen),full(Kcen));
        end
        return
    end
    
    if nargin==2
        selection = 0;
    end
    
    if ischar(index)
        name  = index;
        for k=1:size(obj.nameucen,2)
            if strcmp(name,obj.nameucen{k})
                index = k;
                break;
            end
        end
    elseif iscellstr(index)
        name  = index;
        index = inf*ones(length(index),1);
        for i=1:length(index)
            for k=1:size(obj.nameucen,2)
                if strcmp(name{i},obj.nameucen{k})
                    index(i) = k;
                    break;
                end
            end
        end
    end    
    
    if ~isscalar(index) && ~isvector(index)
        error('DOUBLECENINPUTCONSTRAINT: see help doubleCenInputConstraint')
    end
    
    if any(index>obj.numICen) || any(index<1)
        error('DOUBLECENINPUTCONSTRAINT: check index parameter')
    end    
        
    if selection~=0 && selection ~=1
        error('DOUBLECENINPUTCONSTRAINT: selection must be 0 or 1')
    end
    
    if isempty(obj.rcen)
        Hcen = [];
        Kcen = [];
        warning('MATLAB:paramAmbiguous','DOUBLECENINPUTCONSTRAINT: Polytope P = R^%d',length(index));
        return
    end
    
    if isscalar(index) && selection == 0
        Hcen = obj.Hcen(sum(obj.rcen(1:index-1))+1:sum(obj.rcen(1:index)), index );
        Kcen = obj.Kcen(sum(obj.rcen(1:index-1))+1:sum(obj.rcen(1:index)));
    end
    
    if isscalar(index) && selection == 1
        Hcen = obj.Hcen(:,index);
        mask = any(Hcen~=0,2);
        Hcen = obj.Hcen(mask,:);
        Kcen = obj.Kcen(mask);
    end
    
    if length(index)>1
        Haux= obj.Hcen(sum(obj.rcen)+1:end,:);
        Kaux= obj.Kcen(sum(obj.rcen)+1:end,1);
        mask=zeros(1,obj.numICen);
        for i=1:length(index)
            start=sum(obj.rcen(1:index(i)))-obj.rcen(index(i))+1;
            mask(1,start:start+obj.rcen(index(i))-1)=ones(1,obj.rcen(index(i)));
        end
        mask = all(mask>0.5,1);
        help = Haux(:,mask);
        flag = sparse(all(help~=0,2));
        
        if selection==1 %all
            Hcen = Haux(flag,:);
            Kcen = Kaux(flag,1);
        elseif selection==0 %exclusive
            newhelp=Haux(:,~mask);
            newflag=all(newhelp==0,2);
            okrows=flag' & newflag';
            Hcen=Haux(okrows,mask);
            Kcen=Kaux(okrows,:);
        end
    end
    
    if nargout==1 || nargout==0
        if isempty(Hcen)
            Hcen = [];
            warning('MATLAB:paramAmbiguous','DOUBLECENINPUTCONSTRAINT: Polyhedron Ucen = R^%d',length(index));
        else
            Hcen=Polyhedron(full(Hcen),full(Kcen));
        end
    end
    
end