function [Hcen,Kcen,Ucindex] = doubleCenInputConstraintOnSystem(obj,i)
% DOUBLECENINPUTCONSTRAINTONSYSTEM returns Hcen and  Kcen matrcies defining the Polyhedron Uc
% 
% [Hcen,Kcen,Ucindex] = doubleCenInputConstraintOnSystem(obj,i)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return Hcen and  Kcen matrices and the Polyhedron Uc of centralized inputs that 
% affect subsystem i. 
%
% [Hcen,Kcen,Ucindex] = objlss.DOUBLECENINPUTCONSTRAINTONSYSTEM(i)
%
% USAGE: 
% [Uc] = objlss.doubleCenInputConstraintOnSystem(i)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% i = number or name of the system interested
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% [Hcen, Kcen,Ucindex] Hcen and Kcen matrices defining the Polyhedron Uc
% = { Hcen*ucen<=Kucen }
% Ucindex is the set of centralized inputs affecting subsystem i
%
% [Uc] Polyhedron object [1]
%
%
% see also DOUBLECENINPUTCONSTRAINT ADDCENTRALIZEDINPUTCONSTRAINT
%
% [1] M. Herceg, M. Kvasnica, C. N. Jones, and M. Morari, "Multi-Parametric 
% Toolbox 3.0," in Proceedings of the 12th European Control Conference, 
% 2013, pp. 502-510, http://control.ee.ethz.ch/~mpt

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if ischar (i)
        i = obj.name2index(i);
    end
    
    if isempty(obj.Bcen) && isempty(obj.Dcen)
        Hcen = [];
        Kcen = [];
        Ucindex = [];
        return
    end
    
    if isempty(obj.rcen)
        Hcen = [];
        Kcen = [];
        Ucindex = [];
        return
    end
    
    if ~isempty(obj.Bcen) || ~isempty(obj.Dcen)
        if ~isempty(obj.Bcen)
            Baux = obj.Bcen(sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)),:);
            maskB = any(Baux~=0);
        end        
        if ~isempty(obj.Dcen)
            if isempty(obj.pi)
                pi = obj.ni;
            else
                pi = obj.pi;
            end
            Daux = obj.Dcen(sum(pi(1:i-1))+1:sum(pi(1:i)),:);
            maskD = any(Daux~=0);
            mask = maskB | maskD;
        else
            mask = maskB;
        end
        
        Hcen = [];
        Kcen = [];
        if any(mask~=0)
            for k=find(mask==1)
                % local constraints
                Hcen = [Hcen;obj.Hcen(sum(obj.rcen(1:k-1))+1:sum(obj.rcen(1:k)),:)];
                Kcen = [Kcen;obj.Kcen(sum(obj.rcen(1:k-1))+1:sum(obj.rcen(1:k)),1)];
            end
        end
        Hcen = Hcen(:,mask);
        % coupled constraints
        if sum(obj.rcen) ~= size(obj.Hcen,1)
            Haux = obj.Hcen(sum(obj.rcen)+1:end,mask);
            Kaux = obj.Kcen(sum(obj.rcen)+1:end,1);
            Hcen = [Hcen;Haux(any(Haux~=0,2),:)];
            Kcen = [Kcen;Kaux(any(Haux~=0,2))];
        end
        Ucindex = 1:obj.numICen;
        Ucindex = Ucindex(mask);     
    end
    
    if nargout==1 || nargout==0
        if isempty(Hcen)
            Hcen = [];
            warning('DOUBLECENINPUTCONSTRAINTONSYSTEM: Polyhedron Uc = R^%d',sum(obj.ni(index))); %#ok<WNTAG>
        else
            Hcen = Polyhedron(full(Hcen),full(Kcen));
        end
        return
    end
    
end