function [Hdeltau,Kdeltau] = doubleDeltaUConstraint(obj,index,selection)
% DOUBLDELTAUCONSTRAINT returns Hdeltau and Kdeltau matrices defining the Polyhedron DeltaU
% 
% [Hdeltau,Kdeltau] = doubleDeltaUConstraint(obj,index,selection)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Hdeltau and Kdeltau matrices and the Polyhedron DeltaU. 
% deltau is defined as u(k+1)-u(k).
% 
% [Hdeltau,Kdeltau] = objlss.DOUBLEDELTAUCONSTRAINT(index,selection)
%
% USAGE: 
% [Hdeltau,Kdeltau] = objlss.doubleDeltaUConstraint() return overall deltaU
% constraints
% [Hdeltau,Kdeltau] = objlss.doubleDeltaUConstraint(index) 
% DU = objlss.doubleDeltaUConstraint(index,selection)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% index = vector of numbers or names (as cellarray) of involved subsystems 
% 
% selection = 0 exclusive: return matrices H and K of constraints that 
% involve ONLY systems specified in index
%
% selection = 1 all: return matrices H and K of constraints that 
% involve ALSO systems specified in index
% 
% example: constraint 'DU' between system [1,3,5]
% with index = [1,5]
% selection=0--> return Hdeltau=[] and Kdeltau=[] because there is no constraint 
% between systems 1 and 5 only
% selection=1--> return constraint 'DU' [1,3,5] because it contains indeces 
% 1 and 5
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% [Hdeltau, Kdeltau] Hdeltau and Kdeltau matrices defining the Polyhedron DeltaU
% = { Hdeltau*deltau<=Kdeltau }
% 
% [DU] Polyhedron object [1]
% 
%
% see also ADDDELTAUCONSTRAINT
%
% [1] M. Herceg, M. Kvasnica, C. N. Jones, and M. Morari, "Multi-Parametric 
% Toolbox 3.0," in Proceedings of the 12th European Control Conference, 
% 2013, pp. 502-510, http://control.ee.ethz.ch/~mpt

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin >3
        error('DOUBLDELTAUCONSTRAINT : numbers of input')
    end
    
    if nargin==1
        Hdeltau = obj.Hdeltau;
        Kdeltau = obj.Kdeltau;
        if nargout==1 || nargout==0
            Hdeltau = Polyhedron(full(Hdeltau),full(Kdeltau));
        end
        return
    end
    
    if nargin==2
        selection = 0;
    end
    
    if ischar(index)
        index = obj.name2index(index);
    end
    
    if iscellstr(index)
        vect = [];
        for k=1:size(index,2)
            vect(k) = obj.name2index(index(k));
        end
        index = vect;
    end
    
    if ~isscalar(index) && ~isvector(index)
        error('DOUBLDELTAUCONSTRAINT: see help doubleDeltaUConstraint')
    end
       
    if any(index>obj.numSys) || any(index<1)
        error('DOUBLEDELTAUCONSTRAINT: check index parameter')
    end
    
    if selection~=0 && selection~=1
        error('DOUBLDELTAUCONSTRAINT: selection must be 0 or 1')
    end

    if isempty (obj.rdeltau)
        Hdeltau = [];
        Kdeltau = [];
        warning('MATLAB:paramAmbiguous','DOUBLEDELTAUCONSTRAINT: Polyhedron deltaU = R^%d',sum(obj.mi(index)));
        return
    end
    
    if isscalar(index) && selection==0
            Hdeltau = obj.Hdeltau(sum(obj.rdeltau(1:index-1))+1:sum(obj.rdeltau(1:index)), sum(obj.mi(1:index-1))+1:sum(obj.mi(1:index)));
            Kdeltau = obj.Kdeltau(sum(obj.rdeltau(1:index-1))+1:sum(obj.rdeltau(1:index)));
    end
    
    if isscalar(index) && selection == 1
        Hdeltau = obj.Hdeltau(:,sum(obj.mi(1:index-1))+1:sum(obj.mi(1:index)));
        mask = any(Hdeltau~=0,2);
        
        Hdeltau = obj.Hdeltau(mask,:);
        Kdeltau = obj.Kdeltau (mask);
    end
    
    if length(index)>1
        Haux= obj.Hdeltau(sum(obj.rdeltau)+1:end,:);
        Kaux= obj.Kdeltau(sum(obj.rdeltau)+1:end,1);
        mask=zeros(1,sum(obj.mi));
        for i=1:length(index)
            start=sum(obj.mi(1:index(i)))-obj.mi(index(i))+1;
            mask(1,start:start+obj.mi(index(i))-1)=ones(1,obj.mi(index(i)));
        end
        mask=all(mask>0.5,1);
        help=Haux(:,mask);
        flag=help~=0;
        miindex = obj.mi(index);
        flagtmp = sparse(zeros(size(help,1),length(miindex))==1);
        for i = 1:length(miindex)
            flagtmp(:,i) = any( flag( : , sum(miindex(1:i-1))+1:sum(miindex(1:i)) ) , 2 );
        end
        flag = all(flagtmp,2);
        
        if selection==1 %all
            Hdeltau=Haux(flag,:);
            Kdeltau=Kaux(flag,1);
        elseif selection==0 %exclusive
            newhelp=Haux(:,~mask);
            newflag=all(newhelp==0,2);
            okrows=flag' & newflag';
            Hdeltau=Haux(okrows,mask);
            Kdeltau=Kaux(okrows,:);
        end
    end
    
    if nargout==1 || nargout==0
        if isempty(Hdeltau)
            Hdeltau = [];
            warning('MATLAB:paramAmbiguous','DOUBLEDELTAUCONSTRAINT: Polyhedron deltaU = R^%d',sum(obj.mi(index)));
        else
            Hdeltau = Polyhedron(full(Hdeltau),full(Kdeltau));
        end
    end
    
end