function [He,Ke] = doubleErrorEstimationConstraint(obj,index,selection)
% DOUBLEERRORESTIMATIONCONSTRAINT returns He and Ke matrices defining the Polyhedron E
% 
% [He,Ke] = doubleErrorEstimationConstraint(obj,index,selection)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Returns He and Ke matrices and the Polyhedron E. 
% e is defined as x-xc, where xc is the state estimation
% 
% [He,Ke] = objlss.DOUBLEERRORESTIMATIONCONSTRAINT(index,selection)
%
% USAGE: 
% [He,Ke] = objlss.doubleErrorEstimationConstraint() return overall error
% estimation constraints
% [He,Ke] = objlss.doubleErrorEstimationConstraint(index) 
% X = objlss.doubleErrorEstimationConstraint(index,selection)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% index = vector of numbers or names (as cellarray) of involved subsystems 
% 
% selection = 0 exclusive: return matrices H and K of constraints that 
% involve ONLY systems specified in index
%
% selection = 1 all: return matrices H and K of constraints that 
% involve ALSO systems specified in index
% 
% example: constraint 'E' between system [1,3,5]
% with index = [1,5]
% selection=0--> return He=[] and Ke=[] because there is no constraint 
% between systems 1 and 5 only
% selection=1--> return constraint 'E' [1,3,5] because it contains indeces 
% 1 and 5
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% [He, Ke] He and Ke matrices defining the Polyhedron E
% = { He*e<=Ke }
% 
% [E] Polyhedron object [1]
% 
%
% see also ADDERRORESTIMATIONCONSTRAINT
%
% [1] M. Herceg, M. Kvasnica, C. N. Jones, and M. Morari, "Multi-Parametric 
% Toolbox 3.0," in Proceedings of the 12th European Control Conference, 
% 2013, pp. 502-510, http://control.ee.ethz.ch/~mpt

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin >3
        error('DOUBLEERRORESTIMATIONCONSTRAINT : numbers of input')
    end
    
    if nargin==1
        He = obj.He;
        Ke = obj.Ke;
        if nargout==1 || nargout==0
            He = Polyhedron(full(He),full(Ke));            
        end
        return
    end
    
    if nargin==2
        selection = 0;
    end
    
    if ischar(index)
        index = obj.name2index(index);
    end
    
    if iscellstr(index)
        vect = [];
        for k=1:size(index,2)
            vect(k) = obj.name2index(index(k));
        end
        index = vect;
    end
    
    if ~isscalar(index) && ~isvector(index)
        error('DOUBLEERRORESTIMATIONCONSTRAINT: see help doubleErrorEstimationConstraint')
    end
    
    if any(index>obj.numSys) || any(index<1)
        error('DOUBLEERRORESTIMATIONCONSTRAINT: check index parameter')
    end
    
    if selection~=0 && selection~=1
        error('DOUBLEERRORESTIMATIONCONSTRAINT: selection must be 0 or 1')
    end
    
    if isempty(obj.re)
        He = [];
        Ke = [];
        warning('MATLAB:paramAmbiguous','DOUBLEERRORESTIMATIONCONSTRAINT: Polyhedron E = R^%d',sum(obj.ni(index)));
        return
    end
    if isscalar(index) && selection==0
            He = obj.He(sum(obj.re(1:index-1))+1:sum(obj.re(1:index)), sum(obj.ni(1:index-1))+1:sum(obj.ni(1:index)));
            Ke = obj.Ke(sum(obj.re(1:index-1))+1:sum(obj.re(1:index)));
    end
    
    if isscalar(index) && selection == 1
        He = obj.He(:,sum(obj.ni(1:index-1))+1:sum(obj.ni(1:index)));
        mask = any(He~=0,2);
        
        He = obj.He(mask,:);
        Ke = obj.Ke (mask);
    end
    
    if length(index)>1
        Haux= obj.He(sum(obj.re)+1:end,:);
        Kaux= obj.Ke(sum(obj.re)+1:end,1);
        mask=zeros(1,sum(obj.ni));
        for i=1:length(index)
            start=sum(obj.ni(1:index(i)))-obj.ni(index(i))+1;
            mask(1,start:start+obj.ni(index(i))-1)=ones(1,obj.ni(index(i)));
        end
        mask=all(mask>0.5,1);
        help=Haux(:,mask);
        flag=help~=0;
        niindex = obj.ni(index);
        flagtmp = sparse(zeros(size(help,1),length(niindex))==1);
        for i = 1:length(niindex)
            flagtmp(:,i) = any( flag( : , sum(niindex(1:i-1))+1:sum(niindex(1:i)) ) , 2 );
        end
        flag = all(flagtmp,2);
        
        if selection==1 %all
            He = Haux(flag,:);
            Ke = Kaux(flag,1);
        elseif selection==0 %exclusive
            newhelp = Haux(:,~mask);
            newflag = all(newhelp==0,2);
            okrows  = flag' & newflag';
            He      = Haux(okrows,mask);
            Ke      = Kaux(okrows,:);
        end
    end
    
    if nargout==1 || nargout==0
        if isempty(He)
                He = [];
                warning('MATLAB:paramAmbiguous','DOUBLEERRORESTIMATIONCONSTRAINT: Polyhedron E = R^%d',sum(obj.ni(index)));
        else   
            He = Polyhedron(full(He),full(Ke));
        end
    end
    
end