function [Hd,Kd] = doubleExoConstraint(obj,index,selection)
% DOUBLEEXOCONSTRAINT returns Hd and Kd matrices defining the Polyhedron D
% 
% [Hd,Kd] = doubleExoConstraint(obj,index,selection)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Ruturn Hd and Kd matrices and the Polyhedron D
% 
% [Hd,Kd] = objlss.DOUBLEEXOCONSTRAINT(index,selection)
%
% USAGE: 
% [Hd,Kd] = objlss.doubleExoConstraint() return overall exo constraints
% [Hd,Kd] = objlss.doubleExoConstraint(index)
% D = objlss.doubleExoConstraint(index,selection)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% index = vector of numbers or names (as cellarray) of involved exogenous
% signals 
% 
% selection = 0 exclusive: return matrices H and K of constraints that 
% involve ONLY systems specified in index (default)
%
% selection = 1 all: return matrices H and K of constraints that 
% involve ALSO systems specified in index
% 
% example: constraint 'D' between exo [1,3,5]
% with index = [1,5]
% selection=0--> return Hd=[] and Kd=[] because there is no constraint 
% between systems 1 and 5 only
% selection=1--> return constraint 'D' [1,3,5] because it contains indeces 
% 1 and 5
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% [Hd, Kd] Hd and Kd matrices defining the Polyhedron D
% = { Hd*d<=Kd }
%
% [D] Polyhedron object [1]
% 
% 
% [1] M. Herceg, M. Kvasnica, C. N. Jones, and M. Morari, "Multi-Parametric 
% Toolbox 3.0," in Proceedings of the 12th European Control Conference, 
% 2013, pp. 502-510, http://control.ee.ethz.ch/~mpt

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin >3
        error('DOUBLEEXOCONSTRAINT : numbers of input')
    end
    
    if nargin==1
        Hd=obj.Hd;
        Kd=obj.Kd;
        if nargout==1 || nargout==0
            Hd=Polyhedron(full(Hd),full(Kd));
        end
        return
    end
    
    if nargin==2
        selection = 0;
    end
    
    if ischar(index)
        name  = index;
        for k=1:size(obj.named,2)
            if strcmp(name,obj.named{k})
                index = k;
                break;
            end
        end
    elseif iscellstr(index)
        name  = index;
        index = inf*ones(length(index),1);
        for i=1:length(index)
            for k=1:size(obj.named,2)
                if strcmp(name{i},obj.named{k})
                    index(i) = k;
                    break;
                end
            end
        end
    end
    
    if ~isscalar(index) && ~isvector(index)
        error('DOUBLEEXOCONSTRAINT: see help doubleExoConstraint')
    end
    
    if any(index>obj.numExo) || any(index<1)
        error('DOUBLEEXOCONSTRAINT: check index parameter')
    end 
    
    if selection~=0 && selection ~=1
        error('DOUBLEEXOCONSTRAINT: selection must be 0 or 1')
    end
    
    if isempty(obj.rd)
        Hd = [];
        Kd = [];
        warning('MATLAB:paramAmbiguous','DOUBLEEXOCONSTRAINT: Polyhedron D = R^%d',length(index));
        return
    end
    
    if isscalar(index) && selection==0
        Hd = obj.Hd(sum(obj.rd(1:index-1))+1:sum(obj.rd(1:index)), index );
        Kd = obj.Kd(sum(obj.rd(1:index-1))+1:sum(obj.rd(1:index)));
    end
    
    if isscalar(index) && selection == 1
        Hd = obj.Hd(:,index);
        mask = any(Hd~=0,2);
        Hd = obj.Hd(mask,:);
        Kd = obj.Kd (mask);
    end
    
    if length(index)>1
        Haux= obj.Hd(sum(obj.rd)+1:end,:);
        Kaux= obj.Kd(sum(obj.rd)+1:end,1);
        mask=zeros(1,obj.numExo);
        for i=1:length(index)
            start=sum(obj.rd(1:index(i)))-obj.rd(index(i))+1;
            mask(1,start:start+obj.rd(index(i))-1)=ones(1,obj.rd(index(i)));
        end
        mask = all(mask>0.5,1);
        help = Haux(:,mask);
        flag = sparse(all(help~=0,2));
        
        if selection==1 %all
            Hd = Haux(flag,:);
            Kd = Kaux(flag,1);
        elseif selection==0 %exclusive
            newhelp=Haux(:,~mask);
            newflag=all(newhelp==0,2);
            okrows=flag' & newflag';
            Hd=Haux(okrows,mask);
            Kd=Kaux(okrows,:);
        end
    end
    
    
    if nargout==1 || nargout==0
        if isempty(Hd)
            Hd = [];
            warning('MATLAB:paramAmbiguous','DOUBLEEXOCONSTRAINT: Polyhedron D = R^%d',length(index));
        else
            Hd=Polyhedron(full(Hd),full(Kd));
        end
    end
    
end