function [Hd,Kd,Dindex] = doubleExoConstraintOnSystem(obj,i)
% DOUBLEEXOCONSTRAINTONSYSTEM returns Hd and Kd matrices defining the Polyhedron D
%
% [Hd,Kd,Dindex] = doubleExoConstraintOnSystem(obj,i)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return Hd and Kd matrices and the Polyhedron D of exogenous inputs acting 
% on subsystem i
%
% [Hd,Kd,Dindex] = objlss.DOUBLEEXOCONSTRAINTONSYSTEM(i)
%
% USAGE: 
% D = objlss.doubleExoConstraint(i)
%
% ---------------------------------------------------------------------------
% INPUT
% --------------------------------------------------------------------------- 
% i = number or name of the system interested
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% [Hd, Kd, Dindex] Hd and Kd matrices defining the Polyhedron D
% = { Hd*d<=Kd }
% Dindex is the set of centralized inputs affecting subsystem i
%
% [D] if interested only at the Polyhedron [1]
%
%
% see also DOUBLEEXOCONSTRAINT ADDEXOCONSTRAINT
%
% [1] M. Herceg, M. Kvasnica, C. N. Jones, and M. Morari, "Multi-Parametric 
% Toolbox 3.0," in Proceedings of the 12th European Control Conference, 
% 2013, pp. 502-510, http://control.ee.ethz.ch/~mpt

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if ischar (i)
        i = obj.name2index(i);
    end
        
    if isempty(obj.M) && isempty(obj.N)
        Hd = [];
        Kd = [];
        Dindex = [];
        return
    end
    
    if isempty(obj.rd)
        Hd = [];
        Kd = [];
        Dindex = [];
        return
    end
    
    if ~isempty(obj.M) || ~isempty(obj.N)
        if ~isempty(obj.M)
            Maux = obj.M(sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)),:);
            maskM = any(Maux~=0);
        end
        if ~isempty(obj.N)
            if isempty(obj.pi)
                pi = obj.ni;
            else
                pi = obj.pi;
            end
            Naux = obj.N(sum(pi(1:i-1))+1:sum(pi(1:i)),:);
            maskN = any(Naux~=0);
            mask = maskM | maskN;
        else
            mask = maskM;
        end
        
        Hd = [];
        Kd = [];
        if any(mask~=0)
            for k=find(mask==1)
                % local constraints                
                Hd = [Hd;obj.Hd(sum(obj.rd(1:k-1))+1:sum(obj.rd(1:k)),:)];
                Kd = [Kd;obj.Kd(sum(obj.rd(1:k-1))+1:sum(obj.rd(1:k)),1)];
            end
        end
        Hd = Hd(:,mask);
        % coupled constraints
        if sum(obj.rd) ~= size(obj.Hd,1)
            Haux = obj.Hd(sum(obj.rd)+1:end,mask);
            Kaux = obj.Kd(sum(obj.rd)+1:end,1);
            Hd = [Hd;Haux(any(Haux~=0,2),:)];
            Kd = [Kd;Kaux(any(Haux~=0,2))];
        end
        Dindex = 1:obj.numExo;
        Dindex = Dindex(mask);
    end
    
    if nargout==1 || nargout==0
        if isempty(Hd)
            Hd = [];
            warning('DOUBLEEXOCONSTRAINTONSYSTEM: Polyhedron D = R^%d',sum(obj.ni(index))); %#ok<WNTAG>
        else
            Hd=Polyhedron(full(Hd),full(Kd));
        end
        return
    end
    
end