function [Hu,Ku] = doubleInputConstraint(obj,index,selection)
% DOUBLEINPUTCONSTRAINT returns Hu and Ku matrices defining the Polyhedron U
% 
% [Hu,Ku] = doubleInputConstraint(obj,index,selection)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return Hu and Ku matrices and the Polyhedron U
% 
% [Hu,Ku] = objlss.DOUBLEINPUTCONSTRAINT(index,selection)
%
% USAGE: 
% [Hu,Ku] = objlss.doubleInputConstraint() return overall input constraints
% [Hu,Ku] = objlss.doubleInputConstraint(index) 
% U = objlss.doubleInputConstraint(index,selection)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% index = vector of numbers or names (as cellarray) of involved systems 
% 
% selection = 0 exclusive: return matrices H and K of constraints that 
% involve ONLY systems specified in index
%
% selection = 1 all: return matrices H and K of constraints that 
% involve ALSO systems specified in index
% 
% example: constraint 'U' between system [1,3,5]
% with index = [1,5]
% selection=0--> return Hu=[] and Ku=[] because there is no constraint 
% between systems 1 and 5 only
% selection=1--> return constraint 'U' [1,3,5] because it contains indeces 
% 1 and 5
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% [Hu, Ku] Hu and Ku matrices defining the Polyhedron U
% = { Hu*u<=Ku }
% 
% [U] Polyhedron object [1]
% 
%
% see also ADDINPUTCONSTRAINT
%
% [1] M. Herceg, M. Kvasnica, C. N. Jones, and M. Morari, "Multi-Parametric 
% Toolbox 3.0," in Proceedings of the 12th European Control Conference, 
% 2013, pp. 502-510, http://control.ee.ethz.ch/~mpt

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

   if nargin>3
       error('DOUBLEINPUTCONSTRAINT : numbers of input')
   end
   
   if nargin==1
       Hu=obj.Hu;
       Ku=obj.Ku;
       if nargout==1 || nargout==0
           Hu = Polyhedron(full(Hu),full(Ku));
       end
       return
   end
   
   if nargin==2
       selection = 0;
   end
   
   if ischar(index)
       index=obj.name2index(index);
   end
   
   if iscellstr(index)
       vect=[];
       for k=1:size(index,2)
           vect(k)=obj.name2index(index(k));
       end
       index=vect;
   end
   
   if ~isscalar(index) && ~isvector(index)
       error('DOUBLEINPUTCONSTRAINT: see help doubleInputConstraint')
   end
   
   if any(index>obj.numSys) || any(index<1)
       error('DOUBLEINPUTCONSTRAINT: check index parameter')
   end
   
   if selection~=0 && selection ~=1
       error('DOUBLEINPUTCONSTRAINT: selection must be 0 or 1')
   end
   
   if isempty(obj.ru)
       Hu = [];
       Ku = [];
       warning('MATLAB:paramAmbiguous','DOUBLEINPUTCONSTRAINT: Polyhedron U = R^%d',sum(obj.mi(index)));
       return
   end
      
   if isscalar(index) && selection ==0
       Hu=obj.Hu(sum(obj.ru(1:index-1))+1:sum(obj.ru(1:index)), sum(obj.mi(1:index-1))+1:sum(obj.mi(1:index)));
       Ku=obj.Ku(sum(obj.ru(1:index-1))+1:sum(obj.ru(1:index)));
   end
   
   if isscalar(index) && selection == 1
       Hu = obj.Hu(:,sum(obj.mi(1:index-1))+1:sum(obj.mi(1:index)));
       mask = any(Hu~=0,2);
       
       Hu = obj.Hu(mask,:);
       Ku = obj.Ku (mask);
   end
   
   if length(index)>1
       Haux = obj.Hu(sum(obj.ru)+1:end,:);
       Kaux = obj.Ku(sum(obj.ru)+1:end,1);
       mask =   zeros(1,sum(obj.mi));
       for i=1:length(index)
           start =  sum(obj.mi(1:index(i)))-obj.mi(index(i))+1;
           mask(1,start:start+obj.mi(index(i))-1) = ones(1,obj.mi(index(i)));
       end
       mask =  all(mask>0.5,1);
       help =  Haux(:,mask);
       flag =  help~=0;
       miindex = obj.mi(index);
       flagtmp = sparse(zeros(size(help,1),length(miindex))==1);
       for i = 1:length(miindex)
           flagtmp(:,i) = any( flag( : , sum(miindex(1:i-1))+1:sum(miindex(1:i)) ) , 2 );
       end
       flag = all(flagtmp,2);
        
       if selection==1 %all
           Hu   =   Haux(flag,:);
           Ku   =   Kaux(flag,1);
       elseif selection==0 %exclusive
           newhelp  =   Haux(:,~mask);
           newflag  =   all(newhelp==0,2);
           okrows   =   flag' & newflag';
           Hu       =   Haux(okrows,mask);
           Ku       =   Kaux(okrows,:);
       end
   end
   
   if nargout==1 || nargout==0
       if isempty(Hu)
            Hu = [];
            warning('MATLAB:paramAmbiguous','DOUBLEINPUTCONSTRAINT: Polyhedron U = R^%d',sum(obj.mi(index)));
       else
           Hu = Polyhedron(full(Hu),full(Ku));
       end
   end
   
end