function [Hy,Ky] = doubleOutConstraint(obj,index,selection)
% DOUBLEOUTCONSTRAINT returns Hy and Ky matrices defining the Polyhedron Y
% 
% [Hy,Ky] = doubleOutConstraint(obj,index,selection)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return Hy and Ky matrices and the Polyhedron Y
%
% [Hy,Ky] = objlss.DOUBLEOUTCONSTRAINT(index,selection)
%
% USAGE:  
% [Hy,Ky] = objlss.doubleOutConstraint() return overall out constraints
% [Hy,Ky] = objlss.doubleOutConstraint(index)
% Y = objlss.doubleOutConstraint(index,selection)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% index = vector of numbers or names (as cellarray) of involved systems 
% 
% selection = 0 exclusive: return matrices H and K of constraints that 
% involve ONLY systems specified in index
%
% selection = 1 all: return matrices H and K of constraints that 
% involve ALSO systems specified in index
% 
% example: constraint 'Y' between system [1,3,5]
% with index = [1,5]
% selection=0--> return Hy=[] and Ky=[] because there is no constraint 
% between systems 1 and 5 only
% selection=1--> return constraint 'Y' [1,3,5] because it contains indeces 
% 1 and 5
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% [Hy, Ky] Hy and Ky matrices defining the Polyhedron Y
% = { Hy*y<=Ky }
%
% [Y] Polyhedron object [1]
% 
%
% see also ADDOUTCONSTRAINT 
%
% [1] M. Herceg, M. Kvasnica, C. N. Jones, and M. Morari, "Multi-Parametric 
% Toolbox 3.0," in Proceedings of the 12th European Control Conference, 
% 2013, pp. 502-510, http://control.ee.ethz.ch/~mpt

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin >3
        error('DOUBLEOUTCONSTRAINT: numbers of input')
    end
    
    if nargin==1
        Hy=obj.Hy;
        Ky=obj.Ky;
        if nargout==1 || nargout==0
            Hy = Polyhedron(full(Hy),full(Ky));
        end
        return
    end
    
    if nargin==2
        selection = 0;
    end
    
    if ischar(index)
        index=obj.name2index(index);
    end
    
    if iscellstr(index)
        vect=[];
        for k=1:size(index,2)
            vect(k)=obj.name2index(index(k));
        end
        index=vect;
    end
    
    if ~isscalar(index) && ~isvector(index)
        error('DOUBLEOUTCONSTRAINT: see help doubleOutConstraint')
    end
    
    if any(index>obj.numSys) || any(index<1)
        error('DOUBLEOUTCONSTRAINT: check index parameter')
    end
    
    if selection~=0 && selection ~=1
        error('DOUBLEOUTCONSTRAINT: selection must be 0 or 1')
    end
    
    if isempty(obj.C)
        warning('MATLAB:paramAmbiguous','DOUBLEOUTCONSTRAINT: State feedback y = x: the output constraints are state constraints');
        [ Hy, Ky ] = obj.doubleStateConstraint(index,selection); 
        return;
    else
        if isempty(obj.ry)
            Hy = [];
            Ky = [];
            warning('MATLAB:paramAmbiguous','DOUBLEOUTCONSTRAINT: Polyhedron Y = R^%d',sum(obj.pi(index)));
            return
        end
    end
    
    if isscalar(index)  && selection ==0
        Hy=obj.Hy(sum(obj.ry(1:index-1))+1:sum(obj.ry(1:index)), sum(obj.pi(1:index-1))+1:sum(obj.pi(1:index)));
        Ky=obj.Ky(sum(obj.ry(1:index-1))+1:sum(obj.ry(1:index)));
    end
    
    if isscalar(index) && selection == 1
        Hy = obj.Hy(:,sum(obj.pi(1:index-1))+1:sum(obj.pi(1:index)));
        mask = any(Hy~=0,2);
        Hy = obj.Hy(mask,:);
        Ky = obj.Ky(mask);
    end
    
    
    if length(index)>1
        Haux= obj.Hy(sum(obj.ry)+1:end,:);
        Kaux= obj.Ky(sum(obj.ry)+1:end,1);
        mask=zeros(1,sum(obj.pi));
        for i=1:length(index)
            start=sum(obj.pi(1:index(i)))-obj.pi(index(i))+1;
            mask(1,start:start+obj.pi(index(i))-1)=ones(1,obj.pi(index(i)));
        end
        mask    =   all(mask>0.5,1);
        help    =   Haux(:,mask);
        flag    =   help~=0;
        piindex =   obj.pi(index);
        flagtmp = sparse(zeros(size(help,1),length(piindex))==1);
        for i = 1:length(piindex)
            flagtmp(:,i) = any( flag( : , sum(piindex(1:i-1))+1:sum(piindex(1:i)) ) , 2 );
        end
        flag = all(flagtmp,2);
       
        if selection==1 %all
            Hy=Haux(flag,:);
            Ky=Kaux(flag,1);
        elseif selection==0 %exclusive
            newhelp=Haux(:,~mask);
            newflag=all(newhelp==0,2);
            okrows=flag' & newflag';
            Hy=Haux(okrows,mask);
            Ky=Kaux(okrows,:);
        end
    end
    
    
    if nargout==1 || nargout==0
        if isempty(Hy)
            Hy = [];
            warning('MATLAB:paramAmbiguous','DOUBLEOUTCONSTRAINT: Polyhedron Y = R^%d',sum(obj.pi(index)));
        else
            Hy = Polyhedron(full(Hy),full(Ky));
        end
    end
    
end