function [a,b,c,d] = isCoupled(obj,i,j)
% ISCOUPLED check how two subsystems are coupled
% 
% [a,b,c,d] = objlss.isCoupled(i,j)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Return fours boolean variables
% 
% a (b, c, d) = 1 if there is coupling between subsystem i and j, i.e. Aij
% (Bij, Cij, Dij) are not zero. 0 otherwise
% 
% [a,b,c,d] = objlss.ISCOUPLED(i,j)
%
% USAGE: 
% [a,b] = objlss.isCoupled(i,j) coupling in A and B
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
%  i = number or name of the first subsystem
%  j = number or name of the second subsystem
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
%  [a,b,c,d] logical variables
% 
%
% see also ADDCOUPLING

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin~=3
        error('ISCOUPLED: input parameters')
    end
    
    if ischar(i)==1
        i=name2index(obj,i);
    end
    
    if ischar(j)==1
        j=name2index(obj,j);
    end
    
    if ~obj.couplingmatrix(i,j) % no coupling between system i and j
        a = false;
        b = false;
        c = false;
        d = false;
    else % where is coupling?
        a=full(~all(all(obj.A(sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)),sum(obj.ni(1:j-1))+1:sum(obj.ni(1:j)))==0)));
        if isempty(obj.B)
            b = false;
        else
            b = full(~all(all(obj.B(sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)),sum(obj.mi(1:j-1))+1:sum(obj.mi(1:j)))==0)));
        end
        
        if isempty(obj.pi)
            pi = obj.ni;
        else
            pi = obj.pi;
        end
        
        if isempty(obj.C)
            if i==j
                c = true;
            else
                c = false;
            end
        else
            c = full(~all(all(obj.C(sum(pi(1:i-1))+1:sum(pi(1:i)),sum(obj.ni(1:j-1))+1:sum(obj.ni(1:j)))==0)));
        end
        
        if isempty(obj.D)
            d = false;
        else
            d = full(~all(all(obj.D(sum(pi(1:i-1))+1:sum(pi(1:i)),sum(obj.mi(1:j-1))+1:sum(obj.mi(1:j)))==0)));
        end
    end
end