classdef lss
% LSS design a constrained linear time-invariant large-scale system
%
% objlss = lss(A,B,ni,mi,C,D,pi,name,Ts)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% This function allows you to create an large-scale system object with all 
% its propperties.
%
% objlss = LSS() initialize an empty lss object
%
% USAGE:
%
% objlss = LSS(A,B,ni,mi)
%
% objlss = LSS(A,[],ni,[])
%
% objlss = LSS(A,B,ni,mi,C,D,pi)
%
% objlss = LSS(A,B,ni,mi,C,D,pi,name)
%
% objlss = LSS(A,B,ni,mi,C,D,pi,name,Ts)
%
% objlss = LSS(objsubss)
%
% Return an object of lss class.
% ---------------------------------------------------------------------------
% INPUT
% x(t+1)|dx = Ax(t) + Bu(t)
% y(t)  = Cx(t) + Du(t)
%
% ni, mi, pi, = vector, how the system (the matrices) are partitioned
%
% name = string, a name that you can associate with the system.
%
% Ts = sampling time
% Ts = [] System described time-continuos
% Ts > 0  System discretize with Ts sample time
% Ts = -1 System is yet in discrete form
%
% objsubss : subss object (see help subss)
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
%
% objlss = an lss object
%
% ---------------------------------------------------------------------------
% PROPERTIES
% ---------------------------------------------------------------------------
% name : name of each subsystem
% numSys : number of subsystems
% Ts : sample time
% A, B, C, D, M, N, Bcen, Dcen : matrices of the Linear Time Invariant 
%                                large-scale System
% ni, mi, pi: vectors, partitioning of state, input and output variables
% numExo : number of exogenous signals
% numICen : number of centralized inputs
% Hx, Kx : matrices of polytopic state constraints 
% Hu, Ku : matrices of polytopic input constraints 
% Hdeltau, Kdeltau : matrices of polytopic delta u constraints 
% Hy, Ky : matrices of polytopic output constraints 
% Hd, Kd : matrices of polytopic exogenous signals constraints 
% Hcen, Kcen : matrices of polytopic centralized input constraints 
% He, Ke : matrices of polytopic state estimation error constraints 
% couplingmatrix : matrix of numSys x numSys logical elements, if the ij-th
%                 element is 1, there exist a coupling among subsystem i
%                 and j
%
% namex, nameu, named, namey, nameucen : cell arrays with names of state, 
%             input, exogenous signals, output, centralized input variables
%
%

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it,
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------
   

    properties (SetAccess = private , Hidden = false)
        name;
        numSys;
        Ts;
        
        A;
        B;
        C;
        D;
        M;
        N;
        Bcen;
        Dcen;
        
        ni;
        mi;
        pi;
        numExo;
        numICen;
        
        Hx;
        Kx;
        Hu;
        Ku;
        Hdeltau;
        Kdeltau;
        Hy;
        Ky;
        Hd;
        Kd;        
        Hcen;
        Kcen;
        He;
        Ke;
        
        couplingmatrix = sparse(false(0,0));
    end
    
    properties (SetAccess = private , Hidden = true)
        namex;
        nameu;
        named;
        namey;
        nameucen;
        rx;
        ru;
        ry;
        rd;
        rcen;
        re;
        rdeltau;
    end
    
    methods
        function obj = lss(A,B,ni,mi,C,D,pi,name,Ts)
            global pnpmpcToolboxGlobalOptions;
            if ~isstruct(pnpmpcToolboxGlobalOptions),
                pnpmpc_toolbox_err;
            end
            
            if (nargin==0)
                obj.numSys  = 0;
                obj.numExo  = 0;
                obj.numICen = 0;
                return
            end
            
            if nargin~=4 && nargin~=7 && nargin~=8 && nargin~=9
                error('LSS: input parameters. see help LSS')
            end
            
            if nargin<8
                name = [];
            end
            if nargin<9
                Ts = [];
            end         
            
            if size(ni,1)~=1 && size(ni,2)~=1 
                error('LSS: ni must be a vector')
            end
            
            if ~isempty(B) && ~isequal(size(ni,2),size(mi,2))
                error('LSS: ni and mi must have the same dimensions')
            end
            if ~isequal(size(A,1),size(A,2),sum(ni))
                error('LSS: Arows=%d (or Acolumns=%d) ~= sum(ni)=%d',size(A,1),size(A,2),sum(ni))
            end
            if sum(mi)==0
                if ~isempty(B)
                    error('LSS: Bcolumns=%d ~= sum(mi)=%d',size(B,2),sum(mi))
                end
            else
                if ~isequal(size(B,1),sum(ni))
                    error('LSS: Brows=%d ~= sum(ni)=%d',size(B,1),sum(ni))
                end
                if ~isequal(size(B,2),sum(mi))
                    error('LSS: Bcolumns=%d ~= sum(mi)=%d',size(B,2),sum(mi))
                end
            end
            obj.numSys = max(size(ni));
            
            if isempty(name)
                for j =1:length(ni)
                    obj.name{1,end+1} = ['system_',num2str(j)];
                end
            else
                if ischar(name)
                    for j=1:length(ni)
                        obj.name{1,end+1} = [name,num2str(j)];
                    end
                elseif iscell(name) && length(name)==obj.numSys
                    obj.name = name;
                else
                    error('LSS: The name must be a string')
                end
            end
            
            obj.A   = sparse(A);
            obj.B   = sparse(B);
            obj.ni  = ni;
            obj.mi  = mi;
            
            if isequal(Ts,-1) || isscalar(Ts) || isempty(Ts)
                if Ts==-1
                    disp('LSS: Discrete time system');
                end
                if isscalar(Ts) && Ts~=-1 && Ts>0
                    fprintf(1,'LSS: Time continuous system, discretized with sampling time Ts= %d\n',Ts);
                end
                if isempty(Ts)
                    disp('LSS: Time continuos system');
                end
            else
                error('LSS: Ts must be -1 or scalar or empty');
            end
            
            obj.Ts  = Ts;
            obj.numExo = 0;
            obj.numICen = 0;
            
            
            if nargin>=7
                             
                if ~isempty(C) && ~isequal(size(ni,2),size(pi,2))
                    error('LSS: dimensions of pi')
                end
                if ~isempty(C) && ~isequal (size(C,1),sum(pi))
                    error('LSS: Crows=%d ~=sum(pi)=%d',size(C,1),sum(pi))
                end
                if ~isempty(C) && ~isequal(size(C,2),sum(ni))
                    error('LSS: Columns=%d ~= sum(ni)=%d',size(C,2),sum(ni))
                end
                if ~isempty(C) && ~isempty(D) && ~isequal(size(D,1),sum(pi))
                    error('LSS: Drows=%d ~=sum(pi)=%d',size(D,1),sum(pi))
                end
                if ~isempty(C) && ~isempty(D) && ~isequal(size(D,2),sum(mi))
                    error('LSS: Dcolumns=%d ~= sum(mi)=%d',size(D,2),sum(mi))
                end
                if ~isempty(pi) && sum(pi)==0
                    error('LSS: dimensions of pi, no outputs')
                end   
                
                obj.C = sparse(C);
                if ~isempty(obj.C)
                    obj.D   = sparse(D);
                    obj.pi  = pi;
                end
                
                if isequal(obj.C,eye(sum(ni))) && isequal(obj.D,zeros(sum(ni),sum(mi)))
                    obj.C  = [];
                    obj.D  = [];
                    obj.pi = [];
                end
                
                if isempty(C) && sum(pi)~=0
                    error('LSS: Crows=%d ~=sum(pi)=%d',size(C,1),sum(pi))
                end            
            end
            
            % coupled matrix
            obj.couplingmatrix=sparse(true(obj.numSys));
            for i=1:obj.numSys
                for j=1:obj.numSys
                    [a,b,c,d] = isCoupled(obj,i,j);
                    if (a||b||c||d)==1
                        obj.couplingmatrix(i,j)=true;
                    else
                        obj.couplingmatrix(i,j)=false;
                    end
                end
            end
        end
    end
end