function [ objGraph ] = plot(objlss,indexsys,how)
% PLOT Plots the coupling graph of subsystems for a large-scale system
%
% objGraph = plot(objlss,io)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Plot the coupling graph of subsystems for the lss object objlss. The plot
% method uses the couplingmatrix. 
%
% objGraph = objlss.PLOT
%
% USAGE:
% objGraph = objlss.PLOT(indexsys)
% objGraph = objlss.PLOT(indexsys,how)
%
% ---------------------------------------------------------------------------
% INPUT
% --------------------------------------------------------------------------- 
% objlss : large-scale system object
% indexsys : index of the subsystems to plot. if it is empty, all
% subsystems are plotted
% how : 'io' plots local sensors and local controllers, 'exo' plots
% exogenous signals, 'ucen' plots centralized inputs, 'all' plots all
% signals of the centralized system.
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objGraph : graphViz4Matlab object
%  
%
% This method make use of the toolbox 'graphViz4Matlab'
% http://code.google.com/p/graphviz4matlab/
%
% see also GRAPHVIZ4MATLAB

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% --------------------------------------------------------------------------- 

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin==1 || isempty(indexsys)
        indexsys = 1:objlss.numSys;
    end
    if nargin>=3 && ~isa(how,'char')
        error('PLOT: input arguments')
    end
    
    names          = objlss.name(indexsys);
    couplingmatrix = objlss.couplingmatrix(indexsys,indexsys);
    
    if nargin<=2
        objGraph = graphViz4Matlab('-adjMat',couplingmatrix,'-nodeLabels',names,'-layout',Gvizlayout);
    else
        numSys = length(indexsys);
        switch(how)
            case 'io'
                couplingmatrix = sparse(eye(3*numSys,3*numSys)==1);
                acm = zeros(numSys,numSys)==1;
                bcm = zeros(numSys,numSys)==1;
                ccm = zeros(numSys,numSys)==1;
                dcm = zeros(numSys,numSys)==1;
                for i=1:numSys
                    ii = indexsys(i);
                    for j=1:numSys
                        jj = indexsys(j);
                        [ acm(i,j), bcm(i,j), ccm(i,j), dcm(i,j) ] = objlss.isCoupled(ii,jj);
                    end
                end
                couplingmatrix(1:numSys,1:numSys)                     = acm;
                couplingmatrix(numSys+1:2*numSys,1:numSys)            = bcm;
                couplingmatrix(1:numSys,2*numSys+1:3*numSys)          = ccm;
                couplingmatrix(numSys+1:2*numSys,2*numSys+1:3*numSys) = dcm;
                for i=1:numSys
                    ii = num2str(indexsys(i));
                    nameu{i} = [ 'u_{[' ii ']}' ];
                    namey{i} = [ 'y_{[' ii ']}' ];
                end
                names = [names nameu namey];
            case 'exo'
                numExo = objlss.numExo;
                couplingmatrix = [ couplingmatrix zeros(numSys,numExo)==1 ; zeros(numExo,numSys+numExo)==1 ];
                mcm = zeros(numExo,numSys)==1;
                for i=1:numSys
                    ii = indexsys(i);
                    for j=1:numExo
                        M = objlss.getM(ii,j);
                        N = objlss.getN(ii,j);
                        test = any(M~=0) || any(N~=0);
                        if test
                            mcm(j,i) = test;
                        end
                    end
                end
                couplingmatrix(numSys+1:end,1:numSys) = mcm;
                for j=1:numExo
                    jj = num2str(j);
                    named{j} = [ 'd_{' jj '}' ];
                end
                if numExo>0
                    names = [names named ];
                end
            case 'ucen'
                numIcen = objlss.numICen;
                couplingmatrix = [ couplingmatrix zeros(numSys,numIcen)==1 ; zeros(numIcen,numSys+numIcen)==1 ];
                ucencm = zeros(numIcen,numSys)==1;
                for i=1:numSys
                    ii = indexsys(i);
                    for j=1:numIcen
                        Bcen = objlss.getBcen(ii,j);
                        Dcen = objlss.getDcen(ii,j);
                        test = any(Bcen~=0) || any(Dcen~=0);
                        if test
                            ucencm(j,i) = test;
                        end
                    end
                end
                couplingmatrix(numSys+1:end,1:numSys) = ucencm;
                for j=1:numIcen
                    jj = num2str(j);
                    nameucen{j} = [ 'ucen_{' jj '}' ];
                end
                if numIcen>0
                    names = [names nameucen ];
                end
            case 'all'
                numExo  = objlss.numExo;
                numIcen = objlss.numICen;
                couplingmatrix = sparse([ eye(3*numSys,3*numSys)==1 zeros(3*numSys,numExo+numIcen)==1 ; zeros(numExo+numIcen,3*numSys+numExo+numIcen)==1 ] );
                acm = zeros(numSys,numSys)==1;
                bcm = zeros(numSys,numSys)==1;
                ccm = zeros(numSys,numSys)==1;
                dcm = zeros(numSys,numSys)==1;
                for i=1:numSys
                    ii = indexsys(i);
                    for j=1:numSys
                        jj = indexsys(j);
                        [ acm(i,j), bcm(i,j), ccm(i,j), dcm(i,j) ] = objlss.isCoupled(ii,jj);
                    end
                end
                mcm = zeros(numExo,numSys)==1;
                ncm = zeros(numExo,numSys)==1;
                for i=1:numSys
                    ii = indexsys(i);
                    for j=1:numExo
                        M = objlss.getM(ii,j);
                        N = objlss.getN(ii,j);
                        test = any(M~=0);
                        if test
                            mcm(j,i) = test;
                        end
                        test = any(N~=0);
                        if test
                            ncm(j,i) = test;
                        end
                    end
                end
                bucencm = zeros(numIcen,numSys)==1;
                ducencm = zeros(numIcen,numSys)==1;
                for i=1:numSys
                    ii = indexsys(i);
                    for j=1:numIcen
                        Bcen = objlss.getBcen(ii,j);
                        Dcen = objlss.getDcen(ii,j);
                        test = any(Bcen~=0);
                        if test
                            bucencm(j,i) = test;
                        end
                        test = any(Dcen~=0);
                        if test
                            ducencm(j,i) = test;
                        end
                    end
                end
                
                couplingmatrix(1:numSys,1:numSys)                              = acm;
                couplingmatrix(numSys+1:2*numSys,1:numSys)                     = bcm;
                couplingmatrix(1:numSys,2*numSys+1:3*numSys)                   = ccm;
                couplingmatrix(numSys+1:2*numSys,2*numSys+1:3*numSys)          = dcm;
                
                couplingmatrix(3*numSys+1:3*numSys+numExo,1:numSys)            = mcm;
                couplingmatrix(3*numSys+1:3*numSys+numExo,2*numSys+1:3*numSys) = ncm;
                
                couplingmatrix(3*numSys+numExo+1:end,1:numSys)                 = bucencm;
                couplingmatrix(3*numSys+numExo+1:end,2*numSys+1:3*numSys)      = ducencm;
                
                for i=1:numSys
                    ii = num2str(indexsys(i));
                    nameu{i} = [ 'u_{[' ii ']}' ];
                    namey{i} = [ 'y_{[' ii ']}' ];
                end
                names = [names nameu namey ];
                for j=1:numExo
                    jj = num2str(j);
                    named{j} = [ 'd_{' jj '}' ];
                end
                if numExo>0
                    names = [names named ];
                end
                for j=1:numIcen
                    jj = num2str(j);
                    nameucen{j} = [ 'ucen_{' jj '}' ];
                end
                if numIcen>0
                    names = [names nameucen ];
                end
            otherwise
        end
        
        tmp = [];
        for i=numSys+1:2*numSys
            if sum(couplingmatrix(i,:))==1
                tmp = [ tmp ; i ];
            end
        end
        couplingmatrix(tmp,:) = [];
        couplingmatrix(:,tmp) = [];
        names(tmp)            = [];
        
        objGraph = graphViz4Matlab('-adjMat',couplingmatrix,'-nodeLabels',names);
    end
    
    if nargout == 0;
        clear('objGraph')
    end
end