function obj = removeConstraint( obj , index , flag )
% REMOVECONSTRAINT Removes constraints on state|input|output|exogenous signal|centralized input
%  
% objlss = objlss.removeConstraint(index,flag)
%   
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Remove constraints on state|input|output|exogenous signal|centralized
% input from the lss object objlss
% 
% objlss = objlss.REMOVECONSTRAINT(index,flag)
%
% USAGE:
% objlss = objlss.removeConstraint(index)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% index = number of the involved system, or vector which contains the
% indices of the involved systems. It can be also a string with the name of
% the system or a cell array wich contains the names of the systems. If it
% is empty, all constraints specified in flag will be removed
%
% WARNING: if index=[2,4] this function remove the constraints involving
% subsystems 2 and 4 onely. For example, a constraint involving subsystems
% [2,4,5] will be not removed
% 
% flag = 'state' remove only the state constraints of the subsystems in index 
% (it modifies Hx and Kx)
% 
% flag = 'in' remove only the input constraints of the subsystems in index 
% (it modifies Hu and Ku)
% 
% flag = 'out' remove only the output constraints of the subsystems in index 
% (it modifies Hy and Ky)
% 
% flag = 'exo' remove only the exogenous signal constraints of the subsystems 
% in index (it modifies Hd and Kd). In this case index means the indeces of the
% exogenous signals.
%
% flag = 'cen' remove only the centralized input constraint of the subsystems 
% in index (it modifies Hcen and Kcen). In this case index means the indeces of
% the centralized inputs.
%
% flag = 'error' remove only the error estimation constraints of the subsystems 
% in index (it modifies He and Ke).
%
% flag = 'deltau' remove only the delta u constraints of the subsystems 
% in index (it modifies Hdeltau and Kdeltau).
%  
% if flag is empty, state, input, output constraints are removed
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
%
% objlss with updated matrices of constraints
% 
% 

% Copyright is with the following author(s):
%
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if ( ischar (index) || iscellstr(index)) && (strcmp(flag,'exo') || strcmp(flag,'cen')   )
        error('REMOVECONSTRAINT: if index is a string flag cannot be exo or cen ')
    end
    
    if ischar(index)
        index = obj.name2index(index);
    end
    
    if iscellstr(index)
        vect=[];
        for k=1:size(index,2)
            vect(k)=name2index(index(k));
        end
        index=vect;
    end
    
    if isempty(index)
        switch(flag)
            case 'state'
                obj.Hx = [];
                obj.Kx = [];
                obj.rx = [];
            case 'in'
                obj.Hu = [];
                obj.Ku = [];
                obj.ru = [];
            case 'out'
                obj.Hy = [];
                obj.Ky = [];
                obj.ry = [];
            case 'exo'
                obj.Hd = [];
                obj.Kd = [];
                obj.rd = [];
            case 'cen'
                obj.Hcen = [];
                obj.Kcen = [];
                obj.rcen = [];
            case 'error'
                obj.He = [];
                obj.Ke = [];
                obj.re = [];
            case 'deltau'
                obj.Hdeltau = [];
                obj.Kdeltau = [];
                obj.rdeltau = [];
            otherwise
                error('REMOVECONSTRAINT: flag, see help removeConstraint')
        end
        return
    end
    
    if ~isvector(index) && ~isscalar(index)
        error('REMOVECONSTRAINT: index must be a cell string or vector or scalar')
    end
    
    if nargin==3
        if ~strcmp(flag,'state') && ~strcmp(flag,'in') && ~strcmp(flag,'out') && ~strcmp(flag,'exo') && ~strcmp(flag,'cen') && ~strcmp(flag,'error') && ~strcmp(flag,'deltau')
            error('REMOVECONSTRAINT: flag, see help removeConstraint')
        end
    end
    
    if nargin==2 || strcmp(flag,'state')
        if ~isempty(obj.Hx)
            if isscalar(index) && obj.rx(index)~=0
                if index>obj.numSys
                    error('REMOVECONSTRAINT: system %d in not defined',index)
                end
                obj.Hx(sum(obj.rx(1:index-1))+1:sum(obj.rx(1:index)),:)=[];
                obj.Kx(sum(obj.rx(1:index-1))+1:sum(obj.rx(1:index)))=[];
                obj.rx(index)=0;
            end
            
            if length(index)>1
                Haux= obj.Hx(sum(obj.rx)+1:end,:);
                Kaux= obj.Kx(sum(obj.rx)+1:end,1);
                mask=zeros(1,sum(obj.ni));
                for i=1:length(index)
                    if index(i)> obj.numSys
                        error('REMOVECONSTRAINT: system %d is not defined', index(i))
                    end
                    start=sum(obj.ni(1:index(i)))-obj.ni(index(i))+1;
                    mask(1,start:start+obj.ni(index(i))-1)=ones(1,obj.ni(index(i)));
                end
                mask=all(mask>0.5,1);
                help=Haux(:,mask);
                flag=all(help~=0,2);
                newhelp=Haux(:,~mask);
                newflag=all(newhelp==0,2);
                okrows=flag & newflag;
                obj.Hx=[obj.Hx(1:sum(obj.rx(1:end)),:);Haux(~okrows,:)];
                obj.Kx=[obj.Kx(1:sum(obj.rx(1:end)),1);Kaux(~okrows,1)];
            end
        end
    end
    
    
    if nargin==2 || strcmp(flag,'in')
        if ~isempty(obj.Hu)
            if isscalar(index) && obj.ru(index)~=0
                if index>obj.numSys
                    error('REMOVECONSTRAINT: system %d in not defined',index)
                end
                obj.Hu(sum(obj.ru(1:index-1))+1:sum(obj.ru(1:index)),:)=[];
                obj.Ku(sum(obj.ru(1:index-1))+1:sum(obj.ru(1:index)))=[];
                obj.ru(index)=0;
            end
            
            if length(index)>1
                Haux= obj.Hu(sum(obj.ru)+1:end,:);
                Kaux= obj.Ku(sum(obj.ru)+1:end,1);
                mask=zeros(1,sum(obj.mi));
                for i=1:length(index)
                    if index(i)> obj.numSys
                        error('REMOVECONSTRAINT: system %d is not defined', index(i))
                    end
                    start=sum(obj.mi(1:index(i)))-obj.mi(index(i))+1;
                    mask(1,start:start+obj.mi(index(i))-1)=ones(1,obj.mi(index(i)));
                end
                mask=all(mask>0.5,1);
                help=Haux(:,mask);
                flag=all(help~=0,2);
                newhelp=Haux(:,~mask);
                newflag=all(newhelp==0,2);
                okrows=flag & newflag;
                obj.Hu=[obj.Hu(1:sum(obj.ru(1:end)),:);Haux(~okrows,:)];
                obj.Ku=[obj.Ku(1:sum(obj.ru(1:end)),1);Kaux(~okrows,1)];
            end
        end
    end
    
    
    if nargin==2 || strcmp(flag,'out')
        if ~isempty (obj.Hy)
            if isscalar(index) && obj.ry(index)~=0
                if index>obj.numSys
                    error('REMOVECONSTRAINT: system number %d in not defined',index)
                end
                obj.Hy(sum(obj.ry(1:index-1))+1:sum(obj.ry(1:index)),:)=[];
                obj.Ky(sum(obj.ry(1:index-1))+1:sum(obj.ry(1:index)),1)=[];
                obj.ry(index) = 0;
            end
            
            if length(index)>1
                Haux= obj.Hy(sum(obj.ry)+1:end,:);
                Kaux= obj.Ky(sum(obj.ry)+1:end,1);
                mask=zeros(1,sum(obj.pi));
                for i=1:length(index)
                    if index(i)> obj.numSys
                        error('REMOVECONSTRAINT: system %d is not defined', index(i))
                    end
                    start=sum(obj.pi(1:index(i)))-obj.pi(index(i))+1;
                    mask(1,start:start+obj.pi(index(i))-1)=ones(1,obj.pi(index(i)));
                end
                mask=all(mask>0.5,1);
                help=Haux(:,mask);
                flag=all(help~=0,2);
                newhelp=Haux(:,~mask);
                newflag=all(newhelp==0,2);
                okrows=flag & newflag;
                obj.Hy=[obj.Hy(1:sum(obj.ry(1:end)),:);Haux(~okrows,:)];
                obj.Ky=[obj.Ky(1:sum(obj.ry(1:end)),1);Kaux(~okrows,1)];
            end
        end
    end
    
    
    if nargin ==3 && strcmp(flag,'exo')
        if ~isempty(obj.Hd)
            if isscalar(index) && obj.rd(index) ~=0
                if index>obj.numExo
                    error('REMOVECONSTRAINT: exogenous signal %d in not defined',index)
                end
                obj.Hd(sum(obj.rd(1:index-1))+1:sum(obj.rd(1:index)),:)=[];
                obj.Kd(sum(obj.rd(1:index-1))+1:sum(obj.rd(1:index)),1)=[];
                obj.rd(index) = 0;
            end
            
            if length(index)>1
                Haux= obj.Hd(sum(obj.rd)+1:end,:);
                Kaux= obj.Kd(sum(obj.rd)+1:end,1);
                mask=zeros(1,sum(obj.ri));
                for i=1:length(index)
                    if index(i)> obj.numExo
                        error('REMOVECONSTRAINT: exogenous signal %d is not defined', index(i))
                    end
                    mask(1,index(i))=1;
                end
                mask=all(mask>0.5,1);
                help=Haux(:,mask);
                flag=all(help~=0,2);
                newhelp=Haux(:,~mask);
                newflag=all(newhelp==0,2);
                okrows=flag & newflag;
                obj.Hd=[obj.Hd(1:sum(obj.rd(1:end)),:);Haux(~okrows,:)];
                obj.Kd=[obj.Kd(1:sum(obj.rd(1:end)),1);Kaux(~okrows,1)];
            end
        end
    end
    
    if nargin == 3 && strcmp(flag,'cen')
        if ~isempty(obj.Hcen)
            if isscalar(index) && obj.rcen(index)~=0
                if index>obj.numICen
                    error('REMOVECONSTRAINT: centralized input %d in not defined',index)
                end
                obj.Hcen(sum(obj.rcen(1:index-1))+1:sum(obj.rcen(1:index)),:)=[];
                obj.Kcen(sum(obj.rcen(1:index-1))+1:sum(obj.rcen(1:index)),1)=[];
                obj.rcen (index) = 0;
            end
            
            if length(index)>1
                Haux= obj.Hcen(sum(obj.rcen)+1:end,:);
                Kaux= obj.Kcen(sum(obj.rcen)+1:end,1);
                mask=zeros(1,sum(obj.rcen));
                for i=1:length(index)
                    if index(i)> obj.numICen
                        error('REMOVECONSTRAINT: centralized input %d in not defined', index(i))
                    end
                    mask(1,index(i))=1;
                end
                mask=all(mask>0.5,1);
                help=Haux(:,mask);
                flag=all(help~=0,2);
                newhelp=Haux(:,~mask);
                newflag=all(newhelp==0,2);
                okrows=flag & newflag;
                obj.Hcen=[obj.Hd(1:sum(obj.rcen(1:end)),:);Haux(~okrows,:)];
                obj.Kcen=[obj.Kd(1:sum(obj.rcen(1:end)),1);Kaux(~okrows,1)];
            end
        end
    end
    
    if nargin==2 || strcmp(flag,'error')
        if ~isempty(obj.He)
            if isscalar(index) && obj.re(index)~=0
                if index>obj.numSys
                    error('REMOVECONSTRAINT: system %d in not defined',index)
                end
                obj.He(sum(obj.re(1:index-1))+1:sum(obj.re(1:index)),:)=[];
                obj.Ke(sum(obj.re(1:index-1))+1:sum(obj.re(1:index)))=[];
                obj.re(index)=0;
            end
            
            if length(index)>1
                Haux= obj.He(sum(obj.re)+1:end,:);
                Kaux= obj.Ke(sum(obj.re)+1:end,1);
                mask=zeros(1,sum(obj.ni));
                for i=1:length(index)
                    if index(i)> obj.numSys
                        error('REMOVECONSTRAINT: system %d is not defined', index(i))
                    end
                    start=sum(obj.ni(1:index(i)))-obj.ni(index(i))+1;
                    mask(1,start:start+obj.ni(index(i))-1)=ones(1,obj.ni(index(i)));
                end
                mask=all(mask>0.5,1);
                help=Haux(:,mask);
                flag=all(help~=0,2);
                newhelp=Haux(:,~mask);
                newflag=all(newhelp==0,2);
                okrows=flag & newflag;
                obj.He=[obj.He(1:sum(obj.rx(1:end)),:);Haux(~okrows,:)];
                obj.Ke=[obj.Ke(1:sum(obj.rx(1:end)),1);Kaux(~okrows,1)];
            end
        end
    end
    
    if nargin==2 || strcmp(flag,'deltau')
        if ~isempty(obj.Hdeltau)
            if isscalar(index) && obj.rdeltau(index)~=0
                if index>obj.numSys
                    error('REMOVECONSTRAINT: system %d in not defined',index)
                end
                obj.Hdeltau(sum(obj.rdeltau(1:index-1))+1:sum(obj.rdeltau(1:index)),:)=[];
                obj.Kdeltau(sum(obj.rdeltau(1:index-1))+1:sum(obj.rdeltau(1:index)))=[];
                obj.rdeltau(index)=0;
            end
            
            if length(index)>1
                Haux= obj.Hdeltau(sum(obj.rdeltau)+1:end,:);
                Kaux= obj.Kdeltau(sum(obj.rdeltau)+1:end,1);
                mask=zeros(1,sum(obj.mi));
                for i=1:length(index)
                    if index(i)> obj.numSys
                        error('REMOVECONSTRAINT: system %d is not defined', index(i))
                    end
                    start=sum(obj.mi(1:index(i)))-obj.mi(index(i))+1;
                    mask(1,start:start+obj.mi(index(i))-1)=ones(1,obj.mi(index(i)));
                end
                mask=all(mask>0.5,1);
                help=Haux(:,mask);
                flag=all(help~=0,2);
                newhelp=Haux(:,~mask);
                newflag=all(newhelp==0,2);
                okrows=flag & newflag;
                obj.Hdeltau=[obj.Hdeltau(1:sum(obj.rdeltau(1:end)),:);Haux(~okrows,:)];
                obj.Kdeltau=[obj.Kdeltau(1:sum(obj.rdeltau(1:end)),1);Kaux(~okrows,1)];
            end
        end
    end
    
end