function obj = removeSystem(obj,i)
% REMOVESYSTEM Removes a subsystem from an lss
% 
% objlss = objlss.removeSystem(i)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Remove a subsystem from a lss object objlss. It removes matrices and
% constraints that involve subsystem i
%
% objlss = objlss.REMOVESYSTEM(i)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% i = name or number of the system to remove
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlss without the system i
% 

% Copyright is with the following author(s):
%
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin~=2
        error('REMOVESYSTEM: parameters in input');
    end
    
    if ischar(i)
        i=obj.name2index(i);
    end
    
    if i>obj.numSys
        error('REMOVESYSTEM: System %d does not exist',i)
    end
    
    % matrices
    obj.A ( sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)) , :) = [];
    obj.A ( :, sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)))  = [];
    obj.B ( sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)) , :) = [];
    obj.B ( :, sum(obj.mi(1:i-1))+1:sum(obj.mi(1:i)))  = [];
    
    if ~isempty(obj.C)
        obj.C ( sum(obj.pi(1:i-1))+1:sum(obj.pi(1:i)) , :)=[];
        obj.C ( : , sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)))=[];
    end
    
    if ~isempty(obj.D)
        obj.D ( sum(obj.pi(1:i-1))+1:sum(obj.pi(1:i)) , :)=[];
        obj.D ( : , sum(obj.mi(1:i-1))+1:sum(obj.mi(1:i)))=[];
    end
    
    if ~isempty(obj.M)
        obj.M (sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)),:)=[];
        mask=any(obj.M~=0);
        obj.numExo = size(obj.M,2);
        
        if ~isempty(obj.N)
            obj.N ( sum(obj.pi(1:i-1))+1:sum(obj.pi(1:i)) , :)=[];
            maskN=any(obj.N~=0);
            mask = mask | maskN;
        end
        
        cont = 0;
        for k=1:length(mask)
            if mask(k) == 0
                obj = obj.removeExoInput(k-cont);
                cont = cont + 1;
                warning('MATLAB:note','WARNING: exogenous signal %d is automatically removed',k)
            end
        end
        if all(all(obj.M==0)) && all(all(obj.N==0))
            obj.M = [];
            obj.N = [];
        elseif all(all(obj.N==0)) && isempty(obj.C)
            obj.N=[];
        end
        
        obj.numExo = size(obj.M,2);
        
    end
    
    if ~isempty(obj.Bcen)
        obj.Bcen ( sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)) , :)=[];
        mask=any(obj.Bcen~=0);
        
        if ~isempty(obj.Dcen)
            obj.Dcen ( sum(obj.pi(1:i-1))+1:sum(obj.pi(1:i)) , :)=[];
            maskD = any(obj.Dcen ~= 0);
            mask = mask & maskD;
        end
        
        cont = 0;
        for k=1:length(mask)
            if mask(k) == 0
                obj = obj.removeCentralizedInput(k-cont);
                cont = cont + 1;
                warning('MATLAB:note','WARNING: centralized inpu %d is automatically removed',k)
            end
        end
        
        if all(all(obj.Bcen==0))
            obj.Bcen=[];
        end
        if all(all(obj.Dcen==0))
            obj.Dcen=[];
        end
        obj.numICen = size(obj.Bcen,2);
    end
    
    % state constraints
    if ~isempty(obj.Hx)
        if size(obj.Hx,1)~=sum(obj.rx)
            obj.Hx(:,sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)))=zeros(size(obj.Hx,1),obj.ni(i));
            obj.Hx(sum(obj.rx(1:i-1))+1:sum(obj.rx(1:i)),:)=zeros(obj.rx(i),size(obj.Hx,2));
            obj.Kx(sum(obj.rx(1:i-1))+1:sum(obj.rx(1:i)))=zeros(obj.rx(i),1);
            Haux=obj.Hx(sum(obj.rx)+1:end,:);
            Kaux=obj.Kx(sum(obj.rx)+1:end,1);
            flag=[];
            for j=1:size(Haux,1)
                for k=1:obj.numSys
                    help=Haux(j,sum(obj.ni(1:k-1))+1:sum(obj.ni(1:k)));
                    flag(j,k)=any(help~=0);
                    
                end
            end
            flagrow=(sum(flag,2)>1.5);
            index=0;
            numsys = [];
            daspostare  = Haux(~flagrow,:);
            daspostarek = Kaux(~flagrow);
            
            for j=1:size(daspostare,1)
                for k=1:obj.numSys
                    index=index+1;
                    if flag(j,k)~=0
                        break
                    end
                end
                numsys(end+1,1)=index;
                index=0;
            end
            
            aux=obj.Hx(1:sum(obj.rx),:);
            auxk=obj.Kx(1:sum(obj.rx));
            for j=1:size(numsys,1)
                aux  =  [aux(1:sum(obj.rx(1:numsys(j))),:); daspostare(j,:); aux(sum(obj.rx(1:numsys(j)))+1:sum(obj.rx),:)];
                auxk =  [auxk(1:sum(obj.rx(1:numsys(j)))); daspostarek(j) ; auxk(sum(obj.rx(1:numsys(j)))+1:sum(obj.rx))];
                obj.rx(numsys(j))=obj.rx(numsys(j))+1;
            end
            
            obj.Hx = [aux ; Haux(flagrow,:)];
            obj.Kx = [auxk; Kaux(flagrow)];
        end
        obj.Hx(:,sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)))=[];
        obj.Hx(sum(obj.rx(1:i-1))+1:sum(obj.rx(1:i)),:)=[];
        obj.Kx(sum(obj.rx(1:i-1))+1:sum(obj.rx(1:i)))=[];
        obj.rx(i)=[];
    end
    
    
    % input constraints
    if ~isempty(obj.Hu)
        if size(obj.Hu,1)~=sum(obj.ru)
            obj.Hu(:,sum(obj.mi(1:i-1))+1:sum(obj.mi(1:i)))=zeros(size(obj.Hu,1),obj.mi(i));
            obj.Hu(sum(obj.ru(1:i-1))+1:sum(obj.ru(1:i)),:)=zeros(obj.ru(i),size(obj.Hu,2));
            obj.Ku(sum(obj.ru(1:i-1))+1:sum(obj.ru(1:i)))=zeros(obj.ru(i),1);
            Haux=obj.Hu(sum(obj.ru)+1:end,:);
            Kaux=obj.Ku(sum(obj.ru)+1:end,1);
            flag=[];
            for j=1:size(Haux,1)
                for k=1:obj.numSys
                    help=Haux(j,sum(obj.mi(1:k-1))+1:sum(obj.mi(1:k)));
                    flag(j,k)=any(help~=0);
                    
                end
            end
            flagrow=(sum(flag,2)>1.5);
            index=0;
            numsys = [];
            daspostare  = Haux(~flagrow,:);
            daspostarek = Kaux(~flagrow);
            
            for j=1:size(daspostare,1)
                for k=1:obj.numSys
                    index=index+1;
                    if flag(j,k)~=0
                        break
                    end
                end
                numsys(end+1,1)=index;
                index=0;
            end
            
            aux=obj.Hu(1:sum(obj.ru),:);
            auxk=obj.Ku(1:sum(obj.ru));
            for j=1:size(numsys,1)
                aux  =  [aux(1:sum(obj.ru(1:numsys(j))),:); daspostare(j,:); aux(sum(obj.ru(1:numsys(j)))+1:sum(obj.ru),:)];
                auxk =  [auxk(1:sum(obj.ru(1:numsys(j)))); daspostarek(j) ; auxk(sum(obj.ru(1:numsys(j)))+1:sum(obj.ru))];
                obj.ru(numsys(j))=obj.ru(numsys(j))+1;
            end
            
            obj.Hu = [aux ; Haux(flagrow,:)];
            obj.Ku = [auxk; Kaux(flagrow)];
        end
        obj.Hu(:,sum(obj.mi(1:i-1))+1:sum(obj.mi(1:i)))=[];
        obj.Hu(sum(obj.ru(1:i-1))+1:sum(obj.ru(1:i)),:)=[];
        obj.Ku(sum(obj.ru(1:i-1))+1:sum(obj.ru(1:i)))=[];
        obj.ru(i)=[];
    end
    
    % output constraints
    if ~isempty(obj.Hy)
        if size(obj.Hy,1)~=sum(obj.ry)
            obj.Hy(:,sum(obj.pi(1:i-1))+1:sum(obj.pi(1:i)))=zeros(size(obj.Hy,1),obj.pi(i));
            obj.Hy(sum(obj.ry(1:i-1))+1:sum(obj.ry(1:i)),:)=zeros(obj.ry(i),size(obj.Hy,2));
            obj.Ky(sum(obj.ry(1:i-1))+1:sum(obj.ry(1:i)))=zeros(obj.ry(i),1);
            Haux=obj.Hy(sum(obj.ry)+1:end,:);
            Kaux=obj.Ky(sum(obj.ry)+1:end,1);
            flag=[];
            for j=1:size(Haux,1)
                for k=1:obj.numSys
                    help=Haux(j,sum(obj.pi(1:k-1))+1:sum(obj.pi(1:k)));
                    flag(j,k)=any(help~=0);
                    
                end
            end
            flagrow=(sum(flag,2)>1.5);
            index=0;
            numsys = [];
            daspostare  = Haux(~flagrow,:);
            daspostarek = Kaux(~flagrow);
            
            for j=1:size(daspostare,1)
                for k=1:obj.numSys
                    index=index+1;
                    if flag(j,k)~=0
                        break
                    end
                end
                numsys(end+1,1)=index;
                index=0;
            end
            
            aux=obj.Hy(1:sum(obj.ry),:);
            auxk=obj.Ky(1:sum(obj.ry));
            for j=1:size(numsys,1)
                aux  =  [aux(1:sum(obj.ry(1:numsys(j))),:); daspostare(j,:); aux(sum(obj.ry(1:numsys(j)))+1:sum(obj.ry),:)];
                auxk =  [auxk(1:sum(obj.ry(1:numsys(j)))); daspostarek(j) ; auxk(sum(obj.ry(1:numsys(j)))+1:sum(obj.ry))];
                obj.ry(numsys(j))=obj.ry(numsys(j))+1;
            end
            
            obj.Hy = [aux ; Haux(flagrow,:)];
            obj.Ky = [auxk; Kaux(flagrow)];
        end
        obj.Hy(:,sum(obj.pi(1:i-1))+1:sum(obj.pi(1:i)))=[];
        obj.Hy(sum(obj.ry(1:i-1))+1:sum(obj.ry(1:i)),:)=[];
        obj.Ky(sum(obj.ry(1:i-1))+1:sum(obj.ry(1:i)))=[];
        obj.ry(i)=[];
    end
    
    
    % delta u constraints
    if ~isempty(obj.Hdeltau)
        if size(obj.Hdeltau,1)~=sum(obj.rdeltau)
            obj.Hdeltau(:,sum(obj.mi(1:i-1))+1:sum(obj.mi(1:i)))=zeros(size(obj.Hdeltau,1),obj.mi(i));
            obj.Hdeltau(sum(obj.rdeltau(1:i-1))+1:sum(obj.rdeltau(1:i)),:)=zeros(obj.rdeltau(i),size(obj.Hdeltau,2));
            obj.Kdeltau(sum(obj.rdeltau(1:i-1))+1:sum(obj.rdeltau(1:i)))=zeros(obj.rdeltau(i),1);
            Haux=obj.Hdeltau(sum(obj.rdeltau)+1:end,:);
            Kaux=obj.Kdeltau(sum(obj.rdeltau)+1:end,1);
            flag=[];
            for j=1:size(Haux,1)
                for k=1:obj.numSys
                    help=Haux(j,sum(obj.mi(1:k-1))+1:sum(obj.mi(1:k)));
                    flag(j,k)=any(help~=0);
                    
                end
            end
            flagrow=(sum(flag,2)>1.5);
            index=0;
            numsys = [];
            daspostare  = Haux(~flagrow,:);
            daspostarek = Kaux(~flagrow);
            
            for j=1:size(daspostare,1)
                for k=1:obj.numSys
                    index=index+1;
                    if flag(j,k)~=0
                        break
                    end
                end
                numsys(end+1,1)=index;
                index=0;
            end
            
            aux=obj.Hdeltau(1:sum(obj.rdeltau),:);
            auxk=obj.Kdeltau(1:sum(obj.rdeltau));
            for j=1:size(numsys,1)
                aux  =  [aux(1:sum(obj.rdeltau(1:numsys(j))),:); daspostare(j,:); aux(sum(obj.rdeltau(1:numsys(j)))+1:sum(obj.rdeltau),:)];
                auxk =  [auxk(1:sum(obj.rdeltau(1:numsys(j)))); daspostarek(j) ; auxk(sum(obj.rdeltau(1:numsys(j)))+1:sum(obj.rdeltau))];
                obj.rdeltau(numsys(j))=obj.rdeltau(numsys(j))+1;
            end
            
            obj.Hdeltau = [aux ; Haux(flagrow,:)];
            obj.Kdeltau = [auxk; Kaux(flagrow)];
        end
        obj.Hdeltau(:,sum(obj.mi(1:i-1))+1:sum(obj.mi(1:i)))=[];
        obj.Hdeltau(sum(obj.rdeltau(1:i-1))+1:sum(obj.rdeltau(1:i)),:)=[];
        obj.Kdeltau(sum(obj.rdeltau(1:i-1))+1:sum(obj.rdeltau(1:i)))=[];
        obj.rdeltau(i)=[];
    end
    
    
    % error estimation constraints
    if ~isempty(obj.He)
        if size(obj.He,1)~=sum(obj.re)
            obj.He(:,sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)))=zeros(size(obj.He,1),obj.ni(i));
            obj.He(sum(obj.re(1:i-1))+1:sum(obj.re(1:i)),:)=zeros(obj.re(i),size(obj.He,2));
            obj.Ke(sum(obj.re(1:i-1))+1:sum(obj.re(1:i)))=zeros(obj.re(i),1);
            Haux=obj.He(sum(obj.re)+1:end,:);
            Kaux=obj.Ke(sum(obj.re)+1:end,1);
            flag=[];
            for j=1:size(Haux,1)
                for k=1:obj.numSys
                    help=Haux(j,sum(obj.ni(1:k-1))+1:sum(obj.ni(1:k)));
                    flag(j,k)=any(help~=0);
                    
                end
            end
            flagrow=(sum(flag,2)>1.5);
            index=0;
            numsys = [];
            daspostare  = Haux(~flagrow,:);
            daspostarek = Kaux(~flagrow);
            
            for j=1:size(daspostare,1)
                for k=1:obj.numSys
                    index=index+1;
                    if flag(j,k)~=0
                        break
                    end
                end
                numsys(end+1,1)=index;
                index=0;
            end
            
            aux=obj.He(1:sum(obj.re),:);
            auxk=obj.Ke(1:sum(obj.re));
            for j=1:size(numsys,1)
                aux  =  [aux(1:sum(obj.re(1:numsys(j))),:); daspostare(j,:); aux(sum(obj.re(1:numsys(j)))+1:sum(obj.re),:)];
                auxk =  [auxk(1:sum(obj.re(1:numsys(j)))); daspostarek(j) ; auxk(sum(obj.re(1:numsys(j)))+1:sum(obj.re))];
                obj.re(numsys(j))=obj.re(numsys(j))+1;
            end
            
            obj.He = [aux ; Haux(flagrow,:)];
            obj.Ke = [auxk; Kaux(flagrow)];
        end
        obj.He(:,sum(obj.ni(1:i-1))+1:sum(obj.ni(1:i)))=[];
        obj.He(sum(obj.re(1:i-1))+1:sum(obj.re(1:i)),:)=[];
        obj.Ke(sum(obj.re(1:i-1))+1:sum(obj.re(1:i)))=[];
        obj.re(i)=[];
    end
    
    
    obj.numSys=obj.numSys-1;
    obj.ni(i)=[];
    obj.mi(i)=[];
    obj.name(i)=[];
    obj.couplingmatrix(:,i) = [];
    obj.couplingmatrix(i,:) = [];
    
    if ~isempty(obj.pi)
        obj.pi(i)=[];
    end
    
    if size(obj.namex,2)>=i
        if ~isempty(obj.namex(i))
            obj.namex (i) =[];
        end
    end
    
    if size(obj.nameu,2)>=i
        if ~isempty(obj.nameu(i))
            obj.nameu (i) =[];
        end
    end
    
    if size(obj.namey,2)>=i
        if ~isempty(obj.namey(i))
            obj.namey (i) =[];
        end
    end
    
end