function [ varargout ] = stairsUcen(objlss,struct,opzindex,indexFig)
% STAIRSUCEN Stairstep plot centralized inputs trajectories
% 
% [ fig subplotn plotndec ] = stairsUcen(objlss,struct,opzindex,indexFig)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% --------------------------------------------------------------------------- 
% Stairstep plot of centralized inputs trajectories ucen.
% 
% [ fig subplotn plotndec ] = STAIRSUCEN(objlss,struct,opzindex,indexFig)
% 
% USAGE: 
% STAIRSUCEN(objlss,struct,opzindex,indexFig)
% STAIRSUCEN(objlss,struct)
% [ fig ] = STAIRSUCEN(...)
% [ fig plotn ] = STAIRSUCEN(...)
%
% ---------------------------------------------------------------------------
% INPUT
% --------------------------------------------------------------------------- 
% struct = data to plot, it must be a structure composed by the following
% fields:  
% 1) struct.signals.values = matrix of centralized inputs trajectories ( time x Iucen )
% 2) struct.time   = time
% 3) struct.signals.dimensions = number of states (columns of
% struct.signals.values)
% 
% opzindex = scalar or vector of involved centralized inputs in the plot
% 
% indexFig    = handle of previously created figure in order to plot on 
%               it (optional in every use of the function). For 
%               more details see output parameters
% 
% ---------------------------------------------------------------------------
% OUTPUT
% --------------------------------------------------------------------------- 
% fig = vector of the handles of the opened figures
% plotndec = vector of the handles of the plot in the figure
%
% These outputs are useful to set properties of figures, plots.
%
% 
% see also PLOTUCEN

% Copyright is with the following author(s):
%
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if ~isa(objlss,'lss')
        error('ERROR: first argument must be a lss object')
    end
    
    if nargin<2
        error('ERROR: no valid parameters')
    end
    
    if nargin<3 || isempty(opzindex)
        opzindex = 1:objlss.numICen;
    end
    
    mask = zeros(1,objlss.numICen);
    
    for k=1:length(opzindex)
        if opzindex(k) > objlss.numICen
            error('ERROR: centralized input %d does not exist',opzindex(k))
        end
        mask(1,opzindex(k)) =1;
    end
    mask = any(mask~=0,1);
    
    if nargin<=3
        fig = figure('Name','Figure','NumberTitle','off');
    else
        fig = indexFig;
        figure(fig);
        hold on
    end
    
    X = struct.time;
    Y = struct.signals.values(:,mask);
    
    plotn = stairs(X,Y);
    
    title('stairsUcen','FontSize',16,'FontName','Arial');
    xlabel('t [s]','FontSize',16,'FontName','Arial');
    if length(opzindex)==1
        if isempty(objlss.nameucen)
            ylabel(['u_{cen,' num2str(opzindex) '}'],'FontSize',16,'FontName','Arial');
        else
            ylabel(objlss.nameucen{opzindex},'FontSize',16,'FontName','Arial');
        end
    else
        ylabel('u_{cen}','FontSize',16,'FontName','Arial');
    end
    
    varargout = [];
    if nargout>=1
        varargout{1,1} = fig;
    end
    if nargout>=2
        varargout{1,2} = plotn;
    end
end