function [ varargout ] = stairsX (objlss,rows,columns,struct,opzsystems,opzstates,indexFigSub)    
% STAIRSX Stairstep plot of state trajectories
% 
% [ fig subplotn plotndec ] = stairsX (objlss,rows,columns,struct,opzsystems,opzstates,indexFigSub)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% --------------------------------------------------------------------------- 
% Stairstep plot of state trajectories x, using the names of each state from namex.
% 
% [ fig subplotn plotndec ] = STAIRSX(objlss,rows,columns,struct,opzsystems,opzstates,indexFigSub)
% 
% USAGE: 
% STAIRSX(objlss,rows,columns,struct,opzsystems,opzstates,indexFigSub)
% STAIRSX(objlss,[],[],struct)
% STAIRSX(objlss,rows,columns,struct)
% STAIRSX(objlss,rows,columns,struct,opzsystems,opzstates)
% STAIRSX(objlss,rows,columns,struct,opzsystems)
% STAIRSX(objlss,rows,columns,struct,[],opzstates)
% [ fig ] = STAIRSX(...)
% [ fig subplotn ] = STAIRSX(...)
% [ fig subplotn plotndec ] = STAIRSX(...)
%
% ---------------------------------------------------------------------------
% INPUT
% --------------------------------------------------------------------------- 
% rows, columns = in how many rows per columns you want to split the
% figure, i.e. rows x columns is how many subplot for each figure
% 
% struct = data to plot, it must be a structure composed by the following
% fields:  
% 1) struct.signals.values = matrix of state trajectories ( time x states )
% 2) struct.time   = time
% 3) struct.signals.dimensions = number of states (columns of
% struct.signals.values)
% 
% opzsystems = scalar or vector (string or cell array of string) of
% involved systems in the plot
% 
% opzstates = scalar or vectore of involved states in the plot
% 
% indexFigSub = cell array composed by two element {handleFig,handleSubplot}.
%               This option allows one to plot on a previously created figure. 
%               It is optional in every use of the function.
% 
% ---------------------------------------------------------------------------
% OUTPUT
% --------------------------------------------------------------------------- 
% fig = vector of the handles of the opened figures
% subplotn = matrix of the handles of the opened subplots (in i-th row
%            handles of the i-th figure)
% plotndec = 3D matrix of the handles of the plot in the figure
%            example: plotndec(i,j,k) k-th handle of plot in i-th figure, 
%            j-th subplot
%
% These outputs are useful to set properties of figures, subplots,
% plots.
%
% 
% see also PLOTX

% Copyright is with the following author(s):
%
% (C) 2012 
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if ~isa(objlss,'lss')
        error('ERROR: first argument must be a lss object')
    end
    
    if nargin<4
        error('ERROR: no valid parameters')
    end
    
    if isempty(rows) && ~isempty(columns)
        error('ERROR: rows and columns must be or empty or scalar')
    end
    
    if ~isempty(rows) && isempty(columns)
        error('ERROR: rows and columns must be or empty or scalar')
    end
    
    if isempty(rows)&& isempty(columns)
        allinone = 1;
        rows = 1;
        columns = 1;
    else
        allinone = 0;
    end
    
    if ~isscalar(rows) && ~isscalar (columns)
        error('ERROR: second argument (rows) and third argument (columns) must be scalar')
    end
    
    if length(struct.time) ~= size(struct.signals.values,1)
        error('ERROR: length(struct.time) must be equal to struct.values rows ')
    end
    
    if struct.signals.dimensions ~= size(struct.signals.values,2)
        error('ERROR: struct.dimensions must be equal to struct.values columns')
    end
    
    % empty opzsystems = all systems
    if nargin<=4 || isempty(opzsystems)
        opzsystems = 1:objlss.numSys;
    end
    
    % opzsystems as cell array or string
    if nargin >=5 && iscellstr(opzsystems)
        vect = [];
        for k=1:size(opzsystems,2)
            vect(k) = name2index(opzsystems(k));
        end
        opzsystems = vect;
    end
    
    % opzsystems unique
    if nargin >= 5 && ischar(opzsystems)
        opzsystems = objlss.name2index(opzsystems);
    end
    mask = zeros(1,sum(objlss.ni));
    
    % mask involved systems ( 1xsum(ni) )
    for k=1:length(opzsystems)
        if opzsystems(k) > objlss.numSys
            error('ERROR: system %d does not exist', opzsystems(k))
        end
        mask(1,sum(objlss.ni(1:opzsystems(k)-1))+1:sum(objlss.ni(1:opzsystems(k))))=ones(1,objlss.ni(opzsystems(k)));
    end
    mask = any (mask~=0,1);
    
    % opzstates does not defined or empty, opzstates = all states
    flagWhatStates = 0;
    if nargin <=5  || isempty(opzstates)
        opzstates =1:max(objlss.ni);
        flagWhatStates = 1;
    end
    
    % maskstate is a mask that contains states opzstates
    maskstate = zeros(1,sum(objlss.ni));
    for k=1:length(objlss.ni)
        for j=1:length(opzstates)
            if objlss.ni(k)<opzstates(j)
                if flagWhatStates
                    break;
                end
                warning('MATLAB:paramAmbiguous','WARNING: system %d does not have %d states, it has %d states',k,opzstates(j),objlss.ni(k));
            else
                maskstate (1,sum(objlss.ni(1:k-1))+opzstates(j))=1;
            end
        end
    end
    mask = mask & maskstate;
    
    if struct.signals.dimensions<sum(mask)
        error('ERROR: not enough data to plot')
    end
    
    subpxpag=rows*columns;
    nfinestre = ceil(length(opzsystems)/subpxpag);
    if allinone ==1
        nfinestre = 1;
    end
    fig = zeros(nfinestre,1);
    subplotn = zeros(nfinestre,subpxpag);
    plotndec = subplotn;
    
    if nargin>=7
        if iscell(indexFigSub)
            indexFig = indexFigSub{1,1};
            indexSub = indexFigSub{1,2};
        else
            error('ERROR: {Fig,Subplot}')
        end
        if length(indexFig)~=nfinestre
            error('ERROR: few index Figure');
        end
        if size(indexSub,1)~=nfinestre || size(indexSub,2)~=subpxpag
            error('ERROR: few index Subplot');
        end
    else
        indexFig = [];
        indexSub = [];
    end
    
    
    for i=1:nfinestre
        if isempty(indexFig)
            fig(i) = figure('Name',strcat('Figure', num2str(i)),'NumberTitle','off');
        else
            fig(i) = indexFig(i);
        end
        
        for j=1:subpxpag
            ind = (i-1)*subpxpag+j;
            if ind<=length(opzsystems)
                if isempty(indexSub)
                    subplotn(i,j) = subplot(rows,columns,j,'Parent',fig(i),'FontSize',16,'FontName','Arial');
                else
                    subplotn(i,j) = indexSub(i,j);
                end
                subp = subplotn(i,j);
                hold(subp,'all');
                box(subp,'on');
                
                %title
                if allinone == 0
                    title(subp,objlss.name{opzsystems(ind)},'FontSize',16,'FontName','Arial');
                else
                    title(subp,'stairsX','FontSize',16,'FontName','Arial');
                end
                % x label
                xlabel(subp,'t [s]','FontSize',16,'FontName','Arial');
                
                % y label
                if size(objlss.namex,2)<opzsystems(ind) || isempty (objlss.namex{1,opzsystems(ind)}) || length(opzstates)>1
                    ylabel(subp,'x','FontSize',16,'FontName','Arial');
                else if isscalar(opzstates)
                        ylabel(subp,objlss.namex{opzsystems(ind)}(opzstates,:),'FontSize',16,'FontName','Arial');
                    end
                end
                
                % plot
                help = zeros(1,sum(objlss.ni));
                help(1,sum(objlss.ni(1:opzsystems(ind)-1))+1:sum(objlss.ni(1:opzsystems(ind)))) = ones(1,objlss.ni(opzsystems(ind)));
                help = mask & help;
                
                if allinone == 1
                    help = mask;
                end
                
                X = struct.time;
                Y = struct.signals.values(:,help);
                
                for k=1:size(Y,2)
                    plotndec(i,j,k) = stairs(X, Y(:,k), 'Parent' , subplotn(i,j) );
                end
            else
                break
            end
        end
    end
    
    varargout = [];
    if nargout>=1
        varargout{1,1} = fig;
    end
    if nargout>=2
        varargout{1,2} = subplotn;
    end
    if nargout>=3
        varargout{1,3} = plotndec;
    end
end