function obj = union(obj,indices,newname)
% UNION Joins two or more subsystems in one within an LSS object
% 
% objlss = objlss.union(index,newname)
% 
% -------------------------------------------------------------------------
% DESCRIPTION
% ------------------------------------------------------------------------- 
% Joins two or more subsystems in one within an LSS object
%
% objlss = objlss.UNION(index,newname)
% 
% -------------------------------------------------------------------------
% 
% INPUT
% ------------------------------------------------------------------------- 
% index = vector of numbers or names (as cellarray) of subsystems to join 
% newname = name of the new joint subsystem (as string);
%           If it isn't passed, it is the union of the name of subsystems 
%           that have been combined
% -------------------------------------------------------------------------
%
% OUTPUT
% -------------------------------------------------------------------------  
% objlss updated

% Copyright is with the following author(s):
%
% (C) 2013 
%     Fabio Sarzo, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     fabio.sarzo01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%
% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------


    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin~=2 && nargin~=3
        error('UNION: check number of inputs');
    end
    
    if nargin<3
        newname = [];
    end
    
    if ~isa(obj,'lss')
        error('UNION: System input must be lss object')
    end
    
    if ischar(indices)
        indices = obj.name2index(indices);
    elseif iscellstr(indices)
        vect = [];
        for k=1:size(indices,2)
            vect(k) = obj.name2index(indices(k));
        end
        indices = vect;
    end
    
    if size(indices,2)<2
        warning('UNION: at least 2 indices needed');
        return;
    end
    
    indices = unique(indices);
    
    if indices(end)>obj.numSys
        error('UNION: subsystem %d does not exist',indices(end));
    end
    
    if size(indices,2)>obj.numSys
        error('UNION: too many indices');
    end
    
    
    
    while length(indices)~=1
        j = indices(end);
        i = indices(end-1);
        
        %edit name
        if isempty(newname)
            obj.name{i} = [obj.name{i},obj.name{j}];
        elseif ischar(newname)
            obj.name{i} = newname;
        else
            error('union: The new name must be a string')
        end
        obj.name(j)=[];
        
        %edit couplingmatrix
        couplingmask = false(size(obj.couplingmatrix));
        couplingmask(:,(i)) = obj.couplingmatrix(:,j);
        couplingmask((i),:) = obj.couplingmatrix(j,:);
        obj.couplingmatrix =(obj.couplingmatrix|couplingmask);
        obj.couplingmatrix(:,j) = [];
        obj.couplingmatrix(j,:) = [];
        
        if (j-i)~=1 %not consecutive subsystems
            
            Tx = sparse(zeros(sum(obj.ni),sum(obj.ni)));
            Tx(1:sum(obj.ni(1:i)),1:sum(obj.ni(1:i))) = eye(sum(obj.ni(1:i)),sum(obj.ni(1:i)));
            Tx(sum(obj.ni(1:i))+1:sum(obj.ni(1:i))+obj.ni(j),sum(obj.ni(1:j-1))+1:sum(obj.ni(1:j-1))+obj.ni(j)) = eye(obj.ni(j));
            Tx(sum(obj.ni(1:i))+obj.ni(j)+1:sum(obj.ni(1:i))+obj.ni(j)+sum(obj.ni(i+1:j-1)),sum(obj.ni(1:i))+1:sum(obj.ni(1:i))+sum(obj.ni(i+1:j-1))) = eye(sum(obj.ni(i+1:j-1)));
            Tx(sum(obj.ni(1:j))+1:sum(obj.ni(1:j))+sum(obj.ni(j+1:end)),sum(obj.ni(1:j))+1:sum(obj.ni(1:j))+sum(obj.ni(j+1:end))) = eye(sum(obj.ni(j+1:end)));
            
            Tu = sparse(zeros(sum(obj.mi),sum(obj.mi)));
            Tu(1:sum(obj.mi(1:i)),1:sum(obj.mi(1:i))) = eye(sum(obj.mi(1:i)),sum(obj.mi(1:i)));
            Tu(sum(obj.mi(1:i))+1:sum(obj.mi(1:i))+obj.mi(j),sum(obj.mi(1:j-1))+1:sum(obj.mi(1:j-1))+obj.mi(j)) = eye(obj.mi(j));
            Tu(sum(obj.mi(1:i))+obj.mi(j)+1:sum(obj.mi(1:i))+obj.mi(j)+sum(obj.mi(i+1:j-1)),sum(obj.mi(1:i))+1:sum(obj.mi(1:i))+sum(obj.mi(i+1:j-1))) = eye(sum(obj.mi(i+1:j-1)));
            Tu(sum(obj.mi(1:j))+1:sum(obj.mi(1:j))+sum(obj.mi(j+1:end)),sum(obj.mi(1:j))+1:sum(obj.mi(1:j))+sum(obj.mi(j+1:end))) = eye(sum(obj.mi(j+1:end)));
            
            obj.A = Tx*obj.A/Tx;
            obj.B = Tx*obj.B/Tu;
            
            nitemp = [obj.ni(1,1:i),obj.ni(1,j),obj.ni(1,i+1:end)];
            nitemp(:,j+1) = [];
            obj.ni = nitemp;
            
            mitemp = [obj.mi(1,1:i),obj.mi(1,j),obj.mi(1,i+1:end)];
            mitemp(:,j+1) = [];
            obj.mi = mitemp;
            
            if ~isempty(obj.numExo)
                obj.M = Tx*obj.M;
            end
            
            if ~isempty(obj.numICen)
                obj.Bcen = Tx*obj.Bcen;
            end
            
            if ~isempty(obj.pi)
                Ty = sparse(zeros(sum(obj.pi),sum(obj.pi)));
                Ty(1:sum(obj.pi(1:i)),1:sum(obj.pi(1:i))) = eye(sum(obj.pi(1:i)),sum(obj.pi(1:i)));
                Ty(sum(obj.pi(1:i))+1:sum(obj.pi(1:i))+obj.pi(j),sum(obj.pi(1:j-1))+1:sum(obj.pi(1:j-1))+obj.pi(j)) = eye(obj.pi(j));
                Ty(sum(obj.pi(1:i))+obj.pi(j)+1:sum(obj.pi(1:i))+obj.pi(j)+sum(obj.pi(i+1:j-1)),sum(obj.pi(1:i))+1:sum(obj.pi(1:i))+sum(obj.pi(i+1:j-1))) = eye(sum(obj.pi(i+1:j-1)));
                Ty(sum(obj.pi(1:j))+1:sum(obj.pi(1:j))+sum(obj.pi(j+1:end)),sum(obj.pi(1:j))+1:sum(obj.pi(1:j))+sum(obj.pi(j+1:end))) = eye(sum(obj.pi(j+1:end)));
                
                obj.C = Ty*obj.C/Tx;
                obj.D = Ty*obj.D/Tu;
                
                if ~isempty(obj.numExo)
                    obj.N = Ty*obj.N;
                end
                
                if ~isempty(obj.numICen)
                    obj.Dcen = Ty*obj.Dcen;
                end
                
                pitemp = [obj.pi(1,1:i),obj.pi(1,j),obj.pi(1,i+1:end)];
                pitemp(:,j+1) = [];
                obj.pi = pitemp;
            end
            
            if ~isempty(obj.rx)
                obj.Hx = obj.Hx/Tx;
                %edit rows
                Hxnew = [obj.Hx(1:sum(obj.rx(1:i)),:);obj.Hx(sum(obj.rx(1:j-1))+1:sum(obj.rx(1:j-1))+obj.rx(j),:);obj.Hx(sum(obj.rx(1:i))+1:end,:)];
                Hxnew(sum(obj.rx(1:j))+1:sum(obj.rx(1:j))+obj.rx(j),:) = [];
                obj.Hx = Hxnew;
                Kxnew = [obj.Kx(1:sum(obj.rx(1:i)),:);obj.Kx(sum(obj.rx(1:j-1))+1:sum(obj.rx(1:j-1))+obj.rx(j),:);obj.Kx(sum(obj.rx(1:i))+1:end,:)];
                Kxnew(sum(obj.rx(1:j))+1:sum(obj.rx(1:j))+obj.rx(j),:) = [];
                obj.Kx = Kxnew;
                %edit obj.rx
                rxtemp = [obj.rx(1,1:i),obj.rx(1,j),obj.rx(1,i+1:end)];
                rxtemp(:,j+1) = [];
                obj.rx = rxtemp;
            end
            if ~isempty(obj.ru)
                obj.Hu = obj.Hu/Tu;
                %edit rows
                Hunew = [obj.Hu(1:sum(obj.ru(1:i)),:);obj.Hu(sum(obj.ru(1:j-1))+1:sum(obj.ru(1:j-1))+obj.ru(j),:);obj.Hu(sum(obj.ru(1:i))+1:end,:)];
                Hunew(sum(obj.ru(1:j))+1:sum(obj.ru(1:j))+obj.ru(j),:) = [];
                obj.Hu = Hunew;
                Kunew = [obj.Ku(1:sum(obj.ru(1:i)),:);obj.Ku(sum(obj.ru(1:j-1))+1:sum(obj.ru(1:j-1))+obj.ru(j),:);obj.Ku(sum(obj.ru(1:i))+1:end,:)];
                Kunew(sum(obj.ru(1:j))+1:sum(obj.ru(1:j))+obj.ru(j),:) = [];
                obj.Ku = Kunew;
                %edit obj.ru
                rutemp = [obj.ru(1,1:i),obj.ru(1,j),obj.ru(1,i+1:end)];
                rutemp(:,j+1) = [];
                obj.ru = rutemp;
            end
            if ~isempty(obj.ry)
                obj.Hy = obj.Hy/Ty;
                %edit rows
                Hynew = [obj.Hy(1:sum(obj.ry(1:i)),:);obj.Hy(sum(obj.ry(1:j-1))+1:sum(obj.ry(1:j-1))+obj.ry(j),:);obj.Hy(sum(obj.ry(1:i))+1:end,:)];
                Hynew(sum(obj.ry(1:j))+1:sum(obj.ry(1:j))+obj.ry(j),:) = [];
                obj.Hy = Hynew;
                Kynew = [obj.Ky(1:sum(obj.ry(1:i)),:);obj.Ky(sum(obj.ry(1:j-1))+1:sum(obj.ry(1:j-1))+obj.ry(j),:);obj.Ky(sum(obj.ry(1:i))+1:end,:)];
                Kynew(sum(obj.ry(1:j))+1:sum(obj.ry(1:j))+obj.ry(j),:) = [];
                obj.Ky = Kynew;
                %edit obj.ry
                rytemp = [obj.ry(1,1:i),obj.ry(1,j),obj.ry(1,i+1:end)];
                rytemp(:,j+1) = [];
                obj.ry = rytemp;
            end
            if ~isempty(obj.rdeltau)
                obj.Hdeltau = obj.Hdeltau/Tu;
                %edit rows
                Hdeltaunew = [obj.Hdeltau(1:sum(obj.rdeltau(1:i)),:);obj.Hdeltau(sum(obj.rdeltau(1:j-1))+1:sum(obj.rdeltau(1:j-1))+obj.rdeltau(j),:);obj.Hdeltau(sum(obj.rdeltau(1:i))+1:end,:)];
                Hdeltaunew(sum(obj.rdeltau(1:j))+1:sum(obj.rdeltau(1:j))+obj.rdeltau(j),:) = [];
                obj.Hdeltau = Hdeltaunew;
                Kdeltaunew = [obj.Kdeltau(1:sum(obj.rdeltau(1:i)),:);obj.Kdeltau(sum(obj.rdeltau(1:j-1))+1:sum(obj.rdeltau(1:j-1))+obj.rdeltau(j),:);obj.Kdeltau(sum(obj.rdeltau(1:i))+1:end,:)];
                Kdeltaunew(sum(obj.rdeltau(1:j))+1:sum(obj.rdeltau(1:j))+obj.rdeltau(j),:) = [];
                obj.Kdeltau = Kdeltaunew;
                %edit obj.rdeltau
                rdeltautemp = [obj.rdeltau(1,1:i),obj.rdeltau(1,j),obj.rdeltau(1,i+1:end)];
                rdeltautemp(:,j+1) = [];
                obj.rdeltau = rdeltautemp;
            end
            if ~isempty(obj.re)
                obj.He = obj.He/Tx;
                %edit rows
                Henew = [obj.He(1:sum(obj.re(1:i)),:);obj.He(sum(obj.re(1:j-1))+1:sum(obj.re(1:j-1))+obj.re(j),:);obj.He(sum(obj.re(1:i))+1:end,:)];
                Henew(sum(obj.re(1:j))+1:sum(obj.re(1:j))+obj.re(j),:) = [];
                obj.He = Henew;
                Kenew = [obj.Ke(1:sum(obj.re(1:i)),:);obj.Ke(sum(obj.re(1:j-1))+1:sum(obj.re(1:j-1))+obj.re(j),:);obj.Ke(sum(obj.re(1:i))+1:end,:)];
                Kenew(sum(obj.re(1:j))+1:sum(obj.re(1:j))+obj.re(j),:) = [];
                obj.Ke = Kenew;
                %edit obj.re
                retemp = [obj.re(1,1:i),obj.re(1,j),obj.re(1,i+1:end)];
                retemp(:,j+1) = [];
                obj.re = retemp;
            end
            
            j = i+1;
        end
        
        
        %edit rx/ru/ry/rdeltau/re (state/input/output/deltau/error estimation costraints)
        if ~isempty(obj.rx)
            if sum(obj.rx)<size(obj.Hx,1)
                [Hxcoupl, Kxcoupl, posxl] = extractrowcoupling(obj.Hx(sum(obj.rx)+1:end,:),obj.Kx(sum(obj.rx)+1:end,1),obj.ni,[i j]);
                obj.Hx   = [obj.Hx(1:sum(obj.rx(1:j)),:);Hxcoupl;obj.Hx(sum(obj.rx(1:j))+1:end,:)];
                obj.Kx   = [obj.Kx(1:sum(obj.rx(1:j)),:);Kxcoupl;obj.Kx(sum(obj.rx(1:j))+1:end,:)];
                posx     = find(posxl==1);
                obj.rx(i) = (obj.rx(i)+obj.rx(j)+size(posx,2));
                obj.rx(j) = [];
                obj.Hx((posx+sum(obj.rx)),:) = [];
                obj.Kx((posx+sum(obj.rx))) = [];
            else
                obj.rx(i) = (obj.rx(i)+obj.rx(j));
                obj.rx(j) = [];
            end
        end
        
        if ~isempty(obj.ru)
            if sum(obj.ru)<size(obj.Hu,1)
                [Hucoupl, Kucoupl, posul] = extractrowcoupling(obj.Hu(sum(obj.ru)+1:end,:),obj.Ku(sum(obj.ru)+1:end,1),obj.mi,[i j]);
                obj.Hu   = [obj.Hu(1:sum(obj.ru(1:j)),:);Hucoupl;obj.Hu(sum(obj.ru(1:j))+1:end,:)];
                obj.Ku   = [obj.Ku(1:sum(obj.ru(1:j)),:);Kucoupl;obj.Ku(sum(obj.ru(1:j))+1:end,:)];
                posu     = find(posul==1);
                obj.ru(i) = (obj.ru(i)+obj.ru(j)+size(posu,2));
                obj.ru(j) = [];
                obj.Hu((posu+sum(obj.ru)),:) = [];
                obj.Ku((posu+sum(obj.ru))) = [];
            else
                obj.ru(i) = (obj.ru(i)+obj.ru(j));
                obj.ru(j) = [];
            end
        end
        
        if ~isempty(obj.ry)
            if sum(obj.ry)<size(obj.Hy,1)
                [Hycoupl, Kycoupl, posyl] = extractrowcoupling(obj.Hy(sum(obj.ry)+1:end,:),obj.Ky(sum(obj.ry)+1:end,1),obj.pi,[i j]);
                obj.Hy   = [obj.Hy(1:sum(obj.ry(1:j)),:);Hycoupl;obj.Hy(sum(obj.ry(1:j))+1:end,:)];
                obj.Ky   = [obj.Ky(1:sum(obj.ry(1:j)),:);Kycoupl;obj.Ky(sum(obj.ry(1:j))+1:end,:)];
                posy     = find(posyl==1);
                obj.ry(i) = (obj.ry(i)+obj.ry(j)+size(posy,2));
                obj.ry(j) = [];
                obj.Hy((posy+sum(obj.ry)),:) = [];
                obj.Ky((posy+sum(obj.ry))) = [];
            else
                obj.ry(i) = (obj.ry(i)+obj.ry(j));
                obj.ry(j) = [];
            end
        end
        
        if ~isempty(obj.rdeltau)
            if sum(obj.rdeltau)<size(obj.Hdeltau,1)
                [Hdeltaucoupl, Kdeltaucoupl, posdeltaul] = extractrowcoupling(obj.Hdeltau(sum(obj.rdeltau)+1:end,:),obj.Kdeltau(sum(obj.rdeltau)+1:end,1),obj.mi,[i j]);
                obj.Hdeltau   = [obj.Hdeltau(1:sum(obj.rdeltau(1:j)),:);Hdeltaucoupl;obj.Hdeltau(sum(obj.rdeltau(1:j))+1:end,:)];
                obj.Kdeltau   = [obj.Kdeltau(1:sum(obj.rdeltau(1:j)),:);Kdeltaucoupl;obj.Kdeltau(sum(obj.rdeltau(1:j))+1:end,:)];
                posdeltau     = find(posdeltaul==1);
                obj.rdeltau(i) = (obj.rdeltau(i)+obj.rdeltau(j)+size(posdeltau,2));
                obj.rdeltau(j) = [];
                obj.Hdeltau((posdeltau+sum(obj.rdeltau)),:) = [];
                obj.Kdeltau((posdeltau+sum(obj.rdeltau))) = [];
            else
                obj.rdeltau(i) = (obj.rdeltau(i)+obj.rdeltau(j));
                obj.rdeltau(j) = [];
            end
        end
        
        if ~isempty(obj.re)
            if sum(obj.re)<size(obj.He,1)
                [Hecoupl, Kecoupl, posel] = extractrowcoupling(obj.He(sum(obj.re)+1:end,:),obj.Ke(sum(obj.re)+1:end,1),obj.ni,[i j]);
                obj.He   = [obj.He(1:sum(obj.re(1:j)),:);Hecoupl;obj.He(sum(obj.re(1:j))+1:end,:)];
                obj.Ke   = [obj.Ke(1:sum(obj.re(1:j)),:);Kecoupl;obj.Ke(sum(obj.re(1:j))+1:end,:)];
                pose     = find(posel==1);
                obj.re(i) = (obj.re(i)+obj.re(j)+size(pose,2));
                obj.re(j) = [];
                obj.He((pose+sum(obj.re)),:) = [];
                obj.Ke((pose+sum(obj.re))) = [];
            else
                obj.re(i) = (obj.re(i)+obj.re(j));
                obj.re(j) = [];
            end
        end
        
        obj.numSys = obj.numSys-1;
        
        obj.ni(i) = (obj.ni(i)+obj.ni(j));
        obj.ni(j) = [];
        
        obj.mi(i) = (obj.mi(i)+obj.mi(j));
        obj.mi(j) = [];
        
        if ~isempty(obj.pi)
            obj.pi(i) = (obj.pi(i)+obj.pi(j));
            obj.pi(j) = [];
        end
        
        
        
        indices(end)=[];
    end
    
    obj.couplingmatrix = sparse(obj.couplingmatrix);
    
end


function [H, K, okrows]= extractrowcoupling(Haux,Kaux,nv,index)

    mask=zeros(1,sum(nv));

    for i=1:length(index)
        start = sum(nv(1:index(i)))-nv(index(i))+1;
        mask(1,start:start+nv(index(i))-1) = ones(1,nv(index(i)));
    end
    
    mask    = all(mask>0.5,1);
    help    = Haux(:,mask);
    flag    = help~=0;
    niindex = nv(index);
    flagtmp = sparse(zeros(size(help,1),length(niindex))==1);
    for i = 1:length(niindex)
        flagtmp(:,i) = any( flag( : , sum(niindex(1:i-1))+1:sum(niindex(1:i)) ) , 2 );
    end
    flag = all(flagtmp,2);
    newhelp = Haux(:,~mask);
    newflag = all(newhelp==0,2);
    okrows  = flag' & newflag';
    H      = Haux(okrows,:);
    K      = Kaux(okrows,:);
end