function test = isinside(obj,x,options)
% ISINSIDE Checks if a given point x lies in an RCI set
%
% test = isinside(obj,x,options)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Checks if a given point x lies in the set given by parameterized RCI
% object obj
% 
% USAGE: 
%  
% test = obj.ISINSIDE(x,options)
% test = obj.ISINSIDE(x)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% x = point of the same dimension of the RCI
% 
% options = sdpsettings object for YALMIP (optional)
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% test = 1 if the point is inside, 0 otherwise
%
%
% We use the implicit representation of the Parameterized RCI is proposed in [1]
% [1] S. V. Rakovic and M. Baric, Parameterized Robust Control Invariant
% Sets for Linear Systems: Theoretical Advances and Computational Remarks,
% IEEE Transactions on Automatic Control, vol. 55, n. 7, pp. 1599-1614, 2010
%
% see also PARAMETERIZEDRCI

% Copyright is with the following author(s):
%
% (C) 2012 
%     Stefano Riverso, Alberto Battocchio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, alberto.battocchio01@universitadipavia.it,
%     giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin~=3
        options = sdpsettings;
    end
    
    if size(x,1)~=size(obj.A,1) || size(x,2)~=1
        error('ISINSIDE: parameters dimensions');
    end
    
    lambda = sdpvar(obj.k,size(obj.x0,1));
    xV     = [ obj.x0' reshape(obj.x(:,:,1:obj.k-1),size(obj.A,1),[]) ];
    
    info   = solvesdp([ lambda(:)>=0 ; sum(lambda,2)==1 ; x == (1-obj.alpha)^-1*( obj.bx + sum(repmat(reshape(lambda',1,[]),size(obj.A,1),1).*xV,2) ) ],0,options);
    
    if info.problem == 0
        test = 1;
    elseif info.problem == 1
        test = 0;
    else
        error('ISINSIDE: ERROR SOLVER: %s',info.info);
    end
end