classdef parameterizedRCI
% PARAMETERIZEDRCI compute a parameterized Robust Control Invariant Set for a Linear System as proposed in [1]
% 
% R = parameterizedRCI(A,B,k,X,U,x0,xref,uref,options)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return a parameterized Robust Control Invariant Set
% 
% R = PARAMETERIZEDRCI(A,B,k,X,U,x0)
% R = PARAMETERIZEDRCI(A,B,k,X,U,x0,xref,uref)
% R = PARAMETERIZEDRCI(A,B,k,X,U,x0,xref,uref,options)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - Matrices A and B of the linear system x(t+1)=Ax(t)+Bu(t) with n states
% and m inputs
% - k must be >= of the controllability index of the pair (A,B)
% - X is a cell array defined as {cx,dx} where cx*x<=dx
% - U is a cell array defined as {cu,du} where cu*u<=du
% - x0 is a matrix of dimensions q x n, with q number of points or a cell
% array with first element the matrix x0 and second element 0 if for each
% partial state the origin is in its interior. If this option is selected
% xref and uref will be set to 0. see example2parameterizedRCI
% - xref and uref to set the center of the robust control invariant set.
% otherwise they are optimized
% - options from YALMIP, i.e. sdpsettings object (optional). An LP solver 
%   is needed.
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% R is a parameterizedRCI object
%
% ---------------------------------------------------------------------------
% PROPERTIES
% ---------------------------------------------------------------------------
% A, B, k, cx, dx, cu, du, x0, xref, uref : the inputs are stored in the
%                                           parameterizedRCI object
% x, u, alpha, bx, bu : optimized values of the procedure
% info : object with information of the optimization problem (see solvesdp
%        of YALMIP)
% RP: Polyhedron object of the RCI set, with H-representation and 
%     V-representation
% 
%
% ---------------------------------------------------------------------------
% METHODS
% ---------------------------------------------------------------------------
% ISINSIDE: checks if a given point x lies in the interior of the RCI
% PLOT: plot the robust control invariant set
% UINV: compute the control input to guarantee robust control invariance
% DOUBLE: return the H-representation of the RCI
% 
%
% [1] S. V. Rakovic and M. Baric, Parameterized Robust Control Invariant
% Sets for Linear Systems: Theoretical Advances and Computational Remarks,
% IEEE Transactions on Automatic Control, vol. 55, n. 7, pp. 1599-1614, 2010
    
% Copyright is with the following author(s):
%
% (C) 2012 
%     Stefano Riverso, Alberto Battocchio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, alberto.battocchio01(at)universitadipavia.it,
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------
    
    properties ( SetAccess = private , Hidden = false )
        A      ;
        B      ;
        k      ;
        cx     ;
        dx     ;
        cu     ;
        du     ;
        x0     ;
        xref   ;
        uref   ;
        x      ;
        u      ;
        alpha  ;
        bx     ;
        bu     ;
        info   ;
    end
    
    properties ( SetAccess = private , Hidden = false )
        RP     ;
    end
    
    methods
        
        function obj = parameterizedRCI(A,B,k,X,U,x0,xref,uref,options)
            
            global pnpmpcToolboxGlobalOptions;
            if ~isstruct(pnpmpcToolboxGlobalOptions),
                pnpmpc_toolbox_err;
            end
            
            if nargin~=9
                options = sdpsettings;
            end
            
            if nargin<=7
                xref = [];
                uref = [];
            end
            
            if isempty(B)
                B = zeros(size(A,2),1);
            end
            
            obj.A    = sparse(A);
            obj.B    = sparse(B);
            n        = size(obj.A ,1);
            m        = size(obj.B ,2);
            
            if ~iscell(X) || length(X)~=2
                error('PARAMETERIZEDRCI: X is not a cell array of two elements. see help parameterizedRCI');
            end
            if ~iscell(U) || length(U)~=2
                error('PARAMETERIZEDRCI: U is not a cell array of two elements. see help parameterizedRCI');
            end
            
            obj.cx   = sparse(X{1,1});
            obj.dx   = sparse(X{1,2});
            obj.cu   = sparse(U{1,1});
            obj.du   = sparse(U{1,2});
            
            obj.k    = k;
            
            flag0    = 0;
            if iscell(x0)
                if length(x0)~=2
                    error('PARAMETERIZEDRCI: x0 is a cell array and it should have two elements. see help parameterizedRCI');
                else
                    x0 = [ zeros(1,n) ; x0{1} ];
                    flag0 = 1;
                end
            end
            
            obj.x0   = sparse(x0);
            obj.xref = sparse(xref);
            obj.uref = sparse(uref);
            
            if flag0==1
                obj.xref = sparse(zeros(n,1));
                obj.uref = sparse(zeros(m,1));
            end
            
            rx       = size(obj.cx,1);
            ru       = size(obj.cu,1);
            
            x0       = x0';
            q        = size(x0,2);
            
            CI       = ctrbindex(obj.A,obj.B);
            if k<CI
                error('PARAMETERIZEDRCI: k is less than controllability index %s',num2str(CI));
            end
            
            if ( ~isempty(obj.cx) && (size(obj.cx,2)~=n || size(obj.dx,1)~=rx || size(obj.dx,2)~=1) ) || size(A,2)~=n || size(B,1)~=n || size(obj.x0,2)~=n || ( ~isempty(obj.cu) && (size(obj.cu,2)~=m || size(obj.du,1)~=ru || size(obj.du,2)~=1))
                error('PARAMETERIZEDRCI: parameters dimensions');
            end
            if ~isempty(xref) && ( size(xref,1)~=n || size(xref,2)~=1 )
                error('PARAMETERIZEDRCI: parameters dimensions');
            end
            if ~isempty(uref) && ( size(uref,1)~=m || size(uref,2)~=1 )
                error('PARAMETERIZEDRCI: parameters dimensions');
            end
            
            obj.info.problem = 1;
            
            CONS   = [];
            
            bx     = sdpvar(n,1);
            bu     = sdpvar(m,1);
            alpha  = sdpvar(1,1);
            x      = sdpvar(n,q,k);
            u      = sdpvar(m,q,k);
            lambda = sdpvar(q,q);
            gamma  = sdpvar(rx,k);
            phi    = sdpvar(ru,k);
            
            assign(bx,0);
            assign(bu,0);
            assign(alpha,0);
            assign(x,0);
            assign(u,0);
            assign(lambda,0);
            if ~isempty(gamma)
                assign(gamma,0);
            end
            if ~isempty(phi)
                assign(phi,0);
            end
            
            
            %% (6.5a)
            CONS = [ CONS ; alpha-1<=-eps ; -alpha<=0 ];
            
            
            %% (6.5b)
            CONS = [ CONS ; -repmat(alpha,q,1)+sum(lambda,2)<=0 ];
            
            
            %% (6.5c)
            CONS = [ CONS ; -lambda(:) <= 0 ];
            
            
            if ~isempty(obj.cu)
                %% (6.5d)
                CONS = [ CONS ; obj.cu*bu+sum(phi,2)+alpha*obj.du-obj.du <= 0 ];
                
                %% (6.5e)
                if k==1
                    listper = [1 2];
                else
                    listper = [1 3 2];
                end
                CONS = [ CONS ; sum(reshape(repmat(obj.cu',k*q,1),m,ru*q*k)'.*repmat(reshape(permute(u,listper),m,k*q)',ru,1),2)-reshape(repmat(phi,1,q)',k*q*ru,1) <= 0  ];
            end
            
            
            if ~isempty(obj.cx)
                %% (6.5f)
                CONS = [ CONS ; obj.cx*bx+sum(gamma,2)+alpha*obj.dx-obj.dx <= 0  ];
                
                %% (6.5g)
                CONS = [ CONS ; sum(reshape(repmat(obj.cx',q,1),n,rx*q)'.*repmat(x0',rx,1),2) - reshape(repmat(gamma(:,1)',q,1),q*rx,1) <= 0 ];
                if k~=1
                    if k==2
                        listper = [1 2];
                    else
                        listper = [1 3 2];
                    end
                    CONS = [ CONS ; sum(reshape(repmat(obj.cx',(k-1)*q,1),n,rx*q*(k-1))'.*repmat(reshape(permute(x(:,:,1:k-1),listper),n,(k-1)*q)',rx,1),2) - reshape(repmat(gamma(:,2:k),1,q)',(k-1)*q*rx,1) <= 0  ];
                end
            end
            
            
            %% (6.5h)
            for i9=1:k-1
                for i10=1:q
                    CONS = [ CONS ; x(:,i10,i9+1)-A*x(:,i10,i9)-B*u(:,i10,i9+1)==0 ];
                end
            end
            CONS = [ CONS ; reshape(x(:,:,1),n*q,1)-sum(repmat(A,q,1).*reshape(repmat(x0,n,1),n,n*q)',2)-sum(repmat(B,q,1).*reshape(repmat(u(:,:,1),n,1),m,n*q)',2)==0 ];
            
            
            %% (6.5a)
            CONS = [ CONS ; (eye(n)-A)*bx-B*bu==zeros(n,1) ];
            CONS = [ CONS ; reshape(x(:,:,k),n*q,1)-sum(reshape(repmat(lambda,1,n)',q,n*q)'.*repmat(x0,q,1),2)==zeros(n*q,1) ];
            if ~isempty(xref)
                CONS = [ CONS ; bx==xref ];
            end
            if ~isempty(uref)
                CONS = [ CONS ; bu==uref ];
            end
            
            
            %% () 0 is added to partial state and control sets
            if flag0==1
                for i=1:k
                    CONS = [ CONS ; x(:,1,i) == 0 ];
                    CONS = [ CONS ; u(:,1,i) == 0 ];
                end
            end
            
            
            obj.info  = solvesdp(CONS,alpha,options);
            
            if obj.info.problem==0
                obj.x                 = double(x);
                obj.u                 = double(u);
                obj.alpha             = double(alpha);
                obj.bx                = sparse(double(bx));
                obj.bu                = sparse(double(bu));
                
                obj.u(isnan(obj.u))   = 0;
                obj.x(isnan(obj.x))   = 0;
                obj.bu(isnan(obj.bu)) = 0;
                obj.bx(isnan(obj.bx)) = 0;                
            else
                obj.x      = [];
                obj.u      = [];
                obj.alpha  = [];
                obj.bx     = [];
                obj.bu     = [];
            end
            
            obj.RP = [];
            
        end
        
        
    end
end