function [ uInv, info ] = uInv(obj,x,options)
% UINV Compute the control input associated to an RCI that guarantees robust control invariance
%
% uInv = uInv(obj,x,options)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Compute the control input associated to the parameterized RCI object obj 
% that guarantees robust control invariance
% 
% USAGE: 
%  
% uInv = obj.UINV(x,options)
% uInv = obj.UINV(x)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% x = point of the same dimension of the RCI
% 
% options = sdpsettings object for YALMIP (optional)
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% uInv: value of the control input
% info: Diagnostic information of solvesdp (see YALMIP)
%
% The control input is computed as in [1]
% [1] S. V. Rakovic and M. Baric, Parameterized Robust Control Invariant
% Sets for Linear Systems: Theoretical Advances and Computational Remarks,
% IEEE Transactions on Automatic Control, vol. 55, n. 7, pp. 1599-1614, 2010
%
%
% see also PARAMETERIZEDRCI

% Copyright is with the following author(s):
%
% (C) 2012 
%     Stefano Riverso, Alberto Battocchio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, alberto.battocchio01@universitadipavia.it,
%     giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin~=3
        options = sdpsettings;
    end
    
    if size(x,1)~=size(obj.A,1) || size(x,2)~=1
        error('UINV: parameters dimensions');
    end
    
    lambda = sdpvar(obj.k,size(obj.x0,1),'full');
    xV     = [ obj.x0' reshape(obj.x(:,:,1:obj.k-1),size(obj.A,1),[]) ];
    
    if all(all(xV==0))
        uInv = zeros(size(obj.B,2));
        info = [];
        return;
    end
    
    if all(abs(obj.x0(1,:))<=10^-6) && all(all(abs(permute(obj.x(:,1,:),[1 3 2]))<=10^-6))
        mu     = sdpvar(1);
        CONS   = [ lambda(:)>=0 ; sum(lambda,2)==mu ; x == (1-obj.alpha)^-1*( obj.bx + sum(repmat(reshape(lambda',1,[]),size(obj.A,1),1).*xV,2) ) ; mu>=0 ; mu<=1 ];
        info   = solvesdp(CONS,mu,options);        
    else
        CONS   = [ lambda(:)>=0 ; sum(lambda,2)==1 ; x == (1-obj.alpha)^-1*( obj.bx + sum(repmat(reshape(lambda',1,[]),size(obj.A,1),1).*xV,2) ) ];
        info   = solvesdp(CONS,lambda(:)'*lambda(:),options);
    end
    
    if info.problem == 0
        uV     = reshape(obj.u,size(obj.B,2),[]);
        lambda = double(lambda);
        uInv   = (1-obj.alpha)^-1*( obj.bu + sum(repmat(reshape(lambda',1,[]),size(obj.B,2),1).*uV,2) );
    elseif info.problem == 1
        error('UINV: x is not inside R')
    else
        error('UINV: ERROR SOLVER: %s',info.info);
    end
end