function [ xest, yest, yfilt] = computeEstimation( obj , xt , yt , ut , xtj , ytj , utj , utcen , d )
% COMPUTEESTIMATION compute state estimation
%
% [ xest, yest, yfilt] = computeEstimation( obj , xt , yt , ut , xtj , ytj , utj , utcen , d )
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Compute state and output estimation estimation
% 
% USAGE: 
%  
% [ xest             ] = COMPUTEESTIMATION( obj , xt , yt , ut , xtj )
% [ xest             ] = COMPUTEESTIMATION( obj , xt , yt , ut , xtj , ytj )
% [ xest             ] = COMPUTEESTIMATION( obj , xt , yt , ut , xtj , ytj , utj )
% [ xest             ] = COMPUTEESTIMATION( obj , xt , yt , ut , xtj , ytj , utj , utcen )
% [ xest             ] = COMPUTEESTIMATION( obj , xt , yt , ut , xtj , ytj , utj , utcen , d )
% [ xest, yest       ] = COMPUTEESTIMATION( obj , xt , yt , ut , xtj , ytj , utj , utcen , d )
% [ xest, yest, yfilt] = COMPUTEESTIMATION( obj , xt , yt , ut , xtj , ytj , utj , utcen , d )
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% obj : pnpEstimator object
% xt : local state estimation at time t
% yt : local output measurements at time t
% ut : local control inputs at time t
% xtj : parents' state estimation at time t
% ytj : parents' output measurements at time t
% utj : parents' control inputs at time t
% utcen : centralized control inputs at time t
% d : exogenous inputs not bounded
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% xest : local state estimation at time t+1
% yest : local output estimation at time t+1, yest = C xt
% yfilt : local output filtered at time t+1, yfilt = C xest
%
%
% see also PNPESTIMATOR

% Copyright is with the following author(s):
%
% (C) 2013 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------    

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin<=5
        ytj = zeros(0,1);
    end
    
    if nargin<=6
        utj = zeros(0,1);
    end
    
    if nargin<=7
        utcen = zeros(0,1);
    end
    
    if nargin<=8
        d = zeros(0,1);
    end
    
    d1 = size(yt,1) + size(xtj,1) + size(ytj,1) + size(ut,1) + size(utj,1) + size(utcen,1) + size(d,1);
    if d1~=size(obj.B,2) || size(yt,2)~=1 || size(xtj,2)~=1 || size(ytj,2)~=1 || size(ut,2)~=1 || size(utj,2)~=1 || size(utcen,2)~=1 || size(d,2)~=1 || size(xt,1)~=size(obj.A,2) || size(xt,2)~=1
        error('COMPUTEESTIMATION: error dimensions of input parameters. see obj.estInputs to check inputs for state estimation')
    end
    
    yest  = obj.C*xt;
    
    xest  = obj.A*xt + obj.B*[ yt ; xtj ; ytj ; ut ; utj ; utcen ; d ];
    
    yfilt = obj.C*xest;
    
end