classdef pnpEstimator
% PNPESTIMATOR design a Plug-and-Play estimator as proposed in [1]
% 
% est = pnpEstimator (objsubss,Ej,Cj,Oj,flag)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return a Plug-and-Play estimator
% 
% est = PNPESTIMATOR (objsubss)
% est = PNPESTIMATOR (objsubss,Ej)
% est = PNPESTIMATOR (objsubss,Ej,Cj)
% est = PNPESTIMATOR (objsubss,Ej,Cj,Oj)
% est = PNPESTIMATOR (objsubss,Ej,Cj,Oj,flag)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - subss is a subsystem of a large-scale system
% - Ej is a cell matrix {Hej1,Kej1;Hej2,Kej2;...} such that parent errors 
%   ej are in the sets Ej1 = { ej1 | Hej1*ej1<=Kej1 },
%   Ej2 = { ej2 | Hej2*ej2<=Kej2 }, ... , where ej1 and ej2 are errors of two
%   parent subsystems of local subsystem subss. This parameter is needed
%   for bounded estimation (see also flag.boundedEstimation)
% - Cj is a cell array {Cj1;Cj2;...} where Cj1, Cj2 and ... are the output
%   linear transformation yj1=Cj1xj1, yj2=Cj2xj2. This parameter is needed
%   to design local estimators that depend on parents' subsystems outputs
%   (see also flag.distributedParents)
% - Oj is a cell matrix {Hoj1,Koj1;Hoj2,Koj2;...} such that parents 
%   disturbances dj are in the sets Oj1 = { dj1 | Hoj1*dj1<=Koj1 },
%   Oj2 = { dj2 | Hoj2*dj2<=Koj2 }, ..., where dj1 and dj2 are disturbances 
%   of two parent subsystems of local subsystem subss. This parameter is needed
%   for bounded estimation with exogenous inputs bounded (see also flag)
% - flag is an optional parameter
%    .distributedParents specify parents' subsystems indices such
%       that local estimator receives outputs from these parent subsystems
%       (default is empty)
%    .Qi and .Ri are weights for the optimization problem to compute gain
%       matrix Lii (default Qi and Ri are identity matrices)
%    .boundedEstimation logical value: 0 the estimation is not bounded, 1
%       it is bounded (default 0)
%    .ExoBounded specify indices of exogenous inputs that must be
%       considered as disturbances (default is empty)
%    .norm specify norm to compute matrices Lij. Possible values inf, 1, 2,
%       'F' (default inf)
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% est is a Plug-and-Play estimator for the given subsystem and given
% parameters
%
% ---------------------------------------------------------------------------
% PROPERTIES
% ---------------------------------------------------------------------------
% Most important properties.
% A, B, C : dynamic of the estimator
% couplingA : indeces of parent subsystems
% Nij : parent subsystems to reduce coupling terms
% Li  : gain matrix to observe local subsystem
% Lij : gain matrix in order to reduce coupling terms 
%
% ---------------------------------------------------------------------------
% METHODS
% ---------------------------------------------------------------------------
% CHECKERRORESTIMATION: check if the initial error is allowed
% COMPUTEESTIMATION: compute state estimation
% EIG: compute eigenvalues of estimator
% MRPIS: compute the mRPI set S
% PNPESTIMATOR2SS: compute the ss object of the state estimator
%
% [1] S. Riverso, M. Farina, R. Scattolini, and G. Ferrari-Trecate, 
%     "Plug-and-play distributed state estimation for linear systems," 
%     in Proceedings of the 52nd IEEE Conference on Decision and Control, 
%     2013, pp. 4889-4894.
    
% Copyright is with the following author(s):
%
% (C) 2013 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------
    
    properties (SetAccess = private)
        A              ;    % local dynamics of estimator
        B              ;    % local dynamics of estimator
        C              ;    % output matrix of estimator
        couplingA      ;    % indeces of parent subsystems
        Nij            ;    % parent subsystems to reduce coupling terms
        Exo            ;    % indices of exogenous inputs acting on the subsystem
        ExoBounded     ;    % indeces of exogenous inputs acting on the subsystem as disturbances (subset of Exo)
        ICen           ;    % indeces of centralized signals acting on subsystem
        Li             ;    % gain matrix to observe local subsystem
        Lij            ;    % gain matrix in order to reduce coupling terms
        He             ;    % bounded estimation
        Ke             ;    % bounded estimation
        Ts             ;    % sampling time
    end
    
    properties (SetAccess = private , Hidden = true)
        nj             ;    % number of states of parent subsystems
        mj             ;    % number of inputs of parent subsystems
        pj             ;    % number of outputs of parent subsystems
        Sgen           ;    % generators for computation of S
        S              ;    % set S as mRPI set inside E
        funcins        ;    % function to check if et0 guarantees bounded estimation
        estInputs      ;    % inputs of function COMPUTEESTIMATION
    end
    
    
    
    methods
        
        function est = pnpEstimator(objsubss,Ej,Cj,Oj,flag)
            
            global pnpmpcToolboxGlobalOptions;
            if ~isstruct(pnpmpcToolboxGlobalOptions),
                pnpmpc_toolbox_err;
            end
            
            if nargin==0
                return;
            end
            
            yalmip('clear')
            
            if nargin>5 || nargin<1
                error('PNPESTIMATOR: parameters in input. see help PNPESTIMATOR');
            end
            
            if ~isa (objsubss,'subss')
                error('PNPESTIMATOR: first argument must be a subss object')
            end
            
            if isempty(objsubss.Ts)
                error('PNPESTIMATOR: subss object must be a discrete-time system')
            end
            
            est.Ts = objsubss.Ts;
            
            Ci = full(objsubss.getC);
            if all(all(Ci==0))
                error('PNPESTIMATOR: subss is an unobservable system')
            end
            
            if ~isempty(objsubss.Cij)
                error('PNPESTIMATOR: no output coupling')
            end
            
            if any(any(objsubss.getD~=0))
                error('PNPESTIMATOR: matrix D of subss object must be 0')
            end
            
            if any(any(objsubss.getDcen~=0))
                error('PNPESTIMATOR: matrix Dcen of subss object must be 0')
            end
            
            Ai              = full(objsubss.getA);
            n               = size(Ai,2);
            p               = size(Ci,1);
            est.couplingA   = objsubss.couplingA;
            est.Exo         = objsubss.Exo;
            est.ICen        = objsubss.ICen;
            Ni              = length(est.couplingA);
            njTtrue         = sum(objsubss.nj);
            est.nj          = objsubss.nj;
            
            HE              = full(objsubss.He);
            KE              = full(objsubss.Ke);
            
            if ~isempty(HE)
                Hred            = cddmex('reduce_h',struct('A',HE,'B',KE));
                HE              = Hred.A;
                KE              = Hred.B;
            end
            
            
            if nargin<=4 || isempty(flag)
                flag.distributedParents = [];
                flag.Qi                   = eye(n);
                flag.Ri                   = eye(p);
                flag.boundedEstimation    = 0 ;
                flag.ExoBounded           = [];
                flag.norm                 = inf;
            end
            
            
            if ~isfield(flag,'Qi') || isempty(flag.Qi)
                flag.Qi                   = eye(n);
            else
                flag.Qi = full(flag.Qi);
                if any(size(flag.Qi)~=[n n])
                    error('PNPESTIMATOR: input flag.Qi')
                end
                flag.Qi = (flag.Qi+flag.Qi')/2;
                if any(eig(flag.Qi)<0)
                    error('PNPESTIMATOR: input flag.Qi')
                end
            end
            
            if ~isfield(flag,'Ri') || isempty(flag.Ri)
                flag.Ri                   = eye(p);
            else
                flag.Ri = full(flag.Ri);
                if any(size(flag.Ri)~=[p p])
                    error('PNPESTIMATOR: input flag.Ri')
                end
                flag.Ri = (flag.Ri+flag.Ri')/2;
                if any(eig(flag.Ri)<=0)
                    error('PNPESTIMATOR: input flag.Ri')
                end
            end
            
            if ~isfield(flag,'boundedEstimation')
                flag.boundedEstimation = 0;
            end
            
            if ~isfield(flag,'norm')
                flag.norm = inf;
            end
            
            if flag.norm~=inf && flag.norm~=1 && flag.norm~=2 && flag.norm~='F'
                flag.norm = inf;
                warning('PNPESTIMATOR: flag.norm is wrong or empty. Default value will be used: flag.norm=inf' )
            end
            
            if ~isfield(flag,'ExoBounded')
                est.ExoBounded = [];
                ExoBoundedIndex = [];
            else
                [ est.ExoBounded , ExoBoundedIndex ] = intersect(objsubss.Exo,unique(flag.ExoBounded));
                est.ExoBounded = reshape(est.ExoBounded,1,[]);
            end
            
            [ est.Exo , diffExo ] = setdiff(objsubss.Exo,est.ExoBounded);
            est.Exo = reshape(est.Exo,1,[]);
            
            
            if ( isempty(objsubss.couplingA) || all(all(abs(objsubss.Aij)<=1e-8)) ) && flag.boundedEstimation==0
                %% no coupling terms and no bounded estimation
                est.Li  = -(dlqr(Ai',Ci',flag.Qi,flag.Ri))';
                
                
            else
                %% yes coupling terms or yes bounded estimation
                 
                
                if ~isempty(est.ExoBounded) && flag.boundedEstimation==1
                    %% check if the disturbances are unbounded
                    if isempty(objsubss.Hd)
                        error('PNPESTIMATOR: exogenous inputs are not bounded. It is impossible design robust PNPESTIMATOR estimator')
                    else
                        tmpP = Polyhedron(full(objsubss.Hd),full(objsubss.Kd)).projection(ExoBoundedIndex).computeHRep;
                        HDD  = tmpP.A;
                        KDD  = tmpP.b;
                    end
                    [ j ] = est.checkUnbounded( objsubss.M(:,ExoBoundedIndex) , HDD , KDD );
                    if ~isempty(j)
                        error(['PNPESTIMATOR: exogenous input ' num2str(est.ExoBounded(ExoBoundedIndex(j))) ' is unbounded']);
                    end
                    Wi = Polyhedron2zonotope(Polyhedron(full(HDD),full(KDD)));
                    if isempty(Wi)
                        error('PNPESTIMATOR: exogenous inputs for robust estimation are not zonotope');
                    end
                    Deltai = Wi.G;
                    MiDeltai = objsubss.M(:,ExoBoundedIndex)*Deltai;
                    
                    if ~isempty(objsubss.N)
                        [ j ] = est.checkUnbounded( objsubss.N(:,ExoBoundedIndex) , HDD , KDD );
                        if ~isempty(j)
                            error(['PNPESTIMATOR: exogenous input ' num2str(est.ExoBounded(ExoBoundedIndex(j))) ' is unbounded']);
                        end
                        WiN = Polyhedron2zonotope(Polyhedron(full(HDD),full(KDD)));
                        if isempty(WiN)
                            error('PNPESTIMATOR: exogenous inputs for robust estimation are not zonotope');
                        end
                        DeltaiN = WiN.G;
                        NiDeltai = objsubss.N(:,ExoBoundedIndex)*DeltaiN;
                    else
                        NiDeltai = [];
                    end
                else
                    MiDeltai = [];
                    NiDeltai = [];
                end

                
                
                
                CJ = cell(length(est.Nij),1);
                
                if ~isfield(flag,'distributedParents')
                    flag.distributedParents = [];
                else
                    est.Nij = intersect(unique(flag.distributedParents),est.couplingA);
                    if length(objsubss.couplingA)<length(flag.distributedParents)
                        error('PNPESTIMATOR: the number of elements in distributedParents should be <= %d',length(objsubss.couplingA))
                    end
                end
                
                
                
                
                
                if flag.boundedEstimation==1
                    % use constraints Ei Ej
                    
                    HE = HE./repmat(KE,1,size(HE,2));
                    
                    if ~isempty(est.couplingA)
                        if nargin==1 || isempty(Ej) || ~iscell(Ej) || size(Ej,1)~=Ni || size(Ej,2)~=2
                            error('PNPESTIMATOR: wrong sets Ej from parent subsystems. see help pnpEstimator')
                        end
                    end
                    njT = 0;
                    njR = 0;
                    for j=1:Ni
                        if size(Ej{j,1},1)~=size(Ej{j,2},1)
                            error('PNPESTIMATOR: wrong sets Ej from parent subsystem %d. see help pnpEstimator',j)
                        end
                        njT     = njT + size(Ej{j,1},2);
                        njR     = njR + size(Ej{j,1},1);
                        Ej{j,1} = full(Ej{j,1}./repmat(Ej{j,2},1,size(Ej{j,1},2)));
                        Ej{j,2} = full(ones(size(Ej{j,1},1),1));
                    end
                    if njT~=njTtrue
                        error('PNPESTIMATOR: wrong sets Ej from parent subsystems. see help pnpEstimator')
                    end
                    HJ   = [];
                    KJ   = [];
                    GEij = [];
                    Ejz  = cell(Ni,1);
                    for j=1:Ni
                        Ejtmp = Polyhedron2zonotope(Polyhedron(Ej{j,1},Ej{j,2}));
                        if isempty(Ejtmp)
                            error('PNPESTIMATOR: error estimation constraint of parent subsystem %d is not a zonotope',j);
                        end
                        pEjtmp = Ejtmp.p;
                        Gijtmp = Ejtmp.G;                        
                        if ~all(abs(pEjtmp)<=1e-8)
                            error('PNPESTIMATOR: error estimation constraints of parent subsystem %d is a zonotope not centered in 0',j);
                        end
                        Ejz{j} = zonotope(pEjtmp,Gijtmp);
                        GEij   = blkdiag(GEij,Gijtmp);
                        HJ     = blkdiag( HJ , Ej{j,1} );
                        KJ     = [ KJ ; Ej{j,2} ]; %#ok<AGROW>
                    end
                    
                    pEj = cell(Ni,1);
                    for j = 1:Ni
                        pEj{j,1} = pinv( Ej{j,1} );
                    end   
                    
                else
                    % no use constraints Ei Ej
                    
                    HE   = eye(n);
                    GEij = eye(sum(objsubss.nj));
                    
                    pEj = cell(Ni,1);
                    for j = 1:Ni
                        pEj{j,1} = eye(objsubss.nj(j));
                    end
                    
                    HJ = [ eye(njTtrue) ; -eye(njTtrue) ];
                    KJ = ones(2*njTtrue,1);
                    
                end
                
                if ~isempty(est.Nij) && ( nargin<=2 || ~iscell(Cj) || size(Cj,1)~=length(est.Nij) || size(Cj,2)~=1 )
                    error('PNPESTIMATOR: wrong matrices Cj from parent subsystems. see help pnpEstimator')
                end
                
                est.pj = zeros(1,length(est.Nij));
                for jj=1:length(est.Nij)
                    j = find(est.couplingA==est.Nij(jj));
                    if size(Cj{jj,1},2)~=objsubss.nj(j)
                        error('PNPESTIMATOR: wrong matrix Cj from parent subsystem %d. see help pnpEstimator',j)
                    end
                    CJ{jj,1} = full(Cj{jj,1});
                    est.pj(jj) = size(CJ{jj,1},1);
                end
                
                
                if flag.boundedEstimation==1 && ~isempty(est.Nij) && ~isempty(Oj)
                    
                    pjTtrueO = sum(est.pj);
                    
                    if ~isempty(est.couplingA)
                        if nargin==1 || isempty(Oj) || ~iscell(Oj) || size(Oj,1)~=Ni || size(Oj,2)~=2
                            error('PNPESTIMATOR: wrong sets Oj from parent subsystems. see help pnpEstimator')
                        end
                    end
                    pjTO = 0;
                    pjRO = 0;
                    for j=1:Ni
                        if isempty(Oj{j,1})
                            pjTO = pjTO + est.pj(j);
                            continue;
                        end
                        if size(Oj{j,1},1)~=size(Oj{j,2},1)
                            error('PNPESTIMATOR: wrong sets Oj from parent subsystem %d. see help pnpEstimator',j)
                        end
                        pjTO    = pjTO + size(Oj{j,1},2);
                        pjRO    = pjRO + size(Oj{j,1},1);
                        Oj{j,1} = full(Oj{j,1}./repmat(Oj{j,2},1,size(Oj{j,1},2)));
                        Oj{j,2} = full(ones(size(Oj{j,1},1),1));
                    end
                    if pjTO~=pjTtrueO
                        error('PNPESTIMATOR: wrong sets Oj from parent subsystems. see help pnpEstimator')
                    end
                    
                    GOij  = [];
                    Ojz   = cell(Ni,1);
                    for j=1:Ni
                        if isempty(Oj{j,1})
                            GOij   = blkdiag(GOij,zeros(est.pj(j),0));
                            continue;
                        end
                        Ojtmp = Polyhedron2zonotope(Polyhedron(Oj{j,1},Oj{j,2}));
                        if isempty(Ojtmp)
                            error('PNPESTIMATOR: disturbance constraint of parent subsystem %d is not a zonotope',j);
                        end
                        pOjtmp = Ojtmp.p;
                        Gijtmp = Ojtmp.G;
                        if ~all(abs(pOjtmp)<=1e-8)
                            error('PNPESTIMATOR: disturbance constraints of parent subsystem %d is a zonotope not centered in 0',j);
                        end
                        Ojz{j} = zonotope(pOjtmp,Gijtmp);
                        GOij   = blkdiag(GOij,Gijtmp);
                    end     
                else
                    GOij = [];
                end
                
                                
                
                Fij = full(objsubss.Aij);
                
                if ~isempty(est.Nij)
                    for jj=1:length(est.Nij)
                        j = find(est.couplingA==est.Nij(jj));
                        Lijopt   = sdpvar(n,size(CJ{jj,1},1),'full');
                        Fijtmp   = Fij(:,sum(objsubss.nj(1:j-1))+1:sum(objsubss.nj(1:j))) + Lijopt*CJ{jj,1};
                        iFijtmpj = HE*Fijtmp*pEj{j,1};
                        if flag.norm==inf
                            J = norm(iFijtmpj,inf);
                        elseif flag.norm==1
                            J = norm(iFijtmpj,1);
                        elseif flag.norm==2
                            J = norm(iFijtmpj,2);
                        elseif flag.norm=='F'
                            J = sum(sum( (iFijtmpj).^2 ));
                        end
                        info = solvesdp([],J,sdpsettings('verbose',0));
                        if info.problem~=0
                            error(['PNPESTIMATOR: Lij for parent subsystem %d was not created. Solver error: ' info.info],est.Nij(jj))
                        end
                        est.Lij = [ est.Lij double(Lijopt) ];
                        Fijtmp(abs(double(Fijtmp))<=eps) = 0;
                        Fij(:,sum(objsubss.nj(1:j-1))+1:sum(objsubss.nj(1:j))) = double(Fijtmp);
                    end
                    Fij(isnan(Fij)) = 0;
                    est.Lij(isnan(est.Lij)) = 0;
                end
                
                if ~isempty(est.Lij) && isempty(GOij)
                    GOij = zeros(size(est.Lij,2),0);
                end
                
                [ j ] = est.checkUnbounded( Fij , HJ , KJ );
                if ~isempty(j)
                    error(['PNPESTIMATOR: state_j ' num2str(j) ' is unbounded']);
                end
                
                if all(all(abs(Fij)<=1e-8))
                    % coupling can be completely removed
                    est.Li = -(dlqr(Ai',Ci',flag.Qi,flag.Ri))';
                end
                
                if isempty(est.Li) || max(est.Jzono(est.Li(:),Ai,Ci,Fij,objsubss.nj,HE,pEj,[],CJ,GEij,MiDeltai,NiDeltai,est.Lij*GOij,est.couplingA))>=1
                    mulQR = 10.^(-4:4);
                    tmpJ  = inf;
                    Ltmpi = zeros(n,p);
                    for i=mulQR
                        for j=mulQR
                            tmpLt = -(dlqr(Ai',Ci',i*flag.Qi,j*flag.Ri))';
                            tJ    = max(est.Jzono(tmpLt(:),Ai,Ci,Fij,objsubss.nj,HE,pEj,[],CJ,GEij,MiDeltai,NiDeltai,est.Lij*GOij,est.couplingA));
                            if tmpJ>tJ
                                Ltmpi = tmpLt;
                                tmpJ  = tJ;
                            end
                        end
                    end
                    
                    tau = 1.1:0.1:10;
                    xt0 = Ltmpi(:);
                    for i=tau
                        tJ    = max(est.Jzono(i*Ltmpi(:),Ai,Ci,Fij,objsubss.nj,HE,pEj,[],CJ,GEij,MiDeltai,NiDeltai,est.Lij*GOij,est.couplingA));
                        if tmpJ>tJ
                            xt0   = i*Ltmpi(:);
                        else
                            break;
                        end
                    end
                    
                    [ xtopt , J ] = fmincon(@(xt)max(est.Jzono(xt,Ai,Ci,Fij,objsubss.nj,HE,pEj,[],CJ,GEij,MiDeltai,NiDeltai,est.Lij*GOij,est.couplingA)),xt0,[],[],[],[],-1e3*ones(length(xt0),1),1e3*ones(length(xt0),1),@(xt)est.Jzono(xt,Ai,Ci,Fij,objsubss.nj,HE,pEj,[],CJ,GEij,MiDeltai,NiDeltai,est.Lij*GOij,est.couplingA),optimset('Display','off','Algorithm','active-set','TolFun',1e-3));
                    
                    if J>=0
                        if isempty(est.Nij)
                            error('PNPESTIMATOR: it is impossible to design a Plug-and-Play controller. Nonlinear optimization failed.')
                        else
                            xt0   = [ xtopt(1:n*p) ; est.Lij(:) ];
                            [ xtopt , J ] = fmincon(@(xt)max(est.Jzono(xt,Ai,Ci,objsubss.Aij,objsubss.nj,HE,pEj,est.Nij,CJ,GEij,MiDeltai,NiDeltai,GOij,est.couplingA)),xt0,[],[],[],[],-1e3*ones(length(xt0),1),1e3*ones(length(xt0),1),@(xt)est.Jzono(xt,Ai,Ci,objsubss.Aij,objsubss.nj,HE,pEj,est.Nij,CJ,GEij,MiDeltai,NiDeltai,GOij,est.couplingA),optimset('Display','off','Algorithm','active-set','TolFun',1e-3));
                            if J>=0
                                error('PNPESTIMATOR: it is impossible to design a Plug-and-Play controller. Nonlinear optimization failed.')
                            end
                            est.Lij = reshape(xtopt(p*n+1:end,1),n,[]);
                            totnj = 0;
                            for jj=1:length(est.Nij)
                                j  = find(est.couplingA==est.Nij(jj));
                                Fij(:,sum(objsubss.nj(1:j-1))+1:sum(objsubss.nj(1:j))) = objsubss.Aij(:,sum(objsubss.nj(1:j-1))+1:sum(objsubss.nj(1:j))) + est.Lij(:,totnj+(1:est.pj(jj)))*CJ{jj,1} ;
                                totnj = totnj + est.pj(jj);
                            end
                        end
                    end
                    est.Li = reshape(xtopt(1:n*p),n,p);
                end
                
            end
            
            %% LTI pnp Estimator
            est.Li        = sparse(est.Li);
            est.Lij       = sparse(est.Lij);
            est.A         = sparse( objsubss.getA+est.Li*objsubss.getC );
            est.B         = sparse([ -est.Li Fij -est.Lij objsubss.getB objsubss.Bij objsubss.Bcen objsubss.M(:,diffExo) ]);
            est.estInputs = [ size(est.Li,2) size(Fij,2) size(est.Lij,2) size(objsubss.getB,2) size(objsubss.Bij,2) size(objsubss.Bcen,2) length(diffExo) ];
            est.C         = objsubss.getC;
            est.mj        = objsubss.mj;
            
            clear Ejtmp Ojz Wi tmpP Ejz WiN Ojtmp
            
            if flag.boundedEstimation==0
                est.He      = sparse([eye(n);-eye(n)]);
                est.Ke      = sparse(inf(2*n,1));
                est.funcins = @(et0)(-1);
            else
                [ est.He, est.Ke ] = objsubss.doubleErrorEstimationConstraint;
                k = floor(log10(1e-6)/log10(max(abs(eig(full(est.A))))));
                
                %% Compute set initial error
                est.Sgen = sparse([ Fij*GEij MiDeltai ]);
                if ~isempty(NiDeltai)
                    est.Sgen = sparse([ est.Sgen est.Li*NiDeltai ]);
                end
                if ~isempty(GOij)
                    est.Sgen = sparse([ est.Sgen est.Lij*GOij ]);
                end
                
                if ~isempty(est.Sgen)
                    EE    = full(est.Sgen);
                    Abar  = zeros(n*k,n);
                    qe    = size(est.He,1);
                    r     = ones(k*qe,1);
                    Hebar = zeros(qe*k,n*k);
                    Amul  = eye(n);
                    for i = 1:k
                        Hebar((i-1)*qe+1:i*qe,(i-1)*n+1:i*n) = HE;
                        for j = i:k
                            r((j-1)*qe+1:j*qe) = r((j-1)*qe+1:j*qe) - norm(HE*Amul*EE,inf);
                        end
                        Amul = Amul*est.A;
                        Abar((i-1)*n+1:i*n,:) = Amul;
                    end
                    Abar  = sparse(Abar);
                    Hebar = sparse(Hebar);
                    r     = repmat(r,2,1);
                    est.funcins = @(et0)([Hebar*Abar;-Hebar*Abar]*et0-r);
                else
                    Abar     = zeros(n*k,n);
                    Amul     = est.A;
                    qe       = size(est.He,1);
                    Hebar    = zeros(qe*k,n*k);
                    for i = 1:k
                        Abar((i-1)*n+1:i*n,:) = Amul;
                        Hebar((i-1)*qe+1:i*qe,(i-1)*n+1:i*n) = HE;
                        Amul = Amul*est.A;
                    end
                    Abar     = sparse(Abar);
                    Hebar    = sparse(Hebar);
                    est.funcins = @(et0)([Hebar*Abar;-Hebar*Abar]*et0-ones(2*qe*k,1));
                end
                
            end
            
            yalmip('clear')
        end
        
        
    end
    
    
    
    
    
    
    
    
    
    
    
    
    methods(Static)
        
        
        function [ J , ceq ] = Jzono( xt , Ati , Cti , Fij , nj , Fcali , pEj , Nij , CJ , GEij , MiDeltai , NiDeltai , GOij , couplingA )
            n    = size(Ati,2);
            p    = size(Cti,1);
            Lti  = reshape(xt(1:p*n,1),n,p);
            Ltij = reshape(xt(p*n+1:end,1),n,[]);
            
            totnj = 0;
            for jj=1:length(Nij)
                j  = find(couplingA==Nij(jj));
                pjtmp = size(CJ{jj,1},1);
                Fij(:,sum(nj(1:j-1))+1:sum(nj(1:j))) = Fij(:,sum(nj(1:j-1))+1:sum(nj(1:j))) + Ltij(:,totnj+(1:pjtmp))*CJ{jj,1} ;
                totnj = totnj + pjtmp;
            end
            
            if isempty(NiDeltai)            
                GenTotal = [ Fij*GEij MiDeltai ];
            else
                GenTotal = [ Fij*GEij MiDeltai Lti*NiDeltai ];
            end
                        
            if isempty(Ltij)
                GenTotal = [ GenTotal GOij ];                
            else
                GenTotal = [ GenTotal Ltij*GOij ];
            end
            
            
            kf = floor(log10(1e-6)/log10(max(abs(eig( Ati+Lti*Cti )))));
            
            if kf<0
                total = 1e6;
                totalDist = 1e6;
            else
                total  = 0;
                totalDist = 0;
                Amul   = eye(n);
                for kk=0:kf
                    totalp1 = total;
                    totalDistp1 = totalDist;
                    for j=1:length(nj)
                        total  = total + norm(Fcali*Amul*Fij(:,sum(nj(1:j-1))+1:sum(nj(1:j)))*pEj{j,1},inf);
                    end
                    totalDist = totalDist + norm(Fcali*Amul*GenTotal,inf);
                    if ((total-totalp1)/total)<=1e-6 && ((totalDist-totalDistp1)/totalDist)<=1e-6
                        break;
                    end
                    Amul = (Ati+Lti*Cti)*Amul;
                end
            end
            
            J = [total;totalDist;abs(eig(Ati+Lti*Cti))]-1;
            
            ceq = [];
        end
        
        
        
        function [ j ] = checkUnbounded( matrix , Hmatrix , Kmatrix )
            %% check if the variables are unbounded
            j = [];
            big = 10^7;
            Hmatrix = [eye(size(matrix,2));-eye(size(matrix,2));Hmatrix];
            Kmatrix = [ones(2*size(matrix,2),1)*big;Kmatrix];
            if ~isempty(Hmatrix)
                for i = 1:size(Hmatrix,2)
                    objective    = zeros(1,size(Hmatrix,2));
                    objective(i) = 1;
                    INmin        = struct('obj',objective,'A',Hmatrix,'B',Kmatrix);
                    OUTmin       = cddmex('solve_lp',INmin);
                    objective(i) = -1;
                    INmax        = struct('obj',objective,'A',Hmatrix,'B',Kmatrix);
                    OUTmax       = cddmex('solve_lp',INmax);
                    if OUTmin.xopt(i)==-big || OUTmax.xopt(i)==big
                        if any(matrix(:,i)~=0)
                            j = i;
                            break
                        end
                    end
                end
            end
        end
        
        
        
        
    end
    
    
    
end