classdef pnpctrl
% PNPCTRL design a Plug-and-Play controller as proposed in [1] and [2]
% 
% ctrl = pnpctrl(subss,Xj,flag)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return an unconstrained state-feedback Plug-and-Play controller
% 
% ctrl = PNPCTRL (subss)
% ctrl = PNPCTRL (subss,Xj)
% ctrl = PNPCTRL (subss,Xj,flag)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - subss is a subsystem of a large-scale system
% - Xj is a cell matrix {Hxj1,Kxj1;Hxj2,Kxj2;...} such that parent states 
%   xj are in the sets Xj1 = { xj1 | Hxj1*xj1<=Kxj1 },
%   Xj2 = { xj2 | Hxj2*xj2<=Kxj2 }, ... , where xj1 and xj2 are two
%   parent subsystems of local subsystem subss. This parameter is needed
%   to scale state variables (see also flag.useConstraints)
% - flag is an optional parameter
%    .distributedParents specify parents' subsystems indices such
%       that local controller receives states from these parent subsystems
%       (default is empty) therefore compute gain matrices Kij
%    .Qi and .Ri are weights for the optimization problem to compute gain
%       matrix Kii (default Qi and Ri are identity matrices)
%    .useConstraints logical value: 0 the optimization problem does not take
%       advantages of the knowledge of scaling factors, 1 take advantages
%       (default 0)
%    .norm specify norm to compute matrices Kij. Possible values inf, 1, 2,
%       'F' (default inf)
%    .boundKij bound on the computation of gain matrices Kij
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% ctrl is an unconstrained Plug-and-Play controller for the given subsystem and 
% given parameters
%
% ---------------------------------------------------------------------------
% PROPERTIES
% ---------------------------------------------------------------------------
% Most important properties.
% A, B, M : dynamic of the subsystem
% couplingA : indeces of parent subsystems
% Aij : coupling terms
% Nij : parent subsystems to reduce coupling terms
% Ki  : gain matrix to observe local subsystem
% Kij : gain matrix in order to reduce coupling terms 
%
% ---------------------------------------------------------------------------
% METHODS
% ---------------------------------------------------------------------------
% UK : compute control action
%
% [1] S. Riverso and G. Ferrari-Trecate, "Plug-and-Play distributed model 
%     predictive control with coupling attenuation," Optimal Control 
%     Applications and Methods, p. Submitted, 2013.
% [2] S. Riverso, "Distributed and plug-and-play control for constrained 
%     systems," PhD dissertation, Universita' degli studi di Pavia, 2014.
%

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    properties ( SetAccess = private , Hidden = false )
        A              ;    % local dynamics
        B              ;    % local dynamics
        M              ;    % local dynamics
        Aij            ;    % coupling dynamics
        couplingA      ;    % indeces of parent subsystems
        Nij            ;    % parent subsystems to reduce coupling terms
        Ki             ;    % matrix to stabilize local subsystem
        Kij            ;    % matrix in order to reduce coupling terms
    end
    
    properties ( SetAccess = private , Hidden = true )
        nj             ;    % number of states of parent subsystems
    end
    
    methods
        function ctrl = pnpctrl(subss,Xj,flag)
            
            global pnpmpcToolboxGlobalOptions;
            if ~isstruct(pnpmpcToolboxGlobalOptions),
                pnpmpc_toolbox_err;
            end
            
            if nargin==0
                return;
            end
            
            yalmip('clear')
            
            if nargin>3 || nargin<1
                error('PNPCTRL: parameters in input. see help PNPCTRL');
            end
            
            if ~isa (subss,'subss')
                error('PNPCTRL: first argument must be a subss object')
            end
            
            if isempty(subss.Ts)
                error('PNPCTRL: subss object must be a discrete-time system')
            end
            
            if isempty(subss.B) || all(all(subss.B==0))
                error('PNPCTRL: subss is an autonomous system')
            end
            
            if ~isempty(subss.Bij)
                error('PNPCTRL: no input coupling')
            end
            
            if ~isempty(subss.C)
                warning('PNPCTRL class will design a state-feedback PNPCTRL controller')
            end
            
            if ~isempty(subss.Bcen)
                error('PNPCTRL: subss must not have centralized input')
            end
            
            ctrl.A           = full(subss.A);
            ctrl.B           = full(subss.B);
            ctrl.M           = full(subss.M);
            ctrl.Aij         = full(subss.Aij);
            ctrl.couplingA   = subss.couplingA;
            ctrl.nj          = subss.nj;
            
            n               = size(ctrl.A,2);
            m               = size(ctrl.B,2);
            Ni              = length(ctrl.couplingA);
            njTtrue         = sum(subss.nj);
            
            HX              = full(subss.Hx);
            KX              = full(subss.Kx);
            
            if ~isempty(HX)
                Hred            = cddmex('reduce_h',struct('A',HX,'B',KX));
                HX              = Hred.A;
                KX              = Hred.B;
            end
            
            if nargin<=2 || isempty(flag)
                flag.distributedParents = [];
                flag.Qi                   = [];
                flag.Ri                   = [];
                flag.useConstraints       = 0 ;
                flag.norm                 = inf;
                flag.boundKij             = inf;
            end
            
            if ~isfield(flag,'Qi') || isempty(flag.Qi)
                flag.Qi                   = eye(n);
            else
                flag.Qi = full(flag.Qi);
                if any(size(flag.Qi)~=[n n])
                    error('PNPCTRL: input flag.Qi')
                end
                flag.Qi = (flag.Qi+flag.Qi')/2;
                if any(eig(flag.Qi)<0)
                    error('PNPCTRL: input flag.Qi')
                end
            end
            
            if ~isfield(flag,'Ri') || isempty(flag.Ri)
                flag.Ri                   = eye(m);
            else
                flag.Ri = full(flag.Ri);
                if any(size(flag.Ri)~=[m m])
                    error('PNPCTRL: input flag.Ri')
                end
                flag.Ri = (flag.Ri+flag.Ri')/2;
                if any(eig(flag.Ri)<=0)
                    error('PNPCTRL: input flag.Ri')
                end
            end
            
            if ~isfield(flag,'norm') || ( flag.norm~=inf && flag.norm~=1 && flag.norm~=2 && flag.norm~='F' )
                flag.norm = inf;
                warning('PNPCTRL: flag.norm is wrong or empty. Default value will be used: flag.norm=inf' )
            end
            
            %% bound on norm Kij
            if ~isfield(flag,'boundKij') || flag.boundKij<0 || isempty(flag.boundKij)
                flag.boundKij = inf;
                warning('PNPCTRL: flag.boundKij is wrong or empty. Default value will be used: flag.boundKij=inf' )
            end
            
            Ktmpi = [];
            
            if isempty(subss.couplingA) || all(all(abs(ctrl.Aij)<=1e-8))
                %% no coupling terms
                Ktmpi = -(dlqr(ctrl.A,ctrl.B,flag.Qi,flag.Ri));
            else
                %% coupling terms
                if ~isfield(flag,'distributedParents')
                    flag.distributedParents = [];
                else
                    ctrl.Nij = intersect(unique(flag.distributedParents),ctrl.couplingA);
                    if length(subss.couplingA)<length(flag.distributedParents)
                        error('PNPCTRL: the number of elements in distributedParents should be <= %d',length(subss.couplingA))
                    end
                end
                
                if ~isfield(flag,'useConstraints')
                    flag.useConstraints = 0;
                end
                
                if flag.useConstraints==1
                    % use constraints information
                    
                    HX = HX./repmat(KX,1,size(HX,2));
                    
                    if nargin==1 || isempty(Xj)
                        error('PNPCTRL: wrong sets Xj from parent subsystems. see help pnpctrl')
                    end
                    
                    if ~iscell(Xj) || size(Xj,1)~=Ni || size(Xj,2)~=2
                        error('PNPCTRL: wrong sets Xj from parent subsystems. see help pnpctrl')
                    end
                    njT = 0;
                    njR = 0;
                    for j=1:Ni
                        if size(Xj{j,1},1)~=size(Xj{j,2},1)
                            error('PNPCTRL: wrong sets Xj from parent subsystem %d. see help pnpctrl',j)
                        end
                        njT     = njT + size(Xj{j,1},2);
                        njR     = njR + size(Xj{j,1},1);
                        Xj{j,1} = full(Xj{j,1}./repmat(Xj{j,2},1,size(Xj{j,1},2)));
                        Xj{j,2} = full(ones(size(Xj{j,1},1),1));
                    end
                    if njT~=njTtrue
                        error('PNPCTRL: wrong sets Xj from parent subsystems. see help pnpctrl')
                    end
                    HJ = [];
                    KJ = [];
                    for j=1:Ni
                        HJ = blkdiag( HJ , Xj{j,1} );
                        KJ = [ KJ ; Xj{j,2} ]; %#ok<AGROW>
                    end
                    
                    pXj = cell(Ni,1);
                    for j = 1:Ni
                        pXj{j,1} = pinv( Xj{j,1} );
                    end
                    
                else
                    % no constraints information
                    
                    HX = eye(n);
                    
                    pXj = cell(Ni,1);
                    for j = 1:Ni
                        pXj{j,1} = eye(ctrl.nj(j));
                    end
                    
                    HJ = [ eye(njTtrue) ; -eye(njTtrue) ];
                    KJ = ones(2*njTtrue,1);
                    
                end
                
                
                Fij = ctrl.Aij;
                
                if ~isempty(ctrl.Nij)
                    Nijtmp = [];
                    for jj=1:length(ctrl.Nij)
                        j = find(ctrl.couplingA==ctrl.Nij(jj));
                        Kijopt   = sdpvar(m,subss.nj(j),'full');
                        Fijtmp   = Fij(:,sum(subss.nj(1:j-1))+1:sum(subss.nj(1:j))) + ctrl.B*Kijopt;
                        iFijtmpj = HX*Fijtmp*pXj{j,1};
                        if flag.norm==inf
                            Jij = norm(iFijtmpj,inf);
                            CONS = norm(Kijopt*pXj{j,1},inf)<=flag.boundKij;
                        elseif flag.norm==1
                            Jij = norm(iFijtmpj,1);
                            CONS = norm(Kijopt*pXj{j,1},1)<=flag.boundKij;
                        elseif flag.norm==2
                            Jij = norm(iFijtmpj,2);
                            CONS = norm(Kijopt*pXj{j,1},2)<=flag.boundKij;
                        elseif flag.norm=='F'
                            Jij = sum(sum( (iFijtmpj).^2 ));
                            CONS = sum(sum( Kijopt*pXj{j,1} ))<=flag.boundKij;
                        end
                        info     = solvesdp(CONS,Jij,sdpsettings('verbose',0));
                        if info.problem~=0
                            error(['PNPCTRL: Kij for parent subsystem %d was not created. Solver error: ' info.info],ctrl.Nij(jj))
                        end
                        Ktmp = double(Kijopt);
                        if all(all(Ktmp==0))
                            Nijtmp = [ Nijtmp jj];
                            continue;
                        end
                        ctrl.Kij = [ ctrl.Kij Ktmp ];
                        Fijtmp(abs(double(Fijtmp))<=eps) = 0;
                        Fij(:,sum(subss.nj(1:j-1))+1:sum(subss.nj(1:j))) = double(Fijtmp);
                    end
                    ctrl.Nij(Nijtmp) = [];
                    Fij(isnan(Fij)) = 0;
                    ctrl.Kij(isnan(ctrl.Kij)) = 0;                    
                end
                
                [ j ] = ctrl.checkUnbounded( Fij , HJ , KJ );
                if ~isempty(j)
                    warning(['PNPCTRL: state_j ' num2str(j) ' is unbounded']);
                    error('PNPCTRL: constraints information is not useful. Try with flag.useConstraints=0 ');
                end
                
                
                
                if all(all(abs(Fij)<=1e-8))
                    % coupling can be completely removed
                    Ktmpi = -(dlqr(ctrl.A,ctrl.B,flag.Qi,flag.Ri));
                end
                
                if isempty(Ktmpi) || ctrl.Jzono(Ktmpi(:),ctrl.A,ctrl.B,Fij,subss.nj,HX,pXj,[],ctrl.couplingA)>=1
                    mulQR = 10.^(-4:4);
                    tmpJ  = inf;
                    Ktmpi = zeros(m,n);
                    for i=mulQR
                        for j=mulQR
                            tmpKt = -(dlqr(ctrl.A,ctrl.B,i*flag.Qi,j*flag.Ri));
                            tJ    = ctrl.Jzono(tmpKt(:),ctrl.A,ctrl.B,Fij,subss.nj,HX,pXj,[],ctrl.couplingA);
                            if tmpJ>tJ
                                Ktmpi = tmpKt;
                                tmpJ  = tJ;
                            end
                        end
                    end
                    
                    tau = 1.1:0.1:10;
                    xt0 = Ktmpi(:);
                    for i=tau
                        tJ    = ctrl.Jzono(i*Ktmpi(:),ctrl.A,ctrl.B,Fij,subss.nj,HX,pXj,[],ctrl.couplingA);
                        if tmpJ>tJ
                            xt0   = i*Ktmpi(:);
                        else
                            break;
                        end
                    end
                    
                    [ xtopt , J ] = fmincon(@(xt)ctrl.Jzono(xt,ctrl.A,ctrl.B,Fij,subss.nj,HX,pXj,[],ctrl.couplingA),xt0,[],[],[],[],-1e3*ones(length(xt0),1),1e3*ones(length(xt0),1),@(xt)ctrl.Jzono(xt,ctrl.A,ctrl.B,Fij,subss.nj,HX,pXj,[],ctrl.couplingA),optimset('Display','off','Algorithm','active-set','TolFun',1e-3));
                    
                    
                    if J>=1
                        if isempty(ctrl.Nij)
                            error('PNPCTRL: it is impossible to design a Plug-and-Play controller. Nonlinear optimization failed. If you know variables scaling, i.e. state constraints, try flag.constraints=1')
                        else
                            xt0   = [ xtopt(1:n*m) ; ctrl.Kij(:) ];
                            [ xtopt , J ] = fmincon(@(xt)ctrl.Jzono(xt,ctrl.A,ctrl.B,ctrl.Aij,subss.nj,HX,pXj,ctrl.Nij,ctrl.couplingA),xt0,[],[],[],[],-1e3*ones(length(xt0),1),1e3*ones(length(xt0),1),@(xt)ctrl.Jzono(xt,ctrl.A,ctrl.B,ctrl.Aij,subss.nj,HX,pXj,ctrl.Nij,ctrl.couplingA),optimset('Display','off','Algorithm','active-set','TolFun',1e-3));
                            if J>=1
                                error('PNPCTRL: it is impossible to design a Plug-and-Play controller. Nonlinear optimization failed. If you know variables scaling, i.e. state constraints, try flag.constraints=1')
                            end
                            warning('PNPCTRL: Kij have been changed in order to guarantee PnP conditions and could not respect boundKij')
                            ctrl.Kij = reshape(xtopt(m*n+1:end,1),m,[]);
                            totnj = 0;
                            for jj=1:length(ctrl.Nij)
                                j  = find(ctrl.couplingA==ctrl.Nij(jj));
                                Fij(:,sum(subss.nj(1:j-1))+1:sum(subss.nj(1:j))) = subss.Aij(:,sum(subss.nj(1:j-1))+1:sum(subss.nj(1:j))) + ctrl.B*ctrl.Kij(:,totnj+(1:subss.nj(j))) ;
                                totnj = totnj + subss.nj(j);
                            end
                        end
                    end
                    Ktmpi  = reshape(xtopt(1:n*m),m,n);
                end
            end
            
            
            
            ctrl.A   = sparse(ctrl.A);
            ctrl.B   = sparse(ctrl.B);
            ctrl.M   = sparse(ctrl.M);
            ctrl.Aij = sparse(ctrl.Aij);
            ctrl.Kij = sparse(ctrl.Kij);
            ctrl.Ki  = sparse(Ktmpi);
            
            
            
        end
    end
    
    
    
    
    
    
    
    
    
    
    
    
    methods(Static)
        
        function [ J , ceq ] = Jzono( xt , Ati , Bti , Fij , nj , Fcali , pXj , Nij , couplingA )
            n   = size(Ati,2);
            m   = size(Bti,2);
            Kti = reshape(xt(1:m*n,1),m,n);
            Ktij = reshape(xt(m*n+1:end,1),m,[]);
            
            totnj = 0;
            for jj=1:length(Nij)
                j  = find(couplingA==Nij(jj));
                Fij(:,sum(nj(1:j-1))+1:sum(nj(1:j))) = Fij(:,sum(nj(1:j-1))+1:sum(nj(1:j))) + Bti*Ktij(:,totnj+(1:nj(j)));
                totnj = totnj + nj(j);
            end
            
            kf = floor(log10(1e-6)/log10(max(abs(eig( Ati+Bti*Kti )))));
            
            if kf<0
                total = 1e6;
            else
                total  = 0;
                Amul   = eye(n);
                for k=0:kf
                    totalp1 = total;
                    for j=1:length(nj)
                        total  = total + norm(Fcali*Amul*Fij(:,sum(nj(1:j-1))+1:sum(nj(1:j)))*pXj{j,1},inf);
                    end
                    if ((total-totalp1)/total)<=1e-6
                        break;
                    end
                    Amul = (Ati+Bti*Kti)*Amul;
                end
            end
            
            J = max([total;abs(eig(Ati+Bti*Kti))]);
            
            ceq = [];
        end
        
        
        function [ j ] = checkUnbounded( matrix , Hmatrix , Kmatrix )
            %% check if the variables are unbounded
            j = [];
            big = 10^7;
            Hmatrix = [eye(size(matrix,2));-eye(size(matrix,2));Hmatrix];
            Kmatrix = [ones(2*size(matrix,2),1)*big;Kmatrix];
            if ~isempty(Hmatrix)
                for i = 1:size(Hmatrix,2)
                    objective    = zeros(1,size(Hmatrix,2));
                    objective(i) = 1;
                    INmin        = struct('obj',objective,'A',Hmatrix,'B',Kmatrix);
                    OUTmin       = cddmex('solve_lp',INmin);
                    objective(i) = -1;
                    INmax        = struct('obj',objective,'A',Hmatrix,'B',Kmatrix);
                    OUTmax       = cddmex('solve_lp',INmax);
                    if OUTmin.xopt(i)==-big || OUTmax.xopt(i)==big
                        if any(matrix(:,i)~=0)
                            j = i;
                            break
                        end
                    end
                end
            end
        end
        
    end
    
    
end