function u = uk(objCtrl,x0,xj)
% UK compute the control action as proposed in [1] and [2]
%
% u = uk(objCtrl,x0,xj)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Compute the control action as proposed in [1] and [2], in a decentralized 
% or distributed fashion
%
% u = Kii*x0 + Kij*xj
%
% USAGE:
%
% u = objCtrl.UK(x0)
% u = objCtrl.UK(x0,xj)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% objCtrl - pnpctrl object
% x0 - current state of the subsystem associated with the pnpctrl object
% xj - current state of parent subsystems (needed if Kij is not empty)
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% u - values of the input variables
%
%
% [1] S. Riverso and G. Ferrari-Trecate, "Plug-and-Play distributed model 
%     predictive control with coupling attenuation," Optimal Control 
%     Applications and Methods, p. Submitted, 2013.
% [2] S. Riverso, "Distributed and plug-and-play control for constrained 
%     systems," PhD dissertation, Universita' degli studi di Pavia, 2014.
%

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if isempty(objCtrl.Ki)
        warning('UK: PnP controller is empty') %#ok<WNTAG>
        return
    end
    
    if nargin<2 || nargin>3
        error('UK: parameters in input. see help UK')
    end
    
    if size(x0,1)~=size(objCtrl.A,1) || size(x0,2)~=1
        error('UK: dimension of x0 is wrong')
    end
    
    u = objCtrl.Ki*x0;
    
    if isempty(objCtrl.Kij)
        if nargin==3
            error('UK: objCtrl is a decentralized PnP controller')
        else
            return
        end
    end
    
    if size(xj,1)~=size(objCtrl.Kij,2) || size(xj,2)~=1
        error('UK: dimension of xj is wrong')
    end
    
    u = u + objCtrl.Kij*xj;
    
end