classdef pnpmpc
% PNPMPC design a Plug-and-Play controller as proposed in [1], [2] and [3]
% 
% ctrl = pnpmpc (objsubss,N,Xj,Uj,flag,options)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return a Plug-and-Play controller without setting the terminal penalty and
% terminal region
% 
% ctrl = PNPMPC (objsubss,N)
% ctrl = PNPMPC (objsubss,N,Xj) 
% ctrl = PNPMPC (objsubss,N,Xj,Uj)
% ctrl = PNPMPC (objsubss,N,Xj,Uj,flag)
% ctrl = PNPMPC (objsubss,N,Xj,Uj,flag,options)  
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - objsubss is a subsystem of a large-scale system
% - N is the prediction horizon
% - Xj is a cell matrix {Hxj1,Kxj1;Hxj2,Kxj2;...} such that parent states 
%   xj are in the sets Xj1 = { xj1 | Hxj1*xj1<=Kxj1 },
%   Xj2 = { xj2 | Hxj2*xj2<=Kxj2 }, ... , where xj1 and xj2 are two
%   parent subsystems of local subsystem subss.
% - Uj is a cell matrix {Huj1,Kuj1;Huj2,Kuj2;...} such that parent inputs 
%   uj are in the sets Uj1 = { uj1 | Huj1*uj1<=Kuj1 },
%   Uj2 = { uj2 | Huj2*uj2<=Kuj2 }, ... , where uj1 and uj2 are two
%   parent subsystems of local subsystem subss. Cell array Uj is needed if
%   objsubss has input constraints and flag.distributedParents is not
%   empty.
% - flag is an optional parameter
%    .distributedParents specify parents' subsystems indices such
%       that local controller receives states from these parent subsystems
%       (default is empty) therefore compute gain matrices Kij
%    .LPdesign true design controllers as in [1], false as in [2]. Default
%       true. If false state constraints will be checked as zonotopes
%    .Qi and .Ri are weights for nonlinear optimization problem to compute 
%       gain matrix Kii (default Qi and Ri are identity matrices). These
%       matrix are needed if .LPdesign=false, hence design as in [2]
%    .k cell array {kmin,kmax} in order to try different k for the LP
%       design as in [1]
%    .norm specify norm to compute matrices Kij. Possible values inf, 1, 2,
%       'F' (default inf)
%    .boundKij bound on the computation of gain matrices Kij
%    .ExoBounded specify indices of exogenous inputs that must be
%       considered as disturbances (default is empty)
% - options from YALMIP, i.e. sdpsettings object (optional)
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% ctrl is a Plug-and-Play MPC regulator for the given subsystem
%
% ---------------------------------------------------------------------------
% PROPERTIES
% ---------------------------------------------------------------------------
% Most important properties.
% A, B, M : dynamic of the subsystem
% Aij and Bij : coupling terms
% couplingA and couplingB : indeces of parent subsystems
% N : prediction horizon
% Hxc, Kxc, Hv, Kv : constraints for \hat{x} and v
% Z : parameterizedRCI|epsilon_mRPI object
% 
%
% ---------------------------------------------------------------------------
% METHODS
% ---------------------------------------------------------------------------
% URH: compute the control action
% XFQPMAX: design an ellipsoidal terminal set and a quadratic cost function
% ZEROTERMINAL: design a quadratic cost function, without terminal penalty
%               and terminal set as the set-point 
%
% [1] S. Riverso, M. Farina, and G. Ferrari-Trecate, "Plug-and-Play Model 
%     Predictive Control based on robust control invariant sets,"
%     Automatica, Accepted, 2013.
% [2] S. Riverso, M. Farina, and G. Ferrari-Trecate, "Plug-and-Play 
%     Decentralized Model Predictive Control for Linear Systems," IEEE 
%     Trans. Automat. Contr., vol. 58, no. 10, pp. 2608-2614, 2013.
% [3] S. Riverso, "Distributed and plug-and-play control for constrained 
%     systems," Universita' degli studi di Pavia, 2014.
%
    
% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Stefano Riverso, Alberto Battocchio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, alberto.battocchio01(at)universitadipavia.it,
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------
    
    properties ( SetAccess = private , Hidden = false )
        N              ;    % prediction horizon
        A              ;    % local dynamics
        B              ;    % local dynamics
        Hxc            ;    % tightened state constraints
        Kxc            ;    % tightened state constraints
        Hv             ;    % tightened input constraints
        Kv             ;    % tightened input constraints
        M              ;    % local dynamics
        Hd             ;    % exogenous inputs constraints
        Kd             ;    % exogenous inputs constraints
        Z              ;    % parameterizedRCI or epsilon_mRPI for PnP control
        Aij            ;    % coupling dynamics
        Bij            ;    % input dynamics
        Xj             ;    % parents state constraints
        Uj             ;    % parents input constraints
        couplingA      ;    % indeces of dynamical parent subsystems
        couplingB      ;    % indeces of input coupled parent subsystems
        Nij            ;    % parents subsystems to reduce coupling terms
        Exo            ;    % indeces of exogenous inputs
        ExoBounded     ;    % indeces of exogenous inputs as disturbances
        Kij            ;    % matrix in order to reduce coupling terms
    end
    
    properties ( SetAccess = private , Hidden = true )
        stateCons      ;    % anonymous function for state constraints
        inputCons      ;    % anonymous function for input constraints
        initialCons    ;    % anonymous function for initial constraints for pnp
        dynState       ;    % anonymous function for dynamics
        TerminalSet    ;    % flag for defined terminal set
        CostFunction   ;    % flag for defined cost function
        J              ;    % anonymous function for cost function
        Xcf            ;    % anonymous function for terminal set
        Ki             ;    % matrix for tube-based control
        nj             ;    % state of parents subsystems
    end
    
    
    
    
    
    methods
        function ctrl = pnpmpc(objsubss,N,Xj,Uj,flag,options)
            
            global pnpmpcToolboxGlobalOptions;
            if ~isstruct(pnpmpcToolboxGlobalOptions),
                pnpmpc_toolbox_err;
            end
            
            if nargin==0
                return;
            end
            
            yalmip('clear')
            
            if nargin>6 || nargin<2
                error('PNPMPC: parameters in input. see help PNPMPC');
            end
            
            if nargin==2 || isempty(Xj)
                Xj = {};
            end
            
            if nargin<4 || isempty(Uj)
                Uj = {};
            end
            
            if ~isa (objsubss,'subss')
                error('PNPMPC: first argument must be a subss object')
            end
            
            if isempty(objsubss.Ts)
                error('PNPMPC: subss object must be a discrete-time system')
            end
            
            if isempty(objsubss.B) || all(all(objsubss.B==0))
                error('PNPMPC: subss is an autonomous system')
            end
            
            if ~isempty(objsubss.C)
                warning('PNPMPC class will design a state-feedback PnPMPC controller')
            end
            
            if ~isempty(objsubss.Bcen)
                error('PNPMPC: subss must not have centralized input')
            end
            
            if ~(N>0)
                error('PNPMPC: N (receding horizon) must be positive')
            end
            
            if nargin~=6 || isempty(options)
                options = sdpsettings;
            end
            
            ctrl.N           = N;
            ctrl.A           = full(objsubss.A);
            ctrl.B           = full(objsubss.B);
            ctrl.M           = full(objsubss.M);
            ctrl.Hd          = full(objsubss.Hd);
            ctrl.Kd          = full(objsubss.Kd);
            ctrl.Aij         = full(objsubss.Aij);
            ctrl.Bij         = full(objsubss.Bij);
            ctrl.couplingA   = objsubss.couplingA;
            ctrl.couplingB   = objsubss.couplingB;
            ctrl.nj          = objsubss.nj;
            ctrl.Xj          = Xj;
            ctrl.Uj          = Uj;
            n               = size(ctrl.A,2);
            m               = size(ctrl.B,2);
            Ni              = length(ctrl.couplingA);
            Mi              = length(ctrl.couplingB);
            njTtrue         = sum(objsubss.nj);
            mjTtrue         = sum(objsubss.mj);
            
            HX              = full(objsubss.Hx);
            KX              = full(objsubss.Kx);
            if isempty(HX)
                HX = [eye(n);-eye(n)];
                KX = 1e6*ones(2*n,1);
            end
            HU              = full(objsubss.Hu);
            KU              = full(objsubss.Ku);
            
            HX              = HX./repmat(KX,1,size(HX,2));
            KX              = ones(size(HX,1),1);
            HU              = HU./repmat(KU,1,size(HU,2));
            KU              = ones(size(HU,1),1);
            
            %% check flag
            if nargin<=4 || isempty(flag)
                flag.distributedParents = [];
                flag.LPdesign             = true ;
                flag.Qi                   = [];
                flag.Ri                   = [];                
                kmin                      = ctrbindex(ctrl.A,ctrl.B);
                flag.k                    = { kmin 10*kmin };
                flag.ExoBounded           = [];
                flag.norm                 = inf;
                flag.boundKij             = inf;
            end
            
            %% check exogenous inputs that are disturbances
            if ~isfield(flag,'ExoBounded')
                ctrl.ExoBounded = [];
                ExoBoundedIndex = [];
            else
                [ ctrl.ExoBounded , ExoBoundedIndex ] = intersect(objsubss.Exo,unique(flag.ExoBounded));
                ctrl.ExoBounded = reshape(ctrl.ExoBounded,1,[]);
            end
            
            [ ctrl.Exo , diffExo ] = setdiff(objsubss.Exo,ctrl.ExoBounded);
            ctrl.Exo = reshape(ctrl.Exo,1,[]);
            
            if ~isempty(ctrl.ExoBounded)
                %% check if the disturbances are unbounded
                if isempty(ctrl.Hd)
                    error('PNPMPC: exogenous inputs are not bounded. It is impossible design robust PnPMPC controller')
                else
                    DDset = Polyhedron(full(ctrl.Hd),full(ctrl.Kd)).projection(ExoBoundedIndex);
                    HDD   = DDset.A;
                    KDD   = DDset.b;
                end
                [ j ] = ctrl.checkUnbounded( ctrl.M(:,ExoBoundedIndex) , HDD , KDD );
                if ~isempty(j)
                    error(['PNPMPC: exogenous input ' num2str(ctrl.ExoBounded(ExoBoundedIndex(j))) ' is unbounded']);
                end
            end
            
            if ~isfield(flag,'LPdesign')
                flag.LPdesign  = true;
            end
            
            if flag.LPdesign || ~isempty(ctrl.ExoBounded)
                if ~isfield(flag,'k')
                    kmin   = ctrbindex(ctrl.A,ctrl.B);
                    flag.k = { kmin 10*kmin };
                else
                    if size(flag.k,2)==1
                        flag.k{1,2} = flag.k{1,1};
                    end
                    if ~flag.k{1,1}>0
                        error('PNPMPC: kmin must be positive (see flag.k)')
                    end
                    if ~iscell(flag.k) || size(flag.k,2)~=2
                        error('PNPMPC: k must be a cell array wich contains {kmin,kmax} (see flag.k)')
                    end
                    if flag.k{1,1}>flag.k{1,2}
                        error('PNPMPC: kmax must be > kmin (see flag.k)')
                    end
                end
                kmin = flag.k{1,1};
                kmax = flag.k{1,2};
            end
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            if ( isempty(objsubss.couplingA) || all(all(abs(ctrl.Aij)<=1e-8)) ) && ( isempty(objsubss.couplingB) || all(all(abs(ctrl.Bij)<=1e-8)) ) && isempty(ctrl.ExoBounded)
                
                
                
                
                %% no coupling terms or coupling terms very very small and there are no disturbances
                ctrl.Hxc = sparse(HX);
                ctrl.Kxc = sparse(KX);
                ctrl.Hv  = sparse(HU);
                ctrl.Kv  = sparse(KU);
                % x0==xc(0)
                ctrl.initialCons = @(x0,xc,lambda)( x0-xc(:,1)==0 );
                
                
                
                
            else
                
                
                
                
                %% coupling terms
                
                %% check state constraints Xj from parents
                if ~isempty(objsubss.couplingA)
                    if ~iscell(Xj) || size(Xj,1)~=Ni || size(Xj,2)~=2
                        error('PNPMPC: wrong sets Xj from parents. see help pnpmpc')
                    end
                    njT = 0;
                    njR = 0;
                    for j=1:Ni
                        if size(Xj{j,1},1)~=size(Xj{j,2},1)
                            error('PNPMPC: wrong sets Xj from parent %d. see help pnpmpc',j)
                        end
                        njT     = njT + size(Xj{j,1},2);
                        njR     = njR + size(Xj{j,1},1);
                        Xj{j,1} = full(Xj{j,1}./repmat(Xj{j,2},1,size(Xj{j,1},2)));
                        Xj{j,2} = full(ones(size(Xj{j,1},1),1));
                    end
                    if njT~=njTtrue
                        error('PNPMPC: wrong sets Xj from parents. see help pnpmpc')
                    end
                end
                
                
                %% check input constraints Xj from parents
                if ~isempty(objsubss.couplingB)
                    if ~iscell(Uj) || size(Uj,1)~=Mi || size(Uj,2)~=2
                        error('PNPMPC: wrong sets Uj from parents. see help pnpmpc')
                    end
                    mjT = 0;
                    mjR = 0;
                    for j=1:Mi
                        if size(Uj{j,1},1)~=size(Uj{j,2},1)
                            error('PNPMPC: wrong sets Uj from parent %d. see help pnpmpc',j)
                        end
                        mjT     = mjT + size(Uj{j,1},2);
                        mjR     = mjR + size(Uj{j,1},1);
                        Uj{j,1} = full(Uj{j,1}./repmat(Uj{j,2},1,size(Uj{j,1},2)));
                        Uj{j,2} = full(ones(size(Uj{j,1},1),1));
                    end
                    if mjT~=mjTtrue
                        error('PNPMPC: wrong sets Uj from parents. see help pnpmpc')
                    end
                end
                
                
                %% check if consider distributed gains Kij
                if ~isfield(flag,'distributedParents')
                    flag.distributedParents = [];
                else
                    ctrl.Nij = intersect(unique(flag.distributedParents),ctrl.couplingA);
                    if length(objsubss.couplingA)<length(flag.distributedParents)
                        error('PNPMPC: the number of elements in distributedParents should be <= %d',length(objsubss.couplingA))
                    end
                end
                
                
                %% if want use state constraints as zonotopes, check Qi, Ri and if state constraints are really zonotopes
                if ~flag.LPdesign
                    if ~isfield(flag,'Qi') || isempty(flag.Qi)
                        flag.Qi               = 1e-4*eye(n);
                    else
                        flag.Qi = full(flag.Qi);
                        if any(size(flag.Qi)~=[n n])
                            error('PNPMPC: input flag.Qi')
                        end
                        flag.Qi = (flag.Qi+flag.Qi')/2;
                        if any(eig(flag.Qi)<0)
                            error('PNPMPC: input flag.Qi')
                        end
                    end
                    
                    if ~isfield(flag,'Ri') || isempty(flag.Ri)
                        flag.Ri               = eye(m);
                    else
                        flag.Ri = full(flag.Ri);
                        if any(size(flag.Ri)~=[m m])
                            error('PNPMPC: input flag.Ri')
                        end
                        flag.Ri = (flag.Ri+flag.Ri')/2;
                        if any(eig(flag.Ri)<=0)
                            error('PNPMPC: input flag.Ri')
                        end
                    end
                    
                    %% convert polytopes to zonotopes
                    ZJ = zonotope;
                    GXJ = zonotope;
                    for j = 1:Ni
                        ZJ(j) = Polyhedron2zonotope(Polyhedron(Xj{j,1},Xj{j,2}));
                        if isempty(ZJ(j))
                            error('PNPMPC: state constraints of parent subsystem %d is not a zonotope',j);
                        end
                        ptmp = ZJ(j).p;
                        if any(abs(ptmp)>1e-6)
                            error('PNPMPC: zonotope state constraints of parents subsystems are not centered in 0');
                        end
                        GXJ      = GXJ*ZJ(j);
                    end
                    GXJ = GXJ.G;
                end
                
                
                
                
                %% compute matrices needed for optimization problems
                HJ = [];
                KJ = [];
                for j=1:Ni
                    HJ = blkdiag( HJ , Xj{j,1} );
                    KJ = [ KJ ; Xj{j,2} ]; %#ok<AGROW>
                end
                
                HUJ = [];
                KUJ = [];
                for j=1:Mi
                    HUJ = blkdiag( HUJ , Uj{j,1} );
                    KUJ = [ KUJ ; Uj{j,2} ]; %#ok<AGROW>
                end
                
                if ~flag.LPdesign || ~isempty(ctrl.Nij)
                    pXj = cell(Ni,1);
                    for j = 1:Ni
                        pXj{j,1} = pinv( Xj{j,1} );
                    end
                end
                
                Fij = ctrl.Aij;
                
                
                
                
                
                
                
                %% check if possible reduce coupling
                if ~isempty(ctrl.Nij)
                    %% check norm used for computing Kij
                    if ~isfield(flag,'norm') || ( flag.norm~=inf && flag.norm~=1 && flag.norm~=2 && flag.norm~='F' )
                        flag.norm = inf;
                        warning('PNPMPC: flag.norm is wrong or empty. Default value will be used: flag.norm=inf' )
                    end
                    
                    %% bound on norm Kij
                    if ~isfield(flag,'boundKij') || flag.boundKij<0 || isempty(flag.boundKij)
                        flag.boundKij = inf;
                        warning('PNPMPC: flag.boundKij is wrong or empty. Default value will be used: flag.boundKij=inf' )
                    end
                    
                    ctrl.Kij = zeros(m,njTtrue);
                    
                    Nijtmp = [];
                    for jj=1:length(ctrl.Nij)
                        j = find(ctrl.couplingA==ctrl.Nij(jj));
                        Kijopt   = sdpvar(m,objsubss.nj(j),'full');
                        Fijtmp   = Fij(:,sum(objsubss.nj(1:j-1))+1:sum(objsubss.nj(1:j))) + ctrl.B*Kijopt;
                        if isempty(HX)
                            iFijtmpj = Fijtmp*pXj{j,1};
                        else
                            iFijtmpj = HX*Fijtmp*pXj{j,1};
                        end                        
                        if flag.norm==inf
                            Jij = norm(iFijtmpj,inf);
                            CONS = norm(Kijopt*pXj{j,1},inf)<=flag.boundKij;
                        elseif flag.norm==1
                            Jij = norm(iFijtmpj,1);
                            CONS = norm(Kijopt*pXj{j,1},1)<=flag.boundKij;
                        elseif flag.norm==2
                            Jij = norm(iFijtmpj,2);
                            CONS = norm(Kijopt*pXj{j,1},2)<=flag.boundKij;
                        elseif flag.norm=='F'
                            Jij = sum(sum( (iFijtmpj).^2 ));
                            CONS = sum(sum( Kijopt*pXj{j,1} ))<=flag.boundKij;
                        end
                        info     = solvesdp(CONS,Jij,sdpsettings('verbose',0));
                        if info.problem~=0
                            error(['PNPMPC: Kij for parent subsystem %d was not created. Solver error: ' info.info],ctrl.Nij(jj))
                        end
                        Ktmp = double(Kijopt);
                        if all(all(Ktmp==0))
                            Nijtmp = [ Nijtmp jj];
                            continue;
                        end
                        ctrl.Nij(Nijtmp) = [];
                        ctrl.Kij(:,sum(objsubss.nj(1:j-1))+1:sum(objsubss.nj(1:j))) = double(Kijopt) ;
                        Fijtmp(abs(double(Fijtmp))<=eps) = 0;
                        Fij(:,sum(objsubss.nj(1:j-1))+1:sum(objsubss.nj(1:j))) = double(Fijtmp);
                    end
                    Fij(isnan(Fij)) = 0;
                    ctrl.Kij(isnan(ctrl.Kij)) = 0;
                    
                    
                    %% reduce input constraints depending on attenuation gain matrices Kij
                    if ~isempty(HU) && any(any(abs(ctrl.Aij)))>=1e-8
                        vertexKijXj = ctrl.computeExtreme( ctrl.Kij , HJ , KJ , [] , [] , [] , [] , [] , [] )';
                        KijXj.A     = repmat(HU,size(vertexKijXj,2),1);
                        KijXj.B     = repmat(KU,size(vertexKijXj,2),1)-sum(KijXj.A.*reshape(repmat(vertexKijXj,size(HU,1),1),size(vertexKijXj,1),[])',2);
                        KijXj       = cddmex('reduce_h',KijXj);
                        HU          = KijXj.A;
                        KU          = KijXj.B;
                    end
                end
                
                
                
                
                
                
                
                
                
                
                if all(all(abs(Fij)<=1e-8)) && isempty(ctrl.couplingB) && isempty(ctrl.ExoBounded)
                    
                    
                    %% coupling can be completely removed
                    ctrl.Hxc = HX;
                    ctrl.Kxc = KX;
                    ctrl.Hv  = HU;
                    ctrl.Kv  = KU;
                    % x0==xc(0)
                    ctrl.initialCons = @(x0,xc,lambda)( x0-xc(:,1)==0 );
                    
                    
                else
                    
                    %% coupling cannot be completely removed
                    
                    [ j ] = ctrl.checkUnbounded( Fij , HJ , KJ );
                    if ~isempty(j)
                        error(['PNPMPC: state_j ' num2str(j) ' is unbounded']);
                    end
                    [ j ] = ctrl.checkUnbounded( ctrl.Bij , HUJ , KUJ );
                    if ~isempty(j)
                        error(['PNPMPC: input_j ' num2str(j) ' is unbounded']);
                    end
                    
                    
                    if flag.LPdesign
                        
                        
                        %% polytopes
                        if isempty(ExoBoundedIndex)
                            [ vertexW ] = ctrl.computeExtreme( Fij , HJ , KJ , [] , [] , [] , ctrl.Bij , HUJ , KUJ );
                        else
                            [ vertexW ] = ctrl.computeExtreme( Fij , HJ , KJ , objsubss.M(:,ExoBoundedIndex) , HDD , KDD , ctrl.Bij , HUJ , KUJ );
                        end
                        
                        % try different k
                        for k=kmin:kmax
                            testZi = 1;
                            % compute RCI Zi
                            try
                                Zi = parameterizedRCI(ctrl.A,ctrl.B,k,{HX,KX},{HU,KU},{vertexW,0},zeros(n,1),zeros(m,1),options);
                            catch e
                                warning(e.message)
                                continue;
                            end
                            
                            % in the following, the code check numerical problems
                            if Zi.info.problem==0
                                testZi = 0;
                                if Zi.isinside(zeros(size(ctrl.A,1),1),options)~=1
                                    testZi = 1;
                                    continue;
                                end
                                ctrl.Z    = Zi;
                                
                                bxrep    = repmat(ctrl.Z.bx,1,size(ctrl.Z.x0,1));
                                xV       = (1-ctrl.Z.alpha)^-1*( bxrep + ctrl.Z.x0' );
                                Hxctmp.A    = repmat(HX,size(xV,2),1);
                                if ~isempty(Hxctmp.A)
                                    Hxctmp.B = repmat(KX,size(xV,2),1)-sum(Hxctmp.A.*reshape(repmat(xV,size(HX,1),1),size(xV,1),[])',2);
                                    Hxctmp   = cddmex('reduce_h',Hxctmp);
                                    ctrl.Hxc  = Hxctmp.A;
                                    ctrl.Kxc  = Hxctmp.B;
                                    % compute constraints Xc
                                    for i = 1:ctrl.Z.k-1
                                        xV       =  (1-ctrl.Z.alpha)^-1*( bxrep + ctrl.Z.x(:,:,i) );
                                        Hxctmp.A =  repmat(ctrl.Hxc,size(xV,2),1);
                                        Hxctmp.B =  repmat(ctrl.Kxc,size(xV,2),1)-sum(Hxctmp.A.*reshape(repmat(xV,size(ctrl.Hxc,1),1),size(xV,1),[])',2);
                                        Hxctmp   =  cddmex('reduce_h',Hxctmp);
                                        ctrl.Hxc  =  Hxctmp.A;
                                        ctrl.Kxc  =  Hxctmp.B;
                                    end
                                    if any(ctrl.Hxc*zeros(size(ctrl.A,1),1)>ctrl.Kxc)
                                        testZi = 1;
                                        continue;
                                    end
                                end
                                
                                % compute constraints V
                                ctrl.Hv   = HU;
                                ctrl.Kv   = KU;
                                if ~isempty(ctrl.Hv)
                                    burep    = repmat(ctrl.Z.bu,1,size(ctrl.Z.x0,1));
                                    for i = 1:ctrl.Z.k
                                        uV       =  (1-ctrl.Z.alpha)^-1*( burep + ctrl.Z.u(:,:,i) );
                                        Hvtmp.A  =  repmat(ctrl.Hv,size(uV,2),1);
                                        Hvtmp.B  =  repmat(ctrl.Kv,size(uV,2),1)-sum(Hvtmp.A.*reshape(repmat(uV,size(ctrl.Hv,1),1),size(uV,1),[])',2);
                                        Hvtmp    =  cddmex('reduce_h',Hvtmp);
                                        ctrl.Hv   =  Hvtmp.A;
                                        ctrl.Kv   =  Hvtmp.B;
                                    end
                                    if any(ctrl.Hv*zeros(size(ctrl.B,2),1)>ctrl.Kv)
                                        testZi = 1;
                                        continue;
                                    end
                                end
                                break;
                            end
                        end
                        
                        if testZi==1
                            error('PNPMPC: it is impossible to design a Plug-and-Play controller with kmin=%d and kmax=%d. Last solver error: %s',kmin,kmax,Zi.info.info);
                        end
                        
                        % x0-xc(0) is inside the RCI
                        ctrl.initialCons = @(x0,xc,lambda) ( [ lambda(:)>=0 ; sum(lambda,2) == 1 ; x0-xc == (1-ctrl.Z.alpha)^-1*( ctrl.Z.bx + sum(repmat(reshape(lambda',1,[]),size(ctrl.A,1),1).*[ctrl.Z.x0' reshape(ctrl.Z.x(:,:,1:ctrl.Z.k-1),size(ctrl.Z.A,1),[])],2)) ] );
                        
                        
                        
                        
                    else
                        
                        if isempty(ExoBoundedIndex)
                            GZD = [];
                        else
                            ZD = Polyhedron2zonotope(Polyhedron(full(HDD),full(KDD)));
                            GZD = ZD.G;
                        end
                        
                        
                        %% zonotopes
                        tmpGXJ = blkdiag(GXJ,GZD);
                        mulQR = 10.^(-4:4);
                        tmpJ  = inf;
                        Ktmpi = zeros(m,n);
                        for i=mulQR
                            for j=mulQR
                                tmpKt = -(dlqr(ctrl.A,ctrl.B,i*flag.Qi,j*flag.Ri));
                                tJ    = ctrl.Jzono(tmpKt(:),ctrl.A,ctrl.B,[Fij objsubss.M(:,ExoBoundedIndex)],objsubss.nj,HX,pXj,tmpGXJ,HU,[],ctrl.couplingA);
                                tJ    = max(tJ(1:end-1));
                                if tmpJ>tJ
                                    Ktmpi = tmpKt;
                                    tmpJ  = tJ;
                                end
                            end
                        end
                        
                        xt0 = Ktmpi(:);
                        
                        xtopt  = fmincon(@(xt)max(ctrl.Jzono(xt,ctrl.A,ctrl.B,[Fij objsubss.M(:,ExoBoundedIndex)],objsubss.nj,HX,pXj,tmpGXJ,HU,[],ctrl.couplingA)),xt0,[],[],[],[],-1e3*ones(length(xt0),1),1e3*ones(length(xt0),1),@(xt)ctrl.Conszono(xt,ctrl.A,ctrl.B,[Fij objsubss.M(:,ExoBoundedIndex)],objsubss.nj,HX,pXj,tmpGXJ,HU,[],ctrl.couplingA),optimset('Display','off','Algorithm','active-set','TolFun',1e-4));
                        
                        JJ = ctrl.Conszono(xtopt,ctrl.A,ctrl.B,[Fij objsubss.M(:,ExoBoundedIndex)],objsubss.nj,HX,pXj,tmpGXJ,HU,[],ctrl.couplingA);
                        JJ = max(JJ(1:end-1)+1);
                        if JJ>=1
                            if isempty(ctrl.Nij)
                                error('PNPMPC: it is impossible to design a Plug-and-Play controller. Nonlinear optimization failed. Try using LP optimization, try using flag.LPdesign=1')
                            else
                                xt0   = [ xtopt(1:n*m) ; 0*ctrl.Kij(:) ];
                                xtopt = fmincon(@(xt)max(ctrl.Jzono(xt,ctrl.A,ctrl.B,[objsubss.Aij objsubss.M(:,ExoBoundedIndex)],objsubss.nj,HX,pXj,tmpGXJ,HU,ctrl.Nij,ctrl.couplingA)),xt0,[],[],[],[],-1e3*ones(length(xt0),1),1e3*ones(length(xt0),1),@(xt)ctrl.Conszono(xt,ctrl.A,ctrl.B,[objsubss.Aij objsubss.M(:,ExoBoundedIndex)],objsubss.nj,HX,pXj,tmpGXJ,HU,ctrl.Nij,ctrl.couplingA),optimset('Display','off','Algorithm','active-set','TolFun',1e-4));
                                JJ    = ctrl.Conszono(xtopt,ctrl.A,ctrl.B,[objsubss.Aij objsubss.M(:,ExoBoundedIndex)],objsubss.nj,HX,pXj,tmpGXJ,HU,ctrl.Nij,ctrl.couplingA);
                                JJ    = max(JJ(1:end-1)+1);
                                if JJ>=1
                                    error('PNPMPC: it is impossible to design a Plug-and-Play controller. Nonlinear optimization failed. Try using LP optimization, try using flag.LPdesign=1')
                                end
                                warning('PNPMPC: Kij have been changed in order to guarantee PnP conditions and could not respect boundKij')
                                ctrl.Kij = reshape(xtopt(m*n+1:end,1),m,[]);
                                totnj = 0;
                                for jj=1:length(ctrl.Nij)
                                    j  = find(ctrl.couplingA==ctrl.Nij(jj));
                                    Fij(:,sum(objsubss.nj(1:j-1))+1:sum(objsubss.nj(1:j))) = objsubss.Aij(:,sum(objsubss.nj(1:j-1))+1:sum(objsubss.nj(1:j))) + ctrl.B*ctrl.Kij(:,totnj+(1:objsubss.nj(j))) ;
                                    totnj = totnj + objsubss.nj(j);
                                end
                            end
                        end
                        ctrl.Ki  = reshape(xtopt(1:n*m),m,n);
                        ctrl.Z   = epsilon_mRPI( ctrl.A+ctrl.B*ctrl.Ki , zonotope(zeros(n,1),[Fij objsubss.M(:,ExoBoundedIndex)]*tmpGXJ) , 1e-5 );
                        
                        % compute constraints Xc
                        ctrl.Hxc = HX;
                        ctrl.Kxc = KX - norm(HX,inf)*1e-5;
                        kf = floor(log10(1e-6)/log10(max(abs(eig( ctrl.A+ctrl.B*ctrl.Ki )))));
                        tot  = 0;
                        Amul = eye(n);
                        for k=0:kf
                            totalp1 = tot;
                            tot = tot + norm(HX*Amul*[Fij objsubss.M(:,ExoBoundedIndex)]*tmpGXJ,inf);
                            if ((tot-totalp1)/tot)<=10^-6
                                break;
                            end
                            Amul = (ctrl.A+ctrl.B*ctrl.Ki)*Amul;
                        end
                        ctrl.Kxc = ctrl.Kxc-tot;
                        if any(ctrl.Kxc<=0)
                            error('PNPMPC: it is impossible to design a Plug-and-Play controller. Nonlinear optimization failed. Try using LP optimization, try using flag.LPdesign=1')
                        end
                        
                        % compute constraints V
                        ctrl.Hv   = HU;
                        ctrl.Kv   = KU;
                        [ ~ , GZ , ctrl.Z ] = doublePG(ctrl.Z);
                        for i=1:size(HU,1)
                            ctrl.Kv(i) = ctrl.Kv(i)-norm(GZ'*ctrl.Ki'*HU(i,:)',1);
                        end
                        if any(ctrl.Kv<=0) && ~isempty(ctrl.Kv)
                            error('PNPMPC: it is impossible to design a Plug-and-Play controller. Nonlinear optimization failed. Try using LP optimization, try using flag.LPdesign=1')
                        end
                        
                        clear ZJ CONS Fijtmp Jij Kijopt iFijtmpj 
                        
                        % x0-xc(0) is inside the RPI
                        ctrl.initialCons = @(x0,xc,lambda)( [ x0-xc==ctrl.Z.vertMink.p+ctrl.Z.vertMink.G*lambda ; norm(lambda,inf)<=1 ] );
                        
                        ctrl.Ki = sparse(ctrl.Ki);
                        
                        
                        
                    end
                    
                end
            end
            
            
            
            
            
            %% in the following we build the anonymous functions for better computation of the control action using YALMIP
            
            ctrl.A   = sparse(ctrl.A);
            ctrl.B   = sparse(ctrl.B);
            ctrl.M   = sparse(ctrl.M);
            ctrl.Aij = sparse(ctrl.Aij);
            ctrl.Kij = sparse(ctrl.Kij);
            
            %% in dynState (dynamic of the state) we receive xc(:)
            AA = [];
            BB = [];
            for i=1:N
                AA = blkdiag(AA,ctrl.A);
                BB = blkdiag(BB,ctrl.B);
            end
            
            
            if isempty(ctrl.Exo)
                ctrl.dynState = @(xc,v,d)(xc(n+1:end) == AA*xc(1:end-n)+BB*v);
            else
                MM = [];
                for i=1:N
                    MM = blkdiag(MM,objsubss.M(:,diffExo));
                end
                ctrl.dynState = @(xc,v,d)( xc(n+1:end) == AA*xc(1:end-n)+BB*v+MM*d );
            end
            
            if isempty(objsubss.Hx)
                ctrl.Hxc = [];
                ctrl.Kxc = [];
            else
                ctrl.Hxc = sparse(ctrl.Hxc);
                ctrl.Kxc = sparse(ctrl.Kxc);
            end
            if isempty(objsubss.Hu)
                ctrl.Hv = [];
                ctrl.Kv = [];
            else
                ctrl.Hv = sparse(ctrl.Hv);
                ctrl.Kv = sparse(ctrl.Kv);
            end
            ctrl.Hd  = sparse(ctrl.Hd);
            ctrl.Kd  = sparse(ctrl.Kd);
            
            
            %% constraints on xc
            if ~isempty(ctrl.Hxc)
                Hbx = repmat(ctrl.Hxc,N,1);
                Kbx = repmat(ctrl.Kxc,N,1);
                ctrl.stateCons = @(xc)(sum(Hbx.*(reshape(repmat(xc(:,1:N),size(ctrl.Hxc,1),1),n,[]))',2)<=Kbx);
            else
                ctrl.stateCons = @(xc)([]);
            end
            
            %% constraints on v
            if ~isempty(ctrl.Hv)
                Hbu = sparse(repmat(ctrl.Hv,N,1));
                Kbu = sparse(repmat(ctrl.Kv,N,1));
                ctrl.inputCons = @(v)(sum(Hbu.*(reshape(repmat(v,size(ctrl.Hv,1),1),m,[]))',2)<=Kbu);
            else
                ctrl.inputCons = @(v)([]);
            end
            
            %% Terminal Set and Cost Function are created with own methods
            ctrl.TerminalSet  = false;
            ctrl.CostFunction = false;
            
            yalmip('clear')
            
        end
    end
    
    
    
    
    
    
    
    
    
    
    
    
    methods(Static)
        
        function J = Jzono( xt , Ati , Bti , Fij , nj , Fcali , pFcalj , GW , fUt , Nij , couplingA )
            %% nonlinear optimization problem for zonotope case
            n   = size(Ati,2);
            m   = size(Bti,2);
            Kti  = reshape(xt(1:m*n,1),m,n);
            Ktij = reshape(xt(m*n+1:end,1),m,[]);
            GZ = [];
            
            totnj = 0;
            for jj=1:length(Nij)
                j  = find(couplingA==Nij(jj));
                Fij(:,sum(nj(1:j-1))+1:sum(nj(1:j))) = Fij(:,sum(nj(1:j-1))+1:sum(nj(1:j))) + Bti*Ktij(:,totnj+(1:nj(j)));
                totnj = totnj + nj(j);
            end
            
            kf = floor(log10(1e-6)/log10(max(abs(eig( Ati+Bti*Kti )))));
            
            if kf<0
                total = 1e6;
                tot   = 1e6;
            else
                total  = 0;
                tot    = 0;
                Amul   = eye(n);
                
                for k=0:kf
                    GZ = [ GZ Amul*Fij*GW ];
                    totalp1 = total;
                    for j=1:length(nj)
                        total  = total + norm(Fcali*Amul*Fij(:,sum(nj(1:j-1))+1:sum(nj(1:j)))*pFcalj{j,1},inf);
                    end
                    tot = tot + norm(Fcali*Amul*Fij*GW,inf);
                    if ((total-totalp1)/total)<=1e-6
                        break;
                    end
                    Amul = (Ati+Bti*Kti)*Amul;
                end
                tot = tot + norm(Fcali,inf)*1e-5;
            end
            
            if isempty(fUt)
                J = [ total ; tot ; abs(eig( Ati+Bti*Kti )) ] ;
                return
            end
            
            if kf<0 || max([ total ; tot ; abs(eig( Ati+Bti*Kti )) ])>=10
                lv_max = 1e6;
                J = [ total ; tot ; lv_max ; abs(eig( Ati+Bti*Kti )) ] ;
                return
            end
            
            lv_max = 0;
            for j=1:size(fUt,1)
                lv_max = max([ lv_max  norm(GZ'*Kti'*fUt(j,:)',1) ]);
            end
            
            J = [ total ; tot ; lv_max ; abs(eig( Ati+Bti*Kti )) ];
        end
        
        
        function [ c , ceq ] = Conszono( xt , Ati , Bti , Fij , nj , Fcali , pFcalj , GW , fUt , Nij , couplingA )
            %% constraints for nonlinear optimization problem for zonotope case
            c = pnpmpc.Jzono( xt , Ati , Bti , Fij , nj , Fcali , pFcalj , GW , fUt , Nij , couplingA );
            c = c(1:end-1)-1;
            ceq = [];
        end
        
        
        
        
        
        function [ vertexW ] = computeExtreme( matrix , Hmatrix , Kmatrix , matrixD , HmatrixD , KmatrixD , matrixU , HmatrixU , KmatrixU )
            %% compute extreme of flat polytope
            matrix  = full(matrix);
            matrixD = full(matrixD);
            matrixU = full(matrixU);
            
            tol     = 1e-6;
            
            if isempty(matrixU) && isempty(matrixD)
                P   = Polyhedron(full(Hmatrix),full(Kmatrix));
                tmp = P.V*matrix';
                tmp(abs(tmp)<=tol) = 0;
                tmp = sortrows(tmp);
                tmp = tmp([true; any(abs(diff(tmp))>tol,2)],:);
                P   = Polyhedron(tmp).computeHRep;
            elseif isempty(matrixU)
                P1  = Polyhedron(full(Hmatrix),full(Kmatrix));
                tmp = P1.V*matrix';
                tmp(abs(tmp)<=tol) = 0;
                tmp = sortrows(tmp);
                tmp = tmp([true; any(abs(diff(tmp))>tol,2)],:);
                P1  = Polyhedron(tmp).computeHRep;
                
                P2  = Polyhedron(full(HmatrixD),full(KmatrixD));
                tmp = P2.V*matrixD';
                tmp(abs(tmp)<=tol) = 0;
                tmp = sortrows(tmp);
                tmp = tmp([true; any(abs(diff(tmp))>tol,2)],:);
                P2  = Polyhedron(tmp).computeHRep;
                
                P   = P1 + P2;
            elseif isempty(matrixD)
                P1  = Polyhedron(full(Hmatrix),full(Kmatrix));
                tmp = P1.V*matrix';
                tmp(abs(tmp)<=tol) = 0;
                tmp = sortrows(tmp);
                tmp = tmp([true; any(abs(diff(tmp))>tol,2)],:);
                P1  = Polyhedron(tmp).computeHRep;
                
                P2  = Polyhedron(full(HmatrixU),full(KmatrixU));
                tmp = P2.V*matrixU';
                tmp(abs(tmp)<=tol) = 0;
                tmp = sortrows(tmp);
                tmp = tmp([true; any(abs(diff(tmp))>tol,2)],:);
                P2  = Polyhedron(tmp).computeHRep;
                
                P   = P1 + P2;
            else
                P1  = Polyhedron(full(Hmatrix),full(Kmatrix));
                tmp = P1.V*matrix';
                tmp(abs(tmp)<=tol) = 0;
                tmp = sortrows(tmp);
                tmp = tmp([true; any(abs(diff(tmp))>tol,2)],:);
                P1  = Polyhedron(tmp).computeHRep;
                
                P2  = Polyhedron(full(HmatrixU),full(KmatrixU));
                tmp = P2.V*matrixU';
                tmp(abs(tmp)<=tol) = 0;
                tmp = sortrows(tmp);
                tmp = tmp([true; any(abs(diff(tmp))>tol,2)],:);
                P2  = Polyhedron(tmp).computeHRep;
                
                P3  = Polyhedron(full(HmatrixD),full(KmatrixD));
                tmp = P3.V*matrixD';
                tmp(abs(tmp)<=tol) = 0;
                tmp = sortrows(tmp);
                tmp = tmp([true; any(abs(diff(tmp))>tol,2)],:);
                P3  = Polyhedron(tmp).computeHRep;
                
                P   = P1 + P2 + P3;
            end
            
            P = P.computeHRep;
%             if ~isempty(P.Ae)
%                 P   = Polyhedron([P.A;P.Ae;-P.Ae],[P.b;P.be+1e-6;-P.be+1e-6]).minVRep;
%             end
%             tmp = cddmex('extreme',struct('A',[eye(P.Dim);-eye(P.Dim)],'B',tol*ones(2*P.Dim,1)));
%             P   = Polyhedron([P.V;tmp.V]);
            vertexW = P.minVRep.V;
        end
        
        
        
        
        function [ j ] = checkUnbounded( matrix , Hmatrix , Kmatrix )
            %% check if the variables are unbounded
            j = [];
            big = 10^7;
            Hmatrix = [eye(size(matrix,2));-eye(size(matrix,2));Hmatrix];
            Kmatrix = [ones(2*size(matrix,2),1)*big;Kmatrix];
            if ~isempty(Hmatrix)
                for i = 1:size(Hmatrix,2)
                    objective    = zeros(1,size(Hmatrix,2));
                    objective(i) = 1;
                    INmin        = struct('obj',objective,'A',Hmatrix,'B',Kmatrix);
                    OUTmin       = cddmex('solve_lp',INmin);
                    objective(i) = -1;
                    INmax        = struct('obj',objective,'A',Hmatrix,'B',Kmatrix);
                    OUTmax       = cddmex('solve_lp',INmax);
                    if OUTmin.xopt(i)==-big || OUTmax.xopt(i)==big
                        if any(matrix(:,i)~=0)
                            j = i;
                            break
                        end
                    end
                end
            end
        end
        
    end
    
    
end