function [ u, diagnostics ] = uRH(objCtrl,x0,xj,xjinv,din,xcrefin,vrefin,options)
% URH compute the control action as proposed in [1]
%
% [ u, diagnostics ] = uRH(objCtrl,x0,xj,xjinv,din,xcrefin,vrefin,options)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Compute the control action as proposed in [1-3] in a decentralized or
% distributed fashion
%
% USAGE:
%
% [ u             ] = objCtrl.uRH(x0)
% [ u             ] = objCtrl.uRH(x0,xj)
% [ u             ] = objCtrl.uRH(x0,xj,xjinv)
% [ u             ] = objCtrl.uRH(x0,xj,xjinv,din)
% [ u             ] = objCtrl.uRH(x0,xj,xjinv,din,xcrefin)
% [ u             ] = objCtrl.uRH(x0,xj,xjinv,din,xcrefin,vrefin)
% [ u             ] = objCtrl.uRH(x0,xj,xjinv,din,xcrefin,vrefin,options)
% [ u diagnostics ] = objCtrl.uRH(x0,xj,xjinv,din,xcrefin,vrefin,options)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% objCtrl - pnpmpc object
% x0 - current state of the subsystem equipped with the pnpmpc object
% xj - vector of states of parent subsystems selected during the design 
%      phase with flag.distributedParents
% xjinv - cell array, each element is the state of a parent subsystem. This
%         input is useful if in the design phase flag.LPdesign was true and
%         there are no .ExoBounded disturbances
% din - vector of the exogenous inputs acting on the subsystem
% xcrefin - set-points for the state. It could be a vector of n states, or
%           a matrix of n x N+1 elements.
% vrefin - set-points for the state. It could be a vector of m states, or
%          a matrix of m x N elements.
% options from YALMIP, i.e. sdpsettings object (optional)
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% u - values of the input variables
% diagnostics - struct variable with field 'mpc' and 'uInv'. 
%      Struct 'mpc' contains:
%          * in 'info' the variable returned by solvesdp for the mpc problem, 
%          * in 'J' value of the cost function
%          * in 'xc' and 'v' the optimizers of the mpc problem
%      Struct 'uInv' contains:
%          * in 'info' the variable returned by solvesdp to compute the
%            control action inside the RCI set stored in objCtrl
%
%
% [1] S. Riverso, M. Farina, and G. Ferrari-Trecate, "Plug-and-Play Model 
%     Predictive Control based on robust control invariant sets,"
%     Automatica, Accepted, 2013.
% [2] S. Riverso, M. Farina, and G. Ferrari-Trecate, "Plug-and-Play 
%     Decentralized Model Predictive Control for Linear Systems," IEEE 
%     Trans. Automat. Contr., vol. 58, no. 10, pp. 2608-2614, 2013.
% [3] S. Riverso, "Distributed and plug-and-play control for constrained 
%     systems," Universita' degli studi di Pavia, 2014.
%
    
% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Stefano Riverso, Alberto Battocchio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, alberto.battocchio01@universitadipavia.it,
%     giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if isempty(objCtrl.N)
        warning('URH: PnP controller is empty') %#ok<WNTAG>
        return
    end
    
    if ~objCtrl.TerminalSet || ~objCtrl.CostFunction
        error('URH: cost function and terminal set not defined')
    end
    
    if nargin<2 || nargin>8
        error('URH: parameters in input. see help URH')
    end
    
    if size(x0,1)~=size(objCtrl.A,1)
        error('URH: dimension of x0 is wrong')
    end
    
    if ~(nargin>=3 && ~isempty(xj))
        xj = [];
    end
    
    if ~isempty(xj)
        [ ~ , ii ] = intersect(objCtrl.couplingA,objCtrl.Nij);
        tmp = sum(objCtrl.nj(ii));
        if size(xj,1)~=tmp
            error('URH: dimension of xj is wrong. xj should a vector of %d elements',tmp)
        end
        xj = xj(:,1);
    end
    
    
    xjinvtemp = [];
    if nargin>=4 && ~isempty(xjinv)
        if ~iscell(xjinv)
            xjinv = {xjinv};
        end
        if length(xjinv)~=length(objCtrl.nj)
            error('URH: dimension of xjinv is wrong')
        end
        for j=1:length(objCtrl.nj)
            if isempty(xjinv{j})
                xjinvtemp = [ xjinvtemp ; zeros(objCtrl.nj(j),1) ];
            else
                if size(xjinv{j},1)~=objCtrl.nj(j)
                    error('URH: dimension of xjinv for subsystem %d is wrong ',objCtrl.couplingA(j))
                end
                xjinvtemp = [ xjinvtemp ; xjinv{j} ];
            end
        end
        xjinv = xjinvtemp;
    else
        xjinv = [];
    end
    
    d = zeros(size(objCtrl.Exo,2),objCtrl.N);
    if nargin>=5
        [dr, dc] = size(din);
        if dr~=numel(objCtrl.Exo)
            error('URH: d must be a vector of %d or a matrix %d x %d',numel(objCtrl.Exo),numel(objCtrl.Exo),objCtrl.N)
        end
        if dc>1 && dc ~= objCtrl.N
            error('URH: d must be a vector of %d or a matrix %d x %d',numel(objCtrl.Exo),numel(objCtrl.Exo),objCtrl.N)
        else
            d = din;
        end
        
        if isvector(din)
            d = repmat(din,1,objCtrl.N);
        end
    end
    
    xcref = zeros(size(objCtrl.A,1),objCtrl.N+1);
    if nargin>=6
        [xr, xc] = size(xcrefin);
        if xc>1
            if xr ~= size(objCtrl.A,1) || xc ~= objCtrl.N+1
                error('URH: xcref must be a matrix %d x %d',size(objCtrl.A,1),objCtrl.N+1)
            end
            
            if ~isempty (objCtrl.Hxc)
                if ~all(all(objCtrl.Hxc*xcrefin <= repmat(objCtrl.Kxc,1,objCtrl.N+1)))
                    error('URH: xcref does not respect the state constraints')
                end
            end
            xcref = xcrefin;
        end
        
        if isvector(xcrefin)
            if ~isempty(objCtrl.Hxc)
                if ~all(objCtrl.Hxc*xcrefin<=objCtrl.Kxc)
                    error('URH: xcref does not respect the state constraints')
                end
            end
            xcref = repmat(xcrefin,1,objCtrl.N+1);
        end
    end
    
    
    vref = zeros(size(objCtrl.B,2),objCtrl.N);
    if nargin>=7
        [ur, uc] = size(vrefin);
        if uc>1
            if ur ~= size(objCtrl.B,1) || uc ~= objCtrl.N
                error('URH: vref must be a matrix %d x %d',size(objCtrl.B,1),objCtrl.N)
            end
            if ~isempty (objCtrl.Hv)
                if ~all(all(objCtrl.Hv*vrefin <= repmat(objCtrl.Kv,1,objCtrl.N)))
                    error('URH: vref does not respect the input constraints')
                end
            end
            vref = vrefin;
        end
        
        if isvector(vrefin)
            if ~isempty(objCtrl.Hv)
                if ~all(objCtrl.Hv*vrefin<=objCtrl.Kv)
                    error('URH: vref does not respect the input constraints')
                end
            end
            vref = repmat(vrefin,1,objCtrl.N);
        end
    end
    
    if nargin<8
        options = sdpsettings;
    end
    
    xcopt  = sdpvar(size(objCtrl.A,1),objCtrl.N+1,'full');
    vopt   = sdpvar(size(objCtrl.B,2),objCtrl.N,'full');
    if isa(objCtrl.Z,'parameterizedRCI')
        lambda = sdpvar(objCtrl.Z.k,size(objCtrl.Z.x0,1),'full');
    elseif isa(objCtrl.Z,'epsilon_mRPI')
        lambda = sdpvar(size(objCtrl.Z.vertMink.G,2),1,'full');
    else
        lambda = [];
    end
    
    CONS   = [  objCtrl.dynState(xcopt(:),vopt(:),d(:))        ;
                objCtrl.stateCons(xcopt)                       ;
                objCtrl.inputCons(vopt)                        ;
                objCtrl.Xcf(xcopt(:,objCtrl.N+1),xcref(:,end)) ;
                objCtrl.initialCons(x0,xcopt(:,1),lambda)      ];
    
    info = solvesdp( CONS , objCtrl.J(xcopt,vopt,xcref,vref) , options );
    
    if info.problem~=0
        error(info.info);
    end
    
    v   = double(vopt(:,1));
    xc  = double(xcopt(:,1));
    
    diagnostics = [];
    if nargout==2
        diagnostics.mpc    = info;
        diagnostics.mpc.xc = double(xcopt);
        diagnostics.mpc.v  = double(vopt);
        diagnostics.mpc.J  = objCtrl.J(diagnostics.mpc.xc,diagnostics.mpc.v,xcref,vref);
    end
    
    if isa(objCtrl.Z,'parameterizedRCI')
        uz       = sdpvar(size(objCtrl.Z.B,2),1,'full');
        xV       = [ objCtrl.Z.x0' reshape(objCtrl.Z.x(:,:,1:objCtrl.Z.k-1),size(objCtrl.Z.A,1),[]) ];
        mu       = sdpvar(1);
        if isempty(xjinv)
            % DEMPC
            try
                [ uz , infouInv ] = objCtrl.Z.uInv(x0-xc,options);
            catch
                error('URH: numerical problems with computation of uInv')
            end
        elseif ~isempty(objCtrl.ExoBounded)
            warning('URH: input argument xjinv is not used, since there are disturbances in .ExoBounded')
            % DEMPC
            [ uz , infouInv ] = objCtrl.Z.uInv(x0-xc,options);
        else
            % DIMPC
            if isempty(objCtrl.Kij)
                Fij = objCtrl.Aij;
            else
                Fij = objCtrl.Aij+objCtrl.Z.B*objCtrl.Kij;
            end
            if isempty(objCtrl.Hv)
                infouInv = solvesdp( [ lambda(:)>=0 ; sum(lambda,2)==mu ; mu>=0 ; mu<=1 ; objCtrl.Z.A*(x0-xc)+objCtrl.Z.B*uz+Fij*xjinv == (1-objCtrl.Z.alpha)^-1*( objCtrl.Z.bx + sum(repmat(reshape(lambda',1,[]),size(objCtrl.Z.A,1),1).*xV,2) ) ] , mu , options );
            else
                infouInv = solvesdp( [ lambda(:)>=0 ; sum(lambda,2)==mu ; mu>=0 ; mu<=1 ; objCtrl.Z.A*(x0-xc)+objCtrl.Z.B*uz+Fij*xjinv == (1-objCtrl.Z.alpha)^-1*( objCtrl.Z.bx + sum(repmat(reshape(lambda',1,[]),size(objCtrl.Z.A,1),1).*xV,2) ) ; objCtrl.Z.cu*uz<=objCtrl.Z.du-objCtrl.Z.cu*v ] , mu , options );
            end
        end
        if infouInv.problem~=0
            error(infouInv.info)
        end
        uzi  = double(uz);
        diagnostics.uInv = infouInv;
    elseif isa(objCtrl.Z,'epsilon_mRPI')
        uzi = objCtrl.Ki*(x0-xc);
    else
        uzi = 0;
    end
    
    
    % distributed u
    if isempty(objCtrl.Kij)
        uij = 0;
    else
        uij = objCtrl.Kij*xj;
    end
    
    diagnostics.u.v       = v;
    diagnostics.u.uzi     = uzi;
    diagnostics.u.uij     = uij;
    
    u  = v + uzi + uij ;
    
end