classdef subss
% SUBSS design a constrained linear time-invariant subsystem of a large-scale system
%
% objsubss = subss(A,B,C,D,name,Ts)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------?
% This function allows you to create Subss object with all its properties.
%
% objsubss = SUBSS(A,B,C,D,name,Ts)
%
% USAGE:
%
% objsubss = SUBSS(A,B)
%
% objsubss = SUBSS(A,B,C,D)
%
% objsubss = SUBSS(A,B,C,D,name)
%
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% x+ = Ax + Bu
% y  = Cx + Du
%
% name = string, a name that you can associate with the system.
%
% Ts = sample time.
% Ts = [] System described time-continuos
% Ts > 0  System discretize with Ts sample time
% Ts = -1 System is yet in discrete form
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objsubss = an subss object.
%
% ---------------------------------------------------------------------------
% PROPERTIES
% ---------------------------------------------------------------------------
% name : name of the systems or subsystems
% Ts : sample time
% A, B, C, D, M, N, Bcen, Dcen : matrices of the Linear Time Invariant
%                                subsystem
% Aij, Bij, Cij, Dij : coupling matrices among subsystem i and subsystems j
% couplingA, couplingB, couplingC, couplingD: vectors with the indices of
%         parent subsystems in a large-scale system object
% Exo : indices of exogenous signals in a large-scale system object
% ICen : number of centralized inputs in a large-scale system object
% Hx, Kx : matrices of polytopic state constraints 
% Hu, Ku : matrices of polytopic input constraints 
% Hdeltau, Kdeltau : matrices of polytopic delta u constraints 
% Hy, Ky : matrices of polytopic output constraints 
% Hd, Kd : matrices of polytopic exogenous signals constraints 
% Hcen, Kcen : matrices of polytopic centralized input constraints 
% He, Ke : matrices of polytopic state estimation error constraints 
%
%
% see also SUBSS

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012
%     Alberto Battocchio, Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     alberto.battocchio01(at)universitadipavia.it, stefano.riverso(at)unipv.it, 
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    properties (SetAccess = private)
        name;
        Ts;
        
        A;
        B;
        C;
        D;
        M;
        N;
        Bcen;
        Dcen;
        
        Aij;
        Bij;
        Cij;
        Dij;
        
        couplingA;
        couplingB;
        couplingC;
        couplingD;
        Exo;        
        ICen;
        
        Hx;
        Kx;
        Hu;
        Ku;
        Hdeltau;
        Kdeltau;
        Hy;
        Ky;
        Hd;
        Kd;
        Hcen;
        Kcen;
        He;
        Ke;     
    end
    
    properties ( SetAccess = private , Hidden = true )
        nj ;
        mj ;
        pj ;
    end
    
    
    methods
        function obj = subss(A,B,C,D,name,Ts)
            
            global pnpmpcToolboxGlobalOptions;
            if ~isstruct(pnpmpcToolboxGlobalOptions),
                pnpmpc_toolbox_err;
            end
            
            if nargin==0
                return
            end
            
            if isequal(size(A,1),size(B,1))==0 && ~isempty(B)
                error('SUBSS: A rows=%d ~= B rows=%d',size(A,1),size(B,1))
            end
            
            if isequal(size(A,1),size(A,2))==0
                error('SUBSS: A must be a square matrix')
            end
            
            obj.name = 'subss_{i}';
            
            if nargin==2 %A,B
                obj.A       = A;
                obj.B       = B;
                return
            end
            
            if ~isempty(C) && ~isempty(D)
                if isequal(size(C,1),size(D,1))==0
                    error('SUBSS: C rows=%d ~= D rows=%d',size(C,1),size(D,1))
                end
                
                if isequal (size(D,2),size(B,2))==0
                    error('SUBSS: D columns=%d ~=B columns=%d',size(D,2),size(B,2))
                end
                
                if isequal(size(C,2),size(A,1))==0
                    error('SUBSS: C columns=%d ~= A rows=B columns = %d',size(C,2),size(A,1))
                end
            end
            
            if nargin==4 %A,B,C,D
                obj.A       = A;
                obj.B       = B;
                obj.C       = C;
                obj.D       = D;
                obj.Ts      = [];
                return
            end %nargin==4
            
            if nargin>=5 %A,B,C,D,name,Ts
                if ~isempty(name)
                    obj.name = name;
                end
                
                if nargin==6
                    if isequal(Ts,-1) || isscalar(Ts) || isempty(Ts)
                        obj.Ts=Ts;
                    else
                        error('SUBSS: Ts must be -1 or scalar or empty');
                    end
                else
                    obj.Ts = [];
                end
                obj.A       = A;
                obj.B       = B;
                obj.C       = C;
                obj.D       = D;
                return
            end
        end
    end
end