function [dist, Z1, Z2] = distance(Z1,Z2)
% DISTANCE Distance between two sets
%
% [dist, Z1o, Z2o] = distance(Z1,Z2,Options)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Computes the distance between two sets
%
% d =  min    norm(x1-x2)
%     x1,x2
%      s.t.   x1 in Z1
%             x2 in Z2
%
% USAGE:
%   [dist]          = distance(Z1,Z2)
%   [dist, Z1     ] = distance(Z1,Z2)
%   [dist, Z1, Z2 ] = distance(Z1,Z2)
%
%   Since the H-representation of the zonotope is needed, the function DISTANCE
%   for Polyhedron is executed.
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% Z1             - point, zonotope or zonotope array
% Z2             - point, zonotope or zonotope array
%
% Note: At least one input set must be zonotope or zonotope array!
%
% ---------------------------------------------------------------------------
% OUTPUT                                                                                                    
% ---------------------------------------------------------------------------
% dist      - output of distance function in Polyhedron class
% Z1, Z2    - Zonotope Z1 and Z2 updated with H-representation and 
%             V-representation
%
% see also ZONOTOPE
%

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
% (C) 2012 
%     Stefano Riverso, Davide M. Raimondo
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, davide.raimondo(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------
    
    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if ~isa(Z1, 'zonotope')
        if ~isa(Z2, 'zonotope')
            error('DISTANCE: Argument MUST be a zonotope object');
        else
            tmp = Z1;
            Z1  = Z2;
            Z2  = tmp;
        end
    end
    
    Z1 = Z1.computeHRep;
    
    if isa(Z2, 'double')
        dist = distance@Polyhedron(Z1,Z2);
    else
        Z2 = Z2.computeHRep;
        dist = distance@Polyhedron(Z1,Z2);
    end
    
end