function check = isInside(Z,x0)
% ISINSIDE Checks if a given point lies in the interior of a given zonotope
%
% check = isInside(Z,x0)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Checks if the given point x0 lies in the interior of the zonotope Z
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% Z                 - Zonotopes
% x0                - given point, if not specified: x0=0;
%
%
% ---------------------------------------------------------------------------
% OUTPUT                                                                                                    
% ---------------------------------------------------------------------------
% check - logical statement indicating if {x0=p+G*d,|d|inf<=1} was satisfied
%
% see also ZONOTOPE
%

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
% (C) 2012 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if ~isa(Z, 'zonotope'),
        error('ISINSIDE: input argument MUST be a zonotope');
    end
    
    if nargin==1
        x0 = zeros(size(Z(1).p,1),1);
    else
        x0 = full(x0);
    end
    
    check = zeros(length(Z),1);
    
    for i = 1:length(Z)
        if size(x0,1)~=Z(i).Dim
            error('ISINSIDE: dimension of input argument x0');
        end
        
        ng = ngenerator(Z(i));
        
        IN.A   = [ eye(ng) ; -eye(ng) ; Z(i).G ];
        IN.B   = [ ones(2*ng,1) ; x0-Z(i).p ];
        IN.lin = 2*ng+1:size(IN.B,1);
        IN.obj = 0;
        
        OUT    = cddmex('solve_lp',IN);
        
        if OUT.how==1
            check(i) = 1;
        else
            check(i) = 0;
        end
    end
end