function [ R, Z, Q ] = minus(Z,Q)
% MINUS Pontryagin difference
%
% MINUS  Minkowski (Pontryagin) difference
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
%  Pontryagin difference
%
%   Since the H-representation of the zonotope is needed, the function
%   MINUS for Polyhedron is executed if the second input is a zonotope.
%
% NOTE: If Q ia a zonotope array, Q=[Q1 Q2 ... Qn], then Q is considered
% as the space covered by the union of zonotopes Qi, and the minus.m 
% computes Z-Q according to the definition 
%   Z-Q=\{x\in Z| x+w\in Z \forall w\in Q\}=(Z-Q1)&(Z-Q2)&...&(Z-Qn) 
%
% USAGE:
%   R=Z-Q
%   R=minus(Z,Q)
%   [ R Z Q ] = minus(Z,Q)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% Z                       - Zonotope or zonotope array
% Q                       - Zonotope or zonotope array
%
%
% ---------------------------------------------------------------------------
% OUTPUT                                                                                                    
% ---------------------------------------------------------------------------
% R     - polytope (or polytope array) describing the Pontryagin difference
% Z     - Zonotope updated with H-representation and V-representation
% Q     - Zonotope updated with H-representation and V-representation
%
% see also PLUS, MLDIVIDE
%

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
% (C) 2012 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, davide.raimondo(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if ~isa(Z, 'zonotope')
        if ~isa(Q, 'zonotope')
            error('MINUS: An argument MUST be a zonotope object');
        else
            tmp = Q;
            Q   = Z;
            Z   = tmp;
        end
    end
    
    
    if isa(Q, 'double')
        R(length(Z),1) = zonotope;
        for i=1:length(Z)
            R(i) = zonotope(Z(i).p-Q,Z(i).G);
        end
        return
    end
    
    Z = Z.computeHRep;
    Q = Q.computeHRep;
    
    Zp = zonotope2Polyhedron(Z);
    Qp = zonotope2Polyhedron(Q);
    
    R = minus@Polyhedron(Zp,Qp);
end