function R = mtimes(Z,Q)
% MTIMES Zonotope multiplication
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
%  Three cases could happen:
%   1. Multiplication of two zonotopes Z*Q leads to a higher 
%      dimensional zonotope
%   2. Multiplication of a zonotope by a matrix -- this leads an affine
%      transformation of the generators of zonotope Z
%   3. Multiplication of a zonotope by a scalar -- if the origin is included
%      in the zonotope, the zonotope will be shrinked
%
% USAGE:
%   R = Z*Q
%   R = Z*[1 0; 1 1]
%   R = Z*beta
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% Z     - scalar / matrix / zonotope
% Q     - scalar / matrix / zonotope
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% R   - resulting zonotope
%

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
% (C) 2012 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if ~isa(Z, 'zonotope') && (~isa(Q, 'zonotope') && ~isa(Q, 'double'))
        error('MTIMES: Argument MUST be a zonotope object');
    end
    
    if isa(Z,'double')
        tmp = Z;
        Z   = Q;
        Q   = tmp;
    end
    
    dimZ = length(Z);
    
    if isa(Q, 'zonotope')
        dimQ = length(Q);
        R(dimZ*dimQ,1) = zonotope;
        for i=1:dimZ
            for j=1:dimQ
                R(dimZ*(i-1)+j,1) = zonotope([Z(i).p;Q(j).p],blkdiag(Z(i).G,Q(j).G));
            end
        end
    else
        if size(Q,1)==1 && size(Q,2)==1
            R(dimZ,1) = zonotope;
            for i=1:dimZ
                R(i) = zonotope(Q*Z(i).p,Q*Z(i).G);
            end
        else
            R(dimZ,1) = zonotope;
            for i=1:dimZ
                if size(Q,2)==size(Z(i).G,1)
                    R(i) = zonotope(Q*Z(i).p,Q*Z(i).G);
                else
                    error('MTIMES: matrix error');
                end
            end
        end
    end
    
end