function [ Xu, XuA, Xs, J, H ] = outerApproximation(X,f,W,gf,hf,options)
% OUTERAPPROXIMATION finds an outer approximation of a nonlinear function evaluated on set X and W
% 
% [ Xu XuA Xs J H ] = outerApproximation(X,f,W,gf,hf,options)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Finds an outer approximation of nonlinear function f on zonotope set X
% and W. The computation of the outer approximation uses procedures
% proposed in [1] and [2]
% 
% [ Xu XuA Xs J H ] = outerApproximation(X,f,W,gf,hf,options)
%
% USAGE:
% Xu = outerApproximation(X,f)
% Xu = outerApproximation(X,f,W)
% Xu = outerApproximation(X,f,W,gf,hf)
% Xu = outerApproximation(X,f,W,gf,hf,options)
% [ Xu ] = outerApproximation(X,f,W,gf,hf,options)
% [ Xu XuA ] = outerApproximation(X,f,W,gf,hf,options)
% [ Xu XuA Xs ] = outerApproximation(X,f,W,gf,hf,options)
% [ Xu XuA Xs J ] = outerApproximation(X,f,W,gf,hf,options)
% [ Xu XuA Xs J H ] = outerApproximation(X,f,W,gf,hf,options)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - f handle function f(x,w). y=f(x,w) is a vector. f is a DC function
% - X is a zonotope set. x \in X
% - W is a zonotope set. w \in W
%   if options.split.max_zono>1 the set X can be splitted as in [2], while
%   the set W is not spllited
% - gf and hf are convex functions such that f = gf-hf
%   if gf and hf are empty, the function outerApproximation needs IntLab 
%   toolbox http://www.ti3.tuhh.de/rump/intlab/
% - options
%        .J = Jacobian as a function handle J(x,w)
%        .H = Hessian as a function handle H(x,w)
%        .split.ngenerators    number of generators to split
%        .split.alpha          where to split
%        .split.max_zono       max number of zonotopes for the outer
%                              approximation
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% - Xu outer approximation
% - XuA outer approximation using options.split.max_zono
% - Xs zonotope X splitted in options.split.max_zono zonotopes
% - J if the Jacobian is computed, J is a function handle.
% - H if the Heassian is computed, H is a function handle.
%   J and H can be used in input for future use of the function
%   outerApproximation
%
%
%
%
% see also EXAMPLE1OUTERAPPROXIMATION, EXAMPLE2OUTERAPPROXIMATION, EXAMPLE3OUTERAPPROXIMATION
%
%
% [1] T. Alamo, J. M. Bravo, M. J. Redondo, and E. F. Camacho, "A 
% set-membership state estimation algorithm based on DC programming,"
% Automatica, vol. 44, no. 1, pp. 216-224, Jan. 2008.
%
% [2] D. M. Raimondo, S. Riverso, S. Summers, C. N. Jones, J. Lygeros, and 
% M. Morari, "A set theoretic method for verifying feasibility of a fast 
% explicit nonlinear Model Predictive Controller," in Distributed Decision 
% Making and Control, R. Johansson and A. Rantzer, Eds. Springer, Lecture 
% Notes in Control and Information Sciences vol. 417, 2012, pp. 289-311.
%

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
% (C) 2012 
%     Stefano Riverso, Davide M. Raimondo
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, davide.raimondo(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------
    
    if nargin<2
        error('OUTERAPPROXIMATION: first input must be a zonotope and second input must be a function handle!');
    end
    
    if ~isa(X,'zonotope') || (~isa(W,'zonotope') && ~isempty(W)),
        error('OUTERAPPROXIMATION: Input argument must be a zonotope!');
    end
    
    if length(X)>1 || (~isempty(W) && length(W)>1),
        error('OUTERAPPROXIMATION: This function does not work with zonotope arrays!');
    end
    
    if ~isa(f,'function_handle')
        error('OUTERAPPROXIMATION: f function handle!');
    end
    
    % dimension of variables x and w
    nx = X.Dim;
    if isempty(W)
        nw = 0;
        W = zonotope;
    else
        nw = W.Dim;
    end
    W = W.computeHRep;
    vertexW = W.V;    
    
    x = sym('x',[nx+nw 1]);
    for i=1:nx+nw
        xstr{i} = num2str(i);
    end
    try
        F  = f(x(1:nx),x(nx+1:end));
        nu = size(F,1);
    catch err
        error('OUTERAPPROXIMATION: Z and W are not compatible with f(x,w)');
    end
    
    J = [];
    H = [];
    if nargin==3
        GF = [];
        HF = [];
    elseif nargin==4
        error('OUTERAPPROXIMATION: if g is expressed also h must be expressed');
    elseif nargin>6
        error('OUTERAPPROXIMATION: input parameters');
    end
    
    if nargin>=5
        if isempty(gf) && isempty(hf)
            GF = [];
            HF = [];
        else
            if ~isa(gf,'function_handle') || ~isa(hf,'function_handle')
                error('OUTERAPPROXIMATION: gf and hf function handle!');
            end
            GF = gf(x(1:nx),x(nx+1:end));
            HF = hf(x(1:nx),x(nx+1:end));
        end
    end
    
    if nargin==6
        % isfield jacobian and hessian from input
        if isfield(options,'J')
            J  = options.J;
            Jv = J(zeros(nx),zeros(nu));
            if size(Jv,1)~=nu || size(Jv,2)~=nx+nw
                error('OUTERAPPROXIMATION: dimension of J');
            end
            in = strrep(char(J),'];[','],[');
            J  = eval([in,';']);
        end
        if isfield(options,'H')
            H  = options.H;
            Hv = H(zeros(nx),zeros(nu));
            if size(Hv,1)~=nu*(nx+nw) || size(Hv,2)~=nx+nw
                error('OUTERAPPROXIMATION: dimension of H');
            end
            in = strrep(char(H),'];[','],[');
            H  = eval([in,';']);
        end
    end
    
    % Compute Jacobian and Hessian
    if isempty(J)
        J = jacobian(F,x);
        if isempty(GF)
            Htmp = jacobian(J,x);
            H = Htmp;
            for i=1:nu
                for j=i:nu:size(Htmp,1)
                    H(end+1,:) = Htmp(j,:);
                end
            end
            H(1:size(Htmp,1),:) = [];
            H = convertf(H,nx+nw,xstr);
        end
        J = convertf(J,nx+nw,xstr);
    end
    
    % Approximation using one zonotope see [1]
    [ Xu Q ] = oneStepDC(X,W,vertexW,f,GF,HF,J,H,F,xstr);
    
    if nargin<6 || isempty(isfield(options,'split')) || options.split.max_zono==1
        XuA = [];
        Xs  = X;
        if nargout>=4
            in = char(J);
            for i=nx+1:nx+nw
                in = strrep(in,['x(' xstr{i} ')'],['w(' xstr{i-nx} ')']);
            end
            in = strrep(in,'@(x)','@(x,w)');
            in = strrep(in,'],[','];[');
            J = eval([in,';']);
        end
        
        if nargout>=5
            if ~isempty(H)
                in = char(H);
                for i=nx+1:nx+nw
                    in = strrep(in,['x' xstr{i} ')'],['w(' xstr{i-nx} ')']);
                end
                in = strrep(in,'@(x)','@(x,w)');
                in = strrep(in,'],[','];[');
                H = eval([in,';']);
            end
        end
        return
    end
    
    % Algorithm for a better approximation see [2]
    LIST        = 1;
    val(1)      = norm(Q,'inf');
    now         = 1;
    pI          = X.p;
    GI          = X.G;
    sumZONOoutU = zeros(1,min(options.split.ngenerators,size(GI,2)));
    ZONO{1}     = Xu;
    ZONOIN{1}   = X;
    col         = min(options.split.ngenerators,size(GI,2));
    
    while length(LIST) ~= options.split.max_zono && now ~= -1
        cont = length(ZONO);
        for s=1:col
            tempp = split(zonotope(pI(:,now),GI((now-1)*nx+1:(now-1)*nx+nx,:)),s,options.split.alpha);
            temppZONOS = { tempp(1).p , tempp(2).p };
            tempGZONOS = { tempp(1).G , tempp(2).G };
            pI = [pI temppZONOS{1} temppZONOS{2}];
            GI = [GI ; tempGZONOS{1} ; tempGZONOS{2} ];
            ZONOIN{end+1} = zonotope( temppZONOS{1},tempGZONOS{1} );
            [ ZONO{end+1}, Q ] = oneStepDC( ZONOIN{end} , W , vertexW , f , GF , HF , J , H , F , xstr );
            if ZONO{end}.Dim~=0
                val = [ val norm(Q,'inf') ];
            else
                val = [ val -inf ];
            end
            ZONOIN{end+1} = zonotope( temppZONOS{2},tempGZONOS{2} );
            [ ZONO{end+1}, Q ] = oneStepDC( ZONOIN{end} , W , vertexW , f , GF , HF , J , H , F , xstr );
            if ZONO{end}.Dim~=0
                val = [ val norm(Q,'inf') ];
            else
                val = [ val -inf ];
            end
            sumZONOoutU(s) = ( val(end-1) + val(end));
        end
        [~, index] = min(sumZONOoutU);
        LIST = [LIST(1:find(LIST==now)-1) cont+index*2-1 cont+index*2 LIST(find(LIST==now)+1:end)];
        maxi = -inf;
        now = -1;
        for i=LIST
            if val(i) > maxi
                maxi = val(i);
                now = i;
            end
        end
    end
    
    XuA(1) = ZONO{LIST(1)};
    for i=2:length(LIST)
        XuA = [ XuA ZONO{LIST(i)} ];
    end
    
    if nargout==1
        Xu = XuA;
    end
    
    if nargout>=3
        Xs(1) = ZONOIN{LIST(1)}*W;
        for i=2:length(LIST)
            Xs = [ Xs ZONOIN{LIST(i)}*W ];
        end
    end
    
    if nargout>=4
        in = char(J);
        for i=nx+1:nx+nw
            in = strrep(in,['x(' xstr{i} ')'],['w(' xstr{i-nx} ')']);
        end
        in = strrep(in,'@(x)','@(x,w)');
        in = strrep(in,'],[','];[');
        J = eval([in,';']);
    end
    
    if nargout>=5
        if ~isempty(H)
            in = char(H);
            for i=nx+1:nx+nw
                in = strrep(in,['x' xstr{i} ')'],['w(' xstr{i-nx} ')']);
            end
            in = strrep(in,'@(x)','@(x,w)');
            in = strrep(in,'],[','];[');
            H = eval([in,';']);
        end
    end
    
end





function [out] = convertf(in,nx,xstr)
    in = strrep(char(in),'matrix','');
    for i=1:nx
        in = strrep(in,['x' xstr{i}],['x(' xstr{i} ')']);
    end
    out = eval(['@(x)',in,';']);
end





function [ Xnew, Q ] = oneStepDC(Xtmp,W,vertexW,f,GF,HF,J,H,F,xstr)
    fp = f(Xtmp.p,W.p);
    nu = size(fp,1);
    nw = W.Dim;
    nx = Xtmp.Dim;
    x  = sym('x',[nx+nw 1]);
    xT = sym('x',[1 nx+nw]);
    jp = reshape(J([Xtmp.p;W.p]),nx+nw,nu)';
    Fl = fp+jp*(x-[Xtmp.p;W.p]);
    fl = convertf(Fl,nx+nw,xstr);
    if isempty(GF)
        BB = bounding_box(Xtmp);
        try
            for i=1:size(BB.G,2)
                BBinterval(i,:) = infsup(-BB.G(i,i),BB.G(i,i))+BB.p(i);
            end
        catch err
            error('OUTERAPPROXIMATION: have you installed IntLab? http://www.ti3.tuhh.de/rump/intlab/ ')
        end
        alpha = findAlpha(reshape(H(BBinterval),nx+nw,(nx+nw)*nu)',nu);
        toBeConvex = alpha.*repmat(xT*x,nu,1);
        GF = F + toBeConvex ;
        HF = toBeConvex ;
    end
    
    Xtmp = Xtmp.computeVRep;
    vertexZ = Xtmp.V;
    
    if isempty(vertexW)
        tempMax = zeros(nu,size(vertexZ,1));
        tempMin = zeros(nu,size(vertexZ,1));
    else
        tempMax = zeros(nu,size(vertexZ,1)*size(vertexW,1));
        tempMin = zeros(nu,size(vertexZ,1)*size(vertexW,1));
    end
    
    gf   = convertf(GF,nx+nw,xstr);
    gfp  = gf([Xtmp.p;W.p])';
    JGF  = jacobian(GF,x);
    jgf  = convertf(JGF,nx+nw,xstr);
    jgfp = reshape(jgf([Xtmp.p;W.p]),nx+nw,nu)';
    GFL  = gfp + jgfp*(x-[Xtmp.p;W.p]);
    gfl  = convertf(GFL,nx+nw,xstr);
    
    hf   = convertf(HF,nx+nw,xstr);
    hfp  = hf([Xtmp.p;W.p])';
    JHF  = jacobian(HF,x);
    jhf  = convertf(JHF,nx+nw,xstr);
    jhfp = reshape(jhf([Xtmp.p;W.p]),nx+nw,nu)';
    HFL  = hfp + jhfp*(x-[Xtmp.p;W.p]);
    hfl  = convertf(HFL,nx+nw,xstr);
    
    if isempty(vertexW)
        for ii=1:size(vertexZ,1)
            tempMax(:,ii) = gf(vertexZ(ii,:)')'  - hfl(vertexZ(ii,:)')' - fl(vertexZ(ii,:)')' ;
            tempMin(:,ii) = gfl(vertexZ(ii,:)')' - hf(vertexZ(ii,:)')'  - fl(vertexZ(ii,:)')' ;
        end
    else
        for ii=1:size(vertexZ,1)
            for jj=1:size(vertexW,1)
                tempMax(:,(ii-1)*size(vertexW,1)+jj) = gf([vertexZ(ii,:)';vertexW(jj,:)'])'  - hfl([vertexZ(ii,:)';vertexW(jj,:)'])' - fl([vertexZ(ii,:)';vertexW(jj,:)'])' ;
                tempMin(:,(ii-1)*size(vertexW,1)+jj) = gfl([vertexZ(ii,:)';vertexW(jj,:)'])' - hf([vertexZ(ii,:)';vertexW(jj,:)'])'  - fl([vertexZ(ii,:)';vertexW(jj,:)'])' ;
            end
        end
    end
    
    epsilonMax = max(tempMax,[],2);
    epsilonMin = min(tempMin,[],2);
    
    t = (epsilonMax + epsilonMin) / 2;
    Q = diag((epsilonMax-epsilonMin)/2);
    
    Gu = [jp(:,1:nx)*Xtmp.G jp(:,nx+1:end)*W.G Q];
    
    Xnew = zonotope(fp+t,Gu);
end






function [alpha] = findAlpha(Hess,nu)
    % In order to be convex
    alpha = zeros(nu,1);
    n     = size(Hess,2);
    
    for i=1:nu
        Htmp     = Hess((i-1)*nu+1:i*nu,:);
        alphatmp = inf;
        while abs(alphatmp)>=1e-8
            eigenv   = gershgorin(Htmp);
            alphatmp = max([0 -0.5*min(mid(eigenv)-rad(eigenv))]);
            alpha(i) = alpha(i) + alphatmp;
            Htmp     = Htmp + alpha(i)*eye(n);
        end
    end
end