function [handle,titlehandle] = plotZ(varargin)
% PLOTZ Plots zonotopes in 2D or 3D
%
% handle = plotZ(varargin)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
%
% PLOTZ(Z,OPTIONS) plots zonotope Z if 1<=dimension(Z)<=3
%
% Example
%   plotZ(Z)             - plots Z in default (red) color
%   plotZ(Z,'g',Q,'b')   - plots Z in green and Q in blue color
%   plotZ(ZA,Z)          - plots array of zonotopes PA and a single zonotope P
%   plotZ(ZA,Options)    - plots ZA using options given in the Options structure
%
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% Zonotopes
% Options.extreme_solver  - which method to use for extreme points computation
%                           (see help extreme)
% Options.newfigure   - if 1, each plot command will open a new figure window
% Options.color='r'   - sets default color
% Options.colormap    - sets a different colormap (default: 'hsv')
% Options.gradcolor   - if 1, third coordinate of a 3D polytope is used to
%                       color the polytopes with a gradient coloring (default 0)
% Options.edgecolor   - color of edges (default is black 'k')
% Options.wire=1/0    - plots polytopes in wireframe
% Options.wirestyle   - a string giving style of the wireframe, e.g. '--' or '.-'
% Options.wirecolor   - color of the wireframe (black is default)
% Options.linewidth   - width of the border of polytope (1 is default)
% Options.shade=0-1   - level of transparency (0 - transparent faces, 1 - opaque faces)
% Options.zvalue      - when plotting 2D polytopes and this parameter is given,
%                       it will shift all plotted polytopes along the 3rd
%                       coordinate by this value
% Options.elevate     - if set to 1, every 1D or 2D polytope is elevated by an
%                       increasing value of the z-axis. E.g. plot(P1, P2, P3)
%                       will plot P1 with z=1, P2 with z=2 and P3 with z=3.
%                       default is 0. this option takes precedence from
%                       Options.zvalue.
% Options.xdimension  - whether to plot a section in 2D or 3D (allowed values: 2,3)
%                       default value is 3 if this field is not present
% Options.xsection    - through which states to cut the plot (values e.g. [4 5] for a
%                       cut along x4 and x5). If you have a 3D polytope and want
%                       to intersect it with a plane x3=0, just set
%                       Options.xsection=3 and this will do the trick
% Options.xvalues     - at which values to cut (e.g. [0 10] will cut the plot at x4=0, x5=10)
% Options.verbose     - level of verbosity
% Options.marker      - Which marker to use for 1D plots (no by default)
% Options.linestyle   - Style of the border of each polytope (could be etiehr
%                       '-', ':', ';' or 'none') (default is '-')
% Options.persp_sect  - If set to 1 and Options.xsection is given (i.e. cut is
%                       made, uses perspective plotting (off by default)
% Options.drawnow     - If true (default), the "drawnow" command is used to
%                       refresh current figure after all polytopes are drawn
% Options.sdpsettings - sdpsettings options
%
% Note: If Options is missing or some of the fields are not defined, the default
%       values from MPTOPTIONS will be used
%
% ---------------------------------------------------------------------------
% OUTPUT                                                                                                    
% ---------------------------------------------------------------------------
% handle      - handle of the figure
%
% see also ZONOTOPE
%

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
% (C) 2012 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    nii=nargin;
    Q=varargin{1};
    if ~isa(Q,'zonotope')
        error('PLOT: input argument MUST be a zonotope');
    end
    
    if nii>1
        if isstruct(varargin{nii})
            Options=varargin{nii};
            nii=nii-1;
        else
            Options=[];
        end
    else
        Options=[];
    end
    
    global MPTOPTIONS
    if isempty(MPTOPTIONS)
        MPTOPTIONS = mptopt;
    end
    
    
    lenQ=0; dimZ=0;
    for ii=1:nargin,
        if isa(varargin{ii},'zonotope'),
            dimZ=varargin{ii}.Dim;
            lenQ=lenQ+length(varargin{ii}); % determine how many zonotopes are contained in all input arguments
        end
    end
    
    if ~isfield(Options, 'sdpsettings')
        Options.sdpsettings = sdpsettings('cachesolver',1);
    end
    
    if ~isfield(Options, 'newfigure')
        Options.newfigure = 0;
    end
    
    if ~isfield(Options, 'abs_tol')
        Options.abs_tol = MPTOPTIONS.abs_tol;
    end
    
    if ~isfield(Options, 'wire')
        Options.wire = 0;
    end
    
    if ~isfield(Options, 'edgecolor')
        Options.edgecolor = 'k';
    end
    
    if ~isfield(Options, 'linestyle')
        Options.linestyle = '-';
    end
    
    if ~isfield(Options, 'linewidth')
        Options.linewidth = 1;
    end
    
    if ~isfield(Options, 'axis')
        Options.axis = 'auto';
    end
    
    if ~isfield(Options, 'fontsize')
        Options.fontsize = 10;
    end
    
    if ~isfield(Options, 'drawnow')
        Options.drawnow = 1;
    end
    
    if ~isfield(Options, 'verbose')
        Options.verbose = 0.5;
    end
    
    if ~isfield(Options, 'gradcolor')
        Options.gradcolor = 0;
    end
    
    
    
    if ~isfield(Options, 'color')       % default color
        % to differentiate constraints cycle through the available colors
        % colors='ymcrgbk'; % even white is not Ok, it clash with the background
        
        %Color maps.
        %    hsv        - Hue-saturation-value color map.
        %    hot        - Black-red-yellow-white color map.
        %    gray       - Linear gray-scale color map.
        %    bone       - Gray-scale with tinge of blue color map.
        %    copper     - Linear copper-tone color map.
        %    pink       - Pastel shades of pink color map.
        %    white      - All white color map.
        %    flag       - Alternating red, white, blue, and black color map.
        %    lines      - Color map with the line colors.
        %    colorcube  - Enhanced color-cube color map.
        %    vga        - Windows colormap for 16 colors.
        %    jet        - Variant of HSV.
        %    prism      - Prism color map.
        %    cool       - Shades of cyan and magenta color map.
        %    autumn     - Shades of red and yellow color map.
        %    spring     - Shades of magenta and yellow color map.
        %    winter     - Shades of blue and green color map.
        %    summer     - Shades of green and yellow color map.
        
        if isfield(Options, 'colormap'),
            Options.color = Options.colormap(1:lenQ, :);
        else
            auxcolors = hsv(lenQ);
            colors = auxcolors;
            multiplier=7;
            if mod(size(auxcolors,1),multiplier)==0,
                multiplier=multiplier+1;
            end
            for i=1:lenQ,
                jj=mod(i*multiplier,size(auxcolors,1))+1; % prepare enough colors for all zonotopes, cycle through a given color map
                colors(i,:)=auxcolors(jj,:);
            end
            colors = flipud(colors);
            Options.color = colors;
        end
    else
        if size(Options.color,1)~=length(varargin{1}),
            if size(Options.color,1)>length(varargin{1}),
                Options.color=Options.color(1:lenQ,:);
            else
                Options.color=repmat(Options.color(1, :),lenQ,1);
            end
        end
    end
    
    if ~isfield(Options, 'shade')       % transparency of plots
        if dimZ<=2,
            Options.shade=1;
        else
            Options.shade=0.5;
        end
    end
    if ~isfield(Options,'wirecolor'),
        Options.wirecolor='k';
        iswirecolor=0;
    else
        iswirecolor=1;
    end
    
    if size(Options.color,1)<lenQ,
        error('plot: Not enough colors!');
    end
    if isfield(Options,'xdimension'),
        plottabledimension=min(3,Options.xdimension);
    else
        plottabledimension=3;
    end
    
    if isfield(Options,'xsection'),
        if length(Options.xsection)<dimZ-plottabledimension,
            error(['Options.xsection MUST be a row vector with ' num2str(dimZ-plottabledimension) ' elements!']);
        end
        Indices=Options.xsection;
        plottabledimension=min(plottabledimension,dimZ-length(Indices));
    else
        Indices=plottabledimension+1:dimZ;
    end
    if isfield(Options,'xvalues'),
        Values=Options.xvalues;
    else
        Values=zeros(size(Indices));
    end
    if length(Values)~=length(Indices),
        error('Options.xsection and Options.xvalues MUST be row vectors of same dimension!');
    end
    
    IndicesAll = Indices;
    ValuesAll  = Values;
    
    index=0;
    
    if Options.newfigure,
        figure;  % open new figure window
    else
        newplot; % get current figure (or create new figure)
    end
    
    handle=[];
    
    titlehandle=[];
    cindex=0;
    emptypolymsgctr = 0;
    sectionemptyctr = 0;
    
    if dimZ>2 && Options.verbose>0,
        disp('Plotting...');
    end
    
    while index<nii
        index = index+1;
        Q = varargin{index};
        if ~isa(Q,'zonotope')
            error('PLOT: input argument MUST be a zonotope');
        end
        usercolor=0;
        if index<nii
            usercolor=0;
            if ischar(varargin{index+1})
                color=varargin{index+1};
                usercolor=1;
                index=index+1;
            else
                color=Options.color;
            end
        else
            color=Options.color;
        end
        
        maxlen=length(Q);
        
        for ii=1:maxlen
            cindex=cindex+1;
            if maxlen>1,
                Z = Q(ii);
                if usercolor==0,
                    color = Options.color(cindex,:);
                end
            else
                if usercolor==0,
                    color = Options.color(cindex,:);
                end
                Z = Q;
            end
            
            if all(all(abs(Z.G)<=Options.abs_tol))
                if isempty(indices),      % no section made
                    emptypolymsgctr = 1;
                else
                    sectionemptyctr = 1;  % section is empty
                end
                % warning will be displayed only once at the end of a file
                continue
            end
            
            [ dimZin, nG ] = size(Z.G);
            
            dimZ = min([dimZ,plottabledimension]);
            
            
            Indices = [];
            Values  = [];
            Xplottable = 1:dimZin;
            if ~isempty(IndicesAll)
                for i=1:length(IndicesAll)
                    Xplottable(IndicesAll(i)==Xplottable) = [];
                end
                if length(Xplottable)<dimZin
                    Indices = IndicesAll';
                    Values  = ValuesAll';
                end
            end
            
            if dimZ==1
                if isfield(Options,'xsection')
                    x  = sdpvar(dimZin,1);
                    d  = sdpvar(nG,1);
                    if isempty(Values)
                        vv = plot([ x==Z.p+Z.G*d    ;   norm(d,inf)<=1 ] , x([1,Xplottable]) , [] , [] , Options.sdpsettings );
                    else
                        vv = plot([ x==Z.p+Z.G*d    ;   norm(d,inf)<=1   ;   Z.p(Indices)+Z.G(Indices,:)*d==Values ] , x([1,Xplottable]) , [] , [] , Options.sdpsettings );
                    end
                    x1 = [ max(vv{1}(2,:),[],2) ; min(vv{1}(2,:),[],2) ];
                else
                    x1 = Z.p + [ Z.G ; -Z.G ];
                end
                if Options.elevate==1,
                    x2 = cindex*ones(size(x1));
                else
                    x2 = zeros(size(x1));
                end
                if isempty(Options.marker),
                    h=line(x1,x2,'Color',color(1,:),'LineWidth',5);
                else
                    h=line(x1,x2,'Color',color(1,:),'LineWidth',5,'Marker',Options.marker);
                end
                handle=[handle;h];
            elseif dimZ==2
                shade = Options.shade;
                
                x  = sdpvar(dimZin,1);
                d  = sdpvar(nG,1);
                if isempty(Values)
                    vv = plot([ x==Z.p+Z.G*d    ;   norm(d,inf)<=1 ] , x(Xplottable) , [] , [] , Options.sdpsettings );
                else
                    vv = plot([ x==Z.p+Z.G*d    ;   norm(d,inf)<=1   ;   Z.p(Indices)+Z.G(Indices,:)*d==Values ] , x(Xplottable) , [] , [] , Options.sdpsettings );
                end
                
                x1 = vv{1}(1,:);
                x2 = vv{1}(2,:);
                
                if Options.wire
                    h=line([x1; x1(1)],[x2; x2(1)]);
                    if iswirecolor==0,
                        wcolor=color(1,:);
                    else
                        wcolor=Options.wirecolor;
                    end
                    set(h,'Color',wcolor);
                    set(h,'LineWidth',Options.linewidth);
                    set(h,'LineStyle',Options.wirestyle);
                else
                    if isempty(Indices),
                        if length(Values)==1
                            if Options.persp_sect,
                                if indices==1,
                                    h=patch(values*ones(size(x1)),x1,x2,color(1,:),'EdgeColor',Options.edgecolor,'FaceAlpha',shade,'LineStyle',Options.linestyle);
                                elseif indices==2,
                                    h=patch(x1,values*ones(size(x1)),x2,color(1,:),'EdgeColor',Options.edgecolor,'FaceAlpha',shade,'LineStyle',Options.linestyle);
                                else
                                    h=patch(x1,x2,values*ones(size(x1)),color(1,:),'EdgeColor',Options.edgecolor,'FaceAlpha',shade,'LineStyle',Options.linestyle);
                                end
                            else
                                h=patch(x1,x2,values*ones(size(x1)),color(1,:),'EdgeColor',Options.edgecolor,'FaceAlpha',shade,'LineStyle',Options.linestyle);
                            end
                            zlabel(sprintf('x_{%d}',indices(1)));
                        else
                            h=patch(x1,x2,color(1,:),'EdgeColor',Options.edgecolor,'FaceAlpha',shade,'LineStyle',Options.linestyle);
                        end
                    else
                        if isfield(Options, 'zvalue'),
                            % move the 2D patch along the 3rd coordinate
                            zz = Options.zvalue*ones(size(x1));
                            h=patch(x1,x2,zz,color(1,:),'EdgeColor',Options.edgecolor,'FaceAlpha',shade,'LineStyle',Options.linestyle);
                        elseif Options.elevate==1,
                            % move the 2D patch along the 3rd coordinate
                            zz = cindex*ones(size(x1));
                            h=patch(x1,x2,zz,color(1,:),'EdgeColor',Options.edgecolor,'FaceAlpha',shade,'LineStyle',Options.linestyle);
                        else
                            h=patch(x1,x2,color(1,:),'EdgeColor',Options.edgecolor,'FaceAlpha',shade,'LineStyle',Options.linestyle);
                        end
                    end
                end
                if ~isempty(Indices),
                    Title='Cut through ';
                    for qq=1:length(Indices),
                        Title=[Title sprintf('x_{%d}=%.2f',Indices(qq),Values(qq)) ' '];
                    end
                    titlehandle=title(Title);
                end
                handle=[handle;h];
            elseif dimZ==3
                x  = sdpvar(dimZin,1);
                d  = sdpvar(nG,1);
                if isempty(Values)
                    plot([ x==Z.p+Z.G*d    ;   norm(d,inf)<=1 ] , x(Xplottable) , color , [] , Options.sdpsettings );
                    grid on
                else
                    plot([ x==Z.p+Z.G*d    ;   norm(d,inf)<=1   ;   Z.p(Indices)+Z.G(Indices,:)*d==Values ] , x(Xplottable) , color , [], Options.sdpsettings );
                    grid on
                end
                
                h = findobj(gca,'Type','Patch');
                if ~isempty(h)
                    h = h(1);
                end
                
                if Options.wire
                    if iswirecolor==0,
                        wcolor=color(1,:);
                    else
                        wcolor=Options.wirecolor;
                    end
                    set(h,'EdgeColor',wcolor);
                    set(h,'FaceColor','none');
                else
                    if Options.gradcolor
                        set(h,'EdgeColor',wcolor,'FaceAlpha',Options.shade,'EdgeColor',Options.edgecolor,'LineStyle',Options.linestyle);
                    else
                        set(h,'FaceColor',color,'FaceAlpha',Options.shade,'EdgeColor',Options.edgecolor,'LineStyle',Options.linestyle);
                    end
                end
                
                if ~isempty(Indices)
                    Title='Cut through ';
                    for qq=1:length(Indices),
                        Title=[Title sprintf('x_{%d}=%.2f',Indices(qq),Values(qq)) ' '];
                    end
                    titlehandle=title(Title);
                end
                handle=[handle;h];
            end
        end
    end
    
    axis(Options.axis);
    axislabels = 1:dimZ;
    if dimZ==1,
        xlabel(sprintf('x_{%d}',axislabels(1)),'FontSize',Options.fontsize);
    elseif dimZ==2,
        xlabel(sprintf('x_{%d}',axislabels(1)),'FontSize',Options.fontsize);
        ylabel(sprintf('x_{%d}',axislabels(2)),'FontSize',Options.fontsize);
        grid on
    elseif dimZ==3,
        xlabel(sprintf('x_{%d}',axislabels(1)),'FontSize',Options.fontsize);
        ylabel(sprintf('x_{%d}',axislabels(2)),'FontSize',Options.fontsize);
        zlabel(sprintf('x_{%d}',axislabels(3)),'FontSize',Options.fontsize);
        grid on
    end
    
    h=gcf;
    h1 = get(h,'CurrentAxes');
    set(h1,'Fontname','times');
    set(h1,'Fontsize',min(Options.fontsize,14));
    
    if emptypolymsgctr > 0 && Options.verbose > 0,
        disp('PLOT: Empty zonotope(s) detected!');
    end
    if sectionemptyctr>0 && Options.verbose > 0,
        disp('PLOT: Section through zonotope(s) at the given location is empty! Try to plot the projection (help projection) or specify Options.xvalues.');
    end
    
    if Options.drawnow,
        drawnow
    end
    % If no outputs is asked, clear the variable.
    if nargout == 0;
        clear('handle');
        clear('titlehandle');
    end
end