function R = plus(Z,Q)
% PLUS Minkowski sum of two zonotopes
%
% R = plus(Z,Q)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% R=Z+Q, or R=SUM(Z,Q) computes Minkowski sum of two zonotopes.
%
% NOTE: If Q ia a zonotope array, Q=[Q1 Q2 ... Qn], then Q is considered
% as the space covered with zonotopes Qi, and the plus.m computes Z+Q according
% to the definition Z+Q=\{x+y| x\in Z and y\in Q\}=[(Z+Q1) (Z+Q2) ... (Z+Qn)]
%
% USAGE:
%   R=Z+Q
%   R=plus(Z,Q)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% Z,Q                     - Zonotopes or arrays of zonotopes
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% R   - zonotope (if both Z and Q are zonotoeps)|polytope describing the 
%       Minkowski sum
%
% see also MINUS, EXTREME, MLDIVIDE
%

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
% (C) 2012 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if ~isa(Z, 'zonotope')
        if ~isa(Q, 'zonotope')
            error('PLUS: An argument MUST be a zonotope object');
        else
            tmp = Q;
            Q   = Z;
            Z   = tmp;
        end
    end
    
    dimZ = length(Z);
    if isa(Q, 'zonotope')
        % Z zonotope Q zonotope
        dimQ = length(Q);
        R(dimZ*dimQ,1) = zonotope;
        for i=1:dimZ
            for j=1:dimQ
                R(dimZ*(i-1)+j,1) = zonotope(Z(i).p+Q(j).p,[Z(i).G Q(j).G]);
            end
        end
        return
    elseif isreal(Q)
        % Z zonotope Q real
        R(dimZ,1) = zonotope;
        for i=1:dimZ
            R(i) = zonotope(Z(i).p+Q,Z(i).G);
        end
    else
        error('PLUS: Second rgument MUST be a zonotope object or a real number');
    end
    
end