function Z = reduce(Z,ngen)
% REDUCE Reduces the number of generators of a given zonotope
%
% Z = reduce(Z,ngen)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Reduce the number of generators of a given zonotope
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% Z    - Zonotope
% ngen - number of new generators
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% Z    - zonotope with ngen generators
%
% see also ZONOTOPE, DOUBLEPG, DOUBLEHK
%

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
% (C) 2012 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if ~isa(Z, 'zonotope')
        error('REDUCE: Argument MUST be a zonotope object');
    end
    
    if length(Z)~=1
        error('REDUCE: This function does not work with zonotope arrays!');
    end
    
    G = Z.G;
    if ngen>size(G,2) || ngen<size(G,1)
        error('REDUCE: Number of new generators not valid');
    end
    
    n = size(G,1);    
    
    Greduced = G(:,sum(G,1)~=0);
    
    dime = size(Greduced,2);
    if (dime - floor(dime / n)*n)~=0
        Greduced = [Greduced zeros(size(Greduced,1),1)];
    end
    
    normG = sqrt(sum(Greduced.^2,1));
    [~, ind] = sort(normG,'descend');
    
    Greduced = Greduced(:,ind);
    
    if size(Greduced,2) > ngen
        tmp1 = Greduced(:,1:ngen-n);
        tmp2 = Greduced(:,ngen-n+1:end);
        tmp3 = zeros(n,n);
        for i=1:n
            tmp3(i,i)=sum(abs(tmp2(i,:)));
        end
        Greduced = [tmp1 tmp3];
    end
    
    Z = zonotope(Z.p,Greduced);
    
end