function Zn = split(Z,gener,alpha)
% SPLIT Split of zonotope Z in two zonotope
%
% Zn = split(Z,gener,alpha)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Split of zonotope Z along generator 'gener', splitted at 'alpha'
%
%   Zn = SPLIT(Z) returns the split of a zonotope along the generator
%   'gener', at 'alpha'.
%
% USAGE:
%   Zn = SPLIT(Z)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% Zonotope Z
% gener = generator along to split
% alpha = split the zonotope in two zonotopes, Z1=alpha*Z, Z2=(1-alpha)*Z
%
% ---------------------------------------------------------------------------
% OUTPUT                                                                                                    
% ---------------------------------------------------------------------------
% Zn - split of zonotope Z
%
% see also BISECT

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
% (C) 2012 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if ~isa(Z,'zonotope')
        error('SPLIT: Input argument must be a zonotope!');
    end
    
    if length(Z)~=1
        error('SPLIT: This function does not work with zonotope arrays!');
    end
    
    if nargin == 1
        [ ~ , idx ] = max(max(abs(Z.G)));
        gener = idx;
    end
    
    if nargin < 3
        alpha = 0.5;
    end
    
    if alpha<0 || alpha>1
        error('SPLIT: alpha>=0 && alpha<=1');
    end
    
    GL          = Z.G;
    GR          = Z.G;
    GL(:,gener) = Z.G(:,gener)*alpha;
    GR(:,gener) = Z.G(:,gener)*(1-alpha);
    pL          = (Z.p-GR(:,gener));
    pR          = (Z.p+GL(:,gener));
    
    Zn          = [ zonotope(pL,GL) zonotope(pR,GR) ];
    
end