classdef zonotope < Polyhedron
% ZONOTOPE define a new zonotope object
% 
% Z = zonotope(p,G)
%
% A zonotope is defined as Z = { z | z=p+Gd , ||d||inf<=1 }
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return a zonotope object
% 
% Z = ZONOTOPE(p,G)
% Z = ZONOTOPE(vertices)
% Z = ZONOTOPE(P)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - p and G : matrices that completely define zonotope Z
% - vertices : vertices of the zonotope
% - P : polytope object
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% Z zonotope object
%
% ---------------------------------------------------------------------------
% PROPERTIES
% ---------------------------------------------------------------------------
% - p, G : matrices such that Z = { z | z=p+Gd , ||d||inf<=1 }
% - H, K : matrices such that Z = { z | H*z<=K }
% - vertices : extreme points of zonotope Z
% - xCheb, RCheb : center and ray of the Chebyshev ball
%
% ---------------------------------------------------------------------------
% METHODS
% ---------------------------------------------------------------------------
% see help of each method
% 


% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
% (C) 2012 
%     Stefano Riverso, Davide M. Raimondo
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, davide.raimondo(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

     properties ( SetAccess = private , Hidden = false )
        p       ;
        G       ;
    end
    
    properties ( SetAccess = private , Hidden = false )
        hasGRep = false;
    end
    
    methods     

        function Z = zonotope(par1,par2,par3)
            
            global pnpmpcToolboxGlobalOptions;
            if ~isstruct(pnpmpcToolboxGlobalOptions),
                pnpmpc_toolbox_err;
            end
            
            global MPTOPTIONS
            if isempty(MPTOPTIONS)
                MPTOPTIONS = mptopt;
            end
            
            if nargin>3
                error('ZONOTOPE: input parameters')
            end
            
            if nargin==1 && isa(par1,'Polyhedron')
                tmp = par1(1);
            elseif nargin==3 && isa(par3,'Polyhedron')
                tmp = par3;
            else
                tmp = Polyhedron;
            end
            
            if ~isempty(tmp.Ae)
                error('ZONOTOPE: Polyhedron in input has equality constraints')
            end
            
            Z = Z@Polyhedron('irredundantVRep',tmp.irredundantVRep,'irredundantHRep',tmp.irredundantHRep,'H',[tmp.A tmp.b],'He',[tmp.Ae tmp.be],'V',tmp.V,'R',tmp.R);
            
            if nargin==1 && length(par1)~=1 && (isa(par1,'Polyhedron') || isa(par1,'zonotope'))
                for i=1:length(par1)
                    Z(i) = zonotope(par1(i));
                    if Z(i).Dim==0 && par1(i).Dim~=0
                        error('ZONOTOPE: Polyhedron %d is not a zonotope',i)
                    end
                end
                return
            end
            
            if nargin==1 && isa(par1,'zonotope')
                Z = par1;
                Z.Dim = 0;
                return;
            elseif nargin==1 && isa(par1,'Polyhedron')
                tmp = Polyhedron2zonotope(par1);
                if isempty(tmp)
                    Z = zonotope;
                else
                    Z = tmp;
                end
                return;
            end
            
            if nargin==0
                return;
            end
            
            if nargin>=2
                Z.p   = par1;
                Z.G   = par2;
                
                nc    = size(Z.G,1);
                if nc<1
                    error('ZONOTOPE: Dimension of a zonotope has to be >= 1');
                end
                if size(Z.p,1)~=nc || size(Z.p,2)~=1
                    error('ZONOTOPE: p (center) should be a column vector of length %d',nc);
                end
                
                if issparse(Z.p),
                    Z.p = full(Z.p);
                end
                if issparse(Z.G),
                    Z.G = full(Z.G);
                end
                Z.p(Z.p>=MPTOPTIONS.abs_tol^-1) = 0;
                Z.G(abs(Z.G)<=MPTOPTIONS.abs_tol) = 0;
                
                Z.p(Z.p>=1e6)  =  1e6;
                Z.G(Z.G>=1e6)  =  1e6;
                Z.p(Z.p<=-1e6) = -1e6;
                Z.G(Z.G<=-1e6) = -1e6;
                
                if all(all(Z.G==0)) && all(Z.p==0)
                    Z.p = [];
                    Z.G = [];
                    return
                end
                if size(Z.G,1)~=1
                    Z.G           = Z.G(:,any(Z.G~=0));
                    ind           = all(Z.G<=0);
                    Z.G(:,ind)    = -Z.G(:,ind);
                    [ ~ , index ] = sortrows(abs(Z.G)',1:size(Z.G,1));
                    Z.G           = Z.G(:,index);
                    [ ~ , index ] = sort(sum(abs(Z.G)),'descend');
                    Z.G           = Z.G(:,index);
                    i = 1;
                    while i<size(Z.G,2)
                        j = i+1;
                        while j<=size(Z.G,2)
                            tmp = abs(Z.G(:,i))./abs(Z.G(:,j));
                            if sum(isfinite(tmp))==1
                                j = j+1;
                                continue
                            else
                                tmp(isnan(tmp)) = tmp(1);
                                tmp(isinf(tmp)) = tmp(1);
                                if all(abs(tmp-tmp(1))<=MPTOPTIONS.abs_tol)
                                    Z.G(:,i) = Z.G(:,i) + Z.G(:,j);
                                    Z.G(:,j) = [];
                                else
                                    j = j+1;
                                end
                            end
                        end
                        i = i+1;
                    end
                    if Z.G(1,1)<0
                        Z.G = -Z.G;
                    end
                else
                    Z.G = sum(abs(Z.G));
                end
                
                Z.hasGRep = true;
                
                Z.Dim = size(Z.G,1);
            end
            
            if nargin==3
                Z.p       = par1;
                Z.G       = par2;
                Z.hasGRep = true;
            end
            
        end
        
        
        function new = loadobj(Z)
            new = Z;
        end
            
    end
    
end