function plot(obj,dimx,dimu)
% PLOT Plot an RCI set and the set of the values of the control input that guarantee robust invariance
%
% plot(obj,dimx,dimu)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Plot the RCI set stored in the zonotope RCI object obj and the set of 
% the values of control input that guarantee robust invariance
% 
% USAGE: 
%  
% obj.PLOT
% obj.PLOT(dimx,dimu)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% dimx = indices of the states to plot (optional)
% 
% dimu = indices of the input to plot (optional)
% 
%
% see also ZONOTOPERCI

% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Stefano Riverso, Giuseppe Giorgio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, giuseppe.giorgio01@universitadipavia.it,
%     giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin==3
        if isempty(dimu)
            dimu = 1:min([size(obj.B,2),3]);
        else
            if any(dimu>size(obj.B,2))
                error('PLOT: number of control variables')
            end
        end
        if isempty(dimx)
            dimx = 1:min([size(obj.A,1),3]);
        else
            if any(dimx>size(obj.A,2))
                error('PLOT: number of state variables')
            end
        end
    elseif nargin==2
        dimu = 1:min([size(obj.B,2),3]);
        if isempty(dimx)
            dimx = 1:min([size(obj.A,1),3]);
        else
            if any(dimx>size(obj.A,2))
                error('PLOT: number of state variables')
            end
        end
    elseif nargin==1
        dimu = 1:min([size(obj.B,2),3]);
        dimx = 1:min([size(obj.A,1),3]);
    else
        error('PLOT: number of parameters')
    end
    
    subplot(1,2,1)
    title('Zonotope RCI Set: state')
    hold on    
    p = obj.Zrci.p;
    G = obj.Zrci.G;
    if length(dimx)>1
        d      = sdpvar(size(G,2),1);
        x      = sdpvar(size(obj.A,1),1);
        plot( [ norm(d,inf)<=1 ; x == p + G*d ] , x(dimx) , 'r' );
    else
        plot([p-G,p+G],zeros(1,2),'LineWidth',4,'Color','r')
    end
    if length(dimx)>=1
        xlabel(['x_{' num2str(dimx(1)) '}'],'FontSize',16,'FontName','Arial');
    end
    if length(dimx)>=2
        ylabel(['x_{' num2str(dimx(2)) '}'],'FontSize',16,'FontName','Arial');
    end
    if length(dimx)==3
        zlabel(['x_{' num2str(dimx(3)) '}'],'FontSize',16,'FontName','Arial');
    end
    hold off
    
    
    subplot(1,2,2)
    title('Zonotope RCI Set: control')
    hold on
    v = sdpvar(size(obj.f,1),obj.k);
    d = sdpvar(size(obj.E,2),obj.k);
    x = sdpvar(size(obj.A,1),1);
    u = sdpvar(size(obj.B,2),1);
    CONS = [ obj.Drot*v(:)==x ; u == obj.Mrot*v(:) ];
    for i = 1:obj.k
        CONS = [ CONS ; v(:,i)==obj.E*d(:,i)+obj.f ; norm(d(:,i),'inf')<=1 ];
    end
    if length(dimu)>1        
        plot( CONS , u(dimu) , 'b' );
    else
        dx = sdpvar(size(G,2),1);
        CONS = [ CONS ; norm(dx,inf)<=1 ; x == p + G*dx ];
        info = solvesdp(CONS,-u,sdpsettings('verbose',0));
        if info.problem==0
            maxu = double(u);
            info = solvesdp(CONS,u,sdpsettings('verbose',0));
            if info.problem==0
                minu = double(u);
                plot([minu,maxu],zeros(1,2),'LineWidth',4,'Color','b')
            else
                warning('PLOT: error for subplot Parameterized Robust Control Invariant Set: control')
            end
        else
            warning('PLOT: error for subplot Parameterized Robust Control Invariant Set: control')
        end    
    end
    
    if length(dimu)>=1
        xlabel(['u_{' num2str(dimu(1)) '}'],'FontSize',16,'FontName','Arial');
    end
    if length(dimu)>=2
        ylabel(['u_{' num2str(dimu(2)) '}'],'FontSize',16,'FontName','Arial');
    end
    if length(dimu)==3
        zlabel(['u_{' num2str(dimu(3)) '}'],'FontSize',16,'FontName','Arial');
    end
    hold off
    
end