function [ uInv, info ] = uInv(obj,x,options)
% UINV Compute the control input associated to an RCI that guarantees robust control invariance
%
% uInv = uInv(obj,x,options)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Compute the control input associated to the zonotope RCI object obj 
% that guarantees robust control invariance
% 
% USAGE: 
%  
% uInv = obj.UINV(x,options)
% uInv = obj.UINV(x)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% x = point of the same dimension of the RCI
% 
% options = sdpsettings object for YALMIP (optional)
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% uInv: value of the control input
% info: Diagnostic information of solvesdp (see YALMIP)
%
% The control input is computed as in [1]
% [1] S. V. Rakovic, Robust Control of Constrained Discrete Time Systems: 
% Characterization and Implementation, Imperial College London, University 
% of London, 2005.
%
%
% see also ZONOTOPERCI

% Copyright is with the following author(s):
%
% (C) 2012 
%     Stefano Riverso, Giuseppe Giorgio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, giuseppe.giorgio01@universitadipavia.it,
%     giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    if nargin~=3
        options = sdpsettings;
    end
    
    if size(x,1)~=size(obj.A,1) || size(x,2)~=1
        error('UINV: parameters dimensions');
    end
    
    v = sdpvar(size(obj.f,1),obj.k);
    d = sdpvar(size(obj.E,2),obj.k);
    
    CONS = obj.Drot*v(:)==x ;
    for i = 1:obj.k
        CONS = [ CONS ; v(:,i)==obj.E*d(:,i)+obj.f ; norm(d(:,i),'inf')<=1 ];
    end
    
    info = solvesdp(CONS,norm(v(:),2)^2,options);
    
    if info.problem==0
        uInv = obj.Mrot*double(v(:));    
        uInv(abs(uInv)<=eps) = 0;
    elseif info.problem == 1
        error('UINV: x is not inside R')
    else
        if isinside(obj.Zrci,x) == 1
            error('UINV: ERROR SOLVER: %s',info.info);
        else
            error('UINV: x is not inside R');
        end
    end  
end