classdef zonotopeRCI
% ZONOTOPERCI compute a zonotopic Robust Control Invariant Set for a Linear System as proposed in [1]
% 
% Z = zonotopeRCI(A,B,X,U,W,k,q,options)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Return a zonotopic Robust Control Invariant Set
% 
% Z = ZONOTOPERCI(A,B,X,U,W,k)
% Z = ZONOTOPERCI(A,B,X,U,W,k,q)
% Z = ZONOTOPERCI(A,B,X,U,W,k,q,options)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - Matrices A and B of the linear system x(t+1)=Ax(t)+Bu(t) with n states
% and m inputs
% - k must be >= n
% - X is a cell array defined as {cx,dx} where cx*x<=dx
% - U is a cell array defined as {cu,du} where cu*u<=du
% - W is a cell array defined as {f,E} where w=f+Ed,|d|<=1 describe a
% zonotopic set W. In order to reduce the complexity of zonotopeRCI class,
% one could use {f,E;cw,dw} where cw*w<=dw
% - q is a vector of three elements for the weights of the cost function.
% see [1]. if q is not in input, q = [ 1 1 1 ].
% - options from YALMIP, i.e. sdpsettings object (optional). An LP solver 
%   is needed.
%
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% Z is a zonotopeRCI object
%
% ---------------------------------------------------------------------------
% PROPERTIES
% ---------------------------------------------------------------------------
% A, B, k, cx, dx, cu, du, cw, dw, f, E, q : the inputs are stored in the
%                                            zonotopeRCI object
% info : object with information of the optimization problem (see solvesdp
%        of YALMIP)
% Zrci : zonotope object of the RCI set, with G-representation
% 
%
% ---------------------------------------------------------------------------
% METHODS
% ---------------------------------------------------------------------------
% ISINSIDE: checks if a given point x lies in the interior of the RCI
% PLOT: plot the robust control invariant set
% UINV: compute the control input to guarantee robust control invariance
% 
%
% [1] S. V. Rakovic, Robust Control of Constrained Discrete Time Systems: 
% Characterization and Implementation, Imperial College London, University 
% of London, 2005.
    
% Copyright is with the following author(s):
%
% (C) 2014
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
% (C) 2012 
%     Stefano Riverso, Giuseppe Giorgio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giuseppe.giorgio01(at)universitadipavia.it,
%     giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% --------------------------------------------------------------------------- 
    

    properties (SetAccess = private)
        A      ;
        B      ;
        k      ;
        cx     ;
        dx     ;
        cu     ;
        du     ;
        cw     ;
        dw     ;
        f      ;
        E      ;
        q      ;
        info   ;
        Zrci   ;
    end
    
    
    
    properties (SetAccess = private , Hidden = true)
        Drot   ;
        Mrot   ;
    end
    
     
    
    methods
        function [ obj ] = zonotopeRCI(A,B,X,U,W,k,q,options)
            
            if nargin<8
                options = sdpsettings;
            end
            
            if isempty(B)
                B = zeros(size(A,2),1);
            end
            
            if ~iscell(X) || length(X)~=2
                error('ZONOTOPERCI: X is not a cell array of two elements. see help zonotopeRCI');
            end
            if ~iscell(U) || length(U)~=2
                error('ZONOTOPERCI: U is not a cell array of two elements. see help zonotopeRCI');
            end
            if ~iscell(W) || size(W,2)~=2 || size(W,1)<1 || size(W,1)>2 
                error('ZONOTOPERCI: W is not a cell array of two or four elements. see help zonotopeRCI');
            end
                    
            cx    = X{1,1};
            dx    = X{1,2};
            
            cu    = U{1,1};
            du    = U{1,2};
            
            f     = W{1,1};
            E     = W{1,2};
            
            t     = size(E,2);
            n     = size(A,2);
            m     = size(B,2);
            qu    = size(cu,1);
            qx    = size(cx,1);  
            
            if k<n
                error('ZONOTOPERCI: k is less than n=%s',num2str(n));
            end
            
            if ( ~isempty(cx) && (size(cx,2)~=n || size(dx,1)~=qx || size(dx,2)~=1) ) || size(A,2)~=n || size(B,1)~=n || ( ~isempty(cu) && (size(cu,2)~=m || size(du,1)~=qu || size(du,2)~=1))
                error('ZONOTOPERCI: parameters dimensions');
            end
            
            if isempty(f) || isempty(E) || size(E,1)~=n || size(f,1)~=n
                error('ZONOTOPERCI: parameters dimensions of set W');
            end
            
            if size(W,1)==1
                tmp = zonotope(f,E).computeHRep;
                cw  = tmp.A;
                dw  = tmp.b;
            else
                cw = W{2,1};
                dw = W{2,2};
                if isempty(cw) || isempty(dw) || size(cw,1)~=size(dw,1) || size(cw,2)~=n
                    error('ZONOTOPERCI: parameters dimensions of set W');
                end
            end
            
            qw    = size(cw,1);
            
            if nargin<7 || isempty(q)     
                q = ones(1,3);
            end
            
            qa    = q(1);
            qb    = q(2);
            qp    = q(3);        
            
            alpha = sdpvar(1,1);
            beta  = sdpvar(1,1);
            delta = sdpvar(1,1);
            phi   = sdpvar(1,1);
            T     = sdpvar( qu , t*k     , 'full' );
            M     = sdpvar( m  , n*k     , 'full' );
            L     = sdpvar( qx , t*k     , 'full' );
            D     = sdpvar( n  , n*(k+1) , 'full' );
            Lw    = sdpvar( qw , t       , 'full' );
            
            vone  = ones(t,1);
           
            %% cost function constraints
            CONS = [ qa*alpha + qb*beta + qp*phi <= delta ; 0<=alpha<=1-1e-6 ; 0<=beta<=1-1e-6 ; 0<=phi<=1-1e-6 ; alpha+phi<=1 ; beta+phi<=1 ];
            
            %% input constraints
            if qu~=0
                for i=0:k-1
                    CONS = [ CONS ; -T(:,(i*t)+1:(i+1)*t)<=cu*M(:,(i*n)+1:(i+1)*n)*E<=T(:,(i*t)+1:(i+1)*t) ];
                end
                sumT = 0;
                for i=0:k-1
                    sumT = sumT + T(:,(i*t)+1:(i+1)*t)*vone+cu*M(:,(i*n)+1:(i+1)*n)*f;
                end
                CONS = [ CONS ; sumT<=beta*du ];
            end
            
            %% dynamics
            D(:,1:n) = eye(n);
            CONS = [ CONS ; Lw*vone+cw*D(:,(k*n)+1:(k+1)*n)*f<=phi*dw ; -Lw <= cw*D(:,(k*n)+1:(k+1)*n)*E <= Lw ] ;
            for i=1:k
                sumT = 0;
                for j=0:i-1
                    sumT = sumT + A^(i-1-j)*B*M(:,(j*n)+1:(j+1)*n);
                end
                CONS = [ CONS ;  D(:,(i*n)+1:(i+1)*n)==A^i+sumT ];
            end
            
            
            %% state constraints
            if qx~=0
                for i=0:k-1
                    CONS = [ CONS ; -L(:,(i*t)+1:(i+1)*t)<=cx*D(:,(i*n)+1:(i+1)*n)*E<=L(:,(i*t)+1:(i+1)*t) ];
                end
                sumT=0;
                for i=0:k-1
                    sumT = sumT + L(:,(i*t)+1:(i+1)*t)*vone+cx*D(:,(i*n)+1:(i+1)*n)*f;
                end
                CONS = [ CONS ; sumT<=alpha*dx ];
            end
                      
            info = solvesdp(CONS,delta,options);
            
            if info.problem~=0
                error(info.info);
            end
            
            D  = double(D);
            M  = double(M);
            
            GenRci = [];
            pRci   = 0;
            for i=0:k-1
                GenRci = [ GenRci D(:,(i*n)+1:(i+1)*n)*E ];
                pRci   = pRci   + D(:,(i*n)+1:(i+1)*n)*f;
            end
            
            GenRci = (1-double(phi))^-1*GenRci;
            pRci   = (1-double(phi))^-1*pRci;
            
            GenRci = GenRci(:,any(GenRci~=0));
            GenRci(abs(GenRci)<=eps) = 0;
            
            
            obj.A    = sparse(A) ;
            obj.B    = sparse(B) ;
            obj.k    = k ;
            obj.cx   = sparse(cx) ;
            obj.dx   = sparse(dx) ;
            obj.cu   = sparse(cu) ;
            obj.du   = sparse(du) ;
            obj.cw   = sparse(cw) ;
            obj.dw   = sparse(dw) ;
            obj.f    = sparse(f) ;
            obj.E    = sparse(E) ;
            obj.q    = q ;
            obj.info = info ;
            obj.Zrci = zonotope(pRci,GenRci) ;
            
            
            
            M(abs(M)<=eps) = 0;
            D(abs(D)<=eps) = 0;
            
            D(:,k*n+1:end) = [];
            Dcontr = [];
            Mcontr = [];
            
            for i=k-1:-1:0
                Dcontr = [ Dcontr , D(:,(i*n)+1:(i+1)*n)];
                Mcontr = [ Mcontr , M(:,(i*n)+1:(i+1)*n)];
            end
            
            
            obj.Drot = Dcontr;
            obj.Mrot = Mcontr;
            
            
        end
        
    end
    
end