function [ modelTrucks2Dc, modelTrucks2Dd ] = makeModelTrucks2D(N)
% MAKEMODELTRUCKS2D create model trucks in 2D
%

% Copyright is with the following author(s):
%
% (C) 2012 
%     Stefano Riverso, Alberto Battocchio, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, alberto.battocchio01@universitadipavia.it,
%     giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end 

    if mod(N,sqrt(N))
        error('MAKEMODELTRUCKS2D: sqrt(N) must be integer')
    end    
    
    ele           = sqrt(N);    
    r             = zeros(1,ele);
    r(2)          = 1;
    M1            = toeplitz(r,r);
    
    matrixCoupled = M1;
    for i=2:sqrt(N)
        matrixCoupled                                                          = blkdiag(matrixCoupled,M1);
        matrixCoupled((i-2)*sqrt(N)+1:(i-1)*sqrt(N),(i-1)*sqrt(N)+1:i*sqrt(N)) = eye(sqrt(N));
        matrixCoupled((i-1)*sqrt(N)+1:i*sqrt(N),(i-2)*sqrt(N)+1:(i-1)*sqrt(N)) = eye(sqrt(N));
    end
    
    matrixCoupled = sparse(matrixCoupled==1);
    
    clear r M1
    
    minM          = 8;
    maxM          = 10;
    mink          = 0.5;
    maxk          = 0.5;
    minh          = 0.5;
    maxh          = 0.5;
    
    % constraints on state
    maxx          = 1.5;
    minx          = 1.5;
    maxy          = 1.5;
    miny          = 1.5;
    maxvx         = 0.8;
    minvx         = 0.8;
    maxvy         = 0.8;
    minvy         = 0.8;
    
    % values to choose constraints on input
    maxu          = 1.5;
    minu          = 1.0;
    
    
    M             = minM+rand(1,N)*(maxM-minM);
    kx            = mink+rand(ele,ele-1)*(maxk-mink);
    ky            = mink+rand(ele-1,ele)*(maxk-mink);
    hx            = mink+rand(ele,ele-1)*(maxh-minh);
    hy            = mink+rand(ele-1,ele)*(maxh-minh);
    
    tic
    modelTrucks2Dc     = lss;
    
    % local dynamics for each truck
    for n = 1:N
        i = fix((n-1)/ele)+1;
        j = n-(i-1)*ele;
        kxb = 0;
        kxp = 0;
        kyb = 0;
        kyp = 0;
        hxb = 0;
        hxp = 0;
        hyb = 0;
        hyp = 0;
        if j-1>0
            kxb = kx(i,j-1);
            hxb = hx(i,j-1);
        end
        if j+1<=ele
            kxp = kx(i,j);
            hxp = hx(i,j);
        end
        if i-1>0
            kyb = ky(i-1,j);
            hyb = hy(i-1,j);
        end
        if i+1<=ele
            kyp = ky(i,j);
            hyp = hy(i,j);
        end
        modelTrucks2Dc = modelTrucks2Dc.addSystem ([         0                  1                0                 0           ;
                                               -(kxb+kxp)/M(n)   -(hxb+hxp)/M(n)           0                 0           ;
                                                       0                  0                0                 1           ;
                                                       0                  0         -(kyb+kyp)/M(n)   -(hyb+hyp)/M(n)    ] , [   0          0    ;
                                                                                                                               100/M(n)     0    ;
                                                                                                                                 0          0    ;
                                                                                                                                 0      100/M(n) ] ) ;
    end
    
    % coupling terms among trucks
    for n = 1:modelTrucks2Dc.numSys
        tmp = find(matrixCoupled(n,:)==1);
        for k = tmp
            i = fix((n-1)/ele)+1;
            j = n-(i-1)*ele;
            if k==n-1
                kxb = kx(i,j-1);
                hxb = hx(i,j-1);
                modelTrucks2Dc = modelTrucks2Dc.addCoupling( n , k , [         0                  0                0                 0           ;
                                                                      kxb/M(n)           hxb/M(n)            0                 0           ;
                                                                         0                  0                0                 0           ;
                                                                         0                  0                0                 0           ] , zeros(4,2) );
                continue;
            end
            if k==n+1
                kxp = kx(i,j);
                hxp = hx(i,j);
                modelTrucks2Dc = modelTrucks2Dc.addCoupling( n , k , [         0                  0                0                 0           ;
                                                                      kxp/M(n)           hxp/M(n)            0                 0           ;
                                                                         0                  0                0                 0           ;
                                                                         0                  0                0                 0           ] , zeros(4,2) );
                continue;
            end
            if k<n
                kyb = ky(i-1,j);
                hyb = hy(i-1,j);
                modelTrucks2Dc = modelTrucks2Dc.addCoupling( n , k , [         0                  0                0                 0           ;
                                                                         0                  0                0                 0           ;
                                                                         0                  0            kyb/M(n)           hyb/M(n)       ;
                                                                         0                  0                0                 0           ] , zeros(4,2) );
                continue;
            end
            if k>n
                kyp = ky(i,j);
                hyp = hy(i,j);
                modelTrucks2Dc = modelTrucks2Dc.addCoupling( n , k , [         0                  0                0                 0           ;
                                                                         0                  0                0                 0           ;
                                                                         0                  0            kyp/M(n)           hyp/M(n)       ;
                                                                         0                  0                0                 0           ] , zeros(4,2) );
                continue;
            end
            
        end
    end
    
    
    clear M kx ky hx hy matrixCoupled
    
    % state constraints
    for i=1:modelTrucks2Dc.numSys
        modelTrucks2Dc = modelTrucks2Dc.addStateConstraint(i,[eye(4);-eye(4)],[maxx,maxvx,maxy,maxvy,minx,minvx,miny,minvy]');
    end
    
    % input constraints
    u = linspace(minu,maxu,10);
    for i=1:modelTrucks2Dc.numSys
        ind = fix(length(u)*rand(1)+1);
        modelTrucks2Dc = modelTrucks2Dc.addInputConstraint(i,[eye(2);-eye(2)],[u(ind),u(ind),u(ind),u(ind)]');
    end
    
    time_modelC = toc;
    save([pnpmpcToolboxGlobalOptions.pathInstalled '/examples/pnpmpc/2D/modelTrucks2Dc'],'modelTrucks2Dc','time_modelC')
    
    tic
    modelTrucks2Dd = modelTrucks2Dc.clss2dlss(0.2,'subsystem');
    time_modelDss = toc;
    save([pnpmpcToolboxGlobalOptions.pathInstalled '/examples/pnpmpc/2D/modelTrucks2Dd'],'modelTrucks2Dd','time_modelDss')
    
end