function [ Z, P, indZ, indP ] = Polyhedron2zonotope(P)
% POLYHEDRON2ZONOTOPE Convert a Polyhedron into a zonotope
%
% [ Z P indZ indP ] = Polyhedron2zonotope(P)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Convert a Polyhedron object P into a zonotope object Z
%
%   [ Z P indZ indP ] = POLYHEDRON2ZONOTOPE(P)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% Polyhedron or Polyhderon array P
%
% ---------------------------------------------------------------------------
% OUTPUT                                                                                                    
% ---------------------------------------------------------------------------
% Z zonotope (or Zonotope Array) updated with its H-representation
% P Polyhderon (or Polyhderon Array)
% indZ such that P(indZ) are zonotopes
% indP such that P(indP) are not zonotope
%
% see also ZONOTOPE

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
% (C) 2012 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if ~isa(P, 'Polyhedron')
        error('ISZONOTOPE: Argument MUST be a Polyhedron object');
    end
    
    indZ = [];
    indP = [];
    Z  = zonotope;
    PV = Polyhedron;
    
    if length(P)~=1
        for j=1:length(P)
            if iszonotope(P(j))
                indZ = [indZ j];
                Z    = [ Z P2Z(P(j)) ];
            else
                indP = [indP j];
                PV   = [ PV P(j) ];
            end
        end
        P = PV;
        return;
    end
    
    if ~iszonotope(P)
        indZ = 0;
        indP = 1;
        Z    = [];
        return
    end
    
    indZ = 1;
    indP = 0;
    Z    = P2Z(P);
    P    = [];
    
end



function [ Z, PU ] = P2Z(P)

    global MPTOPTIONS
    if isempty(MPTOPTIONS)
        MPTOPTIONS = mptopt;
    end
    
    nx = P.Dim;
    
    P = P.computeHRep;
    H = P.A;
    K = P.b;
    if all(H*zeros(nx,1)<=K)
        % check if iper rectangular
        H = H./repmat(K,1,size(H,2));
        K = ones(size(H,1),1);
        if all(abs(sum(H,1))<=MPTOPTIONS.abs_tol)
            PU  = P;
            Z   = zonotope(zeros(nx,1),diag(2./sum(abs(H),1)),P);
            return
        end
    end
    
    
    tmp.A          = H;
    tmp.B          = K;
    [ vstr, adjV ] = cddmex('adj_extreme',tmp);
    vertex         = vstr.V;
    
    
    %formulate LMI
    S=sdpvar(nx,nx);                                            %Ellipsoid variables ||Sx+b||<=1
    p=sdpvar(nx,1);                                             %Ellipsoid variables ||Sx+b||<=1
    
    myprog=lmi;
    myprog=myprog + set('S>=10^-6');                            %add constraint for each vertex
    for i=1:size(vertex,1)
        myprog=myprog+set('||S*vertex(i,:)''+p||<1');           %add constraint for each vertex
    end
    if size(vertex,1)<nx
        tmp.A = [eye(nx);-eye(nx)];
        tmp.B = 100*max(max(abs(vertex)))*ones(nx*2,1);
        V     = cddmex('extreme',tmp);
        V     = V.V;
        for i=1:size(V,1)
            myprog=myprog+set('||S*V(i,:)''+p||<1');
        end
    end
    
    options = sdpsettings('verbose',0);
    res = solvesdp(myprog,-logdet(S),options);                  %find solution using LMI solver
    if res.problem~=0
        error(['POLYTOPE2ZONO: ' res.info]);
    end
    Ss = double(S);                                             %extract results
    p  = double(p);                                             %extract results
    p  = -Ss^-1*p;
    
    if issparse(p),
        p = full(p);
    end
    p(abs(p)<=MPTOPTIONS.abs_tol) = 0;
    
    if all(H*zeros(nx,1)<=K)
        % check if iper rectangular
        K = K-H*p;
        H = H./repmat(K,1,size(H,2));
        if all(abs(sum(H,1))<=MPTOPTIONS.abs_tol)
            PU = P;
            Z  = zonotope(p,diag(2./sum(abs(H),1)),P);
            return
        end
    end
    
    V = vertex-repmat(p',size(vertex,1),1);
    
    G = [];
    
    for i=1:size(V,1)
        for jj=adjV{i}
            if jj<=i || any(vstr.lin==jj)
                continue;
            end
            G(:,end+1) = (V(i,:)-V(jj,:))/2';
        end
    end
    if issparse(G),
        G = full(G);
    end
    G(abs(G)<=MPTOPTIONS.abs_tol) = 0;
    G   = G(:,any(G~=0));
    ind = all(G<=0);
    G(:,ind) = -G(:,ind);
    [ ~ , index ] = sortrows(abs(G)',1:size(G,1));
    G = G(:,index);
    [ ~ , index ] = sort(sum(abs(G)),'descend');
    G = G(:,index);
    
    
    i=1;
    while i<size(G,2)
        j = i+1;
        while j<=size(G,2)
            if all(abs(abs(G(:,i))-abs(G(:,j)))<=MPTOPTIONS.abs_tol)
                G(:,j) = [];
            else
                j = j+1;
            end
        end
        i = i+1;
    end
    
    if G(1,1)<0
        G = -G;
    end
    
    Z = zonotope(p,G,P);
end