function status = iszonotope(Pn)
% ISZONOTOPE Check if a polyhedron is a zonotope
%
% status = iszonotope(Pn)
%
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Check if the Polyhedron object Pn is a zonotope
%
% USAGE:
%       status = iszonotope(Pn)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% Pn   - Polyhedron or Polyhedron array
%
% ---------------------------------------------------------------------------
% OUTPUT                                                                                                    
% ---------------------------------------------------------------------------
% status  - 1 if the given Polyhedron is a zonotope, 0 otherwise
%
% see also ZONOTOPE, POLYHEDRON

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
% (C) 2012 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    global MPTOPTIONS
    if isempty(MPTOPTIONS)
        MPTOPTIONS = mptopt;
    end
    
    if ~isa(Pn, 'Polyhedron')
        error('ISZONOTOPE: Argument MUST be a Polyhedron object');
    end
    
    n = length(Pn);
    
    status = ones(n,1);
    
    for j=1:length(Pn)
        Pn(j) = Pn(j).computeHRep;
        H     = Pn(j).A;
        K     = Pn(j).b;
        
        if mod(size(H,1),2)==0
            [H, IX] = sortrows(H);
            K = K(IX);
            
            for i=1:size(H,1)/2
                if max(H(i,:)+H(end+1-i,:))>MPTOPTIONS.abs_tol && max(K(i,1)+K(end+1-i,1))>MPTOPTIONS.abs_tol
                    status(j) = 0;
                    continue;
                end
            end
        else
            status(j) = 0;
            continue;
        end
    end
end