function coindex = ctrbindex(A,B)
% CTRBINDEX Computes the controllability index for the pair (A,B)
%
% coindex = ctrbindex(A,B)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Compute the controllability index for the pair (A,B)
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% A matrix n x n
% B matrix n x m
%  
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% coindex controllability index
%
%
% see also CTRB

% Copyright is with the following author(s):
%
% (C) 2014 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it
% (C) 2012 
%     Stefano Riverso
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin>0 && ~isnumeric(A)
        error('CTRBINDEX: parameters input')
    end   
    
    if verLessThan('matlab', '7.13')
        nargchk(2,2,nargin); %#ok<NCHKI>
    else
        narginchk(2,2);
    end
    
    A = full(A);
    B = full(B);
    if isempty(B)
        coindex = rank(A);
        return
    end
    
    
    % Dimension checking
    n  = size(A,1);
    nu = size(B,2);
    if ~isequal(n,size(A,2),size(B,1)),
        error('CTRBINDEX: dimensions of A and B')
    end
    
    mu         = ones(nu,1);
    co         = zeros(n,n*nu);
    co(:,1:nu) = B;
    tmprank    = rank(co);
    
    if tmprank~=n
        coins  = zeros(n,n,nu);
        for i=1:nu
            coins(:,1,i) = B(:,i);
        end
        for k=1:n-1
            for i=1:nu
                coins(:,k+1,i)       = A * coins(:,k,i);
                co(:,nu+(k-1)*nu+i)  = coins(:,k+1,i);
                tmp                  = rank(co);
                if tmprank<tmp
                    mu(i)            = mu(i) + 1;
                    tmprank          = tmp;
                end
                if tmprank == n
                    break;
                end
            end
            if tmprank == n
                break;
            end
        end
    end
    coindex = max(mu);
end