function [ ctrl, objlss ] = createCtrlPnPMPC4lsmodel( objlsmodel , N , k , m , p , options )
% CREATECTRLPNPMPC4LSMODEL design a Plug-and-Play controller for each subsystem of a large-scale system as proposed in [1]
%
% [ ctrl objlss ] = createCtrlPnPMPC4lsmodel( objlsmodel , N , k , m , p , options )
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Design a Plug-and-Play controller for each subsystem of a large-scale system
%
% The function is written for Parallel Computing Toolbox.
% To take advantages of parallelization, before this function one can run 
% matlabpool(Ncore), with Ncore number of core.
% 
% [ ctrl objlss ] = createCtrlPnPMPC4lsmodel( objlsmodel , N , k , m , p , options )
%
% USAGE:
% ctrl = createCtrlPnPMPC4lsmodel( objlsmodel , N , k , m , p )
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - objlsmodel is a large-scale system object of class LSmodel [2]
% - N is the horizon of the prediction. It can be a vector of lss.numSys
% elements
% - k must be >= of the controllability index of the pair (Aii,Bii). One can
% also set a cell array {kmin,kmax} in order to try different k. If k, kmin
% and kmax are vectors of lss.numSys elements, the i-th k, kmin and kmax are
% used to design the Plug-and-Play controller
% - m is an array of integer numbers. The k-th external input of the
%   LSmodel object is
%       * a centralized input for the lss object if m(k)=0
%       * a exogenous input for the lss object if m(k)=-1
%       * a local input for the i-th subsystem of the lss object if m(k)=i
% - p is an array of integer numbers. The k-th external output of the
%   LSmodel object is
%       * a local output for the i-th subsystem of the lss object if p(k)=i
% - options from YALMIP, i.e. sdpsettings object (optional)
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% - ctrl is an array of lss.numSys elements. The i-th element is a pnpmpc object
% for the i-th subsystem
% - objlss is the lss object obtained from LSmodel object
% 
%
% [1] S. Riverso, M. Farina, and G. Ferrari-Trecate, Plug-and-Play Model 
% Predictive Control based on robust control invariant sets, Pavia, Italy, 
% 2012. url={ arXiv:1210.6927 }
%
% [2] D. Barcelli, N. Bauer, and P. Trnka, WIDE Toolbox. 2012. 
% url = {http://ist-wide.dii.unisi.it/index.php?p=toolboxsp}
%
% see also PNPMPC, CREATECTRLPNPMPC, LSMODEL2LSS
    
% Copyright is with the following author(s):
%
% (C) 2012 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if ~isa(objlsmodel,'LSmodel')
        error('CREATECTRLPNP4LSMODEL: first argument must be a LSmodel object')
    end
    
    if nargin<5 || nargin>6
        error('CREATECTRLPNP: number of input arguments')
    end

    if nargin <=5 || isempty(options)
        options = sdpsettings;
    end
    
    objlss = lsmodel2lss(objlsmodel,m,p);  
    flag.k = k;
    ctrl   = createCtrlPnPMPC( objlss , N , flag , options );
    
end