function objlss = lsmodel2lss( objlsmodel , m , p )
% LSMODEL2LSS Converts an LSmodel object to an lss object
%
% ctrl = lsmodel2lss( objlsmodel , m , p )
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------
% Converts an LSmodel object to an lss object. LSmodel is a class of WIDE
% toolbox [1].
% 
% objlss = lsmodel2lss( objlsmodel , m , p )
%
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% - objlsmodel is a large-scale system object of class LSmodel [1]
% - m is an array of integer numbers. The k-th external input of the
%   LSmodel object is
%       * a centralized input for the lss object if m(k)=0
%       * a exogenous input for the lss object if m(k)=-1
%       * a local input for the i-th subsystem of the lss object if m(k)=i
% - p is an array of integer numbers. The k-th external output of the
%   LSmodel object is
%       * a local output for the i-th subsystem of the lss object if p(k)=i
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% objlss is an lss object
% 
%
% [1] D. Barcelli, N. Bauer, and P. Trnka, WIDE Toolbox, 2012. 
% url = {http://ist-wide.dii.unisi.it/index.php?p=toolboxsp}
%
% see also LSS, LSMODEL
    
% Copyright is with the following author(s):
%
% (C) 2012 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso(at)unipv.it, giancarlo.ferrari(at)unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------


    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end
    
    if nargin<3 || nargin>4
        error('LSMODEL2LSS: first argument must be a LSmodel object')
    end

    if ~isa(objlsmodel,'LSmodel')
        error('LSMODEL2LSS: first argument must be a LSmodel object')
    end
    
    if objlsmodel.nw~=length(m)
        error('LSMODEL2LSS: second argument must be an array of objlsmodel.nw elements')
    end
    
    if objlsmodel.nz~=length(p)
        error('LSMODEL2LSS: third argument must be an array of objlsmodel.nz elements')
    end 
    
    objlss  = lss;
    
    for i = 1:objlsmodel.N
        objlss = objlss.addSystem( objlsmodel.M{i} );
    end
    
    if any(m<-1) || any(m>objlss.numSys)
        error('LSMODEL2LSS: elements of m')
    end
    
    if any(p<-1) || any(p>objlss.numSys)
        error('LSMODEL2LSS: elements of m')
    end
    
    Ts      = objlss.Ts;
    name    = objlss.name;
    namex   = objlss.namex;
    
    ni      = objlss.ni;
   
    sysss   = ss(objlsmodel);
    
    [ Abar, Bbar, Cbar, Dbar ] = ssdata(sysss);
    
    mibar   = zeros(1,objlss.numSys);
    pibar   = zeros(1,objlss.numSys);
    
    n       = sum(ni);
    
    B       = zeros(n,objlsmodel.nw);
    C       = zeros(objlsmodel.nz,n);
    D       = zeros(objlsmodel.nz,objlsmodel.nw);
    Bcen    = zeros(n,objlsmodel.nw);
    Dcen    = zeros(objlsmodel.nz,objlsmodel.nw);
    M       = zeros(n,objlsmodel.nw);
    N       = zeros(objlsmodel.nz,objlsmodel.nw);
    numIcen = 0;
    numExo  = 0;
    nameu   = cell(1,objlss.numSys);
    namey   = cell(1,objlss.numSys);
    for i=1:objlss.numSys
        nameu{i} = {};
        namey{i} = {};
    end
    
    for i=1:objlsmodel.nw
        if m(i)==0
            numIcen           = numIcen + 1;
            Bcen(:,numIcen)   = Bbar(:,i);
            Dcen(:,numIcen)   = Dbar(:,i);
            nameucen{numIcen} = objlsmodel.Wnames{i};
        elseif m(i)==-1
            numExo            = numExo + 1;
            M(:,numExo)       = Bbar(:,i);
            N(:,numExo)       = Dbar(:,i);
            named{numExo}     = objlsmodel.Wnames{i};
        else
            mibar(m(i))       = mibar(m(i)) + 1;
            B(:,sum(mibar))   = Bbar(:,i);
            D(:,sum(mibar))   = Dbar(:,i);
            nameu{m(i)}{length(nameu{m(i)})+1} = objlsmodel.Wnames{i} ;
        end
    end
    
    for i=1:objlsmodel.nz
        pibar(p(i))     = pibar(p(i)) + 1;
        C(sum(pibar),:) = Cbar(i,:);
        namey{p(i)}{length(namey{p(i)})+1} = objlsmodel.Znames{i} ;
    end
    
    Bcen = Bcen(:,1:numIcen);
    Dcen = Dcen(:,1:numIcen);
    M    = M(:,1:numExo);
    N    = N(:,1:numExo);
    B    = B(:,1:objlsmodel.nw-numIcen-numExo);
    D    = D(:,1:objlsmodel.nw-numIcen-numExo);
    
    if isempty(B)
        B = [];
        D = [];
    end
    
    objlss = lss(Abar,B,ni,mibar,C,D,pibar,name,Ts);
    if ~isempty(Bcen)
        objlss = objlss.addCentralizedInput(Bcen,Dcen);
    end
    if ~isempty(M)
        objlss = objlss.addExoInput(M,N);
    end
    
    for j=1:size(Bcen,2)
        objlss = objlss.setNameucen(j,nameucen{j});
    end
    for j=1:size(M,2)
        objlss = objlss.setNamed(j,named{j});
    end
    
    for i=1:objlss.numSys
        objlss = objlss.setNamex(i,namex{i});
        objlss = objlss.setNameu(i,nameu{i}');
        objlss = objlss.setNamey(i,namey{i}');
    end
    
    
    namex = objlss.namex;
    for i = 1:length(namex)
        xtmp = namex{i};
        for j = 1:length(xtmp)
            nmx = xtmp{j};
            for k = 1:length(objlsmodel.signal)
                if strcmp(objlsmodel.signal(k).name,nmx)
                    Hx     = zeros(1,ni(i));
                    Hx(j)  = 1;
                    xmin   = objlsmodel.signal(k).limit.min;
                    if xmin~=-inf
                        objlss = objlss.addStateConstraint(i,-Hx,-xmin);
                    end
                    xmax   = objlsmodel.signal(k).limit.max;
                    if xmax~=inf
                        objlss = objlss.addStateConstraint(i,Hx,xmax);
                    end
                end
            end
        end
    end
    
    
    nameu = objlss.nameu;
    for i = 1:length(nameu)
        utmp = nameu{i};
        for j = 1:length(utmp)
            nmu = utmp{j};
            for k = 1:length(objlsmodel.signal)
                if strcmp(objlsmodel.signal(k).name,nmu)
                    Hu     = zeros(1,objlss.mi(i));
                    Hu(j)  = 1;
                    umin   = objlsmodel.signal(k).limit.min;
                    if umin~=-inf
                        objlss = objlss.addInputConstraint(i,-Hu,-umin);
                    end
                    umax   = objlsmodel.signal(k).limit.max;
                    if umax~=inf
                        objlss = objlss.addInputConstraint(i,Hu,umax);
                    end
                    
                    umin   = objlsmodel.signal(k).limit.smin;
                    if umin~=-inf
                        objlss = objlss.addDeltaUConstraint(i,-Hu,-umin);
                    end
                    umax   = objlsmodel.signal(k).limit.smax;
                    if umax~=inf
                        objlss = objlss.addDeltaUConstraint(i,Hu,umax);
                    end
                end
            end
        end
    end
    
    
    nameucen = objlss.nameucen;
    for j = 1:length(nameucen)
        nmu = nameucen{j};
        for k = 1:length(objlsmodel.signal)
            if strcmp(objlsmodel.signal(k).name,nmu)
                Hu     = zeros(1,numIcen);
                Hu(j)  = 1;
                umin   = objlsmodel.signal(k).limit.min;
                if umin~=-inf
                    objlss = objlss.addCentralizedInputConstraint(j,-Hu,-umin);
                end
                umax   = objlsmodel.signal(k).limit.max;
                if umax~=inf
                    objlss = objlss.addCentralizedInputConstraint(j,Hu,umax);
                end
            end
        end
    end    
    
    
    named = objlss.named;
    for j = 1:length(named)
        nmu = named{j};
        for k = 1:length(objlsmodel.signal)
            if strcmp(objlsmodel.signal(k).name,nmu)
                Hu     = zeros(1,numExo);
                Hu(j)  = 1;
                umin   = objlsmodel.signal(k).limit.min;
                if umin~=-inf
                    objlss = objlss.addExoConstraint(j,-Hu,-umin);
                end
                umax   = objlsmodel.signal(k).limit.max;
                if umax~=inf
                    objlss = objlss.addExoConstraint(j,Hu,umax);
                end
            end
        end
    end  
    
    pi = objlss.pi;
    if ~isempty(pi)
        namey = objlss.namey;
        for i = 1:length(namey)
            ytmp = namey{i};
            for j = 1:length(ytmp)
                nmy = ytmp{j};
                for k = 1:length(objlsmodel.signal)
                    if strcmp(objlsmodel.signal(k).name,nmy)
                        Hy     = zeros(1,pi);
                        Hy(j)  = 1;
                        ymin   = objlsmodel.signal(k).limit.min;
                        if ymin~=-inf
                            objlss = objlss.addOutConstraint(i,-Hy,-ymin);
                        end
                        ymax   = objlsmodel.signal(k).limit.max;
                        if ymax~=inf
                            objlss = objlss.addOutConstraint(i,Hy,ymax);
                        end
                    end
                end
            end
        end
    end
end