function test = isinside(obj,x,options)
% ISINSIDE Checks if a given point x lies in an epsilon-mRPI set.
%
% test = isinside(obj,x,options)
% 
% ---------------------------------------------------------------------------
% DESCRIPTION
% ---------------------------------------------------------------------------  
% Check if a given point x lies in an epsilon-mRPI object.
% The code is based on an implicit representation of the epsilon-mRPI and
% exploits linear programming.
% 
% USAGE: 
%  
% test = obj.ISINSIDE(x,options)
% test = obj.ISINSIDE(x)
% 
% ---------------------------------------------------------------------------
% INPUT
% ---------------------------------------------------------------------------
% x = point of the same dimension of the epsilon-mRPI set
% 
% options = sdpsettings object for YALMIP (optional)
% 
% ---------------------------------------------------------------------------
% OUTPUT
% ---------------------------------------------------------------------------
% test = 1 if the point is inside, 0 otherwise
%
% see also EPSILON_MRPI

% Copyright is with the following author(s):
%
% (C) 2012 
%     Stefano Riverso, Giancarlo Ferrari Trecate
%     Identification and Control of Dynamic Systems Laboratory,
%     Universita' degli Studi di Pavia, Italy
%     stefano.riverso@unipv.it, giancarlo.ferrari@unipv.it
%

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% ---------------------------------------------------------------------------

    global pnpmpcToolboxGlobalOptions;
    if ~isstruct(pnpmpcToolboxGlobalOptions),
        pnpmpc_toolbox_err;
    end

    if nargin~=3
        options = sdpsettings;
    end
    
    if size(x,1)~=size(obj.A,1) || size(x,2)~=1
        error('ISINSIDE: parameters dimensions');
    end
    
    if ~isa(obj.W,'zonotope')
        lambda = sdpvar(obj.s,size(obj.vertMink.V,2));
        xV     = reshape(obj.vertMink.V,size(obj.A,1),[]);
        info   = solvesdp([ lambda(:)>=0 ; sum(lambda,2)==1 ; x == sum(repmat(reshape(lambda',1,[]),size(obj.A,1),1).*xV,2) ],0,options);
    else
        d = sdpvar(size(obj.vertMink.G,2),1);
        info   = solvesdp([x==obj.vertMink.p+obj.vertMink.G*d;norm(d,inf)<=1],0,options);
    end
    
    if info.problem == 0
        test = true;
    elseif info.problem == 1
        test = false;
    else
        error('ISINSIDE: ERROR SOLVER: %s',info.info);
    end
end